// file:        ActionImagesLoader.java
// source file: ActionImagesLoader.java
// object file: ActionImagesLoader.class
//
//
// included objects:
//  ActionImagesLoader
//    Desc: implements a class that loads all action images
//
// history:
//   01.04.97 - first base version finished by Roman Valehrach
//   04.04.97 - added MediaTracker for image-download
//
//   14.04.97 - first version finished by Roman Valehrach
//
//
// critical problems: none
//
// known problems: none
//

import java.awt.*;
import java.awt.image.*;
import java.util.*;
import java.lang.*;
import java.net.*;
import java.io.*;
import java.util.Properties;
// import java.applet.*;
// import sun.applet.*;
import java.applet.Applet;

class ActionImagesLoader extends Resources
{
  // private static final boolean DEBUG = false;

  protected static DefaultProps        props_;
  protected static ResourceLoader      resloader_;

  protected        ImageObserver       myobs_;

  private static final String DELIMITER         = ".";

//private static final String ACTIONS_BASE1     = "envs.wumpus.action" + DELIMITER;
//private static final String ACTIONS_BASE2     = "envs.vacuum.action" + DELIMITER;
  private static final String ACTIONIMAGES_BASE = "actionimages" + DELIMITER;

  private static final String TOTAL_ID          = "TOTAL";
  private static final String NAME_ID           = DELIMITER + "STR";
  private static final String IMAGE_ID          = DELIMITER + "IMG";

/* obsolet - using properties now

  private static final String actions__[][] = { // constants for image-parameter
    {ActionConstants.STR_UP,          "actionimage.ACTION_UP"},
    {ActionConstants.STR_DOWN,        "actionimage.ACTION_DOWN"},
    {ActionConstants.STR_LEFT,        "actionimage.ACTION_LEFT"},
    {ActionConstants.STR_RIGHT,       "actionimage.ACTION_RIGHT"},

    {ActionConstants.STR_WAIT,        "actionimage.ACTION_WAIT"},
    {ActionConstants.STR_GRAB,        "actionimage.ACTION_GRAB"},
    {ActionConstants.STR_CLIMB,       "actionimage.ACTION_CLIMB"},

    {ActionConstants.STR_SHOOT_UP,    "actionimage.ACTION_SHOOT_UP"},
    {ActionConstants.STR_SHOOT_DOWN,  "actionimage.ACTION_SHOOT_DOWN"},
    {ActionConstants.STR_SHOOT_LEFT,  "actionimage.ACTION_SHOOT_LEFT"},
    {ActionConstants.STR_SHOOT_RIGHT, "actionimage.ACTION_SHOOT_RIGHT"},

    {DefaultConstants.STR_QUESTION,    "actionimage.QUESTION"}
  };

  private static final String actionsn__[][] = { // constants for image-parameter
    {ActionConstants.STR_UP,          DefaultConstants.IMG_ACTION_UP},
    {ActionConstants.STR_DOWN,        DefaultConstants.IMG_ACTION_DOWN},
    {ActionConstants.STR_LEFT,        DefaultConstants.IMG_ACTION_LEFT},
    {ActionConstants.STR_RIGHT,       DefaultConstants.IMG_ACTION_RIGHT},

    {ActionConstants.STR_WAIT,        DefaultConstants.IMG_ACTION_WAIT},
    {ActionConstants.STR_GRAB,        DefaultConstants.IMG_ACTION_GRAB},
    {ActionConstants.STR_CLIMB,       DefaultConstants.IMG_ACTION_CLIMB},

    {ActionConstants.STR_SHOOT_UP,    DefaultConstants.IMG_ACTION_SHOOT_UP},
    {ActionConstants.STR_SHOOT_DOWN,  DefaultConstants.IMG_ACTION_SHOOT_DOWN},
    {ActionConstants.STR_SHOOT_LEFT,  DefaultConstants.IMG_ACTION_SHOOT_LEFT},
    {ActionConstants.STR_SHOOT_RIGHT, DefaultConstants.IMG_ACTION_SHOOT_RIGHT},

    {DefaultConstants.STR_QUESTION,   DefaultConstants.IMG_QUESTION}
  };
*/


// Method: ActionImagesLoader Constructor (public)
//
// *Purpose   : intializes a new instance and sets a new image (-resource) 
//              for the object
//              - a MediaTracker will be created automatically
//               (if none is given or if the imageOffset is not -1)
//
// *Parameters: none
//
// *Return    : none
//
// *Variables : resloader_ (ResourceLoader, class) - where the ActionImagesLoader
//                get's its images from
//              imgobs (ImageObserver) - an observer for all images
//
  public ActionImagesLoader()
  {
    this( null );
  }

  public ActionImagesLoader( ImageObserver imgobs ) //throws NullPointerException
  {
    super();

    if (resloader_ == null)
      resloader_ = new ResourceLoader();

    myobs_ = imgobs;

    if (props_ == null)
    {
      Properties prop = DefaultProps.getDefaultProperties();

      if ( prop == null ) // DefaultProperties initialized ?
      {
        System.err.println("Applet-Properties must be set before using ActionImagesLoader! ");
        //throw new NullPointerException();
        return;
      } 
      else {
        props_= new DefaultProps();
      }
    }

    getResources();
  }


// Method: getResources (public)
//
// *Purpose   : gets all possible/needed Action-Images
//
// *Parameters: none
//
// *Return    : Image - a new highlightend Image-instance
//
// *Variables : images (Vector, class) - the current image-resources
//              baseurl_ (URL, class) - the basic URL for loading
//              tracker_ (MediaTracker, class) - for tracking the loading
//              currentTID_ (int, class) - the Tracker-ID for the next image
//              increasedTracker_ (boolean, class) - increase the Tracker-ID ?
//
  public void getResources()
  {
    ImageReference newref= null;
    Image temp = null;
    String tempstr[] = new String[2];
    URL currentURL;

/* 
    if ( DEBUG )
    {
      System.out.println( "Current Simulator Properties - BEGIN" );
      props_.list(System.out);
      System.out.println( "Current Simulator Properties - ENDE" );
    }
*/

    int i;  // = getActionsNumber();
    int j = getActionImagesNumber();

//    if (i < j) j= i;
    j++; // end  < max +1

    for ( i=1; i < j; i++) {
      try
      {
        if ( getActionImageProperty(i, tempstr ) > 0 ) 
        {
          // System.out.println("PICs to load.. "+tempstr[0]+", "+tempstr[1]);

          if ( resloader_ == null ) {
            resloader_ = new ResourceLoader();
          }

          // for ActionImages: use transpareny Filter & use Res.Loaders-TrackIDs
          temp = resloader_.getImage( tempstr[1], -1, 20 ); 
        }
      }
      catch (NullPointerException e) 
      {
        System.err.println("ActionImagesLoader - NullPointerExecption Exception: " + e);
        temp = null;
      }

      if ( temp != null )
      {
//        System.out.println("Loading the action icon >" + 
//          tempstr[0] + "<");

        // add the imag to own resourcelist...
        newref= new ImageReference( tempstr[0], temp, myobs_ );
        addImageReference( newref );
      }
      else {
        if ( tempstr[0] != null )
          System.err.println("Encountered an Error while loading the action icon >" + 
            tempstr[0] + "<");
        else
          System.err.println("Encountered an Error while loading an action icon");
      }
    }
  }

  public int getActionImageProperty(int i, String temp[])
  // get ActionImage (Icons - String-Name and Image (Path/File, links possible)
  {
    String tmp;
    if ( (temp== null) || (temp.length < 2) )
    {
      return -3;
    }

    if ((temp[0]= props_.getProperty( ACTIONIMAGES_BASE + i + NAME_ID )) == null)
    {
      return -2;
    }
    
    if ( (tmp= props_.getProperty( ACTIONIMAGES_BASE + i + IMAGE_ID )) == null)
    {
      return -1;
    }

    if ( ( tmp = props_.checkLink( tmp ) ) == null )
    {
      return 0;
    }

    // System.err.println("ending action-image-name-fetch.. "+temp[0]+", "+temp[1]+", "+tmp);
    temp[1]= tmp;

    return 1;
  }

/* obsolete - for now !
//  get all the actions (in all envs) -> resulting images -> ...
//  if this shall be used - multiple filemanagement is needed...
  public int getActionsNumber()
  {
    return props_.getNumber( ACTION_BASE1 + TOTAL_ID );
    return props_.getNumber( ACTION_BASE2 + TOTAL_ID );
  }
*/

  public int getActionImagesNumber()
  {
    return props_.getNumber( ACTIONIMAGES_BASE + TOTAL_ID, 0 );
  }


// Method: checkLoading (public)
//         checkTracker (public)
//
// *Purpose   : checks the loading of all  Image-Rsources
//
// *Parameters: baseurl (URL/String) - the basic url of the current browser
//              newimg (Image) - the new image
//              newobs (ImageObserver) - a new observer for the image
//
// *Return    : boolean - was the loading successful ?
//
// *Variables : tracker_(MediaTracker, class) - for tracking the loading
//              trackerOffset_(int, class) - the offset for tracker, 
//                                           ^=where the tracker-ID starts
//              currentTID_(int, class) - the Tracker-ID for the next image
//              increasedTracker_ (boolean, class) - increase the Tracker-ID ?
//
  public boolean checkLoading()
  {
    if (resloader_ == null) return true;

    return resloader_.checkLoading();
  }

  public boolean checkTracker()
  {
    if (resloader_ == null) return true;

    return resloader_.checkTracker();
  }

}// End of class ActionImages
