import java.util.*;
import java.awt.*;
import java.awt.image.ImageObserver;

//
//
// Class
// ArrayBackWumpus
// Wumpus-Agent that has an internal represantation of the environment
// it supersedes its pedecessor with improved capabilities in
// reasoning
// It is slightly 'goal - based': if gold was found it returns home
// immediatly.
//
// creation 31.03.1997 
//
// simplified internal logic 04.04.1997
//
// 
//

class ArrayBackWumpus extends ArrayWumpus
{
  static final String type = "ARRAYBACKWUMPUS";
  protected AdvancedAgentField myfield[] [];

  protected Vector wayback, shooter; // represents the way home
  boolean gohome;

  public ArrayBackWumpus( String name, int ammo, Dimension size )
  {
    this( name, ammo, false, size, ( String ) null, ( ImageObserver ) null, type );
  }
 
  public ArrayBackWumpus( String name, int ammo, boolean takerisk, Dimension size )
  {
    this( name, ammo, takerisk, size, ( String ) null, ( ImageObserver ) null, type );
  }
 
  public ArrayBackWumpus( String name, int ammo, Dimension size, String imgref )
  {
    this( name, ammo, false, size, imgref, ( ImageObserver ) null, type );
  }

  public ArrayBackWumpus( String name, int ammo, Dimension size, ImageObserver obs )
  {
    this( name, ammo, false, size, ( String ) null, obs, type );
  }

  public ArrayBackWumpus( String name, int ammo, boolean takerisk, Dimension size, 
                          String imgref, ImageObserver obs )
  {
    this( name, ammo, takerisk, size, imgref, obs, type );
  }// End of ArrayBackWumpus-Constructors

  public ArrayBackWumpus( String name, int ammo, boolean takerisk, Dimension size, 
                          String imgref, ImageObserver obs, String type )
  {
    super( name, ammo, takerisk, size, imgref, obs, type );
  }// End of ArrayBackWumpus-Constructors

  protected void initVars()
  {
    super.initVars();
    if( log == true )
    {
      System.out.println(" ArrayBack: initializing new vars ");
    }
    wayback = new Vector( 20, 10 );
    shooter = new Vector( 3, 1 );
    gohome = false;
  }

  protected void initField( Dimension fieldsize )
  {
    // initialize the field - represantation
	myfield = new AdvancedAgentField [ fieldsize.width ] [ fieldsize.height ];
    if( log == true )
    {
      System.out.println("ArrayBack: width = " + fieldsize.width);
      System.out.println("ArrayBack: height = " + fieldsize.height);
      System.out.println("ArrayBack: allocating myfield");
    }
    for( int i = 0; i < myfield.length; i++ )
	{
	  for( int j = 0; j < myfield[ i ].length; j++ )
	  {
		myfield[ i ] [ j ] = new AdvancedAgentField();
        if( log == true )
        {
           System.out.println("allocated myfield" + i + j );
        }
	  }
	}
    
    if( log == true )
    {
      System.out.println("finished field allocation \n");
    }
	// Agents start at location ( 1, 1 ); this is also the exit
    location = new Point( 1, 1);
    // do still assign x, y their coordinates for clarity
    location.x = 1;
	location.y = 1;
	myfield[ location.x ] [ location.y ].setExit( true );
	myfield[ location.x ] [ location.y ].setVisited( true );
    myfield[ location.x ] [ location.y ].setWumpus( false );
    myfield[ location.x ] [ location.y ].setPit( false );
	myfield[ location.x ] [ location.y ].incVisited();
    
    // up and left are wall because we always start in the upper left corner
    // this has to be removed if the agent should start anywhere in the maze
    myfield[ location.x - 1 ] [ location.y ].setWall( true );
    myfield[ location.x ] [ location.y - 1 ].setWall( true );
    
    // to be able to use update from super shadowed var must
    // be 'initialized here 
    super.myfield = myfield;
  }

  protected void finalize()
  {
    super.finalize();
    myfield = null; 
    wayback = null;
    shooter = null;
  }
  
  public void receivePercepts( Vector percepts )//where the agent receives his percepts
  {
    super.receivePercepts( percepts );
    if( gohome == false )
    {
      updateWayBack();
    }
    if( log == true )
    {
      System.out.println(" May way home includes the following steps: " );
      if( wayback.isEmpty() == false )
      {
        for( int i = 0; i < wayback.size(); i++ )
          System.out.println(" Step " + i + " = " + ( ( Action ) wayback.elementAt( i ) ).getName() + "\n" );
      }
      else
      System.out.println(" I guess I am already at my exit field \n" );
    }
    
    
    if( log == true && getType().equals( "ARRAYBACKWUMPUS" ) )
    { 
      System.out.println("myaction: " + ( ( Action ) actions.elementAt( 0 ) ).getName() + "\n" );
    }
  }	 

  protected void updateField()
  {
    if( log == true && getType().equals( "ARRAYBACKWUMPUS" ) )
    {
      System.out.println( "Starting to work on action at move number: " + ( moves + 1 ) );
      System.out.println( "Calling ancestors update method first" );
    }

    // call dad's version
    super.updateField();
    
    if( log == true )
    {
      System.out.println(" Now starting advanced field updating " );
    }
    // now declare up, down, left, right 
    AdvancedAgentField up, down, left, right;

    // if it bumped and we thought to move: remove last
    // element of or way - back vector
    if( pure.getBump() == true )
    {
      if( lastaction.getName().equalsIgnoreCase( "Right" ) || 
          lastaction.getName().equalsIgnoreCase( "Left" ) ||
          lastaction.getName().equalsIgnoreCase( "Up" ) ||
          lastaction.getName().equalsIgnoreCase( "Down" ) )
      {
        if( log == true )
        {
          System.out.println(" It bumped: removing last element of wayback " );
        }
        if( wayback.isEmpty() == false )
        {
          wayback.removeElementAt( wayback.size() - 1 );
        }
      }
      checkWayBack();
    }
    else
      checkWayBack();

    // update improved fields if stench/breeze were percepted
    // check for stench and update surrounding fields if wumpus lives
	if(  wumpusalive == true )
	{
      if( pure.getStench() == true )
	  {
        myfield[ location.x ] [ location.y ].setStench( true );
      }
    }

    if( pure.getBreeze() == true )
	{
      myfield[ location.x ] [ location.y ].setBreeze( true );
    }

    // set gohome to true if we found a gold
    if( pickedgold == true )
      gohome = true;

    // every ten moves check if something new can be deduced from the
    // percepts we have gotten so far
    if( ( moves % 10 ) == 0 )
    {
      Point checkpoint = new Point( -1 , -1 );
      // check for wumpus and Pit
      // at every field where a stench or a breeze has happened
      if( log == true )
      {
        System.out.println("Starting ' every ten moves field-update-loop'");
      }
      for( int i = 0; i < myfield.length; i++ )
      {
	    for( int j = 0; j < myfield[ i ].length; j++ )
        {
          if( log == true )
          {
            System.out.println(" ArrayBack: Checking field: " + i + " " + j );
          }
		  if( myfield[ i ] [ j ].isBreeze().equals( "YES" ) == true )
          {
            checkpoint.x = i;
            checkpoint.y = j;
            // get surrounding fields
            left  = ( AdvancedAgentField ) getLeftField( checkpoint );
            right = ( AdvancedAgentField ) getRightField( checkpoint );
            up    = ( AdvancedAgentField ) getUpField( checkpoint );
            down  = ( AdvancedAgentField ) getDownField( checkpoint );
            
            // check surrounding fields for Pits
            if( log == true )
            {
              System.out.println( "It breezed at: " + i + " " + j );
              System.out.println( "Checking for new insights" );
            }
            
            // if three fields have no pit the fourth field must contain it
	       if( up.hasPit().equals( "NO" ) && down.hasPit().equals( "NO" ) &&
		       left.hasPit().equals( "NO" ) )
           {
		     right.setPit( true );
           }
	       else if( up.hasPit().equals( "NO" ) && down.hasPit().equals( "NO" ) &&
		            right.hasPit().equals( "NO" ) )
           {
	         left.setPit( true );
           }
	       else if( up.hasPit().equals( "NO" ) && right.hasPit().equals( "NO" ) &&
		            left.hasPit().equals( "NO" ) )
           {
		     down.setPit( true );
           }
	       else if( down.hasPit().equals( "NO" ) && right.hasPit().equals( "NO" ) &&
		            left.hasPit().equals( "NO" ) )
           {
		     up.setPit( true );
           }
         }

          if( ( myfield[ i ] [ j ].isStench().equals( "YES" )  == true ) &&
                wumpusalive == true )
          {
            checkpoint.x = i;
            checkpoint.y = j;

            // get surrounding fields
            left  = ( AdvancedAgentField ) getLeftField( checkpoint );
            right = ( AdvancedAgentField ) getRightField( checkpoint );
            up    = ( AdvancedAgentField ) getUpField( checkpoint );
            down  = ( AdvancedAgentField ) getDownField( checkpoint );
            
            // check surrounding fields for Wumpus
            if( log == true )
            {
              System.out.println( "It stenched at: " + i + " " + j );
              System.out.println( "Checking for new insights" );
            }
            
            // if three fields have no pit the fourth field must contain it
	       if( up.hasWumpus().equals( "NO" ) && down.hasWumpus().equals( "NO" ) &&
		       left.hasWumpus().equals( "NO" ) )
           {
		     right.setWumpus( true );
           }
	       else if( up.hasWumpus().equals( "NO" ) && down.hasWumpus().equals( "NO" ) &&
		            right.hasWumpus().equals( "NO" ) )
           {
	         left.setWumpus( true );
           }
	       else if( up.hasWumpus().equals( "NO" ) && right.hasWumpus().equals( "NO" ) &&
		            left.hasWumpus().equals( "NO" ) )
           {
		     down.setWumpus( true );
           }
	       else if( down.hasWumpus().equals( "NO" ) && right.hasWumpus().equals( "NO" ) &&
		            left.hasWumpus().equals( "NO" ) )
           {
		     up.setWumpus( true );
           }
          }
        }
      }
    }
  }

  protected void reason()
  {
    AgentField left, right, up, down;
    int wumpuscount = 0;
     
    if( log == true )
    {
      System.out.println(" Now starting advanced reasoning " );
      System.out.println(" Should i go home? " + gohome );
      System.out.println(" Did i pick gold? " + pickedgold );
    }
   
    // erase shooter vector if wumpus has died
    if( wumpusalive == false )
      shooter.removeAllElements();    
    
    // check wayback vector for actions
    if( wayback.isEmpty() == false && gohome == true)
    {
      if( log == true )
      {
        System.out.println( " Taking a step towards the exit " );
      }
      actions.addElement( wayback.lastElement() );
      wayback.removeElementAt( wayback.size() - 1 );
      return;
    }

    // check if we reached exit on way home and climb out of the cave
    // if so
    if( wayback.isEmpty() == true && gohome == true )
    {
      if( log == true )
      {
        System.out.println( " I leave the cave now! " );
      }
      actions.addElement( ( Object ) new Action( "Climb" ) );
      return;
    }

    // check shooter vector for actions
    if( shooter.isEmpty() == false )
    {
      if( log == true )
      {
        System.out.println( " I shoot at the wumpus ( hopefully ) " );
      }
      actions.addElement( shooter.lastElement() );
      shooter.removeElementAt( shooter.size() - 1 );
      return;
    }

    // grab gold: this is done again in the super - method but it
    // prevents the agent to shoot the wumpus in case it is in
    // a neighbouring field and arrows suffice to shoot it
    if( pure.getGlitter() == true )
    {
      actions.addElement( ( Object ) new Action( "Grab" ) );
      return;
    }

    // get neighbouring fields
    // this will be done twice if we do not make an action here
    left  = ( AdvancedAgentField ) getLeftField( location );
    if( log == true )
    {
      System.out.println(" left x,y:" + ( location.x - 1 ) + "  " + location.y );
      System.out.println(" Pit: " + left.hasPit() );
      System.out.println(" Wumpus: " + left.hasWumpus() + "\n" );
    }

    right = ( AdvancedAgentField ) getRightField( location );
    if( log == true )
    {
      System.out.println(" right x,y:" + ( location.x + 1 ) + "  " + location.y );
      System.out.println(" Pit: " + right.hasPit() );
      System.out.println(" Wumpus: " + right.hasWumpus() + "\n" );
    }
    
    up    = ( AdvancedAgentField ) getUpField( location );
    if( log == true )
    {
      System.out.println(" up x,y:" + location.x + "  " + ( location.y - 1 ) );
      System.out.println(" Pit: " + up.hasPit() );
      System.out.println(" Wumpus: " + up.hasWumpus() + "\n" );
    }
    
    down  = ( AdvancedAgentField ) getDownField( location );
    if( log == true )
    {
      System.out.println(" down x,y:" + location.x + "  " + ( location.y + 1 ) );
      System.out.println(" Pit: " + down.hasPit() );
      System.out.println(" Wumpus: " + down.hasWumpus() + "\n" );
    }

    // if the location of the wumpus is in any of a number of adjacent field 
    // and we still have enough arrows to shoot in all directions where the
    // wumpus could be and the wumpus still lives
	// kill him 
	// this time the agent checks for two fields of stench and two arrows etc.
    if(pure.getStench() == true && wumpusalive == true )
    {
      if( log == true )
      {
        System.out.println(" Wumpus around; checking " );
      }
      // get all fields where wumpus possibly is
      if( left.hasWumpus().equals( "YES" ) || left.hasWumpus().equals( "MAYBE" ) )
      {
        if( log == true )
        {
          System.out.println(" Wumpus: maybe left ");
        }
        wumpuscount++;
        shooter.addElement( ( Object ) new Action( "Shoot_Left" ) );
      }
      if( right.hasWumpus().equals( "YES" ) || right.hasWumpus().equals( "MAYBE" ) )
      {
        if( log == true )
        {
          System.out.println(" Wumpus: maybe right ");
        }
        wumpuscount++;
        shooter.addElement( (  Object ) new Action( "Shoot_Right" ) );
      }
      if( down.hasWumpus().equals( "YES" ) || down.hasWumpus().equals("MAYBE" ) )
      {
        if( log == true )
        {
          System.out.println(" Wumpus: maybe down ");
        }
        wumpuscount++;
        shooter.addElement( (  Object ) new Action( "Shoot_Down" ) );
      }
      if( up.hasWumpus().equals( "YES" ) || up.hasWumpus().equals( "MAYBE" ) )
      {
        if( log == true )
        {
          System.out.println(" Wumpus: maybe up ");
        }
        wumpuscount++;
        shooter.addElement( (  Object ) new Action( "Shoot_Up" ) );
      }

      // check if the agent has got enough arrows
      // if yes shoot, if not save arrows
      // ( a field with yes will still be dealt with properly )
      if( getArrowNum() < wumpuscount )
      {
        shooter.removeAllElements();
      }
      else // begin to shoot
      {
        if( log == true )
        {
          System.out.println( "I will shoot in the following directions to kill the wumpus: " );
          for( int i = 0; i < shooter.size(); i++ )
            System.out.println( ( ( Action ) shooter.elementAt( i ) ).getName() );
        }
        if( shooter.isEmpty() == false ) // always stay on the save side
        {
          actions.addElement( shooter.lastElement() );
          shooter.removeElementAt( shooter.size() - 1 );
          return;
        }
      }
    }
    // call ancestors reason method
    if( log == true )
    {
      System.out.println(" Advanced reasoning fruitless; starting super.reason() " );
    }
    super.reason();
    
    // set gohome to true if super returned a moving action that leads to a
    // field that has been visited more than five times
    // super reason is guaranteed to return an action
    // as we set gohome to true we must not forget to update the way a last
    // time
    if( actions.isEmpty() == false )
    {
      String help = ( ( Action ) actions.firstElement() ).getName();
      if( help.equals( "Right" ) == true )
      {
        if( myfield[ location.x + 1 ] [ location.y ].visitCount() > 5 )
        {
          if( log == true )
          {
            System.out.println( "I am not content with my poosible actions. I am going home! " );
          }
          gohome = true;
          updateWayBack();
        }
      }
      if( help.equals( "Left" ) == true )
      {
        if( myfield[ location.x - 1 ] [ location.y ].visitCount() > 5 )
        {
          if( log == true )
          {
            System.out.println( "I am not content with my poosible actions. I am going home! " );
          }
          gohome = true;
          updateWayBack();
        }
      }
      if( help.equals( "Up" ) == true )
      {
        if( myfield[ location.x ] [ location.y - 1 ].visitCount() > 5 )
        {
          if( log == true )
          {
            System.out.println( "I am not content with my poosible actions. I am going home! " );
          }
          gohome = true;
          updateWayBack();
        }
      }
      if( help.equals( "Down" ) == true )
      {
        if( myfield[ location.x ] [ location.y + 1 ].visitCount() > 5 )
        {
          if( log == true )
          {
            System.out.println( "I am not content with my poosible actions. I am going home! " );
          }
          gohome = true;
          updateWayBack();
        }
      }
    }

  }// even reasoning ends eventually

  protected void updateWayBack()
  {
    Action thisround;

    // if we visit home during our journey we may cleaar the
    // wayback vector
    if( myfield[ location.x ] [ location.y ].isExit().equals( "YES" ) )
    {
      wayback.removeAllElements();
    }

    //add any moves that do not reverse the last movement
    thisround = ( Action ) actions.firstElement();
    if( thisround.getName().equalsIgnoreCase( "Up" ) == true )
    {
      if( log == true )
      {
        System.out.println(" UpdateWayBack: This round's action was up " );
      }
      if( wayback.isEmpty() == true )
      {
        wayback.addElement( (  Object ) new Action( "Down" )  );
        if( log == true )
        {
          System.out.println(" UpdateWayBack: added down to beginning" );
        }
      }
      else if( ( ( Action ) wayback.lastElement() ).getName()
                   .equalsIgnoreCase( "Up" ) == false )
      {
        wayback.addElement( (  Object ) new Action( "Down" )  );
        if( log == true )
        {
          System.out.println(" UpdateWayBack: added down to wayback " );
        }
      }
      else
      {
        wayback.removeElementAt( wayback.size() - 1 );
        if( log == true )
        {
          System.out.println( "Removed last element of wayback as a result of antagonistic actions" );
        }
      }
      return;
    }
    if( thisround.getName().equalsIgnoreCase( "Down" ) == true )
    {
      if( log == true )
      {
        System.out.println(" UpdateWayBack: This round's action was down " );
      }
      if( wayback.isEmpty() == true )
      {
        wayback.addElement( (  Object ) new Action( "Up" )  );
        if( log == true )
        {
          System.out.println(" UpdateWayBack: added up to beginning" );
        }
      }
      else if( ( ( Action ) wayback.lastElement() ).getName()
                   .equalsIgnoreCase( "Down" ) == false )
      {
        wayback.addElement( (  Object ) new Action( "Up" )  );
        if( log == true )
        {
          System.out.println(" UpdateWayBack: added up to wayback " );
        }
      }
      else
      {
        wayback.removeElementAt( wayback.size() - 1 );
        if( log == true )
        {
          System.out.println( "Removed last element of wayback as a result of antagonistic actions" );
        }
      }
      return;
    }
    if( thisround.getName().equalsIgnoreCase( "Left" ) == true )
    {
      if( log == true )
      {
        System.out.println(" UpdateWayBack: This round's action was left " );
      }
      if( wayback.isEmpty() == true )
      {
        wayback.addElement( (  Object ) new Action( "Right" )  );
        if( log == true )
        {
          System.out.println(" UpdateWayBack: added right to beginning" );
        }
      }
      else if( ( ( Action ) wayback.lastElement() ).getName()
                   .equalsIgnoreCase( "Left" ) == false )
      {
        wayback.addElement( (  Object ) new Action( "Right" )  );
        if( log == true )
        {
          System.out.println(" UpdateWayBack: added right to wayback " );
        }
      }
      else
      {
        wayback.removeElementAt( wayback.size() - 1 );
        if( log == true )
        {
          System.out.println( "Removed last element of wayback as a result of antagonistic actions" );
        }
      }
      return;
    }
    if( thisround.getName().equalsIgnoreCase( "Right" ) )
    {
      if( log == true )
      {
        System.out.println(" UpdateWayBack: This round's action was right " );
      }
      if( wayback.isEmpty() == true )
      {
        wayback.addElement( (  Object ) new Action( "Left" )  );
        if( log == true )
        {
          System.out.println(" UpdateWayBack: added left to beginning" );
        }
      }
      else if( ( ( Action ) wayback.lastElement() ).getName()
                   .equalsIgnoreCase( "Right" ) == false )
      {
        wayback.addElement( (  Object ) new Action( "Left" ) );
        if( log == true )
        {
          System.out.println(" UpdateWayBack: added left to wayback " );
        }
      }
      else
      {
        wayback.removeElementAt( wayback.size() - 1 );
        if( log == true )
        {
          System.out.println( "Removed last element of wayback as a result of antagonistic actions" );
        }
      }
      return;
    }
    if( log == true )
    {
      System.out.println(" Current action is no move: nothing to do to update my way home" );
    }
  }

  protected void checkWayBack()
  {
    Point start;
    String help;

    start = new Point( location.x, location.y );
    if( log == true )
    {
      System.out.println( "Start wayback check at: " + start.x + "/" + start.y );
    }
    for( int i = 0; i < wayback.size(); i++ )
    {
      help = ( ( Action ) wayback.elementAt( wayback.size() - i - 1 ) ).getName();
      if( help.equals( "Up" ) )
      {
        start.y--;
        if( log == true )
        {
          System.out.println( "Would go up to: " + start.x + "/" + start.y );
        }
      }
      else if( help.equals( "Down" ) )
      {
        start.y++;
        if( log == true )
        {
          System.out.println( "Would go down to: " + start.x + "/" + start.y );
        }
      }
      else if( help.equals( "Left" ) )
      {
        start.x--;
        if( log == true )
        {
          System.out.println( "Would go left to: " + start.x + "/" + start.y );
        }
      }
      else if( help.equals( "Right" ) )
      {
        start.x++;
        if( log == true )
        {
          System.out.println( "Would go right to: " + start.x + "/" + start.y );
        }
      }
    }
    if( start.x != 1 || start.y != 1 )
    {
      if( log == true )
      {
        System.out.println( "Error: Would land at: " + start.x + "/" + start.y 
                             + " if there are no obstacles " );
      }
    }
    else
    {
      if( log == true )
      {
        System.out.println( "Way home is still fine" );
      }
    }
  }
}
