import ArraySearchBackWumpus;
import java.util.*;
import java.awt.*;
import java.awt.image.ImageObserver;

//
//
// ArrayDepthWumpus searches for a way home as soon as it finds gold; 
// it uses depth - first search; the first node to be expanded is 
// the one with the smallest manhatten - distance to the home - field
// ( this is also called  a 'greedy' search algorithm ) 
// It is slightly 'goal - based': if gold was found it returns home
// immediatly
//
//

class ArrayDepthWumpus extends ArraySearchBackWumpus
{
  static final String type = "ARRAYDEPTHWUMPUS";
  
  boolean startsearch;

  public ArrayDepthWumpus( String name, int ammo, Dimension size )
  {
    this( name, ammo, false, size, ( String ) null, ( ImageObserver ) null, type );
  }
 
  public ArrayDepthWumpus( String name, int ammo, boolean takerisk, Dimension size )
  {
    this( name, ammo, takerisk, size, ( String ) null, ( ImageObserver ) null, type );
  }
 
  public ArrayDepthWumpus( String name, int ammo, Dimension size, String imgref )
  {
    this( name, ammo, false, size, imgref, ( ImageObserver ) null, type );
  }

  public ArrayDepthWumpus( String name, int ammo, Dimension size, ImageObserver obs )
  {
    this( name, ammo, false, size, ( String ) null, obs, type );
  }

  public ArrayDepthWumpus( String name, int ammo, boolean takerisk,Dimension size,
                                String imgref, ImageObserver obs )
  {
    this( name, ammo, takerisk, size, imgref, obs, type );
  }

  public ArrayDepthWumpus( String name, int ammo, boolean takerisk,Dimension size,
                                String imgref, ImageObserver obs, String type )
  {
    super( name, ammo, takerisk, size, imgref, obs, type );
  }// End of ArrayDepthWumpus-Constructors

  protected void initVars()
  {
    super.initVars();
    if( log == true )
    {
      System.out.println(" ArrayDepth: initializing new vars ");
    }
    startsearch = true;
  }

  protected boolean search( Point recent )
  {
    Point up, down, left, right, next;
    Vector nearestlast;
    boolean done;
    
    done = false;

    if( log == true && startsearch == true )
    {
      System.out.println( "Starting depth first search at: " + recent.x + "/" + recent.y ); 
    }

    if( startsearch == true )
      startsearch = false;

    // do not include fields that have been already searched or
    // that have never been visited
    if( ( myfield[ recent.x ] [ recent.y ].getSearched() == true ) ||
        ( myfield[ recent.x ] [ recent.y ].wasVisited().equals( "NO" ) ) )
    {
      return done;
    }

    // initialize vector to four element - size
    nearestlast = new Vector( 4 ); 

    // mark recent field as visited and check for goal - field
    myfield[ recent.x ] [ recent.y ].setSearched( true );
    if( myfield[ recent.x ] [ recent.y ].isExit().equals( "YES" ) )
    {
      done = true;
      return done;
    }

    // get neighbour coordinates
    up    = new Point( recent.x, recent.y - 1 );
    down  = new Point( recent.x, recent.y + 1 );
    left  = new Point( recent.x - 1, recent.y );
    right = new Point( recent.x + 1, recent.y );

    
    // calculate manhatten - distance
    nearestlast = calcManhatten( up, down, left, right );

    // recurse and build a way at this occassion
    while( nearestlast.isEmpty() == false && done == false )
    {
      next = ( Point ) nearestlast.lastElement();
      nearestlast.removeElementAt( nearestlast.size() - 1 );
      if( next == up )
      {
       if( log == true )
       {
         System.out.println( "Adding 'Up' action to my way home" );
       }
        searchway.addElement( ( Object ) new Action( "Up" ) );
      }
      else if( next == down )
      {
        if( log == true )
        {
          System.out.println( "Adding 'Down' action to my way home" );
        }
        searchway.addElement( ( Object ) new Action( "Down" ) );
      }
      else if( next == left )
      {
        if( log == true )
        {
          System.out.println( "Adding 'Left' action to my way home" );
        }
        searchway.addElement( ( Object ) new Action( "Left" ) );
      }
      else if( next == right )
      {
        if( log == true )
        {
          System.out.println( "Adding 'Right' action to my way home" );
        }
        searchway.addElement( ( Object ) new Action( "Right" ) );
      }

      if( log == true )
      {
        System.out.println( " Continuing search at: " + next.x + "/" + next.y );
      }

      done = search( next );
      
      if( done == false )
      {
        searchway.removeElementAt( searchway.size() - 1 );
      }
    }
    return done;
  }
}

