import ArrayBackWumpus;
import java.util.*;
import java.awt.*;
import java.awt.image.ImageObserver;

//
//
// Class
// ArraySearchBackWumpus
// A frame for Wumpus-Agents that search a way back home as soon as
// they have found some gold.
//
// This class improves the strategy of updating the field once more.
//
// creation 08.04.1997 
// 
//

abstract class ArraySearchBackWumpus extends ArrayBackWumpus
{
  static final String type = "ARRAYSEARCHBACKWUMPUS";
  protected boolean foundway;
  protected AdvancedSearchAgentField myfield[] [];
  protected Vector searchway; // represents the way home

  public ArraySearchBackWumpus( String name, int ammo, Dimension size )
  {
    this( name, ammo, false, size, ( String ) null, ( ImageObserver ) null, type );
  }
 
  public ArraySearchBackWumpus( String name, int ammo, boolean takerisk, Dimension size )
  {
    this( name, ammo, takerisk, size, ( String ) null, ( ImageObserver ) null, type );
  }
 
  public ArraySearchBackWumpus( String name, int ammo, Dimension size, String imgref )
  {
    this( name, ammo, false, size, imgref, ( ImageObserver ) null, type );
  }

  public ArraySearchBackWumpus( String name, int ammo, Dimension size, ImageObserver obs )
  {
    this( name, ammo, false, size, ( String ) null, obs, type );
  }

  public ArraySearchBackWumpus( String name, int ammo, boolean takerisk,Dimension size,
                                String imgref, ImageObserver obs )
  {
    this( name, ammo, takerisk, size, imgref, obs, type );
  }

  public ArraySearchBackWumpus( String name, int ammo, boolean takerisk,Dimension size,
                                String imgref, ImageObserver obs, String type )
  {
    super( name, ammo, takerisk, size, imgref, obs, type );
  }// End of ArraySearchBackWumpus-Constructors

  protected void initVars()
  {
    super.initVars();
    if( log == true )
    {
      System.out.println(" ArraySearchBack: initializing new vars ");
    }
    searchway = new Vector( 5, 5 );
    foundway = false;
  }

  protected void initField( Dimension fieldsize )
  {
    // initialize the field - represantation
	myfield = new AdvancedSearchAgentField [ fieldsize.width ] [ fieldsize.height ];
    if( log == true )
    {
      System.out.println("ArraySearchBack: width = " + fieldsize.width);
      System.out.println("ArraySearchBack: height = " + fieldsize.height);
      System.out.println("ArraySearchBack: allocating myfield");
    }

    for( int i = 0; i < myfield.length; i++ )
	{
	  for( int j = 0; j < myfield[ i ].length; j++ )
	  {
		myfield[ i ] [ j ] = new AdvancedSearchAgentField();
        if( log == true )
        {
          System.out.println("allocated myfield" + i + j );
        }
	  }
	}
    
    if( log == true )
    {
      System.out.println("finished field allocation");
    }

	// Agents start at location ( 1, 1 ); this is also the exit
    location = new Point( 1, 1);
    // do still assign x, y their coordinates for clarity
    location.x = 1;
	location.y = 1;
	myfield[ location.x ] [ location.y ].setExit( true );
	myfield[ location.x ] [ location.y ].setVisited( true );
    myfield[ location.x ] [ location.y ].setWumpus( false );
    myfield[ location.x ] [ location.y ].setPit( false );
	myfield[ location.x ] [ location.y ].incVisited();

    // up and left are wall because we always start in the upper left corner
    // this has to be removed if the agent should start anywhere in the maze
    myfield[ location.x - 1 ] [ location.y ].setWall( true );
    myfield[ location.x ] [ location.y - 1 ].setWall( true );
    
    // to be able to use methods from super - classes shadowed var must
    // be 'initialized' here 
    super.myfield = myfield; // for arraybackwumpus
    ( ( ArrayWumpus ) this ).myfield = myfield; // for arraywumpus
  }

  protected void finalize()
  {
    super.finalize();
    myfield = null;
    searchway = null;
  }
  
  public void receivePercepts( Vector percepts )//where the agent receives his percepts
  {
    super.receivePercepts( percepts );
    if( gohome == true && foundway == false )
    {
      findWayBack();
      // add the action for this round and remove it from wayback
      actions.addElement( wayback.lastElement() );
      wayback.removeElementAt( wayback.size() - 1 );
    }
    if( log == true )
    {
      System.out.println("Searcher finished move: " + moves +"\n\n" );
    }
  }	 

  protected void updateField()
  {
    AdvancedSearchAgentField upleft, upright, downleft, downright;
    AgentField up, down, left, right;

    if( log == true )
    {
      System.out.println( "Starting to work on action at move number: " + ( moves + 1 ) );
      System.out.println( "Calling ancestors update method first" );
    }

    // call dad's version
    super.updateField();

    // using our knowledge that only one wumpus resides in the cave
    // we know that a stench at ( x/y ) and at ( x - 1/y - 1 ) means there is
    // a wumpus at ( x/y - 1 ) etc.
    // the following code is only valid as long as only one wumpus per cave is
    // allowed 

    up        = getUpField( location );
    down      = getDownField( location );
    left      = getLeftField( location );
    right     = getRightField( location );

    upleft    = getUpLeftField( location );
    upright   = getUpRightField( location );
    downleft  = getDownLeftField( location );
    downright = getDownRightField( location );

    if( pure.getStench() == true && wumpusalive == true )
    {
      if( ( upleft.isStench().equals( "YES" ) && upright.isStench().equals( "YES" ) ) ||
          ( upleft.isStench().equals( "YES" ) && left.hasWumpus().equals( "NO" ) ) ||
          ( upright.isStench().equals( "YES" ) && right.hasWumpus().equals( "NO" ) ) ||
          ( upleft.isStench().equals( "YES" ) && ( downleft.isStench().equals( "NO" ) && downleft.wasVisited().equals( "YES" ) ) ) ||
          ( upright.isStench().equals( "YES" ) && ( downright.isStench().equals( "NO" ) && downright.wasVisited().equals( "YES" ) ) ) )
      {
        up.setWumpus( true );
      }

      if( ( upleft.isStench().equals( "YES" ) && downleft.isStench().equals( "YES" ) ) ||
          ( upleft.isStench().equals( "YES" ) && up.hasWumpus().equals( "NO" ) ) ||
          ( downleft.isStench().equals( "YES" ) && down.hasWumpus().equals( "NO" ) ) ||
          ( upleft.isStench().equals( "YES" ) && ( upright.isStench().equals( "NO" ) && upright.wasVisited().equals( "YES" ) ) ) ||
          ( downleft.isStench().equals( "YES" ) && ( downright.isStench().equals( "NO" ) && downright.wasVisited().equals( "YES" ) ) ) )
      {
        left.setWumpus( true );
      }

      if( ( downleft.isStench().equals( "YES" ) && downright.isStench().equals( "YES" ) ) ||
          ( downleft.isStench().equals( "YES" ) && left.hasWumpus().equals( "NO" ) ) ||
          ( downright.isStench().equals( "YES" ) && right.hasWumpus().equals( "NO" ) ) ||
          ( downleft.isStench().equals( "YES" ) && ( upleft.isStench().equals( "NO" ) && upleft.wasVisited().equals( "YES" ) ) ) ||
          ( downright.isStench().equals( "YES" ) && ( upright.isStench().equals( "NO" ) && upright.wasVisited().equals( "YES" ) ) ) )
      {
        down.setWumpus( true );
      }

      if( ( downright.isStench().equals( "YES" ) && upright.isStench().equals( "YES" ) ) ||
          ( downright.isStench().equals( "YES" ) && down.hasWumpus().equals( "NO" ) ) ||
          ( upright.isStench().equals( "YES" ) && up.hasWumpus().equals( "NO" ) ) ||
          ( downright.isStench().equals( "YES" ) && ( downleft.isStench().equals( "NO" ) && downleft.wasVisited().equals( "YES" ) ) ) ||
          ( upright.isStench().equals( "YES" ) && ( upleft.isStench().equals( "NO" ) && upright.wasVisited().equals( "YES" ) ) ) )
      {
        right.setWumpus( true );
      }
    }
  }
   
  protected void findWayBack()
  {
    boolean success;
    Action current;
    
    current = new Action( "NONE" );

    // remove the action already calculated by superclass
    // keep current action in mind if searching is not successfull
    if( actions.isEmpty() == false )
    {
      current = ( Action ) actions.firstElement();
      if( log == true )
      {
        System.out.println( " Action calculated so far : " + current.getName() ); 
      }
    }
    actions.removeAllElements();

    success = search( location );
    
    if( log == true )
    {
      for( int i = 0; i < searchway.size(); i++ )
      {
        System.out.println( " searchway: " + i + " " +
                              ( ( Action ) searchway.elementAt( i ) ).getName() );
      }
      System.out.println( " wayback size: " + wayback.size() );
    }

    if( success == true )  // this should always be the case
    {
      if( log == true )
      {
        System.out.println( " success = : " + success ); 
      }
      if( ( wayback.size() + 1 ) >= searchway.size() )
      {
        if( log == true )
        {
          System.out.println( "Removing all elements from wayback vector" );
          System.out.println( "to store shorter way home" );
          System.out.println( "The way I found spares me " + ( wayback.size() - searchway.size() ) + " moves." );
        }

        wayback.removeAllElements();
        for( int i = 0; i < searchway.size(); i++ )
        {
          wayback.addElement( searchway.elementAt( searchway.size() - i - 1 ) );
        }

        searchway = null;
      }
    }
    else
    {
      // append the current action
      if( log == true )
      {
        System.out.println( "Search was not successfull!!!!!");
        System.out.println( "Restoring precalculated action: " + current.getName() ); 
      }
      wayback.addElement( ( Object ) current );
    }
    foundway = true;
  }

  public Vector calcManhatten( Point up, Point down, Point left, Point right )
  {
    Vector giveback;
    Point got[ ], help;
    int biggest;
    
    got = new Point[ 4 ];
    got[ 0 ] = up;
    got[ 1 ] = down;
    got[ 2 ] = left;
    got[ 3 ] = right;

    giveback = new Vector( 4 );
    
    // sort point - array; smallest element at position 3
    for( int i = 0; i < got.length - 1; i++ )
    {
      biggest = i;
      for( int j = i + 1; j < got.length; j++ )
      {
        if( ( Math.abs( 1 - got[ j ].x ) + Math.abs( 1 - got[ j ].y ) ) >
            ( Math.abs( 1 - got[ biggest ].x ) + Math.abs( 1 - got[ biggest ].y ) ) )
        {
          biggest = j;
        }
        help = got[ i ];
        got[ i ] = got[ biggest ];
        got[ biggest ] = help;
      }
    }

    for( int i = 0; i < got.length; i++ )
    {
      if( log == true )
      {
        System.out.println( " got[ " + i + " ]: " + got[ i ].x + "/" + got[ i ].y );
      }
      giveback.addElement( ( Object ) got[ i ] );
    }

    return giveback;
  }

  // getxxxField methods return the field that is to 
  // the up/down-right or to the up/down-left from the agent's current location
  protected AdvancedSearchAgentField getUpLeftField( Point of )
  {
    return myfield[ of.x - 1 ] [ of.y - 1 ];
  }

  protected AdvancedSearchAgentField getUpRightField( Point of )
  {
    return myfield[ of.x + 1 ] [ of.y - 1 ];
  }

  protected AdvancedSearchAgentField getDownLeftField( Point of )
  {
    return myfield[ of.x - 1 ] [ of.y + 1 ];
  }

  protected AdvancedSearchAgentField getDownRightField( Point of )
  {
    return myfield[ of.x + 1 ] [ of.y + 1 ];
  }

  abstract protected boolean search( Point recent );
}

