import java.util.*;
import java.awt.*;
import java.awt.image.ImageObserver;

//
//
// Class
// ArrayWumpus
// Wumpus-Agent that has an internal represantation of the environment
// 12.03.1997 
//
// 20.03.1997 eliminated known bugs and simplified agent's logic
//
// 31.03.1997 added history-bookkeeping
//
// To represent the environment the agent uses an array that is
// updated every round according to the percepts the agent gets.
// 
// This agent implements a basic strategy that allows successors
// of this class to build upon it.
//
//

class ArrayWumpus extends WumpusAgent
{
  static final String type = "ARRAYWUMPUS";
  static final boolean log = false; // set to true if the agents should output
                                    // its actions to the console

  protected boolean wumpusalive, pickedgold, takerisk;
  protected AgentField myfield[][];
  protected Point location;
  protected WumpusPurePercept pure;
  protected Action lastaction, action;
  protected int moves, lastscore;

  public ArrayWumpus( String name, int ammo, Dimension size )
  {
    this( name, ammo, false, size, ( String ) null, ( ImageObserver ) null );
  }
 
  public ArrayWumpus( String name, int ammo, boolean takerisk, Dimension size )
  {
    this( name, ammo, takerisk, size, ( String ) null, ( ImageObserver ) null );
  }
 
  public ArrayWumpus( String name, int ammo, Dimension size, String imgref )
  {
    this( name, ammo, false, size, imgref, ( ImageObserver ) null );
  }

  public ArrayWumpus( String name, int ammo, Dimension size, ImageObserver obs )
  {
    this( name, ammo, false, size, ( String ) null, obs );
  }

  public ArrayWumpus( String name, int ammo, boolean takerisk, Dimension size, 
                      String imgref, ImageObserver obs )
  {
    super( name, type, ammo, imgref, obs );
    this.takerisk = takerisk;
    initVars();
    initField( size );
  }// End of ArrayWumpus-Constructors

  public ArrayWumpus( String name, int ammo, boolean takerisk, Dimension size, 
                      String imgref, ImageObserver obs, String type )
  {
    super( name, type, ammo, imgref, obs );
    this.takerisk = takerisk;
    initVars();
    initField( size );
  }

  protected void initVars()
  {
	// provide initializations that are needed for every of the
	// possible constructors
	moves = 0;
    wumpusalive = true;
    pickedgold = false;
    lastaction = new Action( "NONE" ); // this is after all the start
    lastscore = getScore();
    if( log == true )
    {
      System.out.println( "I am of type: " + getType() );
    }
  }

  protected void initField( Dimension fieldsize )
  {
    // initialize the field - represantation
	myfield = new AgentField [ fieldsize.width ] [ fieldsize.height ];
    if( log == true );
    {
      System.out.println("width = " + fieldsize.width);
      System.out.println("height = " + fieldsize.height);
      System.out.println("allocated myfield");
    }
    for( int i = 0; i < myfield.length; i++ )
	{
	  for( int j = 0; j < myfield[ i ].length; j++ )
	  {
		myfield[ i ] [ j ] = new AgentField();
        if( log == true );
        {
          System.out.println("allocated myfield" + i + j );
        }
	  }
	}
    
    if( log == true )
    {
      System.out.println(" Finished field aloocation " );
    }
	
    // Agents start at location ( 1, 1 ); this is also the exit
    location = new Point( 1, 1);
    
    // do still assign x, y their coordinates for clarity
    location.x = 1;
	location.y = 1;
	myfield[ location.x ] [ location.y ].setExit( true );
	myfield[ location.x ] [ location.y ].setVisited( true );
    myfield[ location.x ] [ location.y ].setWumpus( false );
    myfield[ location.x ] [ location.y ].setPit( false );
	myfield[ location.x ] [ location.y ].incVisited();
    
    // up and left are wall because we always start in the upper left corner
    // this has to be removed if the agent should start anywhere in the maze
    myfield[ location.x - 1 ] [ location.y ].setWall( true );
    myfield[ location.x ] [ location.y - 1 ].setWall( true );
    if( log == true )
    {
      System.out.println("ArrayWumpus: end of initfield");
    }
  }

  protected void finalize()
  {
    // return menory for the field represantation
    myfield = null;
  }
  
  public void receivePercepts( Vector percepts )//where the agent receives his percepts
  {
    super.receivePercepts( percepts );
	pure = new WumpusPurePercept( percepts );
    if( log == true )
    {
	  System.out.println("Received Scream? " + pure.getScream() );
      System.out.println("Received Glitter? " + pure.getGlitter() );
      System.out.println("Received Stench? " + pure.getStench() );
      System.out.println("Received Breeze? " + pure.getBreeze() );
      System.out.println("Received Bump? " + pure.getBump() + "\n" );
    }
    updateField();
    reason();
    updateHistory( ( Object ) action );
    if( log == true && getType().equals( "ARRAYWUMPUS" ) )
    { 
      System.out.println("Returned action: " + ( ( Action ) actions.elementAt( 0 ) ).getName() + "as move nr: " + moves + "\n" );
    }
  }	 
  
  protected void updateField() // update environment - representation
  {
    AgentField left, right, up, down;
    boolean moved = false;
    moves++;

    if( log == true && getType().equals( "ARRAYWUMPUS" ) )
    {
      System.out.println( "Starting to work on action at move number: " + moves );
    }

    if( log == true )
    {
      System.out.println( " Now starting to update field " );
    }

    if( pickedgold == false )
    {
      if( lastscore >= getScore() )
      {
        lastscore = getScore();
      }
      else
      {
        pickedgold = true;
        if( log == true )
        {
          System.out.println( getType() + ": Grabbed a piece of gold " );
        }
      }
    }


    if( moves > 1 ) // actions is empty in the first round
    {
       lastaction = ( Action ) actions.elementAt( 0 );
    }
	actions.removeAllElements();

    if( log == true )
    {
      System.out.println("my last action was: " + lastaction.getName() );
    }
    // get neighbouring fields
    left  = getLeftField( location );
    right = getRightField( location );
    up    = getUpField( location );
    down  = getDownField( location );
    
    // check if someone killed the wumpus
	if( pure.getScream() == true )
	{
      if( log == true )
      {
        System.out.println(" wumpus was killed " );
      }
      for( int i = 0; i < myfield.length; i++ )
	  {
		for( int j = 0; j < myfield[ i ].length; j++ )
		{
		  myfield[ i ] [ j ].setWumpus( false );
		}
	  }
	  wumpusalive = false;
	}

	// check if a bump was received and update map
	if( pure.getBump() == true )
	{
      if( log == true )
      {
        System.out.println(" My last action let me bump into a wall " );
      }
	  if( lastaction.getName().equals( "Up" ) )
	  	up.setWall( true );
      else if( lastaction.getName().equals( "Down" ) )
		down.setWall( true );
	  else if( lastaction.getName().equals( "Left" ) )
		left.setWall( true );
	  else // last action was "Right"
		right.setWall( true );
	  
      // leave routine -> there is nothing to update if we
	  // stayed in the same field and the kill of a wumpus
	  // by another agent is already handled
	  return;
	}
	else // there was no bumb
	{
       if( log == true )
       {
      	 System.out.println("No wall encountered during last action" );
       }
      // if we shot last round and heard no scream
      // we can update one field and that is all
      // this is because wall fields may occur everywhere on the map
      if( wumpusalive == true )
      {
        if( log == true )
        {
          System.out.println(" wumpus still lives " );
        }
        if( lastaction.getName().equals( "Shoot_Up" ) )
        {

          myfield[ location.x ] [ location.y - 1 ].setWumpus( false );    
          return;
        }
        if( lastaction.getName().equals( "Shoot_Down" ) )
        {
          myfield[ location.x ] [ location.y + 1 ].setWumpus( false );
          return;
        }
        if( lastaction.getName().equals( "Shoot_Right" ) )
        {
          myfield[ location.x + 1 ] [ location.y ].setWumpus( false );
          return;
        }
        if( lastaction.getName().equals( "Shoot_Left" ) )
        {
          myfield[ location.x - 1 ] [ location.y ].setWumpus( false );
          return;
        }
      }

	  // set new location if we moved
	  if( lastaction.getName().equals( "Up" ) )
      {
        location.y--;
        moved = true;
      }
	  else if( lastaction.getName().equals( "Down" ) )
      {
        location.y++;
        moved = true;
      }
	  else if( lastaction.getName().equals( "Left" ) )
      {
        location.x--;
        moved = true;
      }
	  else if( lastaction.getName().equals( "Right" ) )
      {
        location.x++;
        moved = true;
      }
      
      if( moved == true )
      {
        if( log == true )
        {
          System.out.println(" I moved "  );
          System.out.println("My new location is: " + location.x + "/" + location.y  );
        }
        // mark new location as visited etc.
        myfield[ location.x ] [ location.y ].setWumpus( false );
        myfield[ location.x ] [ location.y ].setPit( false );
	    myfield[ location.x ] [ location.y ].setVisited( true );
        myfield[ location.x ] [ location.y ].incVisited();
        
        // get new neighbouring fields
        left  = getLeftField( location );
        right = getRightField( location );
        up    = getUpField( location );
        down  = getDownField( location );
      }
	}

	// check for stench and update surrounding fields if wumpus lives
	if(  wumpusalive == true )
	{
      if( log == true )
      {
        System.out.println(" I'm checking for stench now" );
      }
      if( pure.getStench() == true )
	  {
	    // if three fields have no wumpus the fourth field must contain it
	    if( up.hasWumpus().equals( "NO" ) && down.hasWumpus().equals( "NO" ) &&
		    left.hasWumpus().equals( "NO" ) )
		{
          if( log == true )
          {
            System.out.println(" The wumpus is located to my right" );
          }
		  right.setWumpus( true );
		}
	    else if( up.hasWumpus().equals( "NO" ) && down.hasWumpus().equals( "NO" ) &&
		         right.hasWumpus().equals( "NO" ) )
		{
          if( log == true )
          {
            System.out.println(" The wumpus is located to my left" );
          }
		  left.setWumpus( true );
		}
	    else if( up.hasWumpus().equals( "NO" ) && right.hasWumpus().equals( "NO" ) &&
		         left.hasWumpus().equals( "NO" ) )
		{
          if( log == true )
          {
            System.out.println(" The wumpus is located downwards" );
          }
	      down.setWumpus( true );
		}
	    else if( down.hasWumpus().equals( "NO" ) && right.hasWumpus().equals( "NO" ) &&
		         left.hasWumpus().equals( "NO" ) )
		{
          if( log == true )
          {
            System.out.println(" The wumpus is located upwards" );
          }
		  up.setWumpus( true );
		}
	  }
	  else // surrounding fields are wumpusless
	  {
        if( log == true )
        {
          System.out.println(" Surrounding fields are wumpusfree" );
        }
        up.setWumpus( false );
		down.setWumpus( false );
		left.setWumpus( false );
		right.setWumpus( false );
	  }
	}
	
	// check for breeze and update surrounding fields
    if( log == true )
    {
      System.out.println(" I'm checking for breeze now" );
    }
	if( pure.getBreeze() == true )
	{
	  // if three fields have no pit the fourth field must contain it
	  if( up.hasPit().equals( "NO" ) && down.hasPit().equals( "NO" ) &&
		  left.hasPit().equals( "NO" ) )
	  {
        if( log == true )
        {
          System.out.println(" A pit is to my right" );
        }
		right.setPit( true );
	  }
	  else if( up.hasPit().equals( "NO" ) && down.hasPit().equals( "NO" ) &&
		       right.hasPit().equals( "NO" ) )
	  {
        if( log == true )
        {
          System.out.println(" A pit is to my left" );
        }
		left.setPit( true );
	  }
	  else if( up.hasPit().equals( "NO" ) && right.hasPit().equals( "NO" ) &&
		       left.hasPit().equals( "NO" ) )
	  {
        if( log == true )
        {
          System.out.println(" A pit is one field below" );
        }
		down.setPit( true );
	  }
	  else if( down.hasPit().equals( "NO" ) && right.hasPit().equals( "NO" ) &&
		       left.hasPit().equals( "NO" ) )
	  {
        if( log == true )
        {
          System.out.println(" A pit is one field above" );
        }
		up.setPit( true );
	  }
	}
	else // surrounding fields are pitless
	{
      if( log == true )
      {
        System.out.println(" Surrounding fields are pitless ");
      }
	  up.setPit( false );
	  down.setPit( false );
	  right.setPit( false );
	  left.setPit( false );
	}
  } // updateField	

  protected void reason()
  {
    AgentField left, right, up, down;
 	int couldright, couldleft, couldup, coulddown, moveno;
    
    // get neighbouring fields
    left  = getLeftField( location );
    if( log == true && getType().equals( "ARRAYWUMPUS" ) )
    {
      System.out.println(" left x,y:" + ( location.x - 1 ) + "  " + location.y );
      System.out.println(" Pit: " + left.hasPit() );
      System.out.println(" Wumpus: " + left.hasWumpus() + "\n" );
    }
    right = getRightField( location );
    if( log == true && getType().equals( "ARRAYWUMPUS" ) )
    {
      System.out.println(" right x,y:" + ( location.x + 1 ) + "  " + location.y );
      System.out.println(" Pit: " + right.hasPit() );
      System.out.println(" Wumpus: " + right.hasWumpus() + "\n" );
    }
    up    = getUpField( location );
    if( log == true && getType().equals( "ARRAYWUMPUS" ) )
    {
      System.out.println(" up x,y:" + location.x + "  " + ( location.y - 1 ) );
      System.out.println(" Pit: " + up.hasPit() );
      System.out.println(" Wumpus: " + up.hasWumpus() + "\n" );
    }
    down  = getDownField( location );
    if( log == true && getType().equals( "ARRAYWUMPUS" ) )
    {
      System.out.println(" down x,y:" + location.x + "  " + ( location.y + 1 ) );
      System.out.println(" Pit: " + down.hasPit() );
      System.out.println(" Wumpus: " + down.hasWumpus() + "\n" );
    }
    
    // always pick up gold 
	if( pure.getGlitter() == true )
	{
      if( log == true )
      {
        System.out.println(" It glitters. I try to grab the gold" );
      }
	  actions.addElement( ( Object ) ( action = new Action( "Grab" ) ) );
	  return;
	}

    // what to do if we start besides a pit or a wumpus
    // we do this after the check for glitter
    if( moves == 1 ) 
    {
      if( pure.getBreeze() == true )
      {
        // leave if even the first step is not safe
        actions.addElement( ( Object ) ( action = new Action( "Climb" ) ) );
        return;
      }

      if( pure.getStench() == true )
      {
        // shoot to the right ( down would be equally well )
        actions.addElement( ( Object ) ( action = new Action( "Shoot_Right" ) ) );
        return;
      }
       
    }


	// if gold has been taken before, exit if possible
	if( myfield[ location.x ] [ location.y ].isExit().equals( "YES" ) &&
	    ( getScore() > 0 || moves > 1000 ) )
	{
      if( log == true )
      {
        System.out.println("Exiting now" );
      }
      actions.addElement( ( Object ) ( action = new Action( "Climb" ) ) );
	  return;
	}

	// if the location of the wumpus is in an adjacent field 
    // and we still have arrows and the wumpus still lives
	// kill him ( each agent has at least 1 arrow )
	// do not check for two fields of stench and two arrows etc.
    if( arrowsLeft() == true && wumpusalive == true )
    {
      if( up.hasWumpus().equals( "YES" ) )
      {
        if( log == true )
        {
          System.out.println(" Shooting up to the kill the wumpus " );
        }
        actions.addElement( ( Object ) ( action = new Action( "Shoot_Up" ) ) );
        return;
      }
	  if( down.hasWumpus().equals( "YES" ) )
      {
        if( log == true )
        {
          System.out.println(" Shooting down to the kill the wumpus " );
        }
	    actions.addElement( ( Object ) ( action = new Action( "Shoot_Down" ) ) );
	    return;
      }
      if( right.hasWumpus().equals( "YES" ) )
      {
        if( log == true )
        {
          System.out.println(" Shooting to the right to the kill the wumpus " );
        }
	    actions.addElement( ( Object ) ( action = new Action( "Shoot_Right" ) ) );
	    return;
      }
	  if( left.hasWumpus().equals( "YES" ) )
      {
        if( log == true )
        {
          System.out.println(" Shooting to the left to kill the wumpus " );
        }
	    actions.addElement( ( Object ) ( action = new Action( "Shoot_Left" ) ) );
	    return;
      }
    }

	// decide the direction in which to go
	// first move to those fields not yet visited than those with the
	// lowest visit count
	// check right, up, left, down respectively
	// include hasWumpus in checks; maybe you forgot your arrows
    if( right.hasWumpus().equals( "NO" ) && right.hasPit().equals( "NO" ) && 
		right.isWall().equals( "NO" ) && right.wasVisited().equals( "NO" ) )
	{
	  actions.addElement( ( Object ) ( action = new Action( "Right" ) ) );
      if( log == true )
      {
        System.out.println(" Going to the right to visit a new field " );
      }
	  return;
	}
	else // check if field is a possible canditate
	{
	  if( right.hasWumpus().equals( "YES" ) || right.hasPit().equals( "YES" ) || 
		  right.isWall().equals( "YES" ) )
	  {
        if( log == true )
        {
          System.out.println(" Moving to the right field would be lethal" );
        }
		couldright = -1; 
	  }
	  else
	  {
		couldright = right.visitCount();
        if( log == true )
        {
          System.out.println(" Could go right though I have been there " + right.visitCount() + " times" );
        }
	  }
	}

    if( up.hasWumpus().equals( "NO" ) && up.hasPit().equals( "NO" ) && 
		up.isWall().equals( "NO" ) && up.wasVisited().equals( "NO" ) )
	{
	  actions.addElement( ( Object ) ( action = new Action( "Up" ) ) );
      if( log == true )
      {
        System.out.println(" Going up to visit a new field " );
      }
	  return;
	}
	else // check if field is a possible canditate
	{
	  if( up.hasWumpus().equals( "YES" ) || up.hasPit().equals( "YES" ) || 
		  up.isWall().equals( "YES" ) )
	  {
        if( log == true )
        {
          System.out.println(" Moving to the field above would be lethal" );
        }
		couldup = -1; 
	  }
	  else
	  {
		couldup = up.visitCount();
        if( log == true )
        {
          System.out.println(" Could go up though I have been there " + up.visitCount() + " times" );
        }
	  }
	}

    if( left.hasWumpus().equals( "NO" ) && left.hasPit().equals( "NO" ) && 
		left.isWall().equals( "NO" ) && left.wasVisited().equals( "NO" ) )
	{
	  actions.addElement( ( Object ) ( action = new Action( "Left" ) ) );
	  if( log == true )
      {
        System.out.println(" Going left to visit a new field " );
      }
      return;
	}
		else // check if field is a possible canditate
	{
	  if( left.hasWumpus().equals( "YES" ) || left.hasPit().equals( "YES" ) || 
		  left.isWall().equals( "YES" ) )
	  {
        if( log == true )
        {
          System.out.println(" The left field would be lethal" );
        }
		couldleft = -1; 
	  }
	  else
	  {
		couldleft = left.visitCount();
        if( log == true )
        {
          System.out.println(" Could go left though i have been there " + left.visitCount() + " times" );
        }
	  }
	}
    if( down.hasWumpus().equals( "NO" ) && down.hasPit().equals( "NO" ) && 
		down.isWall().equals( "NO" ) && down.wasVisited().equals( "NO" ) )
	{
	  actions.addElement( ( Object ) ( action = new Action( "Down" ) ) );
      if( log == true )
      {
        System.out.println(" Going down to visit a new field " );
      }
	  return;
	}
	else // check if field is a possible canditate
	{
	  if( down.hasWumpus().equals( "YES" ) || down.hasPit().equals( "YES" ) || 
		  down.isWall().equals( "YES" ) )
	  {
        if( log == true )
        {
          System.out.println(" Moving to the field below would be lethal" );
        }
		coulddown = -1; 
	  }
	  else
	  {
		coulddown = down.visitCount();
        if( log == true )
        {
          System.out.println(" Could go down though I have been there " + down.visitCount() + " times" );
        }
	  }
	}

	// if no new field is available enter the one that was at least
	// often visited i.e. has the smallest visit count
	// if this count is bigger than 10 venture into any possible
	// field that could be safe
	moveno = getMove( couldright, couldup, couldleft, coulddown );
	if( moveno == couldright )
    {
	  actions.addElement( ( Object ) ( action = new Action( "Right" ) ) );
      if( log == true )
      {
        System.out.println(" Going right because right field was visited the least often" );
      }
	  return;
	}

	if( moveno == couldup )
    {
	  actions.addElement( ( Object ) ( action = new Action( "Up" ) ) );
      if( log == true )
      {
        System.out.println(" Going up because right field was visited the least often" );
      }
	  return;
	}
	if( moveno == couldleft )
    {
	  actions.addElement( ( Object ) ( action = new Action( "Left" ) ) );
      if( log == true )
      {
        System.out.println(" Going left because right field was visited the least often" );
      }
	  return;
	}
	if( moveno == coulddown )
    {
	  actions.addElement( ( Object ) ( action = new Action( "Down" ) ) );
      if( log == true )
      {
        System.out.println(" Going down because right field was visited the least often" );
      }
	  return;
	}
  } // reason

  private int getMove( int right, int up, int left, int down )
  {
	// return the lowest of four numbers that is bigger than zero
	// if this number is bigger than five return 0 if any number is zero,
    // but only if we have not yet found a piece of gold and our
    // strategy is to take risks
	int lowest;
	lowest = right;
	if( ( lowest > up && up > 0 ) || ( lowest <= 0 && up > 0 ) ) 
	  lowest = up;
	if( lowest > left && left > 0 || ( lowest <= 0 && left > 0 ) )
	  lowest = left;
	if( lowest > down && down > 0 || ( lowest <= 0 && down > 0 ) )
	  lowest = down;
	if( lowest > 5 && ( up == 0 || down == 0 || right == 0 || left == 0 ) &&
        pickedgold == false && takerisk == true )
	  return 0;
	else 
	  return lowest;
  }

  // getxxxField methods return the field that is to 
  // the right, to the left, up or down from the agent's current location
  protected AgentField getLeftField( Point of )
  {
    return myfield[ of.x - 1 ] [ of.y ];
  }

  protected AgentField getRightField( Point of )
  {
    return myfield[ of.x + 1 ] [ of.y ];
  }

  protected AgentField getUpField( Point of )
  {
    return myfield[ of.x ] [ of.y - 1 ];
  }

  protected AgentField getDownField( Point of )
  {
    return myfield[ of.x ] [ of.y + 1 ];
  }

  // for improved performance the log member has been set to final
  // which should allow compilers to optimize the if( log... ) 
  // statements away;
  // if it is of interest to log the actions an agent takes under
  // special circumstances you would have to redefine log as:
  // protected boolean log;
  // add the line: log = boolean_value to method initVars;
  // call this method to enable/disable logging
  /*
  public void setLog( boolean change )
  {
    if( log != change )
    {
      if( change == true )
      {
        System.out.println( "Logging now enabled" );
        log = change;
      }
      else
      {
        System.out.println( "Logging now disabled" );
        log = change;
      }
    }
  }
  */
}