// file:        BackGrayFilter.java
// source file: BackGrayFilter.java
// object file: BackGrayFilter.class
//
//
// included objects:
//  BackGrayFilter
//    Desc: a filter that grays the transparent regions of an image
//          - these should mostly be the regions surrounding a image.
//          useful to gray backgrounds, i.e. backgrounds of action icons 
//
// history:
//   28.03.97 - first version finished by Roman Valehrach
//
//
// critical problems: none
//
// known problems: none
//

import java.awt.*;
import java.awt.image.*;

class BackGrayFilter extends RGBImageFilter 
{
// Method: BackGrayFilter Constructor (public)
//
// *Purpose   : intializes a new instance and sets the FilterModel to the
//              Indexed version
//
// *Parameters: none
//
// *Return    : none
//
// *Variables : canFilterIndexColorModel (boolean, class) - defines whether
//                       the images are filter as a whole or pixel by pixel
//
//
  public BackGrayFilter() {
    canFilterIndexColorModel= true;
  }

// Method: filterRGB (public)
//
// *Purpose   : filters the current rgb-color value and sets the
//              color to transparent if white
//
// *Parameters: x (int) - the x-position of the current pixel
//              y (int) - the y-position of the current pixel
//              rgb (int) - rgb-value of the image at the given possition
//
// *Return    : int - the grayed value (rgb value added with white)
//
// *Variables : none
//
  public int filterRGB( int x, int y, int rgb ) {
    int a= (rgb & 0xff000000);	// mask alpha-value
    int r= (rgb & 0x00ff0000);	// mask red-value
    int g= (rgb & 0x0000ff00); 	// mask green-value
    int b= (rgb & 0x000000ff);	// mask blue-value

    if ((r > 0xf00000) && (g > 0x00f000) && (b > 0x0000f0))
    {
      return 0x40ffffff;
    }
    else {
      return (a | r | g | b); // join it all together again and return it
      // return (0xc0000000 | r | g | b); // join it all together again and return it
    }
  }// End of FilterRGB
 
}// End of class BackGrayFilter
