// file:        ColorConstants.java
// source file: ColorConstants.java
// object file: ColorConstants.class
//
//
// included objects:
//  ColorConstants
//    Desc: includes all default color constants necessary in the project 
//
//
// history:
//   08.04.97 - first version finished by Roman Valehrach
//   09.04.97 - added new default colors missing in first version
//              corret LIGHTGREEN <=> "LIGHTGRAY" error
//
// critical problems: none
//
// known problems: none
//

import java.awt.Color;

class ColorConstants extends Object
{

// Variable space:
  public static DefaultProps props__;
  private Color  color_;
  private String colorstr_;


//pubilc Default-constants

  static final int    DEFAULT_RETVALUE     = 0x00808080;
  static final String DEFAULT_RETVALUE_STR = "colors.GRAY";


// --- project color model: -------------------------------------------------------
  static final Color WHITE       = Color.white;
  // static final Color LIGHTGRAY   = Color.lightGray; 
  // Color LIGHTGRAY is a bit too light .. defining one a bit darker
  static final Color LIGHTGRAY   = new Color(0xa0, 0xa0, 0xa0);
  static final Color GRAY        = Color.gray;
  static final Color DARKGRAY    = Color.darkGray;
  static final Color BLACK       = Color.black;

  static final Color RED         = Color.red;
  static final Color GREEN       = Color.green;
  static final Color BLUE        = Color.blue;

  static final Color CYAN        = Color.cyan;
  static final Color MAGENTA     = Color.magenta;
  static final Color YELLOW      = Color.yellow;

  static final Color ORANGE      = Color.orange;
  static final Color PINK        = Color.pink;
 
  static final Color LIGHTGREEN  = new Color(0x14, 0xff, 0x05);
  static final Color DARKRED     = new Color(0xcc, 0x00, 0x00);
  static final Color VIOLET      = new Color(0xc8, 0x0a, 0xc8);
  static final Color BROWN       = new Color(0x66, 0x33, 0x00);
  static final Color GOLD        = new Color(0xff, 0xff, 0x66);
 
  // Environment-specific:
  static final Color FOREGROUND  = LIGHTGREEN;
  static final Color BACKGROUND  = GRAY;
  static final Color FRAMECOLOR  = LIGHTGREEN;
  static final Color WALLCOLOR   = BROWN;
  static final Color CROSSCOLOR  = DARKRED; 

  // Wumpus-specific:
  static final Color PITCOLOR    = BLACK; 
  static final Color DEATHCOLOR  = LIGHTGRAY;
  static final Color WUMPUSCOLOR = GRAY;
  static final Color GOLDCOLOR   = GOLD;

  // Vacuum-specific:
  static final Color DUSTCOLOR = GRAY;


// -- constants for class-use only
  static final int WHITE_VALUE       = 0;
  static final int LIGHTGRAY_VALUE   = 3;
  static final int GRAY_VALUE        = 2;
  static final int DARKGRAY_VALUE    = 4;
  static final int BLACK_VALUE       = 1;

  static final int RED_VALUE         = 5;
  static final int GREEN_VALUE       = 6;
  static final int BLUE_VALUE        = 7;

  static final int CYAN_VALUE        = 8;
  static final int MAGENTA_VALUE     = 9;
  static final int YELLOW_VALUE      = 10;
  static final int ORANGE_VALUE      = 11;
  static final int PINK_VALUE        = 12;
 
  static final int LIGHTGREEN_VALUE  = 20;
  static final int DARKRED_VALUE     = 21;
  static final int GOLD_VALUE        = 22;
  static final int VIOLET_VALUE      = 23;
  static final int BROWN_VALUE       = 24;
 
  // Envrionment-Specific:
  static final int FOREGROUND_VALUE  = 30;
  static final int BACKGROUND_VALUE  = 31;

  static final int FRAMECOLOR_VALUE  = 32;
  static final int WALLCOLOR_VALUE   = 33;
  static final int DEATHCOLOR_VALUE  = 34;
  static final int CROSSCOLOR_VALUE  = 35; 

  // Wumpus-specific:
  static final int PITCOLOR_VALUE    = 36; 
  static final int WUMPUSCOLOR_VALUE = 37; 
  static final int GOLDCOLOR_VALUE   = 38; 

  // Vacuum-specific:
  static final int DUSTCOLOR_VALUE   = 39; 

// -- constants for class-use only


// Method: ColorConstants Constructor (public)
//
// *Purpose   : initializes the new instance
//
// *Parameters: name (String, optional) - the name of the new color
//
// *Return    : none
//
// *Variables : color (String) - the starting color of the instance
//              baseurl (URL) - the baseURL of the current applet 
//              - nessessary for base-related URLs (Default-Databases)
//              databases (String, Array) all the names of the necessary
//                                        default files
//
  public ColorConstants()
  {
    if ( props__ == null )
      props__ = new DefaultProps();

    setColor( DEFAULT_RETVALUE_STR );
  }

  public ColorConstants( String color )
  {
    if ( props__ == null )
      props__ = new DefaultProps();

    setColor( color );
  }

  public ColorConstants( String color, int defaultValue )
  {
    if ( props__ == null )
      props__ = new DefaultProps();

    colorstr_ = color;
    color_ = getColor( colorstr_, DEFAULT_RETVALUE );
  }

// Method: setColor (public)
//
// *Purpose   : sets the base-color of the current FieldObject
//              * if an invalid color was given the DEFAULT_RETVALUE 
//                will be set
//
// *Parameters: color (String) - the new base-color
//
// *Return    : none
//
// *Variables : color (String) - the new base-color
//
  public void setColor ( String color )
  {
    if (  color == null )
      colorstr_ = DEFAULT_RETVALUE_STR;
    else 
      colorstr_ = color;

    color_ = getColor( colorstr_, DEFAULT_RETVALUE );
  }

 
// Method: getValue (public)
//
// *Purpose   : returns the base-color of the current FieldObject
//              * if an invalid color was given the DEFAULT_RETVALUE 
//                will be set
//
// *Parameters: color (String) - the new base-color
//
// *Return    : int - the number representing the color
//
// *Variables : color_ (Color) - the color's internal value is hashed
//
  public int getValue()
  {
    return color_.getRGB();
  }

  public int getRGB()
  {
    return color_.getRGB();
  }
 
// Method: getString (public)
//
// *Purpose   : returns the base-color of the current FieldObject
//              * if an invalid color was given the DEFAULT_RETVALUE-String
//                will be returned
//
// *Parameters: none
//
// *Return    : String - the string representing the current color
//
// *Variables : colorstr_ (String) - the color's string value
//
  public String getString()
  {
    if (  colorstr_ == null )
      setColor( DEFAULT_RETVALUE_STR );
 
    return colorstr_;
  }

 
// Method: getColor (public)
//
// *Purpose   : returns the base-color of the current FieldObject
//              * if an invalid color was given the DEFAULT_RETVALUE-String
//                will be returned
//
// *Parameters: none
//
// *Return    : Color - the decoded color
//
// *Variables : color (String) - the new base-color
//
  public Color getColor()
  {
    if ( color_ == null )
      setColor( DEFAULT_RETVALUE_STR );

    return color_;
  }
 
// Method: getColor (public, static)
//
// *Purpose   : decodes the given String ID (nm) to a new Color-Instance
//              * if an invalid color was given the DEFAULT_RETVALUE
//                will be returned
//
// *Parameters: color (String) - string id of the color to decode
//
// *Return    : Color - the decoded color
//
// *Variables : none
//
  public static Color getColor( String color )
  {
    return getColor( color, DEFAULT_RETVALUE );
  }
 
  public static Color getColor(String nm, int v) 
  {
    Integer intval = null;
    int i;

    if ( nm.startsWith( "colors." ) ) {
      if ( props__ == null )
        props__ = new DefaultProps();

      i = props__.getNumber( nm, v );
//    i = (intval != null) ? intval.intValue() : v;
      return new Color((i >> 16) & 0xFF, (i >> 8) & 0xFF, (i >> 0) & 0xFF);
    } else {
//      return (Color)(new ColorConstants( nm ));
      return String2Color( nm );
    }
  }

// Method: getPropHandle (public, static)
//         getDefaultProperties (public, static)
//
// *Purpose   : returns the handle to the DefaultProps-instance
//
// *Parameters: none
//
// *Return    : DefaultProps - the current static instance of default
//              properties
//
// *Variables : props_ (String) - the new base-color
//
  public static synchronized DefaultProps getPropHandle()
  {
    return props__;
  }

  public static synchronized DefaultProps getDefaultProps()
  {
    return props__;
  }


// Method: String2Color (public, static)
//
// *Purpose   : converts the given color-String to a Color class
//              * if an invalid color was given the DEFAULT_RETVALUE-color 
//                will be returned
//
// *Parameters: color (String) - the new base-color
//
// *Return    : Color - the decoded color
//
// *Variables : none
//
  public static Color String2Color( String value )
  {
    if ( value != null ) {
 
      if ( value.equalsIgnoreCase( "WHITE" ) )        return WHITE;
      if ( value.equalsIgnoreCase( "LIGHTGRAY" ) )    return LIGHTGRAY;
      if ( value.equalsIgnoreCase( "GRAY" ) )         return GRAY;
      if ( value.equalsIgnoreCase( "DARKGRAY" ) )     return DARKGRAY;
      if ( value.equalsIgnoreCase( "BLACK" ) )        return BLACK;

      if ( value.equalsIgnoreCase( "RED" ) )          return RED;
      if ( value.equalsIgnoreCase( "GREEN" ) )        return GREEN;
      if ( value.equalsIgnoreCase( "BLUE" ) )         return BLUE;

      if ( value.equalsIgnoreCase( "CYAN" ) )         return CYAN;
      if ( value.equalsIgnoreCase( "MAGENTA" ) )      return MAGENTA;
      if ( value.equalsIgnoreCase( "YELLOW" ) )       return YELLOW;

      if ( value.equalsIgnoreCase( "PINK" ) )         return PINK;
      if ( value.equalsIgnoreCase( "ORANGE" ) )       return ORANGE;

      if ( value.equalsIgnoreCase( "LIGHTGREEN" ) )   return LIGHTGREEN;
      if ( value.equalsIgnoreCase( "BROWN" ) )        return BROWN;
      if ( value.equalsIgnoreCase( "DARKRED" ) )      return DARKRED;
      if ( value.equalsIgnoreCase( "VIOLET" ) )       return VIOLET;
      if ( value.equalsIgnoreCase( "GOLD" ) )         return GOLD;
 
      if ( value.equalsIgnoreCase( "FOREGROUND" ) )   return FOREGROUND;
      if ( value.equalsIgnoreCase( "BACKGROUND" ) )   return BACKGROUND;
    
      if ( value.equalsIgnoreCase( "FRAMECOLOR" ) )   return FRAMECOLOR;
      if ( value.equalsIgnoreCase( "WALLCOLOR" ) )    return WALLCOLOR;
      if ( value.equalsIgnoreCase( "DEATHCOLOR" ) )   return DEATHCOLOR;
      if ( value.equalsIgnoreCase( "CROSSCOLOR" ) )   return CROSSCOLOR;

      if ( value.equalsIgnoreCase( "PITCOLOR" ) )     return PITCOLOR;
      if ( value.equalsIgnoreCase( "WUMPUSCOLOR" ) )  return WUMPUSCOLOR;
      if ( value.equalsIgnoreCase( "GOLDCOLOR" ) )    return GOLDCOLOR;

      if ( value.equalsIgnoreCase( "DUSTCOLOR" ) )    return DUSTCOLOR;
    }
 
    return new Color( DEFAULT_RETVALUE );
  }// End of String2Color

/* the following code is for internal use only and obsolet !!! : SKIP */

/*
// Method: int2Color (public, static)
//
// *Purpose   : converts the given color-value to a Color class
//              * if an invalid color was given the DEFAULT_RETVALUE-color 
//                will be returned
//
// *Parameters: color (String) - the new base-color
//
// *Return    : Color - the decoded color
//
// *Variables : none
//
  public static Color int2Color( int value )
  {
    switch( value )
    {
      case WHITE_VALUE:        return WHITE;
      case LIGHTGRAY_VALUE:    return LIGHTGRAY;
      case GRAY_VALUE:         return GRAY;
      case DARKGRAY_VALUE:     return DARKGRAY;
      case BLACK_VALUE:        return BLACK;

      case RED_VALUE:          return RED;
      case GREEN_VALUE:        return GREEN;
      case BLUE_VALUE:         return BLUE;

      case CYAN_VALUE:         return CYAN;
      case MAGENTA_VALUE:      return MAGENTA;
      case YELLOW_VALUE:       return YELLOW;

      case ORANGE_VALUE:       return ORANGE;
      case PINK_VALUE:         return PINK;
 
      case LIGHTGREEN_VALUE:   return LIGHTGREEN;
      case DARKRED_VALUE:      return DARKRED;
      case VIOLET_VALUE:       return VIOLET;
      case BROWN_VALUE:        return BROWN;
      case GOLD_VALUE:         return GOLD;

      case FOREGROUND_VALUE:   return FOREGROUND;
      case BACKGROUND_VALUE:   return BACKGROUND;
 
      case FRAMECOLOR_VALUE:   return FRAMECOLOR;
      case WALLCOLOR_VALUE:    return WALLCOLOR;
      case DEATHCOLOR_VALUE:   return DEATHCOLOR;
      case CROSSCOLOR_VALUE:   return CROSSCOLOR;
 
      case PITCOLOR_VALUE:     return PITCOLOR;
      case WUMPUSCOLOR_VALUE:  return WUMPUSCOLOR;
      case GOLDCOLOR_VALUE:    return GOLDCOLOR;
 
      case DUSTCOLOR_VALUE:    return DUSTCOLOR;
 
      default:                 return new Color( DEFAULT_RETVALUE );
    }
  }// End of int2Color
*/

/*
// Method: int2String (publicstatic )
//
// *Purpose   : returns the color's internal value
//              * if an invalid color was given the DEFAULT_RETVALUE 
//                will be returned
//
// *Parameters: color (String) - the new base-color
//
// *Return    : Color - the decoded color
//
// *Variables : none
//
  public static String int2String( int value )
  {
    switch( value )
    {
      case WHITE_VALUE:        return "WHITE";
      case LIGHTGRAY_VALUE:    return "LIGHTGRAY";
      case GRAY_VALUE:         return "GRAY";
      case DARKGRAY_VALUE:     return "DARKGRAY";
      case BLACK_VALUE:        return "BLACK";

      case RED_VALUE:          return "RED";
      case GREEN_VALUE:        return "GREEN";
      case BLUE_VALUE:         return "BLUE";

      case CYAN_VALUE:         return "CYAN";
      case MAGENTA_VALUE:      return "MAGENTA";
      case YELLOW_VALUE:       return "YELLOW";

      case ORANGE_VALUE:       return "ORANGE";
      case PINK_VALUE:         return "PINK";

      case LIGHTGREEN_VALUE:   return "LIGHTGREEN";
      case DARKRED_VALUE:      return "DARKRED";
      case VIOLET_VALUE:       return "VIOLET";
      case BROWN_VALUE:        return "BROWN";
      case GOLD_VALUE:         return "GOLD";

      case FOREGROUND_VALUE:   return "FOREGROUND";
      case BACKGROUND_VALUE:   return "BACKGROUND";

      case FRAMECOLOR_VALUE:   return "FRAMECOLOR";
      case WALLCOLOR_VALUE:    return "WALLCOLOR";
      case DEATHCOLOR_VALUE:   return "DEATHCOLOR";
      case CROSSCOLOR_VALUE:   return "CROSSCOLOR";

      case PITCOLOR_VALUE:     return "PITCOLOR";
      case WUMPUSCOLOR_VALUE:  return "WUMPUSCOLOR";
      case GOLDCOLOR_VALUE:    return "GOLDCOLOR";

      case DUSTCOLOR_VALUE:    return "DUSTCOLOR";

      default:                 return DEFAULT_RETVALUE_STR;
    }
  }
*/

/*
// Method: String2int (public, static)
//
// *Purpose   : returns the color's internal value
//              * if an invalid color was given the DEFAULT_RETVALUE 
//                will be returned
//
// *Parameters: color (String) - the new base-color
//
// *Return    : Color - the decoded color
//
// *Variables : none
//
  public static int String2int( String value )
  {
    if ( value != null ) {

      if ( value.equalsIgnoreCase( "WHITE" ) )        return WHITE_VALUE;
      if ( value.equalsIgnoreCase( "LIGHTGRAY" ) )    return LIGHTGRAY_VALUE;
      if ( value.equalsIgnoreCase( "GRAY" ) )         return GRAY_VALUE;
      if ( value.equalsIgnoreCase( "DARKGRAY" ) )     return DARKGRAY_VALUE;
      if ( value.equalsIgnoreCase( "BLACK" ) )        return BLACK_VALUE;

      if ( value.equalsIgnoreCase( "RED" ) )          return RED_VALUE;
      if ( value.equalsIgnoreCase( "GREEN" ) )        return GREEN_VALUE;
      if ( value.equalsIgnoreCase( "BLUE" ) )         return BLUE_VALUE;

      if ( value.equalsIgnoreCase( "CYAN" ) )         return CYAN_VALUE;
      if ( value.equalsIgnoreCase( "MAGENTA" ) )       return MAGENTA_VALUE;
      if ( value.equalsIgnoreCase( "YELLOW" ) )       return YELLOW_VALUE;

      if ( value.equalsIgnoreCase( "PINK" ) )         return PINK_VALUE;
      if ( value.equalsIgnoreCase( "ORANGE" ) )       return ORANGE_VALUE;


      if ( value.equalsIgnoreCase( "LIGHTGREEN" ) )   return LIGHTGREEN_VALUE;
      if ( value.equalsIgnoreCase( "DARKRED") )       return DARKRED_VALUE;
      if ( value.equalsIgnoreCase( "VIOLET" ) )       return VIOLET_VALUE;
      if ( value.equalsIgnoreCase( "BROWN" ) )        return BROWN_VALUE;
      if ( value.equalsIgnoreCase( "GOLD" ) )         return GOLD_VALUE;


      if ( value.equalsIgnoreCase( "FOREGROUND" ) )   return FOREGROUND_VALUE;
      if ( value.equalsIgnoreCase( "BACKGROUND" ) )   return BACKGROUND_VALUE;

      if ( value.equalsIgnoreCase( "FRAMECOLOR" ) )   return FRAMECOLOR_VALUE;
      if ( value.equalsIgnoreCase( "WALLCOLOR" ) )    return WALLCOLOR_VALUE;
      if ( value.equalsIgnoreCase( "DEATHCOLOR" ) )   return DEATHCOLOR_VALUE;
      if ( value.equalsIgnoreCase( "CROSSCOLOR" ) )   return CROSSCOLOR_VALUE;


      if ( value.equalsIgnoreCase( "PITCOLOR" ) )     return PITCOLOR_VALUE;
      if ( value.equalsIgnoreCase( "WUMPUSCOLOR" ) )  return WUMPUSCOLOR_VALUE;
      if ( value.equalsIgnoreCase( "GOLDCOLOR" ) )    return GOLDCOLOR_VALUE;

      if ( value.equalsIgnoreCase( "DUSTCOLOR" ) )    return DUSTCOLOR_VALUE;
    }

    return DEFAULT_RETVALUE;
  }
*/

} // End of Class ColorConstants
