// file:        DefaultProps.java
// source file: DefaultProps.java
// object file: DefaultProps.class
//
//
// included objects:
//  DefaultProps
//    Desc: includes all default color constants necessary in the project,
//          this class also takes over the handling of static numbers
//          for every imageresource-group 
//
//
// history:
//   14.04.97 - first version finished by Roman Valehrach
//
// critical problems: none
//
// known problems: the current implementation only recognizes link-loops
//                 of first order !!
//           i.e.: link1 -> link1  => break
//           i.e.: link1 -> link2 -> ... -> link1  => no break :-{
//
//       Therefore the maximum Link-Depth has been set to 10!!
//
//
// additional notes: 
//   This class needs support from World! (first caller)
//   Is there an equivalent already implemented ? 
//   If YES this class should extend it...
//
//   This class is not really an extension of the class Properties
//   because it hides the multiple throws from the user
//   (There is no need to try 5 Exceptions for just one call anymore...)
//

import java.io.IOException;
import java.io.InputStream;
import java.io.OutputStream;
import java.io.DataInputStream;
import java.io.PrintStream;
import java.util.Properties;
import java.util.Vector;
import java.net.URL;
import java.io.FileNotFoundException;
import java.net.MalformedURLException;

class DefaultProps extends Object // Properties 
{
  // private static final boolean DEBUG = false;

  protected static URL base_url__;
  protected static Properties default_props__;

  protected static Vector static_num__;

  protected static final String DELIMITER         = ".";

  protected static String defaults_file__         = "default_files.cfg";
  protected static final String DEFAULTS_BASE     = "defaults"+ DELIMITER +"files";

  protected static final String TOTAL_ID          = DELIMITER + "TOTAL";
  protected static final String NAME_ID           = DELIMITER + "STR";
  protected static final String IMAGE_ID          = DELIMITER + "IMG";
  protected static final String COLOR_ID          = DELIMITER + "COLOR";

  protected static final String LINK_ID           = "LINK_";
  protected static final int    LINK_ID_LEN       = LINK_ID.length();
  protected static final int    MAX_DEPTH         = 10;

// Method: DefaultProps Constructor (public)
//
// *Purpose   : initializes the new instance
//
// *Parameters: name (String, optional) - the name of the new color
//
// *Return    : none
//
// *Variables : color (String) - the starting color of the instance
//              base_url__ (URL) - the baseURL of the current applet 
//              - nessessary for base-related URLs (Default-Databases)
//              databases (String, Array) all the names of the necessary
//                                        default files
//
  public DefaultProps()
  {
    if ( default_props__ == null ) 
      System.err.println("Applet-Properties must be initalized!");

    //  default_props__ = new Properties();

    if (static_num__ == null)
      resetStatics();
  }

/*
  public DefaultProps(Properties defaults)
  {
    if ( default_props__ != null ) { // ignore the defaults given
  //TODO: finde a way to copy the defaultmap =>
  //      new_instance => old_defaults(newdefauls(old_defaults.defaults))
  //    or             => old_defaults(old_defaults.defaults(newdefauls))
    } else
      default_props__ = new Properties(defaults);
  }
*/

  public DefaultProps( URL baseurl )
  {
    this( baseurl, null );
  }

  public DefaultProps( URL baseurl, String databases[] )
  {
    int i;

    if ( (base_url__ != null) && (baseurl != null) )
      base_url__= baseurl;
    else if (base_url__ == null)
      base_url__= baseurl;

    if ( (databases != null) && (databases.length > 0 ) )
    {
      for ( i=0; i< databases.length; i++ )
      {
        default_props__= loadURLDefaults( default_props__, 
                                          base_url__, 
                                          databases[i] );
      }
    } 
    else if (default_props__ == null) //initialize only once
    {
      loadDatabase();
    }

    if (static_num__ == null)
      resetStatics();
  }

  public static int getNumber( Properties props, String tmp, 
    int defaultValue )
  {
    try {
      Integer intval;
//      System.out.println("Trying to decode number-property: >" + tmp+"<");

      if ( (tmp= getProperty( props, tmp ) ) == null) {
//         System.out.println("Decode unsuccessful, returning default : >" + defaultValue + "<");
         return defaultValue;
      }


      intval = decode( tmp );
      int i = (intval != null) ? intval.intValue() : defaultValue;
//      System.out.println("Decode successful: >" + x + "< - orig: >"+ tmp +"<");
      return i; 
    }
    catch (Exception e)
    {
    }
    return defaultValue;
  }

  public static int getNumber( String tmp, int defaultValue )
  {
    return getNumber( default_props__, tmp, defaultValue );
  }

  public static int getNumber( Properties props, String tmp )
  {
    return getNumber( default_props__, tmp, -1 );
  }

  public static int getNumber(String tmp)
  {
    return getNumber( default_props__, tmp, -1);
  }

//public static void loadDatabase()
  public void loadDatabase()
  {
    default_props__ = getDatabases( default_props__, 
                                    base_url__, 
                                    defaults_file__ );
  }

  public static Properties getDatabases(
    Properties defaultProp, 
    URL baseurl, 
    String startpage) 
  {
    Properties props= defaultProp;
    String tmp;
    int i,j;

    props= loadURLDefaults(props, baseurl, startpage);

/*
    if ( DEBUG )
    {
      System.out.println( "Current Simulator Properties - BEGIN" );
      props.list(System.out);
      System.out.println( "Current Simulator Properties - ENDE" );
//      j=getNumber( props, "defaults_files_TOTAL", 0 );
    }
*/

    j= getNumber( props, DEFAULTS_BASE + TOTAL_ID, 0 );
    j++; // counting from 1 to omega ... the number j is 
         // for the bounce check in the next for -> increment
    

    for ( i = 1; i < j; i++ )  {
      if ( (tmp=getProperty( props, DEFAULTS_BASE + DELIMITER + i + NAME_ID ) ) != null) {
        props= loadURLDefaults(props, base_url__, tmp);
      }
    }

    return props;
  }

  public static void loadDefaultProperties(String new_databases[])
  {
    int i;

    if ( (new_databases != null) && (new_databases.length > 0 ) )  {
      for ( i=0; i< new_databases.length; i++ )  {
        default_props__= loadURLDefaults(default_props__, base_url__, new_databases[i]);
      }
    }
  }

 
  public synchronized void load(InputStream in) throws IOException {
    default_props__.load(in);
  }

  public synchronized void save(OutputStream out, String header) {
    default_props__.save(out, header);
  }

  public static Properties loadURLDefaults( Properties props, 
                                            URL baseurl, String file )
  {
    URL test_stream_url= null;
    String value= null;
    DataInputStream propFile=null;
    DataInputStream dis=null;
// with JDK 1.1 use this:
//      BufferedReader dis=null;

    try {
      test_stream_url = new URL(baseurl, file);
     
/*
      if ( DEBUG )
      {
        System.out.println ("Loading Defaults - Current text of URL: " + 
          test_stream_url.toString() );

// with JDK 1.0.1-- use this:
        dis = new DataInputStream(test_stream_url.openStream());
// with JDK 1.1 use this:
//      BufferedReader dis= new BufferedReader(new InputStreamReader(test_stream_url.openStream()));

        System.out.println ("Loading Defaults - while begins ");
        while ((value = dis.readLine()) != null) {
          System.out.println ( value );
        }
      }
*/

      if ( test_stream_url != null )
      {
        propFile = new DataInputStream(test_stream_url.openStream());
        if ( props == null ){
          props= new Properties();  //System.getProperties());
        } else {
          props= new Properties( props );
        }

        props.load(propFile);

        // set the system properties
        // System.setProperties(p);
/*
        if ( DEBUG ) 
        {
          System.out.println("All properties:");
          props.list(System.out);
        }
*/
      }
    }
    catch (MalformedURLException e) {
      System.err.println("Properties-File nicht ladbar, fehlerhafte URL: " + e.getMessage());
      e.printStackTrace();
    } 
    catch (FileNotFoundException e) {
      System.err.println("DefaultProperties - File not found Error: " + e.getMessage());
    } 
    catch (IOException e) {
      System.err.println("DefaultProperties - IO Exception: " + e.getMessage());
    } 
    catch (SecurityException e) {
      System.err.println("DefaultProperties - Security Exception: " + e.getMessage());
    }
    catch (Exception e) { // and just everthing else
      System.err.println("DefaultProperties - Common Exception: " + e.getMessage());
    }
    finally {
/*
      if (dis != null)  
        dis.close(); 
      if (propFile != null)  
        propFile.close(); 
*/
    }

    return props;
  }


  public static String getProperty( Properties props, 
    String property, String defaultValue )
  {
    try {
      String retval= props.getProperty(property);
      if ( retval == null ) 
      {
        System.err.println("Failed to get the Porperty: >" + property + 
          "< - using default: >"+defaultValue+"<");
        return defaultValue;
      }
      else
        return retval;
    }
    catch (Exception e) {
      return defaultValue;
    }
  }

  public static String getPropertyNoOutput( Properties props, 
    String property, String defaultValue )
  {
    try {
      String retval= props.getProperty(property);
      if ( retval == null ) 
      {
        // System.err.println("Failed to get the Porperty: >" + property + 
        //  "< - using default: >"+defaultValue+"<");
        return defaultValue;
      }
      else
        return retval;
    }
    catch (Exception e) {
      return defaultValue;
    }
  }

  public static String getProperty( String property, String defaultValue )
  {
    return getProperty( default_props__, property, defaultValue );
  }

  public static String getProperty( Properties props, String property )
  {
    return getProperty( props, property, null );
  }

  public static String getProperty( String property )
  {
    return getProperty( default_props__, property, null );
  }

  public static String getPropertyNoOutput( String property, String defaultValue )
  {
    return getPropertyNoOutput( default_props__, property, defaultValue );
  }

  public static String getPropertyNoOutput( Properties props, String property )
  {
    return getPropertyNoOutput( props, property, null );
  }

  public static String getPropertyNoOutput( String property )
  {
    return getPropertyNoOutput( default_props__, property, null );
  }

  public static void list(Properties props, PrintStream out)
  {
    try {
      props.list(out);
    }
    catch (Exception e) {
    }
  }

  public void list(PrintStream out)
  {
    try {
      default_props__.list(out);
    }
    catch (Exception e) {
    }
  }

// Method: getDefaultProperties (public, static)
//
// *Purpose   : returns the handle to the DefaultProps-instance
//
// *Parameters: none
//
// *Return    : DefaultProps - the current static instance of default
//              properties
//
// *Variables : props_ (String) - the new base-color
//
  public static synchronized Properties getDefaultProperties()
  {
    try {
/*
      if (default_props__ == null)
      {
        Properties props= null;

        default_props__ = getDatabases( props, 
                                        base_url__, 
                                        defaults_file__ );
      }
*/

      return default_props__;
    } catch (Exception e) {
    }

    return null;
  }

  public static URL getBaseURL ()
  {
    return base_url__;
  }

  /** USE THIS ONE - BECAUSE NOT EVERY BROWSER HAS THE FULL Integer IMPLEMENTATION..
   * Decodes a string into an Integer. Deals with decimal, hexadecimal,
   * and octal numbers.
   * @param nm the string to decode
   * @since   JDK1.1
   */
  public static Integer decode(String nm) throws NumberFormatException {
    if (nm.startsWith("0x")) {
      return Integer.valueOf(nm.substring(2), 16);
    }
    if (nm.startsWith("#")) {
      return Integer.valueOf(nm.substring(1), 16);
    }
    if (nm.startsWith("0") && nm.length() > 1) {
      return Integer.valueOf(nm.substring(1), 8);
    }
    if (nm.startsWith("o") && nm.length() > 1) {
      return Integer.valueOf(nm.substring(1), 8);
    }

    return Integer.valueOf(nm);
  }

  public String decodeLink( String firstSearch )
  {
    String tmp= null;

    // System.err.println( "Decoding base.. >" + firstSearch + "<");

    if ( (firstSearch == null) ||
         ( (tmp= getProperty( firstSearch )) == null) )
      return null;
    else 
      return checkLink( default_props__, tmp );
  }

  public static String decodeLink(Properties props, String firstSearch )
  {
    String tmp= null;

    // System.err.println( "Decoding base.. >" + firstSearch + "<");

    if ( (firstSearch == null) ||
         ( (tmp= getProperty( props, firstSearch )) == null) )
      return null;
    else 
      return checkLink( props, tmp);
  }


  public String checkLink( String firstSearch )
  {
    return checkLink(default_props__, firstSearch );
  }

  public static String checkLink (Properties props, String firstSearch )
  {
    String tmp= firstSearch;
    String last = tmp;
    int i=1;

    // System.err.println( "Decoded to >" + tmp + "<");
    while ( (tmp != null) &&
            (tmp.length() > LINK_ID_LEN) &&
            (tmp.startsWith( LINK_ID ))  )
    { // in case of a link get the ref (links mustn't be links_to_links_to...)
      // System.out.println( "Link encountered... ( depth >"+i+"<)decoding now...");
      if ( (tmp= props.getProperty( tmp.substring( LINK_ID_LEN ) )) == null)
      {
        // System.out.println( "No Property found for Link.. ");
        return null;
      }
      if ( tmp.equals(last) )
      {
        System.err.println( "A direct 'LINK_'-loop (1:1, search-depth#"+ i +") was found in the system-properties! ");
        System.err.println( "The link: "+tmp+" is unresolveable!");
        return null;
      }
      if (i > MAX_DEPTH) // 1st order Link-Loop => break
      {
        System.err.println( "A possible 'LINK_'-loop (depth #"+i+") was found in the system-properties! ");
        System.err.println( "The current link: "+tmp+" is unresolveable!");
        System.err.println( "Maximum search-depth reached... Giving up...");
        return null;
      }
      last = tmp;
      i++;
    }

    // System.err.println( "Finished ... >" + tmp + "<");

    return tmp;
  }

  public static String getLinkableProperty( Properties props, String get, String defaultVal  )
  {
    String tmpstr;

    return ( (tmpstr= decodeLink( props, get )) == null ) ? defaultVal : tmpstr;
  }

  public String getLinkableProperty( String get, String defaultVal  )
  {
    String tmpstr;

    return ( (tmpstr= decodeLink( get )) == null ) ? defaultVal : tmpstr;
  }


  public String decodeColorProperty( String base, int i, String defaultVal )
  {
    String tmpstr;

    return ( (tmpstr= decodeLink( base + DELIMITER + i + COLOR_ID )) == null ) ? defaultVal : tmpstr;
  }


  // returns the static number for a given id-string
  public static int getStatic( String staticname )
  {
    int i;
    int retvalue = 0;
    boolean found = false;

    String tmp;

    if ( (staticname == null) || (static_num__ == null) )
    {
      // System.err.println("no searched: "+ staticname+ " resources: " + static_num__);
      return retvalue;
    }

    for ( i = 0; i < static_num__.size() && !found ; i+=2 )
    {
      if (static_num__.elementAt(i) instanceof String)
      {
        tmp = (String)(static_num__.elementAt(i));

        if (   (staticname.equals(tmp)) && (i+1 < static_num__.size()) 
            && (static_num__.elementAt( i+1 ) instanceof Integer) )
        {
          retvalue = ((Integer)(static_num__.elementAt( i+1 ))).intValue();
          found = true;
        }
      }
    }

    // System.out.println("Searched for static: "+ staticname+ ", current value: " + retvalue + ", value found: "+ found +", size: "+ static_num__.size());
    return retvalue;
  }

  
  public static int setStatic( String staticname, int newvalue )
  {
    int i;
    int retvalue = 0;
    boolean found = false;

    String tmp;

    if ( (staticname == null) || (static_num__ == null) )
    {
      // System.err.println("no searched: "+ staticname+ " resources: " + static_num__);
      return retvalue;
    }

    for ( i = 0; i < static_num__.size() && !found ; i+=2 )
    {
      if (static_num__.elementAt(i) instanceof String)
      {
        tmp = (String)(static_num__.elementAt(i));
        if ( (staticname.equals(tmp)) && (i+1 < static_num__.size()) )
        {
          retvalue = newvalue;
          static_num__.setElementAt( (Object)(new Integer( retvalue )), i+1 );
          found = true;
        }
      }
    }

    if (!found) {
      retvalue = newStatic( staticname, newvalue );
    }

    // System.out.println("Searched: "+ staticname+ ", new value: " + retvalue + ", value found: "+ found +", size: "+ static_num__.size());
    return retvalue;
  }

  public static int increaseStatic( String staticname, int incvalue )
  {
    int i;
    int retvalue = 0;
    boolean found = false;

    String tmp;

    if ( (staticname == null) || (static_num__ == null) )
    {
      // System.err.println("no searched: "+ staticname+ " resources: " + static_num__);
      return retvalue;
    }

    for ( i = 0; i < static_num__.size() && !found ; i+=2 )
    {
      if (static_num__.elementAt(i) instanceof String)
      {
        tmp = (String)(static_num__.elementAt(i));
        if (   (staticname.equals(tmp)) && (i+1 < static_num__.size()) 
            && (static_num__.elementAt( i+1 ) instanceof Integer) )
        {
          retvalue = ((Integer)(static_num__.elementAt( i+1 ))).intValue() + incvalue;
          static_num__.setElementAt( (Object)(new Integer( retvalue )), i+1 );
          found = true;
        }
      }
    }

    if (!found) {
      retvalue = newStatic( staticname, incvalue );
    }

    // System.out.println("Searched: "+ staticname+ ", inc.value: " + retvalue + ", value found: "+ found +", size: "+ static_num__.size());
    return retvalue;
  }

  public static int newStatic( String staticname, int newvalue )
  {
    int retvalue = 0;

    if ( (staticname == null) || (static_num__ == null) )
    {
      // System.err.println("no searched: "+ staticname+ " resources: " + static_num__);
      return retvalue;
    }

    static_num__.addElement( (Object)staticname );
    static_num__.addElement( (Object)(new Integer( newvalue )) );
    
    return newvalue;;
  }

  // remove all currently stored static values 
  public static void resetStatics()
  {
    // System.out.println("Resetting the static variables!" );
    static_num__ = new Vector( 6, 4 ); 
  }

}//End of class DefaultProps

