// file:        DrawableResources.java
// source file: DrawableResources.java
// object file: DrawableResources.class
//
//
// included objects:
//  DrawableResource
//    Desc: implements an drawable reference to a single image (=>standard)
//
//
// history:
//   14.04.97 - first version finished by Roman Valehrach
//
//
// critical problems: none
//
// known problems: none
//

import java.awt.Image;
import java.awt.image.ImageObserver;
import java.awt.Graphics;
import java.awt.Rectangle;
import java.awt.Color;


class DrawableResources extends Resources
{
  protected int phase_;

  DrawableResources()
  {
    super();
    phase_ = 0;
  }

  public boolean setImagePhase( int newphase )
  {
    int max = getImageReferencesSize();
    int last = phase_;
    if ( (phase_ = newphase) >= max )
    {
      phase_ = last;
      return false;
    } else {
      return true;
    }
  }

  public int getImagePhase()
  {
    return phase_;
  }

  public void advanceImagePhase()    // Ring-Advancement 1,2,3...x,1,2,3...
  {
    int max = getImageReferencesSize();
    phase_++;
    if ( phase_ >= max )
      phase_ = 0;
  }

  public boolean draw( Graphics g, Rectangle area )
  {
    int max = getImageReferencesSize();
    boolean notfound = true;
    DrawableReferenceIfc tmp;

    if (phase_ >= max)
    {
      phase_ = max - 1;
    }

    while ( notfound && (phase_ > -1) )
    {
      tmp = (DrawableReferenceIfc)getImageReferenceAt( phase_ );

      if ( ( tmp instanceof DrawableReferenceIfc ) &&
             tmp.draw( g, area ) )
      {
        notfound = false;
        return true;
      } else 
      {
        phase_--;
      }
    }
    return false;
  }

  public void drawnot( Graphics g, Rectangle area )
  {
    int max = getImageReferencesSize();
    boolean notfound = true;
    DrawableReferenceIfc tmp;

    if (phase_ >= max)
    {
      phase_ = max - 1;
    }

    while ( notfound && (phase_ > -1) )
    {
      tmp = (DrawableReferenceIfc)getImageReferenceAt( phase_ );

      if ( ( tmp instanceof DrawableReferenceIfc ) &&
             tmp.draw( g, area ) )
      {
        notfound = false;
        return;
      } else {
        phase_--;
      }
    }
  }


}// End of class DrawableResources
