// file:        Field.java
// source file: Field.java
// object file: Field.class
//
//
// included objects:
//  Field
//    Desc: implements the FieldObject-container for 
//          (Agent-) Environments
//          this container can store FieldObjects (strict instance checking)
//          and is extended by new Bases (->FieldBaseObject)
//
//
// history:
//   22.02.97 - first version finished by Roman Valehrach
//
//   04.03.97 - removed some errors (String-Compares with String.equals() only,
//              don't use (String == Konstant)
//   05.03.97 - appended code for slicing up Fields for fieldobjects_
//   07.03.97 - added reference (+handling) for images
//   11.03.97 - Fields only have a rect-drawspace of 1 -> width-1/1 -> height-1
//              => now fields display border properly
//   12.03.97 - new extensions for displaying images (as per protocol 11.03.97)
//   20.03.97 - added & extended function for handling resource names
//   23.03.97 - added a minSize method, added multiple wall images support
//   27.03.97 - changed displaying style of field objects in empty field in 
//              accordance to managment review on 27.03.97
//   01.04.97 - new way of displaying FieldObjects *on* pits 
//   02.04.97 - added setRes, getRes
//   04.04.97 - added double buffered output/changed setArea
//   05.04.97 - changed variables name space & Interface
//   06.04.97 - added getImage, getObserver 
//
//   14.04.97 - fitted for major redesign
//
//   all above changes made by Roman
//
//
// critical problems: 
//   * Netscape does not excute the clearRect method -> old obj. images 
//     stay in field
//   * JDK 1.0.1-- appletviewers and Java-enabled Browsers have problems
//     with setColor -> sometimes the field is painted in green
//
// known problems:
//   * Fields might have problems displaying (calculating space) if the 
//     Fields are very small - debugging code yet to come
//   * still missing a handling routine for String.Prints if Field too small
//
//   * only the last FieldObject (that is no FieldBase) is draw filling
//     the environment *decides* which one fill be draw by adding/removing
//
//   * ALL exception handling missing
//
//
// to think about:
//   * aspect ratio is only guaranteed in squares
//     => Fields AND Images best have square-size (x=y)
//
//   * this is FieldBase's problem not !:
//     * Walls and Pits should always fill the Field
//     * What about Exit-Image ?
//
//   * How to copy Wall images in a given space - wall-paper effect
//

import java.util.Vector;
import java.awt.*;
import java.awt.image.*;
import java.net.*;

import java.applet.Applet;


class Field extends Object implements Drawable
{

  // private static final boolean DEBUG = false;

  private static final boolean CREATE_SUB_GRAPHIC = false;

// class (member)variables section  (* marked with a UNDERSCORE >_<:
  private   String type_;              // ("WALL", "SPACE", "PIT", "EXIT", ...)
  protected Vector fieldobjects_;

  public static ResourceLoader resloader;
  protected String    resname_;          // for compatibility
  protected boolean   drawready = false; // was the graphics/area set ?
  protected Graphics  g_;                // own graphics object for drawing
  protected Rectangle grect_;            // storage for setArea parameters
  protected Rectangle drawrect_;         // storage for current drawing params

  protected ImageObserver myobs_;

  // reference to the current image & its name and the default imagemap

  protected final static String DEFAULT_BACKGROUND_STR = "colors.field.SPACECOLOR";
  protected final static String DEFAULT_FRAMECOLOR_STR = "colors.field.FRAMECOLOR";


  protected boolean list_valid_;      // sign to skip unneeded getTrueObjectlist 

  protected int     list_valid2_;     // this one is for the Active/Passive (=>Sub)FieldLists
                                      // -> list_valid_ invalidates objectlist2
                                      // too... list_valid2_ represents the id for the SubList
                                      // 0 = invalid, 
                                      // 1= Active/Passive-List
                                      // 2= Active-List
                                      // 3= Passive-List
  protected FieldObject[] objectlist_;
  protected FieldObject[] objectlist2_;

  protected int num_bases   = -1; // the number of FieldBaseObjects
  protected int num_passive = -1; // the number of passive FieldObjects
  protected int num_active  = -1; // the number of active  FieldObjects
  protected int num_other   = -1; // the number of all other FieldObjects

  // references for double-buffered screen output
// currently doublebuffering isn't used
//  protected Image         doublebuffer_;         // own offscreen double buffer
//  protected Graphics      doublebuffer_g_;

  // variables for speeded graphics
  protected boolean noclear_  = false; // this is set when no overall-clearing
                                       // of the field (via clearRect) is 
                                       // needed (i.e. for Walls - fill the
                                       // whole field)
//protected boolean nochange_ = false; // have the FieldObjects changed
                                       // since the last draw (If YES => clear
                                       // the Field

  protected Color     frame_color_;
  protected Color     background_;

// Constants Section:
  protected static final int xFIELDOBJECT_DISTANCE = 1; // _DISTANCE between each fields
  protected static final int yFIELDOBJECT_DISTANCE = 1; // _DISTANCE between each fields
  protected static final int xBORDER_DISTANCE = 1;      // _DISTANCE between border and field
  protected static final int yBORDER_DISTANCE = 1;      // _DISTANCE between border and field

  protected static final int xMIN_SPRITE_SIZE = 16; // minimum sizes for recognizable
  protected static final int yMIN_SPRITE_SIZE = 16; // FieldObject-images


// *** Implementation Section: ***

// Method: Field Constructor (public)
//
// *Purpose   : intializes an instance of Field - creates a new Vector,
//              and sets the type, nulls image resources 
//
// *Parameters: fieldtype (String) - defines the type of the Field-Object
//              imgref (String) - default reference to an imagemap
//              obs (ImageObserver) - observer for the image
//
// *Return    : none
//
// *Variables : fieldobjects_ (Vector, class) - a list of FieldObjects in
//                                                the Field-instance
//              type_ (String, class) - the type of the Field-instance
//              resname_(String, class) - default reference to an imagemap
//              myimage_ (Image, class) - pointer to a resource (null)
//
  public Field( String fieldtype, String resource, ImageObserver observer ) 
  {
    fieldobjects_= new Vector( 3, 2 );

    // background_ = Color.white;
    background_ = ColorConstants.getColor( defaultBackgroundName() );
    frame_color_= ColorConstants.getColor( defaultFrameColorName() );
    freeSublists();

    if (resloader == null)
      resloader = new ResourceLoader();

    if (resource != null)
      resname_ = resource;

    if (observer != null)
      setObserver( observer );

    setType(fieldtype);
    // resloader.getResource( this, null );
  }

  public Field( String fieldtype, String resource ) 
  {
    this( fieldtype, resource, null );
  }
  public Field( String fieldtype, ImageObserver observer ) 
  {
    this( fieldtype, null, observer );
  }
  public Field( String fieldtype ) 
  {
    this( fieldtype, null, null );
  }


// Method: setType (public)
//
// *Purpose   : sets the type of the Field-instance
//              it only sets a new type if it is not (null)
//              - in case of a null the type is automatically 'SPACE'
//
// *Parameters: newtype (String) - the new type of the Field-instance
//
// *Return    : none
//
// *Variables : type_ (String, class) - the type of the Field-instance
//
  public void setType(String newtype) {
    if ( newtype != null ) 
    {
      type_= newtype;

      removeAllObjectsAndBase();
      FieldObject fld=null;

      if ( getType().equalsIgnoreCase("WALL") )
      {
        noclear_ = true;
      }
      createFieldBase();
    } else {
      type_= "SPACE";
    }
  }

  protected void createFieldBase()
  {
    FieldObject[] fieldbases = null;
    int i = 0;
    String type = getType();

    if (type == null) return;

    if ( type.equalsIgnoreCase("WALL") )
    {
      fieldbases = new FieldObject[1];
      fieldbases[i] = new Wall(resname_, getObserver());
    }
/*
    else
    if ( type.equalsIgnoreCase("EXIT") )
    {
      fieldbases = new FieldObject[1];
      fieldbases[i] = new Exit(resname_, getObserver());
    }
    else
    if ( type.equalsIgnoreCase("PIT") )
    {
      fieldbases = new FieldObject[1];
      fieldbases[i] = new Pit(resname_, getObserver());
    }
*/
    if (fieldbases != null)
    {
      while ( i < fieldbases.length )
      {
        this.addObjectNoCheck( fieldbases[i++] );
      }
    }
  }


// Method: getType (public)
//
// *Purpose   : returns the type of the Field-instance
//
// *Parameters: none
//
// *Return    : String - the type of the Field-instance
//
// *Variables : type_ (String, class) - the type of the Field-instance
//
  public String getType() {
    return (type_);
  }


// Method: setRes (public)
//
// *Purpose   : sets the current image's path/name of the FieldObject 
//              instance
//              this method also sets the current default color
//
//              * if param is (null) it searches for the defaultResName()
//               and if that does not help it returns the EMPTY_IMAGE_PROP
//                and if that does not solve it just returns DEFAULT_IMAGE
//
// *Parameters: String - the name of the current image
//  
// Return     : none
//
// *Variables : resname_(String, class) - stores the name of the image
//
//
  public void setRes(String resname)
  {
    FieldObject tmpfld= getFirstFieldBase();

    this.resname_ = resname;
    
    if ( tmpfld != null )
      tmpfld.setRes( resname );

  }//End of setRes

// Method: getRes (public)
//
// *Purpose   : returns the fist FieldBases resource name
//
// *Parameters: none
//
// *Return    : String - the name of the current image
//
// *Variables : resname_(String, class) - stores the name of the image
//
  public String getRes()
  {
    FieldObject tmpfld= getFirstFieldBase();
    String ret;
    
    if ( tmpfld != null ) 
    {
      ret = tmpfld.getRes();
      // System.out.println( getType() +": Returning getRes-quest: >" + ret+"<");
    }
    else
    {
      // System.out.println( getType() +": Returning Fielddef - >EMPTY<  current size>"+getSize()+"<  bases>"+size()+"<");
      ret = "EMPTY";
    }

    resname_ = ret;
    return ret;
  }

// Method: defaultFrameColorName  (protected)
//         defaultBackgroundName  (protected)
//
// *Purpose   : returns a default property identifing string for a color
//              - good for beeing overwritten by subclasses to set an other
//                default property identifing string 
//                (the string to be looked up in the system-property list
//
// *Parameters: none
//
// *Return    : String - the new default color
//
// *Variables : all the nessessary id-strings
//
  protected String defaultFrameColorName()  { return DEFAULT_FRAMECOLOR_STR; }
  protected String defaultBackgroundName()  { return DEFAULT_BACKGROUND_STR; }


// Method: decodeColor (public) - unused
//
// *Purpose   : returns the color-instance represented by the color-id string
//              * defaultValue is the color-value the Color shall have in
//                the case that NO (decodeable) color-id was found 
//
// *Parameters: decode (String) - the string to decode to a color
//              defaultStr (String) - the default-string to decode on null
//
// *Return    : Color - the decoded color
//
// *Variables : none
//
/* not used in the current implemenation - commented out for a leaner code-model
  public Color decodeColor( String color, int defaultValue )
  {
    return ColorConstants.getColor( color, defaultValue );
  }
*/

// Method: set/getResource (public) - unused
//
// *Purpose   : sets the drawing-resource for the current field instance
//
// *Parameters: newres (Resources) - the new (drawing) resource
//          or: none
//
// *Return    : none
//          or: Resources - the current (drawing) resource
//
// *Variables : myimage_ (ImageReference, class) - reference to a resource
//
/* not used in the current implemenation - commented out for a leaner code-model
  public void setResource( Resources newres )
  {
    FieldObject tmpfld= null;
    int objectcount= size();
    int i;

    for ( i=0; i<objectcount; i++ )   // if it is a the Base of the Field
    {
      tmpfld= (FieldObject) fieldobjects_.elementAt(i);

      if ( tmpfld instanceof FieldBaseObject )
      {
        tmpfld.setResource( newres );
        // System.out.println("Setting FieldBase: " + tmpfld.getName());
      }
    }
  }
*/

/* not used in the current implemenation - commented out for a leaner code-model
  public Resources getResource()
  {
    FieldObject tmpfld= getFirstFieldBase();
    
    if ( tmpfld instanceof FieldObject )
      return tmpfld.getResource();
    else
      return null;
  }
*/


/* - obsolete - use loadResources() of FieldBaseObject-instance
  public void giveImage( Image newimg, ImageObserver newobs ) 
  {
    FieldObject tmpfld= null;
    int objectcount= size();
    int i;

    for ( i=0; i<objectcount; i++ )   // if it is a the Base of the Field
    {
      tmpfld= (FieldObject) fieldobjects_.elementAt(i);

      if ( tmpfld instanceof FieldBaseObject )
      {
        tmpfld.giveImage( newimg, newobs );
      }
    }
  }
 
  public void giveImage( Image newimg ) 
  {
    giveImage( newimg, (ImageObserver) null );

  }// End of giveImage
*/ 

// Method: set/getObserver (public)
//
// *Purpose   : sets/returns the observer of the current image (-resource)
//              set method ignores null-argument 
//              (-> use setImageObserverNoCheck or removeImageObserver)
//
// *Parameters: ImageObserver - the new/current observer
//
// *Return    : --#---
//
// *Variables : myimage (StdReference, class) - reference to a resource
//
// *addional note: this may later implement a default observer
//
  public void setObserver( ImageObserver newobs )
  {
    if (newobs != null)
      myobs_= newobs;
  }

  public ImageObserver getObserver()
  {
    return myobs_;
  }

// Method: getImage (public) - obsolete
//
// *Purpose   : returns the current image (-resource) for the FieldObject
//              method even checks wheter the image is valid and returns
//              (null) if not
//
// *Parameters: none
//
// *Return    : null - former: Image - the current image
//
// *Variables : myimage_ (ImageReference, class) - reference to a resource
//
/* - obsolete - use getResource() of FieldBaseObject-instance
  public Image getImage()
  {
    FieldObject tmpfld= getFirstFieldBase();
    
    if ( tmpfld != null )
      return tmpfld.getImage();
    else
      return null;
  }
*/

// Method: getFirstFieldBase (public)
//
// *Purpose   : returns the FIRST FieldObject that is a FieldBase
//
// *Parameters: newobject (FieldObject) - new FieldObject to be added
//
// *Return    : FieldObject - the first FieldBaseObject found
//
// *Variables : fieldobjects_ (Vector, class) - a list of FieldObjects in
//                                                the Field-instance
//
  public FieldObject getFirstFieldBase()
  {
    FieldObject ret= null, tmpfld= null;
    int objectcount= size();
    int i;

    for ( i=0; i<objectcount; i++ )   // if it is a the Base of the Field
    {
      tmpfld= (FieldObject) fieldobjects_.elementAt(i);

      if ( tmpfld instanceof FieldBaseObject )
      {
         ret= tmpfld;
         return tmpfld;  // stop searching HERE
      }
    }
    return ret;
  }// End of getFirstFieldBase

  public FieldObject getFirstObject() // Bases are excluded !!!
  {
    FieldObject tmpfld= null;
    int objectcount= size();
    int i;

    for ( i=0; i<objectcount; i++ )   // if it is a the Base of the Field
    {
      tmpfld= (FieldObject) fieldobjects_.elementAt(i);

      if ( (tmpfld instanceof FieldObject) && 
           !(tmpfld instanceof FieldBaseObject) )
      {
         return tmpfld;  // stop searching HERE
      }
    }
    return null;
  }// End of getFirstObject

  public int size() // includes FieldBase = TrueSize of Vector fieldobjects_
  {
    if ( fieldobjects_ != null )
      return fieldobjects_.size();
    else 
      return 0;
  }
    
  public int getSize() // all FieldObjects' size (excluded are FieldBaseObjects)
  {
    FieldObject tmpfld= null;


    if ( (num_passive == -1) || (num_active == -1) )
    {
      int retcount= 0;
      int max= size();
      int i;

      for ( i=0; i<max; i++) 
      {
        tmpfld = ((FieldObject)fieldobjects_.elementAt(i));
        if ( (tmpfld instanceof FieldObject)   && 
             (  (tmpfld instanceof PassiveObject)
              ||(tmpfld instanceof ActiveObject)
             )
           )
        {
          retcount++;
        }
      }
      return retcount;
    } else {
      return (num_passive + num_active);
    }

  }// End of getSize

  public int getFieldBaseSize() // FieldBaseObjects' size
  {
    FieldObject tmpfld= null;


    if ( num_bases == -1)
    {
      int retcount= 0;
      int max= size(); // validates fieldobjects_ too
      int i;

      // if (DEBUG) System.out.println("Recalculating the number of FieldBaseObjects.");

      for ( i=0; i<max; i++) 
      {
        tmpfld = ((FieldObject)fieldobjects_.elementAt(i));
        if (  (tmpfld instanceof FieldBaseObject) )
        {
          retcount++;
        }
      }
      return retcount;
    }
    else {
      return num_bases;
    }

  }// End of getFieldBaseSize

  public int getActiveSize() // all ActiveObjects' size
  {
    FieldObject tmpfld= null;


    if ( num_active == -1)
    {
      int retcount= 0;
      int max= size(); // validates fieldobjects_ too
      int i;

      // if (DEBUG) System.out.println("Recalculating the number of active FieldObjects.");

      for ( i=0; i<max; i++) 
      {
        tmpfld = ((FieldObject)fieldobjects_.elementAt(i));
        if (  (tmpfld instanceof ActiveObject) )
        {
          retcount++;
        }
      }
      return retcount;
    }
    else {
      return num_active;
    }

  }// End of getActiveSize

  public int getPassiveSize() // all PassiveObjects' size
  {
    FieldObject tmpfld= null;

    if ( num_passive == -1)
    {
      int retcount= 0;
      int max= size(); // validates fieldobjects_ too
      int i;
      // if (DEBUG) System.out.println("Recalculating the number of passive FieldObjects.");

      for ( i=0; i<max; i++) 
      {
        tmpfld = ((FieldObject)fieldobjects_.elementAt(i));
        if (  (tmpfld instanceof PassiveObject) )
        {
          retcount++;
        }
      }
      return retcount;
    }
    else {
      return num_passive;
    }

  }// End of getPassiveSize

// Method: addObject (public)
//
// *Purpose   : adds a FieldObject to the Vector-List - checks for type WALL
//
// *Parameters: newobject (FieldObject) - new FieldObject to be added
//
// *Return    : none
//
// *Variables : fieldobjects_ (Vector, class) - a list of FieldObjects in
//                                                the Field-instance
//              type_ (String, class) - the type of the Field-instance
//              error (boolean) - storage if the remove was succesful
//
  public void addObject( FieldObject newobject ) {
    boolean error=false;

    if ( getType().equalsIgnoreCase("WALL") ) 
    {
      error= true; // a Wall can't hold objects - it's allready a solid
    } else {
      addObjectNoCheck( newobject );
    }
  }

// Method: addObjectNoCheck (public)
//
// *Purpose   : adds a Fields (Base)Object to the Vector-List
//              therer is no checking on the current FieldType
//
// *Parameters: newobject (FieldObject) - new FieldObject to be added
//
// *Return    : none
//
// *Variables : fieldobjects_ (Vector, class) - a list of FieldObjects in
//                                                the Field-instance
//              type_ (String, class) - the type of the Field-instance
//              error (boolean) - storage if the remove was succesful
//
  public void addObjectNoCheck( FieldObject newobject ) {
    boolean error= false;

    if (newobject instanceof FieldObject) // check for null and abstract base-class
    {
      if (fieldobjects_ != null) {
        fieldobjects_.addElement((Object)newobject);
        // if newobject cannot be added (runtime fault, no such object, 
        // check and handle exceptions
      } else {
        fieldobjects_= new Vector(3, 3);
        fieldobjects_.addElement((Object)newobject);
      }
    }

    freeSublists();
    // list_valid_  = false;   // the list-save is NOT current...
    // list_valid2_ = 0;       // and all sublists are invalid too...
    // nochange_ = false;

  }// End of addFieldBaseObject

// Method: removeObject (public)
//
// *Purpose   : removes a FieldObject from the Field-Vector
//
// *Parameters: object2remove (FieldObject) - Object to be removed
//
// *Return    : none
//
// *Variables : fieldobjects_ (Vector, class) - a list of FieldObjects in
//                                                the Field-instance
//              type_ (String, class) - the type of the Field-instance
//              error (boolean) - storage if the remove was succesful
//              oppvalue (boolean) - storage for the return from remove
//
  public void removeObject( FieldObject object2remove ) {
    boolean error= false;
    boolean oppvalue;

    int tmpi,i;

//    if ( DEBUG )
//      System.out.println("Field >"+getType()+"<: entering remove a FieldObject, " +
//        " - total of objects: "+ fieldobjects_.size() +" ...");

    if ( object2remove != null ) 
    {
//      if ( DEBUG )
//        System.out.println("Field >"+getType()+"<: removing F.O. >" +
//          object2remove.getName() +"< (type: >"+ object2remove.getType() +"<)");

      oppvalue= fieldobjects_.removeElement(object2remove);
      if ( oppvalue == false )           // for debugging purpose
      {
        i= fieldobjects_.size();
        System.err.println("Field >"+getType()+"<: Error removing >" +
          object2remove.getName() +"< (type: >"+ object2remove.getType() +
          "<)");
/*
        System.out.println("  Listing FieldObjects (total numer: "+i+")");
        for (tmpi=0 ; tmpi < i; tmpi++) 
        {
           System.out.println("  FieldObject #"+tmpi+": name:>"+
            ((FieldObject)(fieldobjects_.elementAt(tmpi))).getName() +"< type:>"+
            ((FieldObject)(fieldobjects_.elementAt(tmpi))).getType() +"<");
        }// end of for
        System.out.println("  Finished listing");
*/
      }// end if 
    } else {
      error= true;
    }

//    if ( DEBUG )
//      System.out.println("Field >"+getType()+"<: leaving after removal, " +
//          " - total of objects now: "+ fieldobjects_.size());

    freeSublists();
    // list_valid_  = false;   // the list-save is NOT current...
    // list_valid2_ = 0;       // and all sublists are invalid too...
    // nochange_ = false;

  }// End of removeObject

// Method: removeAllObjects (public)
//
// *Purpose   : returns the active and passive Fieldobjects
//
// *Parameters: none
//
// *Return    : boolean - successful ?
//
// *Variables : fieldobjects_ (Vector, class) - a list of FieldObjects
//
  public boolean removeAllObjects() 
  {
    int i= 0;
    FieldObject tmpfld;
    //init return array

    while ( (tmpfld= getFirstObject()) != null )
    {
       if ( fieldobjects_.removeElement( (Object) tmpfld ) == false)
       { /* exit */
         freeSublists();
         // list_valid_  = false;   // the list-save is NOT current...
         // list_valid2_ = 0;       // and all sublists are invalid too...
         return false;
       }
    }

    freeSublists();
    // list_valid_  = false;   // the list-save is NOT current...
    // list_valid2_ = 0;       // and all sublists are invalid too...
    // nochange_ = false;

    return true;
  }// End of removeAllObjects (exclude FieldBases)

  public boolean removeAllObjectsAndBase() 
  {
    int i= 0;
    FieldObject tmpfld;
    //init return array

    fieldobjects_.removeAllElements();

    freeSublists();
    // list_valid_  = false;   // the list-save is NOT current...
    // list_valid2_ = 0;       // and all sublists are invalid too...
    // nochange_ = false;

    return fieldobjects_.isEmpty();
  }// End of removeAllObjectsAndBase

// Method: get[All]ObjectList (public)
//
// *Purpose   : returns a sorted list of all FieldObjects...
//              FieldBaseObject -> ActiveObject -> PassiceObject
//              -> everything else unsorted...
//
// *Parameters: none
//
// *Return    : array of FieldObject
//
// *Variables : fieldobjects_ (Vector, class) - a list of FieldObjects in
//                                                the Field-instance
//              objectlist (array FieldObject) - stores the FieldObjects for
//                                               return
//              i (int) - counter, for current fieldobjects
//
  public FieldObject[] getAllObjectList() {
    FieldObject tmpfld;
    boolean[] marks;
    int i= 0;
    int num = 0;
    int max = size();
   
    if (list_valid_ == false) // refetch basic list !!!
    {
      num_bases   = 0;
      num_active  = 0;
      num_passive = 0;
      num_other   = 0;

      //init return array
      objectlist_= new FieldObject[max];
      marks = new boolean[max];
      for ( i=0; i < marks.length; i++ )
        marks[i]= false;              // initialize marks-list

      // assign all fieldobjects elements to array
      for ( i=0; (i < max) && (num < max); i++ ) 
      {
        tmpfld= (FieldObject)fieldobjects_.elementAt(i);      
        if  (tmpfld instanceof FieldBaseObject)
        {
          objectlist_[num]= tmpfld;
          marks[i] = true;
          num++;
          num_bases++;
        }
      } // end for

      for ( i=0; (i < max) && (num < max); i++ ) 
      {
        tmpfld= (FieldObject)fieldobjects_.elementAt(i);      
        if  (tmpfld instanceof PassiveObject)
        {
          objectlist_[num]= tmpfld;
          marks[i] = true;
          num++;
          num_passive++;
        }
      } // end for

      for ( i=0; (i < max) && (num < max); i++ ) 
      {
        tmpfld= (FieldObject)fieldobjects_.elementAt(i);      
        if  (tmpfld instanceof ActiveObject)
        {
          objectlist_[num]= tmpfld;
          marks[i] = true;
          num++;
          num_active++;
        }
      } // end for

      for ( i=0; (i < max) && (num < max); i++ ) 
      {
        tmpfld= (FieldObject)fieldobjects_.elementAt(i);      
        if  ( (tmpfld instanceof FieldObject) && (marks[i] == false) )
        {
          objectlist_[num]= tmpfld;
          marks[i] = true;
          num++;
          num_other++;
        }
      } // end for

      list_valid_  = true;    // the save is current...
      list_valid2_ = 0;       // but sublists are NOT!...
    }

    return objectlist_;

  }// End of getAllObjectList

  public void freeSublists()  // clears and invalidates the sublists 
  {
    objectlist_  = null; // new FieldObject[0];
    objectlist2_ = null; // new FieldObject[0];
    list_valid_  = false;   // the list-save is NOT current...
    list_valid2_ = 0;       // and all sublists are invalid too...

    num_bases   = -1;
    num_active  = -1;
    num_passive = -1;
    num_other   = -1;
  }

// Method: getObjectList (public)
//
// *Purpose   : returns a sorted list of all active & passive FieldObjects...
//              ActiveObject -> PassiceObject
//
// *Parameters: none
//
// *Return    : array of FieldObject
//
// *Variables : fieldobjects_ (Vector, class) - a list of FieldObjects in
//                                                the Field-instance
//              objectlist (array FieldObject) - stores the FieldObjects for
//                                               return
//              i (int) - counter, for current fieldobjects
//
  public FieldObject[] getObjectList() {
    FieldObject tmpfld = null;
    int i = 0;
    int num=0;

    if (list_valid_ == false) // refetch basic list !!!
    {
      getAllObjectList();
    }


    if (list_valid2_ != 1)   // is the current Sublist-ID set ?
    {
      objectlist2_ = new FieldObject[ getActiveSize() + getPassiveSize() ];

      int max;
      if (objectlist_ != null)
        max = objectlist_.length;
      else 
        return new FieldObject[0];

      // assign all fieldobjects elements to array
      for ( i=0; i < max; i++ ) 
      {
        tmpfld = objectlist_[ i ];
        if ( (tmpfld instanceof ActiveObject)  ||
             (tmpfld instanceof PassiveObject) )
          objectlist2_[num++] = tmpfld;
      } // end for

      list_valid2_ = 1;
    }

    return objectlist2_;
  }// End of getObjectList

// Method: setArea (public)
//
// *Purpose   : creates a new Graphics g object from the parentg,
//              having the dimension specified in newarea
//
// *Parameters: newarea (Rectangle) - the new area with which a graphics 
//                                    object shall be created
//              parentg (Graphics) - graphics object from which the new
//                                   graphics is deduced
//
// *Return    : none
//
// *Variables : g (Graphics) - stores the graphics object of the instance
//              grect_ (Rectangle) - stores the parameters from newarea for
//                                  later use
//
  public void setArea( Rectangle newarea, Graphics parentg ) 
  {
    // save the rect parameter for later use
    if ( parentg != null )
      grect_= new Rectangle(newarea.x, newarea.y, newarea.width, newarea.height);

    if ( parentg != null )
    {
/*
      doublebuffer_= null;
      if ( doublebuffer_ == null ) 
      {
*/
        if ( CREATE_SUB_GRAPHIC )
        {
          g_= parentg.create(newarea.x, newarea.y, newarea.width, newarea.height);
          drawrect_= new Rectangle(0, 0, grect_.width, grect_.height);
        } else {
          g_= parentg;
          drawrect_= new Rectangle(grect_.x, grect_.y, grect_.width, grect_.height);
        }
/*
      } else {
        g_= parentg;
        drawrect_= new Rectangle(0, 0, grect_.width, grect_.height);
      }
*/
    }

    if ( (g_ != null) && (grect_!= null) )
      drawready = true;
    else
      drawready = false;
  }

  public void setArea( Rectangle newarea, Graphics parentg, Applet app ) 
  {
    setArea(newarea, parentg);
    myobs_= app;

//    doublebuffer_= app.createImage(newarea.width, newarea.height);
//    doublebuffer_g_= doublebuffer_.getGraphics();
  }// End of setArea


// Method: paintAll (public)
//
// *Purpose   : paint Field and all embbeded FieldObjects
//              calls paintSelf and paintFieldObjects (only if necessary) 
//         
// *Parameters: none
//
// *Return    : none
//
// *Variables : type_ (String, class) - the type of the Field-instance
//
  public void paintAll() {
    // draws the Environment and starts the
    // paintAll of all it's fields

    if ( drawready != true ) return;

    Rectangle tmprect= drawrect_;
    boolean offscreen= false;
    Graphics cur_g= g_;
    Color cur_col= g_.getColor();
    
//    if ( DEBUG )
//      System.out.println("Field >"+getType()+"<: entering paintAll function.");
 
/* currently NO double-buffering!!
    if ( doublebuffer_ != null ) { 
      if ( doublebuffer_g_ != null )
        cur_g= doublebuffer_g_;
      else
        cur_g= doublebuffer_.getGraphics();
    }
*/

    paintBounds(cur_g, tmprect); // draws the bounds of the field
    paintFieldObjects(cur_g, tmprect);

// currently doublebuffering isn't used
/*
   if ( doublebuffer_ != null ) {
      g_.drawImage( doublebuffer_, grect_.x, grect_.y, getObserver() );
      System.out.println("Field >"+getType()+"<: parm grect: " +grect_.x+ "/" +grect_.y);
    }
*/      
    //StdImageReference.drawCross( g_, new Rectangle(grect_.x, grect_.y+grect_.height/2, grect_.width/2, grect_.height/2), null, 1 );

    g_.setColor( cur_col );

//    if ( DEBUG )
//      System.out.println("Field >"+getType()+"<: leaving paintAll function.");
  }// End of paintAll


// Method: finalize (public)
//
// *Purpose   : only calls the graphics methode g_.finalize,
//              just a small garbage routine
//         
// *Parameters: none
//
// *Return    : none
//
// *Variables : g (graphics, class) - the graphics object of the instance
//
  public void finalize() throws Throwable
  {
    g_.finalize();
//    doublebuffer_.finalize();
//    doublebuffer_g_.finalize();
//    g.finalize();
    super.finalize();
  }// End of finalize


// Method: getMinSize (public)
//
// *Purpose   : returns the minimal size needed by a Field
//              
// *Parameters: none
//
// *Return    : Dimension - the size (in pixel) needed for the given
//                          Field
//
// *Variables : 
//
  public Dimension getMinSize() {
    Dimension minsize;
    int objectcount= size() + 1;

    if ( objectcount == 0 ) {
      objectcount= 2*xBORDER_DISTANCE + xMIN_SPRITE_SIZE*1;  
      minsize= new Dimension(objectcount, objectcount);
    } else {
      minsize= getSpace(objectcount);

      minsize.width=  2*xBORDER_DISTANCE + xMIN_SPRITE_SIZE*minsize.width + 
                      xFIELDOBJECT_DISTANCE*(minsize.width-1);
      minsize.height= 2*yBORDER_DISTANCE + yMIN_SPRITE_SIZE*minsize.height + 
                      yFIELDOBJECT_DISTANCE*(minsize.height-1);
    } // end if

    return minsize;
  }// End of getMinSize


// Methods: paintFieldObjects (private)
//
// *Purpose   : paints all fieldobjects_ and the special graphics image
//              of Exit-Fields
//
// *Parameters: g (graphics) - the graphics object of the instance
//              grect (Rectangle) - the current drawarea-parameters
//
// *Return    : none
//
// *Variables : type_ (String, class) - the type of the Field-instance
//
//              rowcols (Dimension) - the number of objects in columns/rows
//              borderdis (Dimension) - the current _DISTANCE to the border
//              objectpos (Point) - the position of the object (row/column)
//              tmprect (Rectangle) - stores the currnent Image-Pos./Size
//              objectcounter (int) - number of current objects in Field
//
  private void paintFieldObjects(Graphics g, Rectangle grect) {
    Dimension rowcols= null;
    Dimension objectdim=  new Dimension(0, 0);
    Dimension borderdist= new Dimension(xBORDER_DISTANCE, yBORDER_DISTANCE);
    Point     objectpos=  new Point(0, 0);
    Rectangle tmprect=    new Rectangle(0, 0, 0, 0);

    FieldObject tmpfld= null, drawfld= null, basefld= null;
    boolean nobasefld = false;

    // count all drawable fieldobjects_
    if (!list_valid_)
    {
      getAllObjectList();
    }

    int i;
    int objectcount    = 0;  /* the number of total FieldObjects            */
    int stdobjectcount = 0;  /* the number of passive & active FieldObjects */
    int num_painted    = 0;  /* the number of objects painted */

    if (objectlist_ != null)
      objectcount= objectlist_.length;
    if ( objectcount < 1 )  // only run through the rest IF there are any Objects
      return;

    stdobjectcount= getPassiveSize() + getActiveSize();

    // draw the one FieldBaseObject that is FILLING the Field first !!!
    if ( (objectcount - stdobjectcount) > 0)
    {
      for (i=0; i<objectlist_.length; i++) {  // or just a FieldObject to big for it's jacket
        tmpfld = objectlist_[i];
        if ( tmpfld instanceof FieldBaseObject && 
             tmpfld instanceof FieldFillingIfc &&
             ((FieldFillingIfc) (tmpfld)).fillsField() 
           ) 
        {
          num_painted++;  // advance FieldObjectscount for field that dont neet a matrix-space..
          tmpfld.draw( g, new Rectangle( grect.x+1, 
                                         grect.y+1,
                                         (grect.width  - 2), 
                                         (grect.height - 2) ) );
          if ( nobasefld == false )
            drawfld= tmpfld;
        }
      }

//    if ( false && drawfld != null ) // is there a FieldObject that is filling the Field
//    {
//      drawfld.draw( g, 
//      new Rectangle( grect.x+1,       grect.y+1,
//                     grect.width - 2, grect.height - 2));
//    }

    }

    i = objectcount - num_painted;
    if (i < 1) return;

    if (i < 4)
      i=4;

    // now split up space for fieldobjects_
    rowcols= getSpace(i);

//    if ( DEBUG )
//      System.out.println("Field: Paint "+i+" FieldOjects in >"+rowcols.width+"< Rows and >"+rowcols.height+"< Columns");

    objectdim.width=  (grect.width  - ((2*borderdist.width)  +((rowcols.width  -1) *xFIELDOBJECT_DISTANCE)) ) / rowcols.width;
    objectdim.height= (grect.height - ((2*borderdist.height) +((rowcols.height -1) *yFIELDOBJECT_DISTANCE)) ) / rowcols.height;

    if (objectdim.width < objectdim.height) {  // get equi-size
      objectdim.height= objectdim.width;
    } else {
      objectdim.width= objectdim.height;
    }

    borderdist.width=  grect.x + (grect.width  - (((objectdim.width  + xFIELDOBJECT_DISTANCE) * rowcols.width)  - xFIELDOBJECT_DISTANCE))/2;
    borderdist.height= grect.y + (grect.height - (((objectdim.height + yFIELDOBJECT_DISTANCE) * rowcols.height) - yFIELDOBJECT_DISTANCE))/2;

//    if ( DEBUG )
//      System.out.println("Field: >"+getType()+"<: Standard FieldObject Dimensions ("+objectdim.width+","+objectdim.height+")");

    // no need to change these constant values in the following loop
    // drawing all FieldObjects that don't fill the current Field
    tmprect.width=  objectdim.width;
    tmprect.height= objectdim.height;

// now draw FieldBases - that DON'T fill the Field !!!
    if ( (objectcount - (stdobjectcount + num_painted)) > 0)
    {
      for ( i=0; i<objectcount; i++ ) {  // if it is a the Base of the Field
        tmprect.x= borderdist.width + (xFIELDOBJECT_DISTANCE + objectdim.width) * objectpos.x; 
        tmprect.y= borderdist.height + (yFIELDOBJECT_DISTANCE + objectdim.height) * objectpos.y; 

        tmpfld= objectlist_[i];

        if (  (tmpfld instanceof FieldObject)
           && (tmpfld instanceof FieldBaseObject)
           && (!(tmpfld instanceof FieldFillingIfc)
           ||  !((FieldFillingIfc) (tmpfld)).fillsField()) 
           )
        {
/*
          if ( DEBUG )
            System.out.println("Field: >"+getType()+
              "<: drawing base FieldObject with ("+tmprect.x+","+tmprect.y+
              "/"+tmprect.width+","+tmprect.height+")");
*/

          tmpfld.draw( g,tmprect );
          num_painted++;  // advance FieldObjectscount for field that dont neet a matrix-space..

          if ( objectpos.x < (rowcols.width - 1) ) // if row-position isn't at max 
          { 
            objectpos.x= objectpos.x + 1;     // increase row-position
          } else { 
            objectpos.x= 0;                   // else new row
            objectpos.y= objectpos.y + 1;
          }
        }
      }
    }

    if ((objectcount - num_painted) < 1) return;

    // now draw all FieldObjects - that fill the field!!!
    for (i=getFieldBaseSize(); i<objectcount; i++) {  // or just a FieldObject to big for it's jacket
      tmpfld = objectlist_[i];
      if ( (tmpfld instanceof FieldFillingIfc) &&
           ((FieldFillingIfc) (tmpfld)).fillsField()
         ) 
      {
        if (  (tmpfld instanceof FieldObject) && 
             !(tmpfld instanceof FieldBaseObject) ) 
        {
          num_painted++;  // advance FieldObjectscount for field that dont neet a matrix-space..

          tmpfld.draw( g, new Rectangle( grect.x+1, 
                                         grect.y+1,
                                         grect.width - 2, 
                                         grect.height - 2));
        }
      }
    }

    if ((objectcount - num_painted) < 1) return;

// now draw all other FieldObjects - the rest!!!
    for ( i= getFieldBaseSize(); i<objectcount; i++ )     // if it is a the Base of the Field
    {
      tmprect.x= borderdist.width + (xFIELDOBJECT_DISTANCE + objectdim.width) * objectpos.x; 
      tmprect.y= borderdist.height + (yFIELDOBJECT_DISTANCE + objectdim.height) * objectpos.y; 

      tmpfld= objectlist_[i];

      if (  (tmpfld instanceof FieldObject)     && 
           !(tmpfld instanceof FieldBaseObject) &&
           (!(tmpfld instanceof FieldFillingIfc) ||
            !((FieldFillingIfc) (tmpfld)).fillsField()) 
         )
      {
/*
        if ( DEBUG )
          System.out.println("Field: >"+getType()+
            "<: drawing std. FieldObject with ("+tmprect.x+","+tmprect.y+
            "/"+tmprect.width+","+tmprect.height+")");
*/

        tmpfld.draw( g,tmprect );

        if (objectpos.x < (rowcols.width - 1)) // if row-position isn't at max
        { 
          objectpos.x= objectpos.x + 1;     // increase row-position
        } else { 
          objectpos.x= 0;                   // else new row
          objectpos.y= objectpos.y + 1;
        }
      }
    }

  }// End of paintFieldObjects


/*
  public void invalidateDraw()
  {
    // nochange_ = false;
  }
*/

// Methods: paintBounds (private)
//
// *Purpose   : paints the Field itself (only Exit-image is not painted ->
//              see paintfieldobjects_
//
// *Parameters: g (graphics) - the graphics object of the instance
//              grect (Rectangle) - the current drawarea-parameters
//
// *Return    : none
//
// *Variables : cur_type (String, class) - the type of the Field-instance
//
  private void paintBounds(Graphics g, Rectangle grect) 
  {
    if ( !noclear_ )
    {
//      g.clearRect( grect.x+1, grect.y+1, grect.width-2, grect.height-2 );
      g.setColor( background_ );
      g.fillRect( grect.x+1, grect.y+1, grect.width-2, grect.height-2 );

      // nochange_ = true;
    }

    g.setColor( frame_color_ );
    g.drawRect( grect.x, grect.y, grect.width-1, grect.height-1 );

    // StdImageReference.drawCross( g, grect, null, 1 );
  }
 

// Methods: getSpace (public, static)
//
// *Purpose   : determins the practical number of rows and columns from
//              a given number of objects in an area
//
// *Parameters: objectscount (in) - number of objects in an area
//
// *Return    : Dimension - with the number of rows (heigth) and colums 
//                          (width) determined from the number of objects
//
// *Variables : rowcol (Point) - stores the row/col values for return
//              rowsize (int) - the max size of a single row in the area
//              fract (double) - the fraction of the square - edge (round up)
//              tmp (int) - stores a value for checking if rows can be left out
//
  public static Dimension getSpace( int objectcount ) {
    Dimension rowcol = new Dimension(0,0);
    int rowsize= 0;
    double fract= 0;
    int tmp= 0;

    if ( objectcount == 0 ) { // faster, just to save the calcuation
      return rowcol;
    } // end if zero objects

    if ( objectcount == 1 ) { // now for just one object in area
      rowcol.width= 1;
      rowcol.height= 1;
      return rowcol;
    } // end if one objects

    if (objectcount == 2) { // now for just two objects in area
      rowcol.width= 2;
      rowcol.height= 1;
      return rowcol;
    } // end if two objects

    if (objectcount == 3) { // now for just three objects in area
      rowcol.width= 2;
      rowcol.height= 2;
      return rowcol;
    } // end if three objects

    if (objectcount == 4) {// now for just four objects in area
      rowcol.width= 2;
      rowcol.height= 2;
      return rowcol;
    } // end if four objects


  // and now for a general calculation routine (allways leave out zero-objects)
    fract= Math.sqrt((double)objectcount); // get the square-edges fraction
    rowsize= roundUp(fract);  // and round up to the next natural number

                  // determined from the number of Objects there should be:
    rowcol.width= rowsize;    // x Rows to be displayed
    rowcol.height= rowsize;   // y Columns to be displayed

    
    tmp= (rowsize*rowsize);                    // now check if some rows are
    while( objectcount <= (tmp= tmp-rowsize) ) // not needed and substract them
    {
      rowcol.height= rowcol.height - 1; //  Att: subs used for speed only
//      tmp= tmp - rowsize;             //       avoiding multiplys/divisions
    } // end while

    return rowcol;
  }// End of getSpace


// Method: roundUp (public, static)
//
// *Purpose   : rounds up a fractioned Number (double) to the next higher
//              natural number
//
// *Parameters: num (double) - the value to be rounded up
// *Return    : int - the value rounded up
//
// *Variables : retnum (int) - stores the number to return
//
  public static int roundUp( double num ) { 
    int retnum= 0;

    retnum= (int) num;            // determin natural number (rounded down)

    if ( (num - (double)retnum) > 0 ) {  // determine fraction
      retnum= retnum + 1;              // if fraction then add 1
    } // end if

    return retnum;
  }// End of roundUp


}// End of class Field
