// file:        Gold.java
// source file: Gold.java
// object file: Gold.class
//
//
// included objects:
//  Gold
//    Desc: implements the passive object - Gold, which is what agents 
//          desperatly trying to get in WumpusEnvironments
//
//
// history:
//   22.02.97 - first version finished by Chris
//
//   05.03.97 - development off. continued by Roman:
//              added functions for image drawing
//   12.03.97 - new extensions for displaying images (as per protocol 11.03.97) - Roman
//   31.03.97 - added new default constants - Roman
//   08.08.97 - added default drawing color - Roman
//   11.04.97 - changed Constructors back to old design - property methods
//              set/getRes and set/getImageObserver will handle optional
//              paramethers now - Roman
//   14.04.97 - fitted for major redesign - Roman
//
// critical problems: none
//
// known problems: none
//

import java.awt.*;
import java.awt.image.ImageObserver;
import java.util.*;
import java.lang.*;

class Gold extends PassiveObject
{
  // private final static boolean DEBUG = false;
  private final static boolean IN_ONE = false;

  private final static String TYPE_GOLD = "GOLD";


  private final static String DEFAULT_COLOR_STR = "colors.fieldobject.GOLDCOLOR";


// Method: Gold Constructor (public)
//
// *Purpose   : intializes a new instance and sets a new image (-resource)
//              for the object
//
// *Parameters: name (String) - name of the new instance
//              resource (String) - name of the resource for the FieldObject
//              observer (ImageObserver) - observer for the FieldObject
//
// *Return    : none
//
// *Variables : none
//
//   with set/get-methods:
//              imgref (String) - default reference to an imagemap
//              obs (ImageObserver) - observer for the image
//
  public Gold( String name , String resource, ImageObserver observer )
  {
    super( name, TYPE_GOLD, resource );
  }

  public Gold( String name , String resource )
  {
    super( name, TYPE_GOLD, resource, null );
  }

  public Gold( String name , ImageObserver observer )
  {
    super( name, TYPE_GOLD, null, observer );
  }

  public Gold( String name )
  {
    super( name, TYPE_GOLD, null, null );
  }


// Method: defaultResName()  (protected)
//         defaultColorName  (protected)
//
// *Purpose   : returns a default property identifing string for a 
//              resource and color (mostl often the only *NEW* things
//              - good for beeing overwritten by subclasses to set an other
//                default property identifing string 
//                (the string to be looked up in the system-property list
//
// *Parameters: none
//
// *Return    : String - the new default reource ot color
//
// *Variables : all the nessessary id-strings
//
  public String defaultColorName() { return DEFAULT_COLOR_STR; }


// Method: drawPrimitive (public)
//
// *Purpose   : draws a graphic primitive (a default form of displaying)
//              to the graphics device 
//
// *Parameters: g (Graphics) - the graphics to draw the objects to
//              area (Rectangle) - the position and dimensions of the
//                                 object on screen
//              foreground (Color) - the drawingcolor to use
//
// *Return    : whether the drawing of the Image was successfull or not
//
// *Variables : myimage (ImageReference, class) - pointer to a resource
//
  public boolean drawPrimitive( Graphics g, Rectangle area )
  {
    return ( drawPrimitive(g, area, getColor() ) );
  }

  public boolean drawPrimitive( 
    Graphics g, Rectangle area, Color foreground
  )
  {
//    if ( g == null ) return false;

    if ( foreground != null)
      g.setColor( foreground );

    //Primitive: a simple Box with a sloped top
    if (area.width < 10) return false;
    if (area.height< 10) return false;

    try {
      Rectangle box = new Rectangle( area.x, 
                                     area.y, 
                                     area.width - (area.width/6),
                                     area.height/3 );
      int slope = box.height/3;
      box.x += (area.width  - box.width)/2;
      box.y += (area.height - box.height)/2;
      Point help = new Point(box.x, box.y);

      if (IN_ONE)
      {
        Polygon bar = new Polygon();
        bar.addPoint( help.x, help.y );
        help.y += box.height;
        bar.addPoint( help.x, help.y );
        help.x += box.width;
        bar.addPoint( help.x, help.y );
        help.y = box.y;
        bar.addPoint( help.x, help.y );
        help.x -= slope;
        help.y -= slope;
        bar.addPoint( help.x, help.y );
        help.x = box.x + slope;
        bar.addPoint( help.x, help.y );

        g.fillPolygon( bar );

        g.setColor(Color.black);
        g.drawPolygon( bar );
      } 
      else 
      {
        Polygon top = new Polygon();
        top.addPoint( help.x, help.y );
        help.x += box.width;
        top.addPoint( help.x, help.y );
        help.x -= slope;
        help.y -= slope;
        top.addPoint( help.x, help.y );
        help.x = box.x + slope;
        top.addPoint( help.x, help.y );

        g.fillPolygon( top );
        g.fillRect( box.x, box.y, box.width, box.height );

        g.setColor(Color.black);
        g.drawPolygon( top );
        g.drawRect( box.x, box.y, box.width, box.height );
      }

    } catch (ArithmeticException e)
    {
      System.err.println("Error calculating (Gold-)Bar: " + e.getMessage());
      return false;
    }

    return true;
  }


}// End of class Gold
