// file:        GrayFilter.java
// source file: GrayFilter.java
// object file: GrayFilter.class
//
//
// included objects:
//  GrayFilter
//    Desc: implements a graying filter for imagemaps
//          and includes an alpha-Filter for white
//
//
// history:
//   28.03.97 - first version finished by Roman Valehrach
//
//
// critical problems: none
//
// known problems: none
//

import java.awt.*;
import java.awt.image.*;

class GrayFilter extends RGBImageFilter {
// Method: GrayFilter Constructor (public)
//
// *Purpose   : intializes a new instance and sets the FilterModel to the
//              Indexed version
//
// *Parameters: none
//
// *Return    : none
//
// *Variables : canFilterIndexColorModel (boolean, class) - defines whether
//                       the images are filter as a whole or pixel by pixel
//
  public GrayFilter() {
    canFilterIndexColorModel= true;
  }


// Method: FilterRGB (public)
//
// *Purpose   : filters the current rgb-color value -> graying
//              by adding/mixing in white (0xff, 0xff, 0xff)
//
// *Parameters: x (int) - the x-position of the current pixel
//              y (int) - the y-position of the current pixel
//              rgb (int) - rgb-value of the image at the given possition
//
// *Return    : int - the grayed value (rgb value added with white)
//
// *Variables : none
//
  public int filterRGB( int x, int y, int rgb ) {
    int a= (rgb & 0xff000000);	// mask alpha-value
    int r= (rgb & 0x00ff0000);	// mask red-value 
    int g= (rgb & 0x0000ff00); 	// mask green-value
    int b= (rgb & 0x000000ff);	// mask blue-value
/*
    if ((r > 0xee0000) && (g > 0x00ee00) && (b > 0x0000ee)) {
      a= 0x00000000;
    }
*/
    r= ((r + 0xff0000)/2) & 0x00ff0000; // red-value: increase 
    g= ((g + 0x00ff00)/2) & 0x0000ff00; // green-value: increase 
    b= ((b + 0x0000ff)/2) & 0x000000ff; // blue-value: increase 

    return (a | r | g | b); // join it all together again and return it
  }// End of filterRGB
 
}// End of class GrayFilter