import HashSearchBackWumpus;
import java.util.*;
import java.awt.*;
import java.awt.image.ImageObserver;

//
//
// HashBreadthWumpus searches for a way home as soon as it finds gold; 
// it uses breadth first search; the nodes that are expanded are added
// to the queaue with the one with the smallest manhatten - distance to
// the home - field first.
// ( this is also called  a 'greedy' search algorithm ) 
//
// It is slightly 'goal - based': if gold was found it returns home
// immediatly
//
//

class HashBreadthWumpus extends HashSearchBackWumpus
{
  static final String type = "HASHBREADTHWUMPUS";
 
  protected Vector searchqueue;
  AdvancedSearchBAgentFieldList location;

  public HashBreadthWumpus( String name, int ammo, Dimension size )
  {
    this( name, ammo, size, ( String ) null, ( ImageObserver ) null, type );
  }
 
  public HashBreadthWumpus( String name, int ammo, Dimension size, String imgref )
  {
    this( name, ammo, size, imgref, ( ImageObserver ) null, type );
  }

  public HashBreadthWumpus( String name, int ammo, Dimension size, ImageObserver obs )
  {
    this( name, ammo, size, ( String ) null, obs, type );
  }

  public HashBreadthWumpus( String name, int ammo, Dimension size, String imgref, ImageObserver obs )
  {
    this( name, ammo, size, imgref, obs, type );
  }

  public HashBreadthWumpus( String name, int ammo, Dimension size, String imgref, ImageObserver obs, String type )
  {
    super( name, ammo, size, imgref, obs, type);
  }// End of HashBreadthWumpus-Constructorsors

  protected void initField( Dimension fieldsize )
  {
    AdvancedSearchBAgentFieldList help;
    cavesize = new Dimension();
    cavesize.height = fieldsize.height;
    cavesize.width = fieldsize.width;

	// initialize the field - represantation
	myfield = new HashList();

	// Agents start at location ( 1, 1 ); this is also the exit
    if( log == true )
    {
      System.out.println( " Creating start field and adding it to table " );
    }

	location = new AdvancedSearchBAgentFieldList( 1, 1 );
    location.setWumpus( false );
    location.setPit( false );
    location.setExit( true );
    location.setWall( false );
    location.setVisited( true );
    location.incVisited();
    myfield.addElement( location );

    // up and left are wall because we always start in the upper left corner
    // this has to be removed if the agent should start anywhere in the maze
    help = new AdvancedSearchBAgentFieldList( 0, 1 );
    help.setWall( true );
    myfield.addElement( help );
    help = new AdvancedSearchBAgentFieldList( 1, 0 );
    help.setWall( true );
    myfield.addElement( help );
   
    super.myfield = myfield;
    ( ( HashBackWumpus ) this ).myfield = myfield;
    ( ( HashWumpus ) this ).myfield = myfield;

    super.location = location;
    ( ( HashBackWumpus ) this ).location = location;
    ( ( HashWumpus ) this ).location = location;

    if( log == true )
    {
      System.out.println("HashBreadthWumpus: end of initfield");
    }
  }

  protected void finalize()
  {
    super.finalize();
    myfield = null;
    searchway = null;
  }
  
  protected boolean search( Point recent )
  {
    Point found;
    boolean success;

    success = false;
    searchqueue = new Vector( 5, 5 );
    found = searchBreadth( recent );
    if( found.x >= 0 && found.y >= 0 )
    {
      searchway = generateWay( found, recent );
      success = true;
    }
    return success;
  }

  protected Point searchBreadth( Point recent )
  {
    Vector nearestlast;
    Point workpoint, failed;
    AdvancedSearchBAgentFieldList help, up, down, left, right, next;
    Point cup, cdown, cleft, cright;
    boolean found;
     
    found = false;
    failed = new Point( -1, -1 );

    help = ( AdvancedSearchBAgentFieldList ) myfield.getElement( recent.x, recent.y );
    if( log == true )
    {
      System.out.println( "Starting breadth first search at: " + recent.x + "/" + recent.y );
    }

    // check starting field 
    if( help.isExit().equals( "YES" ) )
    {
      return recent;
    }

    // put starting point in Vector - ' queue '
    // and mark  it as worked
    help.setSearched( true );
    searchqueue.addElement( ( Object ) help );

    // initialize vector to four element - size
    nearestlast = new Vector( 4 ); 

    do
    {
      // get first Element and remove it from queue
      recent = ( ( AdvancedSearchBAgentFieldList ) searchqueue.firstElement() ).getPlace();    
      searchqueue.removeElementAt( 0 );
      help = ( AdvancedSearchBAgentFieldList ) myfield.getElement( recent.x, recent.y );
      if( log == true )
      {
        System.out.println( "Got Point from queue: " + recent.x + "/" + recent.y );
        System.out.println( "Checking for exit" );
      }

      if( help.isExit().equals( "YES" ) )
      {
        if( log == true )
        {
          System.out.println( "Field is exit" );
        }
        found = true;
      }
      else
      {
        if( log == true )
        {
          System.out.println( "Field is not exit" );
        }
      }
      
      // get neighbour coordinates
      up    = ( AdvancedSearchBAgentFieldList ) myfield.getElement( recent.x, recent.y - 1 );
      if( up.getSearched() == false )
      {
        up.setParent( recent );
      }
      down  =  ( AdvancedSearchBAgentFieldList ) myfield.getElement( recent.x, recent.y + 1 );
      if( down.getSearched() == false)
      {
        down.setParent (recent ); 
      }
      left  =  ( AdvancedSearchBAgentFieldList ) myfield.getElement( recent.x - 1, recent.y );
      if( left.getSearched() == false )
      {
        left.setParent( recent );
      }
      right =  ( AdvancedSearchBAgentFieldList ) myfield.getElement( recent.x + 1, recent.y );
      if( right.getSearched() == false )
      {
        right.setParent( recent );
      }

      // get neighbour coordinates
      cup    = new Point( recent.x, recent.y - 1 );
      cdown  = new Point( recent.x, recent.y + 1 );
      cleft  = new Point( recent.x - 1, recent.y );
      cright = new Point( recent.x + 1, recent.y );

      // calculate manhatten - distance
      nearestlast = calcManhatten( cup, cdown, cleft, cright );
     
      // put them on the queue if they are not already there
      // and if they have been visited at least once
      for( int i = 0; i < nearestlast.size(); i++ )
      {
        workpoint = ( Point ) nearestlast.elementAt( nearestlast.size() - i - 1 ); 
        help = ( AdvancedSearchBAgentFieldList ) myfield.getElement( workpoint.x, workpoint.y );
        if( help.wasVisited().equals( "YES" ) )
        {
          if( help.getSearched() == false )
          {
            help.setSearched( true );
            searchqueue.addElement( ( Object ) help );
            if( log == true )
            {
              System.out.println(" queue added " + help.xcoord + help.ycoord );
            }
          }
        }
      }
    } while ( searchqueue.isEmpty() == false && found == false );

    if( found == true )
    {
      return recent;
    }
    else
    {
      return failed;
    }
  }
  
  protected Vector generateWay( Point goal, Point first )
  {
    Vector path;
    Point parent, start, work;
    AdvancedSearchBAgentFieldList help;
    
    path = new Vector( 5, 5 );
    start = first;
    work = goal;
    if( log == true )
    {
      System.out.println( "Generating way home starting with: " + work.x + "/" + work.y + "\n" );
    }

    do
    {
       help = ( AdvancedSearchBAgentFieldList )myfield.getElement( work.x, work.y );
       parent = help.getParent();
       if( log == true )
       {
         System.out.println(" Generate way recent: " + work.x + "/" + work.y );
         System.out.println(" Generate way parent: " + parent.x + "/" + parent.y );
       }

       if( ( parent.x == work.x ) && ( parent.y == work.y - 1 ) )
       {
         path.insertElementAt( new Action( "Down" ), 0 );
       }
       else if( ( parent.x == work.x ) && ( parent.y == work.y + 1 ) )
       {
         path.insertElementAt( new Action( "Up" ), 0 );
       }
       else if( ( parent.x == work.x - 1 ) && ( parent.y == work.y ) )
       {
         path.insertElementAt( new Action( "Right" ), 0 );
       }
       else if( ( parent.x == work.x + 1 ) && ( parent.y == work.y ) )
       {
         path.insertElementAt( new Action( "Left" ), 0 );
       }
       else // an error has happend
       {
         if( log == true )
         {
           System.out.println( "GenerateWay: Should never be here!" );
         }
         return path;
       }
       work = parent;
       if( log == true )
       {
         System.out.println( "Way generation continues with: " + work.x + "/" + work.y );
       }
       if( path.isEmpty() == false )
       {
         for( int i = 0; i < path.size(); i++ )
         if( log == true )
         {
           System.out.println( "path[ " + i + " ] is: " + ( ( Action )path.elementAt( i ) ).getName() );
         }
       }
    }while ( work.x != first.x || work.y != first.y );
    return path;
  }

   // getxxxField methods return the field that is to 
  // the up/down-right or to the up/down-left from the agent's current location
  protected AdvancedSearchAgentFieldList getUpLeftField()
  {
    AdvancedSearchBAgentFieldList returnupleft;
    updateLocation();
    if( ( returnupleft = ( AdvancedSearchBAgentFieldList ) myfield.getElement( location.xcoord - 1, location.ycoord - 1 ) )
           == null )
    {
      returnupleft = new AdvancedSearchBAgentFieldList( location.xcoord - 1, location.ycoord - 1 );
      if( wumpusalive == false )
        returnupleft.setWumpus( false );
      myfield.addElement( returnupleft );
      if( log == true )
      {
        System.out.println( " Created new upleft field ");
      }
    }
    else
    {
      if( log == true ) 
      {
        System.out.println( " Upleft field already existed ");
      }
    }
    return returnupleft;
  }

  protected AdvancedSearchAgentFieldList getUpRightField()
  {
    AdvancedSearchBAgentFieldList returnupright;
    updateLocation();
    if( ( returnupright = ( AdvancedSearchBAgentFieldList ) myfield.getElement( location.xcoord + 1, location.ycoord -1 ) )
           == null )
    {
      returnupright = new AdvancedSearchBAgentFieldList( location.xcoord + 1, location.ycoord - 1 );
      if( wumpusalive == false )
        returnupright.setWumpus( false );
      myfield.addElement( returnupright );
      if( log == true )
      {
        System.out.println( " Created new upright field ");
      }
    }
    else
    {
      if( log == true )
      {
        System.out.println( " Upright field already existed ");
      }
    }
    return returnupright;
  }

  protected AdvancedSearchAgentFieldList getDownLeftField()
  {
    AdvancedSearchBAgentFieldList returndownleft;
    updateLocation();
    if( ( returndownleft = ( AdvancedSearchBAgentFieldList ) myfield.getElement( location.xcoord - 1, location.ycoord + 1 ) )
           == null )
    {
      returndownleft = new AdvancedSearchBAgentFieldList( location.xcoord - 1, location.ycoord + 1 );
      if( wumpusalive == false )
        returndownleft.setWumpus( false );
      myfield.addElement( returndownleft );
      if( log == true )
      {
         System.out.println( " Created new downleft field ");
      }
    }
    else
    {
      if( log == true )
      {
        System.out.println( " downleft field already existed ");
      }
    }
    return returndownleft;
  }

  protected AdvancedSearchAgentFieldList getDownRightField()
  {
    AdvancedSearchBAgentFieldList returndownright;
    updateLocation();
    if( ( returndownright = ( AdvancedSearchBAgentFieldList ) myfield.getElement( location.xcoord + 1, location.ycoord + 1 ) )
           == null )
    {
      returndownright = new AdvancedSearchBAgentFieldList( location.xcoord + 1, location.ycoord + 1 );
      if( wumpusalive == false )
        returndownright.setWumpus( false );
      myfield.addElement( returndownright );
      if( log == true )
      {
        System.out.println( " Created new downright field ");
      }
    }
    else
    {
      if( log == true )
      {
        System.out.println( " Downright field already existed ");
      }
    }
    return returndownright;
  }

  // getxxxField methods return the field that is to 
  // the right, to the left, up or down from the agent's current location
  protected AgentFieldList getLeftField()
  {
    AdvancedSearchBAgentFieldList returnleft;
    updateLocation();
    if( ( returnleft = ( AdvancedSearchBAgentFieldList ) myfield.getElement( location.xcoord - 1, location.ycoord ) )
           == null )
    {
      returnleft = new AdvancedSearchBAgentFieldList( location.xcoord - 1, location.ycoord );
      if( wumpusalive == false )
        returnleft.setWumpus( false );
      myfield.addElement( returnleft );
      if( log == true )
      {
        System.out.println( " Created new left field ");
      }
    }
    else
    {
      if( log == true ) 
      {
        System.out.println( " Left field already existed ");
      }
    }
    return returnleft;
  }

  protected AgentFieldList getRightField()
  {
    AdvancedSearchBAgentFieldList returnright;
    updateLocation();
    if( ( returnright = ( AdvancedSearchBAgentFieldList ) myfield.getElement( location.xcoord + 1, location.ycoord ) )
           == null )
    {
      returnright = new AdvancedSearchBAgentFieldList( location.xcoord + 1, location.ycoord );
      if( wumpusalive == false )
        returnright.setWumpus( false );
      myfield.addElement( returnright );
      if( log == true )
      {
        System.out.println( " Created new right field ");
      }
    }
    else
    {
      if( log == true )
      {
        System.out.println( " right field already existed ");
      }
    }
    return returnright;
  }

  protected AgentFieldList getUpField()
  {
    AdvancedSearchBAgentFieldList returnup;
    updateLocation();
    if( ( returnup = ( AdvancedSearchBAgentFieldList ) myfield.getElement( location.xcoord, location.ycoord - 1 ) )
           == null )
    {
      returnup = new AdvancedSearchBAgentFieldList( location.xcoord, location.ycoord - 1 );
      if( wumpusalive == false )
        returnup.setWumpus( false );
      myfield.addElement( returnup );
      if( log == true )
      {
         System.out.println( " Created new up field ");
      }
    }
    else
    {
      if( log == true )
      {
        System.out.println( " up field already existed ");
      }
    }
    return returnup;
  }

  protected AgentFieldList getDownField()
  {
    AdvancedSearchBAgentFieldList returndown;
    updateLocation();
    if( ( returndown = ( AdvancedSearchBAgentFieldList ) myfield.getElement( location.xcoord, location.ycoord + 1 ) )
           == null )
    {
      returndown = new AdvancedSearchBAgentFieldList( location.xcoord, location.ycoord + 1 );
      if( wumpusalive == false )
        returndown.setWumpus( false );
      myfield.addElement( returndown );
      if( log == true )
      {
        System.out.println( " Created new down field ");
      }
    }
    else
    {
      if( log == true )
      {
        System.out.println( " down field already existed ");
      }
    }
    return returndown;
  }

  protected void updateLocation()
  {
    super.updateLocation();
    location = ( AdvancedSearchBAgentFieldList ) super.location;
  }
}

