import HashSearchBackWumpus;
import java.util.*;
import java.awt.*;
import java.awt.image.ImageObserver;

//
//
// HashDepthIteratedWumpus
//
//
class HashDepthIteratedWumpus extends HashSearchBackWumpus
{
  static final String type = "HASHDEPTHITERATEDWUMPUS";
  protected int maxdepth, currentdepth, iterationdepth;

  public HashDepthIteratedWumpus( String name, int ammo, Dimension size )
  {
    this( name, ammo, size, ( String ) null, ( ImageObserver ) null, type );
  }
 
  public HashDepthIteratedWumpus( String name, int ammo, Dimension size, String imgref )
  {
    this( name, ammo, size, imgref, ( ImageObserver ) null, type );
  }

  public HashDepthIteratedWumpus( String name, int ammo, Dimension size, ImageObserver obs )
  {
    this( name, ammo, size, ( String ) null, obs, type );
  }

  public HashDepthIteratedWumpus( String name, int ammo, Dimension size, String imgref, ImageObserver obs )
  {
    this( name, ammo, size, imgref, obs, type );
  }

  public HashDepthIteratedWumpus( String name, int ammo, Dimension size, String imgref, ImageObserver obs, String type )
  {
    super( name, ammo, size, imgref, obs, type);
  }// End of HashBackWumpus-Constructorsors

  protected void initVars()
  {
    super.initVars();
    if( log == true )
    {
      System.out.println(" HashDepthIterated: initializing new vars ");
    }
    maxdepth = 0;
    currentdepth = -1;
    iterationdepth = 0;
  }
  protected boolean search( Point recent )
  {
    AdvancedSearchAgentFieldList help;
    boolean success;
    success = false;
    maxdepth = wayback.size() + 1;
    if( log == true )
    {
      System.out.println( "Starting iterated depth search at: " + recent.x + "/" + recent.y ); 
    }
    while( success == false && iterationdepth <= maxdepth )
    {
      success = searchIterated( recent );
      iterationdepth++;
      if( log == true )
      {
        System.out.println( "Current iterationdepth: " + iterationdepth );
      }

      // do not forget to clear the searched - flag before the
      // next iteration starts
      for( int i = 0; i < cavesize.width; i++ )
      {
        for( int j = 0; j < cavesize.height; j++ )
        {
		  if( ( help = ( AdvancedSearchAgentFieldList ) myfield.getElement( i , j ) ) != null )
          {
            help.setSearched( false );
          }
        }
      }
    } 
    return success;
  }

  protected boolean searchIterated( Point recent )
  {
    Point up, down, left, right, next;
    AdvancedSearchAgentFieldList help;
    Vector nearestlast;
    boolean done;
    
    done = false;

    // how deep are we already
    currentdepth++;
    if( log == true )
    {
      System.out.println( " Now searching at depth: " + currentdepth );
    }

    // do not include fields that have been searched already or
    // that have never been visited
    help = ( AdvancedSearchAgentFieldList ) myfield.getElement( recent.x, recent.y );
    if( help.getSearched() == true || help.wasVisited().equals( "NO" ) )
    {
      currentdepth--;
      return done;
    }

    // initialize vector to four element - size
    nearestlast = new Vector( 4 ); 

    // mark recent field as visited and check for goal - field
    help.setSearched( true );
    if( help.isExit().equals( "YES" ) )
    {
      if( log == true )
      {
        System.out.println( " Found home - field " );
      }
      done = true;
      return done;
    }

    // if field is not goal check if maximum - depth is reached
    if( currentdepth == iterationdepth )
    {
       currentdepth--;
       return done;
    }

    // get neighbour coordinates
    up    = new Point( recent.x, recent.y - 1 );
    down  = new Point( recent.x, recent.y + 1 );
    left  = new Point( recent.x - 1, recent.y );
    right = new Point( recent.x + 1, recent.y );

    
    // calculate manhatten - distance
    nearestlast = calcManhatten( up, down, left, right );

    // recurse
    while( nearestlast.isEmpty() == false && done == false )
    {
      next = ( Point ) nearestlast.lastElement();
      nearestlast.removeElementAt( nearestlast.size() - 1 );
      if( next == up )
      {
        if( log == true )
        {
          System.out.println( "Adding 'Up' action to my way home" );
        }
        searchway.addElement( ( Object ) new Action( "Up" ) );
      }
      else if( next == down )
      {
        if( log == true )
        {
          System.out.println( "Adding 'Down' action to my way home" );
        }
        searchway.addElement( ( Object ) new Action( "Down" ) );
      }
      else if( next == left )
      {
        if( log == true )
        {
          System.out.println( "Adding 'Left' action to my way home" );
        }
        searchway.addElement( ( Object ) new Action( "Left" ) );
      }
      else if( next == right )
      {
        if( log == true )
        {
          System.out.println( "Adding 'Right' action to my way home" );
        }
        searchway.addElement( ( Object ) new Action( "Right" ) );
      }
      if( log == true )
      {
        System.out.println( " Continuing search at: " + next.x + "/" + next.y );
      }
      done = searchIterated( next );
      if( done == false )
      {
        searchway.removeElementAt( searchway.size() - 1 );
      }
    }
    if( done == false )
    {
      currentdepth--;
    }
    return done;
  }

}

