import HashBackWumpus;
import java.util.*;
import java.awt.*;
import java.awt.image.ImageObserver;

//
// HashSearchBackWumpus 
//
// A frame for Wumpus-Agents that search a way back home as soon as
// they have found some gold.
//
// This class improves the strategy of updating the field once more.
//
//

abstract class HashSearchBackWumpus extends HashBackWumpus
{
  static final String type = "HASHSEARCHBACKWUMPUS";

  protected boolean foundway;
  protected Vector searchway; // represents the way home
  AdvancedSearchAgentFieldList location;

  public HashSearchBackWumpus( String name, int ammo, Dimension size )
  {
    this( name, ammo, size, ( String ) null, ( ImageObserver ) null, type );
  }
 
  public HashSearchBackWumpus( String name, int ammo, Dimension size, String imgref )
  {
    this( name, ammo, size, imgref, ( ImageObserver ) null, type );
  }

  public HashSearchBackWumpus( String name, int ammo, Dimension size, ImageObserver obs )
  {
    this( name, ammo, size, ( String ) null, obs, type );
  }

  public HashSearchBackWumpus( String name, int ammo, Dimension size, String imgref, ImageObserver obs )
  {
    this( name, ammo, size, imgref, obs, type );
  }

  public HashSearchBackWumpus( String name, int ammo, Dimension size, String imgref, ImageObserver obs, String type )
  {
    super( name, ammo, size, imgref, obs, type);
  }// End of HashBackWumpus-Constructors

  protected void initVars()
  {
    super.initVars();
    if( log == true )
    {
      System.out.println(" ArraySearchBack: initializing new vars ");
    }
    searchway = new Vector( 5, 5 );
    foundway = false;
  }

  protected void initField( Dimension fieldsize )
  {
    AdvancedSearchAgentFieldList help;
    cavesize = new Dimension();
    cavesize.height = fieldsize.height;
    cavesize.width = fieldsize.width;

	// initialize the field - represantation
	myfield = new HashList();

	// Agents start at location ( 1, 1 ); this is also the exit
    if( log == true )
    {
      System.out.println( " Creating start field and adding it to table " );
    }

	location = new AdvancedSearchAgentFieldList( 1, 1 );
    location.setWumpus( false );
    location.setPit( false );
    location.setExit( true );
    location.setWall( false );
    location.setVisited( true );
    location.incVisited();
    myfield.addElement( location );

    // up and left are wall because we always start in the upper left corner
    // this has to be removed if the agent should start anywhere in the maze
    help = new AdvancedSearchAgentFieldList( 0, 1 );
    help.setWall( true );
    myfield.addElement( help );
    help = new AdvancedSearchAgentFieldList( 1, 0 );
    help.setWall( true );
    myfield.addElement( help );
   
    super.myfield = myfield;
    ( ( HashWumpus ) this ).myfield = myfield;

    super.location = location;
    ( ( HashWumpus ) this ).location = location;

    if( log == true )
    {
      System.out.println("HashWumpus: end of initfield");
    }
  }

protected void finalize()
  {
    super.finalize();
    myfield = null;
    searchway = null;
  }
  
  public void receivePercepts( Vector percepts )//where the agent receives his percepts
  {
    super.receivePercepts( percepts );
    if( gohome == true && foundway == false )
    {
      findWayBack();
      // add the action for this round and remove it from wayback
      actions.addElement( wayback.lastElement() );
      wayback.removeElementAt( wayback.size() - 1 );
    }
    if( log == true )
    {
      System.out.println("Searcher finished move: " + moves +"\n\n" );
    }
  }	 

  protected void updateField()
  {
    AdvancedSearchAgentFieldList upleft, upright, downleft, downright;
    AgentField up, down, left, right;

    if( log == true )
    {
      System.out.println( "Starting to work on action at move number: " + ( moves + 1 ) );
      System.out.println( "Calling ancestors update method first" );
    }

    // call dad's version
    super.updateField();

    // using our knowledge that only one wumpus resides in the cave
    // we know that a stench at ( x/y ) and at ( x - 1/y - 1 ) means there is
    // a wumpus at ( x/y - 1 ) etc.
    // the following code is only valid as long as only one wumpus per cave is
    // allowed 

    up        = getUpField();
    down      = getDownField();
    left      = getLeftField();
    right     = getRightField();

    upleft    = getUpLeftField();
    upright   = getUpRightField();
    downleft  = getDownLeftField();
    downright = getDownRightField();

    if( pure.getStench() == true && wumpusalive == true )
    {
      if( ( upleft.isStench().equals( "YES" ) && upright.isStench().equals( "YES" ) ) ||
          ( upleft.isStench().equals( "YES" ) && left.hasWumpus().equals( "NO" ) ) ||
          ( upright.isStench().equals( "YES" ) && right.hasWumpus().equals( "NO" ) ) ||
          ( upleft.isStench().equals( "YES" ) && ( downleft.isStench().equals( "NO" ) && downleft.wasVisited().equals( "YES" ) ) ) ||
          ( upright.isStench().equals( "YES" ) && ( downright.isStench().equals( "NO" ) && downright.wasVisited().equals( "YES" ) ) ) )
      {
        up.setWumpus( true );
      }

      if( ( upleft.isStench().equals( "YES" ) && downleft.isStench().equals( "YES" ) ) ||
          ( upleft.isStench().equals( "YES" ) && up.hasWumpus().equals( "NO" ) ) ||
          ( downleft.isStench().equals( "YES" ) && down.hasWumpus().equals( "NO" ) ) ||
          ( upleft.isStench().equals( "YES" ) && ( upright.isStench().equals( "NO" ) && upright.wasVisited().equals( "YES" ) ) ) ||
          ( downleft.isStench().equals( "YES" ) && ( downright.isStench().equals( "NO" ) && downright.wasVisited().equals( "YES" ) ) ) )
      {
        left.setWumpus( true );
      }

      if( ( downleft.isStench().equals( "YES" ) && downright.isStench().equals( "YES" ) ) ||
          ( downleft.isStench().equals( "YES" ) && left.hasWumpus().equals( "NO" ) ) ||
          ( downright.isStench().equals( "YES" ) && right.hasWumpus().equals( "NO" ) ) ||
          ( downleft.isStench().equals( "YES" ) && ( upleft.isStench().equals( "NO" ) && upleft.wasVisited().equals( "YES" ) ) ) ||
          ( downright.isStench().equals( "YES" ) && ( upright.isStench().equals( "NO" ) && upright.wasVisited().equals( "YES" ) ) ) )
      {
        down.setWumpus( true );
      }

      if( ( downright.isStench().equals( "YES" ) && upright.isStench().equals( "YES" ) ) ||
          ( downright.isStench().equals( "YES" ) && down.hasWumpus().equals( "NO" ) ) ||
          ( upright.isStench().equals( "YES" ) && up.hasWumpus().equals( "NO" ) ) ||
          ( downright.isStench().equals( "YES" ) && ( downleft.isStench().equals( "NO" ) && downleft.wasVisited().equals( "YES" ) ) ) ||
          ( upright.isStench().equals( "YES" ) && ( upleft.isStench().equals( "NO" ) && upright.wasVisited().equals( "YES" ) ) ) )
      {
        right.setWumpus( true );
      }
    }
  }
   
  protected void findWayBack()
  {
    boolean success;
    Action current;
    Point start = new Point( location.xcoord, location.ycoord );
    
    current = new Action( "NONE" );
    // remove the action already calculated by superclass
    // keep current action in mind if searching is not successfull
    if( actions.isEmpty() == false )
    {
      current = ( Action ) actions.firstElement();
      if( log == true )
      {
        System.out.println( " Action calculated so far : " + current.getName() ); 
      }
    }
    actions.removeAllElements();

    success = search( start );
    
    if( log == true )
    {
      for( int i = 0; i < searchway.size(); i++ )
      {
        System.out.println( " searchway: " + i + " " +
                              ( ( Action ) searchway.elementAt( i ) ).getName() );
      }
      System.out.println( " wayback size: " + wayback.size() );
    }

    if( success == true )  // this should always be the case
    {
      if( log == true )
      {
        System.out.println( " success = : " + success ); 
      }
      if( ( wayback.size() + 1 ) >= searchway.size() )
      {
        if( log == true )
        {
          System.out.println( "Removing all elements from wayback vector" );
          System.out.println( "to store shorter way home" );
          System.out.println( "The way I found spares me " + ( wayback.size() - searchway.size() ) + " moves." );
        }

        wayback.removeAllElements();
        for( int i = 0; i < searchway.size(); i++ )
        {
          wayback.addElement( searchway.elementAt( searchway.size() - i - 1 ) );
        }
        searchway = null;
      }
      else 
      {
        // append the current action
        if( log == true )
        {
          System.out.println( "Search found no shorter way!!!!");
          System.out.println( "Restoring precalculated action: " + current.getName() ); 
        }
        wayback.addElement( ( Object ) current );
      }
    }
    else
    {
      // append the current action
      if( log == true )
      {
        System.out.println( "Search was not successfull!!!!!");
        System.out.println( "Restoring precalculated action: " + current.getName() ); 
      }
      wayback.addElement( ( Object ) current );
    }
    foundway = true;
  }

  public Vector calcManhatten( Point up, Point down, Point left, Point right )
  {
    Vector giveback;
    Point got[ ], help;
    int biggest;
    
    got = new Point[ 4 ];
    got[ 0 ] = up;
    got[ 1 ] = down;
    got[ 2 ] = left;
    got[ 3 ] = right;

    giveback = new Vector( 4 );
    
    // sort point - array; smallest element at position 3
    for( int i = 0; i < got.length - 1; i++ )
    {
      biggest = i;
      for( int j = i + 1; j < got.length; j++ )
      {
        if( ( Math.abs( 1 - got[ j ].x ) + Math.abs( 1 - got[ j ].y ) ) >
            ( Math.abs( 1 - got[ biggest ].x ) + Math.abs( 1 - got[ biggest ].y ) ) )
        {
          biggest = j;
        }
        help = got[ i ];
        got[ i ] = got[ biggest ];
        got[ biggest ] = help;
      }
    }

    for( int i = 0; i < got.length; i++ )
    {
      if( log == true )
      {
        System.out.println( " got[ " + i + " ]: " + got[ i ].x + "/" + got[ i ].y );
      }
      giveback.addElement( ( Object ) got[ i ] );
    }

    return giveback;
  }

  // getxxxField methods return the field that is to 
  // the up/down-right or to the up/down-left from the agent's current location
  protected AdvancedSearchAgentFieldList getUpLeftField()
  {
    AdvancedSearchAgentFieldList returnupleft;
    updateLocation();
    if( ( returnupleft = ( AdvancedSearchAgentFieldList ) myfield.getElement( location.xcoord - 1, location.ycoord - 1 ) )
           == null )
    {
      returnupleft = new AdvancedSearchAgentFieldList( location.xcoord - 1, location.ycoord - 1 );
      if( wumpusalive == false )
        returnupleft.setWumpus( false );
      myfield.addElement( returnupleft );
      if( log == true )
      {
        System.out.println( " Created new upleft field ");
      }
    }
    else
    {
      if( log == true ) 
      {
        System.out.println( " Upleft field already existed ");
      }
    }
    return returnupleft;
  }

  protected AdvancedSearchAgentFieldList getUpRightField()
  {
    AdvancedSearchAgentFieldList returnupright;
    updateLocation();
    if( ( returnupright = ( AdvancedSearchAgentFieldList ) myfield.getElement( location.xcoord + 1, location.ycoord -1 ) )
           == null )
    {
      returnupright = new AdvancedSearchAgentFieldList( location.xcoord + 1, location.ycoord - 1 );
      if( wumpusalive == false )
        returnupright.setWumpus( false );
      myfield.addElement( returnupright );
      if( log == true )
      {
        System.out.println( " Created new upright field ");
      }
    }
    else
    {
      if( log == true )
      {
        System.out.println( " Upright field already existed ");
      }
    }
    return returnupright;
  }

  protected AdvancedSearchAgentFieldList getDownLeftField()
  {
    AdvancedSearchAgentFieldList returndownleft;
    updateLocation();
    if( ( returndownleft = ( AdvancedSearchAgentFieldList ) myfield.getElement( location.xcoord - 1, location.ycoord + 1 ) )
           == null )
    {
      returndownleft = new AdvancedSearchAgentFieldList( location.xcoord - 1, location.ycoord + 1 );
      if( wumpusalive == false )
        returndownleft.setWumpus( false );
      myfield.addElement( returndownleft );
      if( log == true )
      {
         System.out.println( " Created new downleft field ");
      }
    }
    else
    {
      if( log == true )
      {
        System.out.println( " downleft field already existed ");
      }
    }
    return returndownleft;
  }

  protected AdvancedSearchAgentFieldList getDownRightField()
  {
    AdvancedSearchAgentFieldList returndownright;
    updateLocation();
    if( ( returndownright = ( AdvancedSearchAgentFieldList ) myfield.getElement( location.xcoord + 1, location.ycoord + 1 ) )
           == null )
    {
      returndownright = new AdvancedSearchAgentFieldList( location.xcoord + 1, location.ycoord + 1 );
      if( wumpusalive == false )
        returndownright.setWumpus( false );
      myfield.addElement( returndownright );
      if( log == true )
      {
        System.out.println( " Created new downright field ");
      }
    }
    else
    {
      if( log == true )
      {
        System.out.println( " Downright field already existed ");
      }
    }
    return returndownright;
  }


  // getxxxField methods return the field that is to 
  // the right, to the left, up or down from the agent's current location
  protected AgentFieldList getLeftField()
  {
    AdvancedSearchAgentFieldList returnleft;
    updateLocation();
    if( ( returnleft = ( AdvancedSearchAgentFieldList ) myfield.getElement( location.xcoord - 1, location.ycoord ) )
           == null )
    {
      returnleft = new AdvancedSearchAgentFieldList( location.xcoord - 1, location.ycoord );
      if( wumpusalive == false )
        returnleft.setWumpus( false );
      myfield.addElement( returnleft );
      if( log == true )
      {
        System.out.println( " Created new left field ");
      }
    }
    else
    {
      if( log == true ) 
      {
        System.out.println( " Left field already existed ");
      }
    }
    return returnleft;
  }

  protected AgentFieldList getRightField()
  {
    AdvancedSearchAgentFieldList returnright;
    updateLocation();
    if( ( returnright = ( AdvancedSearchAgentFieldList ) myfield.getElement( location.xcoord + 1, location.ycoord ) )
           == null )
    {
      returnright = new AdvancedSearchAgentFieldList( location.xcoord + 1, location.ycoord );
      if( wumpusalive == false )
        returnright.setWumpus( false );
      myfield.addElement( returnright );
      if( log == true )
      {
        System.out.println( " Created new right field ");
      }
    }
    else
    {
      if( log == true )
      {
        System.out.println( " right field already existed ");
      }
    }
    return returnright;
  }

  protected AgentFieldList getUpField()
  {
    AdvancedSearchAgentFieldList returnup;
    updateLocation();
    if( ( returnup = ( AdvancedSearchAgentFieldList ) myfield.getElement( location.xcoord, location.ycoord - 1 ) )
           == null )
    {
      returnup = new AdvancedSearchAgentFieldList( location.xcoord, location.ycoord - 1 );
      if( wumpusalive == false )
        returnup.setWumpus( false );
      myfield.addElement( returnup );
      if( log == true )
      {
         System.out.println( " Created new up field ");
      }
    }
    else
    {
      if( log == true )
      {
        System.out.println( " up field already existed ");
      }
    }
    return returnup;
  }

  protected AgentFieldList getDownField()
  {
    AdvancedSearchAgentFieldList returndown;
    updateLocation();
    if( ( returndown = ( AdvancedSearchAgentFieldList ) myfield.getElement( location.xcoord, location.ycoord + 1 ) )
           == null )
    {
      returndown = new AdvancedSearchAgentFieldList( location.xcoord, location.ycoord + 1 );
      if( wumpusalive == false )
        returndown.setWumpus( false );
      myfield.addElement( returndown );
      if( log == true )
      {
        System.out.println( " Created new down field ");
      }
    }
    else
    {
      if( log == true )
      {
        System.out.println( " down field already existed ");
      }
    }
    return returndown;
  }

  protected void updateLocation()
  {
    super.updateLocation();
    location = ( AdvancedSearchAgentFieldList ) super.location;
  }

  abstract protected boolean search( Point recent );

}

