import WumpusAgent;
import java.util.*;
import java.awt.*;
import java.awt.image.ImageObserver;

//
//
// HashWumpus
//
// Wumpus-Agent that has an internal represantation of the environment
//
// To represent the environment the agent uses an array that is
// updated every round according to the percepts the agent gets.
// 
// This agent implements a basic strategy that allows successors
// of this class to build upon it.
//

class HashWumpus extends WumpusAgent
{

  static final String type = "HASHWUMPUS";
  static final boolean log = false; // set to true if the agents should output
                                    // its actions to the console

  protected boolean wumpusalive;
  protected HashList myfield;
  protected AgentFieldList location;
  protected WumpusPurePercept pure;
  protected Action lastaction, action;
  protected Dimension cavesize;
  protected int moves, lastscore;
  protected boolean pickedgold;

  public HashWumpus( String name, int ammo, Dimension size )
  {
    this( name, ammo, size, ( String ) null, ( ImageObserver ) null, type );
  }
 
  public HashWumpus( String name, int ammo, Dimension size, String imgref )
  {
    this( name, ammo, size, imgref, ( ImageObserver ) null, type );
  }

  public HashWumpus( String name, int ammo, Dimension size, ImageObserver obs )
  {
    this( name, ammo, size, ( String ) null, obs, type );
  }

  public HashWumpus( String name, int ammo, Dimension size, String imgref, ImageObserver obs )
  {
    this( name, ammo, size, imgref, obs, type );
  }

  public HashWumpus( String name, int ammo, Dimension size, String imgref, ImageObserver obs, String type )
  {
    super( name, type, ammo, imgref, obs );
    initVars();
    initField( size );
  }// End of ReflexWumpus-Constructors

  protected void initVars()
  {
	// provide initializations that are needed for every of the
	// possible constructors
	moves = 0;
    lastscore = getScore();
    wumpusalive = true;
    lastaction = new Action( "NONE" ); // this is after all the start
  }

  protected void initField( Dimension fieldsize )
  {
    AgentFieldList help;
    cavesize = new Dimension();
    cavesize.height = fieldsize.height;
    cavesize.width = fieldsize.width;

	// initialize the field - represantation
	myfield = new HashList();

    // Agents start at location ( 1, 1 ); this is also the exit
    if( log == true )
    {
      System.out.println( " Creating start field and adding it to table " );
    }

	location = new AgentFieldList( 1, 1 );
    location.setWumpus( false );
    location.setPit( false );
    location.setExit( true );
    location.setWall( false );
    location.incVisited();
    myfield.addElement( location );

    // up and left are wall because we always start in the upper left corner
    // this has to be removed if the agent should start anywhere in the maze
    help = getUpField();
    help.setWall( true );
    help = getLeftField();
    help.setWall( true );
   
    if( log == true )
    {
      System.out.println("HashWumpus: end of initfield");
    }
  }

  protected void finalize()
  {
    // return memory for field representation
    myfield = null;
    location = null;
  }

  public void receivePercepts( Vector percepts )//where the agent receives his percepts
  {
    super.receivePercepts( percepts );
	pure = new WumpusPurePercept( percepts );
    if( log == true )
    {
      System.out.println("pure Scream:" + pure.getScream() );
      System.out.println("pure Glitter:" + pure.getGlitter() );
      System.out.println("pure Stench:" + pure.getStench() );
      System.out.println("pure Breeze:" + pure.getBreeze() );
      System.out.println("pure Bump:" + pure.getBump() + "\n" );
    }
    updateField();
    reason();
    updateHistory( ( Object ) action );
    if( log == true )
    {
      System.out.println("myaction: " + ( ( Action ) actions.elementAt( 0 ) ).getName() );
      System.out.println(" This is my move nr:" + moves );
    }
  }	 
  
  protected void updateField() // update environment - representation
  {
    AgentFieldList left, right, up, down, help;
    boolean moved = false;

    if( log == true )
    {
      System.out.println(" Starting round at location: " + location.xcoord + "/" + location.ycoord );
    }

    moves++;

    if( pickedgold == false )
    {
      if( lastscore >= getScore() )
      {
        lastscore = getScore();       
      }
      else
      {
        if( log == true )
        {
          System.out.println( "Terrific: I picked up a piece of gold" );
        }
        pickedgold = true;
      }
    }

    if( moves > 1 ) // actions is empty in the first round
    {
       lastaction = ( Action ) actions.elementAt( 0 );
    }
	actions.removeAllElements();

    if( log == true )
    {
      System.out.println("lastaction was" + lastaction.getName() );
    }
    
    // get neighbouring fields
    // create them if they do not exist yet
    left  = getLeftField();
    right = getRightField();
    up    = getUpField();
    down  = getDownField();

    // check if someone killed the wumpus
    // only already existent fields are marked wumpusless
    // new fields are marked at creation
	if( pure.getScream() == true )
	{
	  for( int x = 0; x < cavesize.width; x++ )
      {
        for( int y = 0; y < cavesize.height; y++ )
        {
          if( ( help = myfield.getElement( x, y ) ) != null )
            help.setWumpus( false );
        }
      }
	  wumpusalive = false;
	}

	// check if a bump was received and update map
	if( pure.getBump() == true )
	{
      if( log == true )
      {
        System.out.println(" Smashed into a wall! " );
      }
	  if( lastaction.getName().equals( "Up" ) )
      {
        up.setWall( true );
        if( log == true )
        {
          System.out.println(" Wall was at up ( " + up.xcoord + "/" + up.ycoord + " )" );
        }
      }
      else if( lastaction.getName().equals( "Down" ) )
      {
        down.setWall( true );
        if( log == true )
        {
          System.out.println(" Wall was at down ( " + down.xcoord + "/" + down.ycoord + " )" );
        }
      }
	  else if( lastaction.getName().equals( "Left" ) )
      {
        left.setWall( true );
        if( log == true )
        {
          System.out.println(" Wall was at left ( " + left.xcoord + "/" + left.ycoord + " )" );
        }
      }
	  else // last action was "Right"
      {
        right.setWall( true );
        if( log == true )
        {
          System.out.println(" Wall was at right ( " + right.xcoord + "/" + right.ycoord + " )" );
        }
      }
	  // leave routine -> there is nothing to update if we
	  // stayed in the same field and the kill of a wumpus
	  // by another agent is already handled
	  return;
	}
	else // there was no bumb
	{
      // if we shot last round and heard no scream
      // we can update one neighbouring field 
      if( wumpusalive == true )
      {
        if( lastaction.getName().equals( "Shoot_Up" ) )
        {
          if( ( help = myfield.getElement( location.xcoord, location.ycoord - 1 ) ) == null )
          {
            help = new AgentFieldList( location.xcoord, location.ycoord - 1 );
            myfield.addElement( help );
          }
          help.setWumpus( false );
          return;
        }
        if( lastaction.getName().equals( "Shoot_Down" ) )
        {
          if( ( help = myfield.getElement( location.xcoord, location.ycoord + 1 ) ) == null )
          {
            help = new AgentFieldList( location.xcoord, location.ycoord + 1 );
            myfield.addElement( help );
          }
          help.setWumpus( false );
          return;
        }
        if( lastaction.getName().equals( "Shoot_Right" ) )
        {
          if( ( help = myfield.getElement( location.xcoord + 1, location.ycoord ) ) == null )
          {
            help = new AgentFieldList( location.xcoord + 1, location.ycoord );
            myfield.addElement( help );
          }
          help.setWumpus( false );
          return;
        }
        if( lastaction.getName().equals( "Shoot_Left" ) )
        {
          if( ( help = myfield.getElement( location.xcoord - 1, location.ycoord ) ) == null )
          {
            help = new AgentFieldList( location.xcoord - 1, location.ycoord );
            myfield.addElement( help );
          }
          help.setWumpus( false );
          return;
        }
      }

	  // set new location if last action was a move
	  if( lastaction.getName().equals( "Up" ) )
      {
        location = up;
        moved = true;
        if( log == true )
        {
          System.out.println(" lastaction was an up move " );
        }
      }
	  else if( lastaction.getName().equals( "Down" ) )
      {
        location = down;
        moved = true;
        if( log == true )
        {
          System.out.println(" lastaction was a down move " );
        }
      }
      else if( lastaction.getName().equals( "Left" ) )
      {
        location = left;
        moved = true;
        if( log == true )
        {
          System.out.println(" lastaction was a left move " );
        }
      }
	  else if( lastaction.getName().equals( "Right" ) )
      {
        location = right;
        moved = true;
        if( log == true )
        {
          System.out.println(" lastaction was a right move " );
        }
      }

	  // mark new location as visited if we have moved
      // also update neighbours
      if( moved == true )
      {
	    location.setWumpus( false );
        location.setPit( false );
        location.setVisited( true );
        location.incVisited();
    
        left  = getLeftField();
        right = getRightField();
        up    = getUpField();
        down  = getDownField();
        if( log == true )
        {
          System.out.println("new location: " + location.xcoord + "/"  + location.ycoord  );
        }
      }
	}

	// check for stench and update surrounding fields if wumpus lives
   if(  wumpusalive == true )
	{
      if( log == true )
      {
        System.out.println(" checking for stench " );
      }
      if( pure.getStench() == true )
	  {
	    // if three fields have no wumpus the fourth field must contain it
	    if( up.hasWumpus().equals( "NO" ) && down.hasWumpus().equals( "NO" ) &&
		    left.hasWumpus().equals( "NO" ) )
        { 
          if( log == true )
          {
            System.out.println(" The wumpus is located to my right" );
          }
		  right.setWumpus( true );
		}
	    else if( up.hasWumpus().equals( "NO" ) && down.hasWumpus().equals( "NO" ) &&
		         right.hasWumpus().equals( "NO" ) )
		{
          if( log == true )
          {
            System.out.println(" The wumpus is located to my left" );
          }
		  left.setWumpus( true );
		}
	    else if( up.hasWumpus().equals( "NO" ) && right.hasWumpus().equals( "NO" ) &&
		         left.hasWumpus().equals( "NO" ) )
		{
          if( log == true )
          {
            System.out.println(" The wumpus is located downwards" );
          }
	      down.setWumpus( true );
		}
	    else if( down.hasWumpus().equals( "NO" ) && right.hasWumpus().equals( "NO" ) &&
		         left.hasWumpus().equals( "NO" ) )
		{
          if( log == true )
          {
            System.out.println(" The wumpus is located upwards" );
          }
		  up.setWumpus( true );
		}
	  }
	  else // surrounding fields are wumpusless
	  {
        if( log == true )
        {
          System.out.println(" Surrounding fields are wumpusfree" );
        }
        up.setWumpus( false );
		down.setWumpus( false );
		left.setWumpus( false );
		right.setWumpus( false );
	  }
	}
	
	// check for breeze and update surrounding fields
	if( pure.getBreeze() == true )
	{
      if( log == true )
      {
        System.out.println( "Checking for breeze." );
      }
	  // if three fields have no pit the fourth field must contain it
	  if( up.hasPit().equals( "NO" ) && down.hasPit().equals( "NO" ) &&
		  left.hasPit().equals( "NO" ) )
	  {
        if( log == true )
        {
          System.out.println(" A pit is to my right" );
        }
		right.setPit( true );
	  }
	  else if( up.hasPit().equals( "NO" ) && down.hasPit().equals( "NO" ) &&
		       right.hasPit().equals( "NO" ) )
	  {
        if( log == true )
        {
          System.out.println(" A pit is to my left" );
        }
		left.setPit( true );
	  }
	  else if( up.hasPit().equals( "NO" ) && right.hasPit().equals( "NO" ) &&
		       left.hasPit().equals( "NO" ) )
	  {
        if( log == true )
        {
          System.out.println(" A pit is one field below" );
        }
		down.setPit( true );
	  }
	  else if( down.hasPit().equals( "NO" ) && right.hasPit().equals( "NO" ) &&
		       left.hasPit().equals( "NO" ) )
	  {
        if( log == true )
        {
          System.out.println(" A pit is one field above" );
        }
		up.setPit( true );
	  }
	}
	else // surrounding fields are pitless
	{
      if( log == true )
      {
        System.out.println(" Surrounding fields are pitless ");
      }
	  up.setPit( false );
	  down.setPit( false );
	  right.setPit( false );
	  left.setPit( false );
	}
  } // updateField	

  protected void reason()
  {
    AgentFieldList left, right, up, down;
    int couldright, couldleft, couldup, coulddown, moveno;    
    
    // create surrounding fields if they do not exist yet
    // this should not really ever happen
    // get neighbouring fields
    left  = getLeftField();
    if( log == true )
    {
      System.out.println(" left x,y:" + ( left.xcoord ) + "  " + left.ycoord );
      System.out.println(" Pit: " + left.hasPit() );
      System.out.println(" Wumpus: " + left.hasWumpus() );
      System.out.println(" Times visited: " + left.visitCount() );
    }

    right = getRightField();
    if( log == true )
    {
      System.out.println(" right x,y:" + ( right.xcoord ) + "  " + right.ycoord );
      System.out.println(" Pit: " + right.hasPit() );
      System.out.println(" Wumpus: " + right.hasWumpus() );
      System.out.println(" Times visited: " + right.visitCount() );
    }

    up    = getUpField();
    if( log == true )
    {
      System.out.println(" up x,y:" + up.xcoord + "  " + ( up.ycoord ) );
      System.out.println(" Pit: " + up.hasPit() );
      System.out.println(" Wumpus: " + up.hasWumpus() );
      System.out.println(" Times visited: " + up.visitCount() );
    }

    down  = getDownField();
    if( log == true )
    {
      System.out.println(" down x,y:" + down.xcoord + "  " + ( down.ycoord ) );
      System.out.println(" Pit: " + down.hasPit() );
      System.out.println(" Wumpus: " + down.hasWumpus() );
      System.out.println(" Times visited: " + down.visitCount() );
    }
	
    // always pick up gold 
	if( pure.getGlitter() == true )
	{
      if( log == true )
      {
        System.out.println(" It glitters. I try to grab the gold" );
      }
	  actions.addElement( ( Object ) ( action = new Action( "Grab" ) ) );
	  return;
	}

	// if gold has been taken before, exit if possible
	// also exit if already more than 100 moves have passed
    if( location.isExit().equals( "YES" ) )
	{
      if( getScore() > 0 || moves > 100 )
      {
        if( log == true )
        {
          System.out.println("Exiting now" );
        }
        actions.addElement( ( Object ) ( action = new Action( "Climb" ) ) );
	    return;
      }
	}
    

	// if the location of the wumpus is in an adjacent field 
    // he is not already dead and there is an arrow left
	// kill him ( each agent has at least 1 arrow )
	// do not check for two fields of stench and two arrows etc.
    if( arrowsLeft() == true && wumpusalive == true )
    {
      if( up.hasWumpus().equals( "YES" ) )
      {
        if( log == true )
        {
          System.out.println(" Shooting up to the kill the wumpus " );
        }
	    actions.addElement( ( Object ) ( action = new Action( "Shoot_Up" ) ) );
	    return;
      }
	  if( down.hasWumpus().equals( "YES" ) )
      {
        if( log == true )
        {
          System.out.println(" Shooting down to the kill the wumpus " );
        }
	    actions.addElement( ( Object ) ( action = new Action( "Shoot_Down" ) ) );
	    return;
      }
      if( right.hasWumpus().equals( "YES" ) )
      {
        if( log == true )
        {
          System.out.println(" Shooting to the right to kill the wumpus " );
        }
	    actions.addElement( ( Object ) ( action = new Action( "Shoot_Right" ) ) );
	    return;
      }
	  if( left.hasWumpus().equals( "YES" ) )
      {
        if( log == true )
        {
          System.out.println(" Shooting to the left to kill the wumpus " );
        }
	    actions.addElement( ( Object ) ( action = new Action( "Shoot_Left" ) ) );
	    return;
      }
    }

	// decide the direction in which to go
	// first move to those fields not yet visited than those with the
	// lowest visit count
	// check left, down, up, right respectively
	// include hasWumpus in checks; maybe you forgot your arrows
    if( left.hasWumpus().equals( "NO" ) && left.hasPit().equals( "NO" ) && 
		left.isWall().equals( "NO" ) && left.wasVisited().equals( "NO" ) )
	{
	  actions.addElement( ( Object ) ( action = new Action( "Left" ) ) );
      if( log == true )
      {
        System.out.println(" Going to the left to visit a new field " );
      }
	  return;
	}
	else // check if field is a possible canditate
	{
	  if( left.hasWumpus().equals( "YES" ) || left.hasPit().equals( "YES" ) || 
		  left.isWall().equals( "YES" ) )
	  {
        if( log == true )
        {
          System.out.println(" Moving left would be lethal or a bump" );
        }
		couldleft = -1; 
	  }
	  else
	  {
		couldleft = left.visitCount();
        if( log == true )
        {
          System.out.println(" Could go left though I have been there " + left.visitCount() + " times" );
        }
	  }
	}

    if( down.hasWumpus().equals( "NO" ) && down.hasPit().equals( "NO" ) && 
		down.isWall().equals( "NO" ) && down.wasVisited().equals( "NO" ) )
	{
	  actions.addElement( ( Object ) ( action = new Action( "Down" ) ) );
      if( log == true )
      {
        System.out.println(" Going to the field below to visit a new field " );
      }
	  return;
	}
	else // check if field is a possible canditate
	{
	  if( down.hasWumpus().equals( "YES" ) || down.hasPit().equals( "YES" ) || 
		  down.isWall().equals( "YES" ) )
	  {
        if( log == true )
        {
          System.out.println(" Moving down would be lethal or a bump" );
        }
		coulddown = -1; 
	  }
	  else
	  {
		coulddown = down.visitCount();
        if( log == true )
        {
          System.out.println(" Could go down though I have been there " + down.visitCount() + " times" );
        }
	  }
	}
    if( right.hasWumpus().equals( "NO" ) && right.hasPit().equals( "NO" ) && 
		right.isWall().equals( "NO" ) && right.wasVisited().equals( "NO" ) )
	{
	  actions.addElement( ( Object ) ( action = new Action( "Right" ) ) );
      if( log == true )
      {
        System.out.println(" Going to the right to visit a new field " );
      }
	  return;
	}
	else // check if field is a possible canditate
	{
	  if( right.hasWumpus().equals( "YES" ) || right.hasPit().equals( "YES" ) || 
		  right.isWall().equals( "YES" ) )
	  {
        if( log == true )
        {
          System.out.println(" Moving to the right field would be lethal or a bump" );
        }
		couldright = -1; 
	  }
	  else
	  {
		couldright = right.visitCount();
        if( log == true )
        {
          System.out.println(" Could go right though I have been there " + right.visitCount() + " times" );
        }
	  }
	}
    if( up.hasWumpus().equals( "NO" ) && up.hasPit().equals( "NO" ) && 
		up.isWall().equals( "NO" ) && up.wasVisited().equals( "NO" ) )
	{
	  actions.addElement( ( Object ) ( action = new Action( "Up" ) ) );
      if( log == true )
      {
        System.out.println(" Going to field above to visit a new field " );
      }
	  return;
	}
	else // check if field is a possible canditate
	{
	  if( up.hasWumpus().equals( "YES" ) || up.hasPit().equals( "YES" ) || 
		  up.isWall().equals( "YES" ) )
	  {
        if( log == true )
        {
          System.out.println(" Moving up would be lethal or a bump" );
        }
		couldup = -1; 
	  }
	  else
	  {
		couldup = up.visitCount();
        if( log == true )
        {
          System.out.println(" Could go up though I have been there " + up.visitCount() + " times" );
        }
	  }
	}


    // if no new field is available enter the one that was at least
	// often visited i.e. has the smallest visit count
	// if this count is bigger than 10 venture into any possible
	// field that could be safe if no gold has been found yet
    moveno = getMove( couldright, couldup, couldleft, coulddown );  
	if( moveno == couldleft )
    {
      if( log == true )
      {
        System.out.println(" Going left because left field was visited the least often" );
      }
	  actions.addElement( ( Object ) ( action = new Action( "Left" ) ) );
	  return;
	}
	if( moveno == coulddown )
    {
      if( log == true )
      {
        System.out.println(" Going down because the field below was visited the least often" );
      }
	  actions.addElement( ( Object ) ( action = new Action( "Down" ) ) );
	  return;
	}
    if( moveno == couldright )
    {
      if( log == true )
      {
        System.out.println(" Going right because right field was visited the least often" );
      }
	  actions.addElement( ( Object ) ( action = new Action( "Right" ) ) );
	  return;
	}
    if( moveno == couldup )
    {
      if( log == true )
      {
        System.out.println(" Going up because the field above visited the least often" );
      }
	  actions.addElement( ( Object ) ( action = new Action( "Up" ) ) );
	  return;
	}
  } // reason
 
  private int getMove( int right, int up, int left, int down )
  {
	// return the lowest of four numbers that is bigger than zero
	// if this number is bigger than ten return 0 if any number is zero,
    // but only if we have not yet found a piece of gold
	int lowest;
	lowest = left;
	if( ( lowest > down && down > 0 ) || ( lowest <= 0 && down > 0 ) ) 
	  lowest = down;
	if( lowest > right && right > 0 || ( lowest <= 0 && right > 0 ) )
	  lowest = right;
	if( lowest > up && up > 0 || ( lowest <= 0 && up > 0 ) )
	  lowest = up;
	if( lowest > 10 && ( up == 0 || down == 0 || right == 0 || left == 0 ) &&
        pickedgold == false )
	  return 0;
	else 
	  return lowest;
  }

  // getxxxField methods return the field that is to 
  // the right, to the left, up or down from the agent's current location
  protected AgentFieldList getLeftField()
  {
    AgentFieldList returnleft;
    if( ( returnleft = myfield.getElement( location.xcoord - 1, location.ycoord ) )
           == null )
    {
      returnleft = new AgentFieldList( location.xcoord - 1, location.ycoord );
      if( wumpusalive == false )
        returnleft.setWumpus( false );
      myfield.addElement( returnleft );
      if( log == true )
      {
        System.out.println( " Created new left field ");
      }
    }
    else
    {
      if( log == true ) 
      {
        System.out.println( " Left field already existed ");
      }
    }
    return returnleft;
  }

  protected AgentFieldList getRightField()
  {
    AgentFieldList returnright;
    if( ( returnright = myfield.getElement( location.xcoord + 1, location.ycoord ) )
           == null )
    {
      returnright = new AgentFieldList( location.xcoord + 1, location.ycoord );
      if( wumpusalive == false )
        returnright.setWumpus( false );
      myfield.addElement( returnright );
      if( log == true )
      {
        System.out.println( " Created new right field ");
      }
    }
    else
    {
      if( log == true )
      {
        System.out.println( " right field already existed ");
      }
    }
    return returnright;
  }

  protected AgentFieldList getUpField()
  {
    AgentFieldList returnup;
    if( ( returnup = myfield.getElement( location.xcoord, location.ycoord - 1 ) )
           == null )
    {
      returnup = new AgentFieldList( location.xcoord, location.ycoord - 1 );
      if( wumpusalive == false )
        returnup.setWumpus( false );
      myfield.addElement( returnup );
      if( log == true )
      {
         System.out.println( " Created new up field ");
      }
    }
    else
    {
      if( log == true )
      {
        System.out.println( " up field already existed ");
      }
    }
    return returnup;
  }

  protected AgentFieldList getDownField()
  {
    AgentFieldList returndown;
    if( ( returndown = myfield.getElement( location.xcoord, location.ycoord + 1 ) )
           == null )
    {
      returndown = new AgentFieldList( location.xcoord, location.ycoord + 1 );
      if( wumpusalive == false )
        returndown.setWumpus( false );
      myfield.addElement( returndown );
      if( log == true )
      {
        System.out.println( " Created new down field ");
      }
    }
    else
    {
      if( log == true )
      {
        System.out.println( " down field already existed ");
      }
    }
    return returndown;
  }

   // for improved performance the log member has been set to final
  // which should allow compilers to optimize the if( log... ) 
  // statements away;
  // if it is of interest to log the actions an agent takes under
  // special circumstances you would have to redefine log as:
  // protected boolean log;
  // add the line: log = boolean_value to method initVars;
  // call this method to enable/disable logging
  /*
  public void setLog( boolean change )
  {
    if( log != change )
    {
      if( change == true )
      {
        System.out.println( "Logging now enabled" );
        log = change;
      }
      else
      {
        System.out.println( "Logging now disabled" );
        log = change;
      }
    }
  }
  */
}
