// file:        HistoryItem.java
// source file: HistoryItem.java
// object file: HistoryItem.class
//
//
// included objects:
//  HistoryItem
//    Desc: implements an object representing actions - the value is 
//          equivalent to the action
//
//
// history:
//   07.04.97 - first version finished by Roman Valehrach
//
//
// critical problems: none
//
// known problems: none
//

import java.util.*;
import java.lang.*;

class HistoryItem extends Object
{
  protected int value_;
  protected int validity_; 

  // private static final boolean DEBUG = false;

// --- actions - String constansts: ---------------------------------------
  static final String ACTION_STR_UP          = "UP";
  static final String ACTION_STR_DOWN        = "DOWN";
  static final String ACTION_STR_LEFT        = "LEFT";
  static final String ACTION_STR_RIGHT       = "RIGHT";
 
  static final String ACTION_STR_WAIT        = "WAIT";
  static final String ACTION_STR_CLIMB       = "CLIMB";
  static final String ACTION_STR_GRAB        = "GRAB";
 
  static final String ACTION_STR_SUCK        = "SUCK";
  static final String ACTION_STR_TURN_OFF    = "TURN_OFF";
 
  static final String ACTION_STR_SHOOT_UP    = "SHOOT_UP";
  static final String ACTION_STR_SHOOT_DOWN  = "SHOOT_DOWN";
  static final String ACTION_STR_SHOOT_LEFT  = "SHOOT_LEFT";
  static final String ACTION_STR_SHOOT_RIGHT = "SHOOT_RIGHT";
 
 
// --- actions - binary coded constansts: ---------------------------------
  static final int ACTION_BASE  = 0x00;
  static final int ACTION_UP          = 0x01 | ACTION_BASE;
  static final int ACTION_DOWN        = 0x02 | ACTION_BASE;
  static final int ACTION_LEFT        = 0x04 | ACTION_BASE;
  static final int ACTION_RIGHT       = 0x08 | ACTION_BASE;
 
  static final int ACTION_BASE2 = 0x10;
  static final int ACTION_WAIT        = 0x00 | ACTION_BASE2;
  static final int ACTION_CLIMB       = 0x01 | ACTION_BASE2;
  static final int ACTION_GRAB        = 0x02 | ACTION_BASE2;
  static final int ACTION_SUCK        = 0x04 | ACTION_BASE2;
  static final int ACTION_TURN_OFF    = 0x08 | ACTION_BASE2;
 
  static final int ACTION_SHOOT = 0x20;
  static final int ACTION_SHOOT_UP    = ACTION_UP    | ACTION_SHOOT;
  static final int ACTION_SHOOT_DOWN  = ACTION_DOWN  | ACTION_SHOOT;
  static final int ACTION_SHOOT_LEFT  = ACTION_LEFT  | ACTION_SHOOT;
  static final int ACTION_SHOOT_RIGHT = ACTION_RIGHT | ACTION_SHOOT;


// Method: HistoryItem Constructor (public)
//
// *Purpose   : intializes a new instance and sets private variables
//
// *Parameters: value (String, excl.-option) - value of the new instance
//              name  (String, excl.-option) - name of the new instance
//              valid (int, optional) - the state of validity 
//
// *Return    : none
//
// *Variables : value (String, class) - stores the HistoryItem's value
//              validity_ (int, class) - stores the HistoryItem's status
//
  public HistoryItem(int value)
  {
    this.value_ = value;
    this.validity_ = 0;
  }

  public HistoryItem( int value, boolean valid )
  {
    this.value_ = value;
    setValid( valid );
  }

  public HistoryItem( String name )
  {
    this.value_ = String2action( name );
    this.validity_ = 0;
  }

  public HistoryItem( String name, boolean valid ) 
  {
    this.value_ = String2action( name );
    setValid( valid );
  }// End of HistoryItem-Constructors


// Method: getValue (public)
//
// *Purpose   : returns the value of the current action instance
//
// *Parameters: none
//
// *Return    : int - value of the item - instance
//
// *Variables : name (String, class) - stores the HistoryItem's value
//
  public int getValue()
  {
    return this.value_;
  }


// Method: getName (public)
//
// *Purpose   : returns the string the value represents
//
// *Parameters: none
//
// *Return    : int - value of the item - instance
//
// *Variables : name (String, class) - stores the HistoryItem's value
//
  public String getName()
  {
    return action2String( this.value_ );
  }

// Method: getValid (public)
//
// *Purpose   : returns wether the current item-instance is valid
//              * if the item is yet un-checked if will always return false
//
// *Parameters: none
//
// *Return    : boolean - status of the item
//
// *Variables : validity_ (int, class) - stores the HistoryItem's status
//
  public boolean getValid()
  {
    switch ( this.validity_ )
    {
      case  0:  return false;
      case  1:  return true;
      case -1:  return false;
      default:  return false;
    }
  }

// Method: setValid (public)
//
// *Purpose   : sets the validity of the current instance
//
// *Parameters: valid (boolean) - the new status of the item-instance
//
// *Return    : none
//
// *Variables : validity_ (int, class) - stores the HistoryItem's status
//
  public void setValid( boolean valid )
  {
    if ( valid == true)
      this.validity_ =  1;
    else
      this.validity_ = -1;
  }


// Method: getChecked (public)
//
// *Purpose   : returns wether the current item-instance has been set
//              properly (status Valid is set)
//
// *Parameters: none
//
// *Return    : boolean - status of the item
//
// *Variables : validity_ (int, class) - stores the HistoryItem's status
//
  public boolean getChecked()
  {
    if ( (this.validity_ != 1) && (this.validity_ != -1) )
      return false;
    else
      return true;
  }// End of getValid


// Method: getValidity (public)
//
// *Purpose   : returns wether the current item-instance is valid
//              * if the item is yet un-checked if will always return false
//
// *Parameters: none
//
// *Return    : boolean - status of the item
//
// *Variables : validity_ (int, class) - stores the HistoryItem's status
//
  public int getValidity()
  {
    switch ( this.validity_ )
    {
      case  0:  return  0;
      case  1:  return  1;
      case -1:  return -1;
      default:  return  0;
    }
  }

// Method: action2String (public, static)
//
// *Purpose   : converts an action (constant value) to a String value
//
// *Parameters: action (int) - the constant value of the action
//   
// *Return    : String - the string representing the action
//
// *Variables : none
//
  public static String action2String(int action)
  {
    switch(action)
    {
      case ACTION_UP   :        return ACTION_STR_UP;
      case ACTION_DOWN :        return ACTION_STR_DOWN;
      case ACTION_LEFT :        return ACTION_STR_LEFT;
      case ACTION_RIGHT:        return ACTION_STR_RIGHT;

      case ACTION_CLIMB:        return ACTION_STR_CLIMB;
      case ACTION_GRAB :        return ACTION_STR_GRAB;
      case ACTION_WAIT :        return ACTION_STR_WAIT;

      case ACTION_SUCK    :     return ACTION_STR_SUCK;
      case ACTION_TURN_OFF:     return ACTION_STR_TURN_OFF;

      case ACTION_SHOOT_UP   :  return ACTION_STR_SHOOT_UP;
      case ACTION_SHOOT_DOWN :  return ACTION_STR_SHOOT_DOWN;
      case ACTION_SHOOT_LEFT :  return ACTION_STR_SHOOT_LEFT;
      case ACTION_SHOOT_RIGHT:  return ACTION_STR_SHOOT_RIGHT;

      default:           return(null);
    }
  }

// Method: String2action (public, static)
//
// *Purpose   : converts an action String to a (constant) value
//
// *Parameters: action (int) - the actions string
//   
// *Return    : int - the action representing the action string
//
// *Variables : none
//
  public static int String2action(String action)
  {
    if (action.equalsIgnoreCase(ACTION_STR_UP))       return ACTION_UP;
    if (action.equalsIgnoreCase(ACTION_STR_DOWN))     return ACTION_DOWN;
    if (action.equalsIgnoreCase(ACTION_STR_LEFT))     return ACTION_LEFT;
    if (action.equalsIgnoreCase(ACTION_STR_RIGHT))    return ACTION_RIGHT;

    if (action.equalsIgnoreCase(ACTION_STR_CLIMB))    return ACTION_CLIMB;
    if (action.equalsIgnoreCase(ACTION_STR_GRAB))     return ACTION_GRAB;
    if (action.equalsIgnoreCase(ACTION_STR_WAIT))     return ACTION_WAIT;

    if (action.equalsIgnoreCase(ACTION_STR_SUCK))     return ACTION_SUCK;
    if (action.equalsIgnoreCase(ACTION_STR_TURN_OFF)) return ACTION_TURN_OFF;

    if (action.equalsIgnoreCase(ACTION_STR_SHOOT_UP))    return ACTION_SHOOT_UP;
    if (action.equalsIgnoreCase(ACTION_STR_SHOOT_DOWN))  return ACTION_SHOOT_DOWN;
    if (action.equalsIgnoreCase(ACTION_STR_SHOOT_LEFT))  return ACTION_SHOOT_LEFT;
    if (action.equalsIgnoreCase(ACTION_STR_SHOOT_RIGHT)) return ACTION_SHOOT_RIGHT;

    return(0);
  }
}//End of class HistoryItem
