// file:        HistoryMap.java
// source file: HistoryMap.java
// object file: HistoryMap.class
//
//
// included objects:
//  HistoryMap
//    Desc: implements an history map for Wumpus/Vacuum-Worlds agents
//
// history:
//   07.04.97 - first version finished by Roman
//   
// critical problems: none
//
// known problems: none
//

import java.util.Vector;
import java.util.Enumeration;

class HistoryMap extends Object
{
  // private static final boolean DEBUG = false;

  protected Vector history_items_;
  protected int    counter_;         // for previous Item search and getElement
  protected int    valid_counter_;   // points to the first invalid item

// --- actions - String constansts: ---------------------------------------
  static final String ACTION_STR_UP          = "UP";
  static final String ACTION_STR_DOWN        = "DOWN";
  static final String ACTION_STR_LEFT        = "LEFT";
  static final String ACTION_STR_RIGHT       = "RIGHT";
 
  static final String ACTION_STR_WAIT        = "WAIT";
  static final String ACTION_STR_CLIMB       = "CLIMB";
  static final String ACTION_STR_GRAB        = "GRAB";
 
  static final String ACTION_STR_SUCK        = "SUCK";
  static final String ACTION_STR_TURN_OFF    = "TURN_OFF";
 
  static final String ACTION_STR_SHOOT_UP    = "SHOOT_UP";
  static final String ACTION_STR_SHOOT_DOWN  = "SHOOT_DOWN";
  static final String ACTION_STR_SHOOT_LEFT  = "SHOOT_LEFT";
  static final String ACTION_STR_SHOOT_RIGHT = "SHOOT_RIGHT";

 
// --- actions - binary coded constansts: ---------------------------------
  static final int ACTION_BASE  = 0x00;
  static final int ACTION_UP          = 0x01 | ACTION_BASE;
  static final int ACTION_DOWN        = 0x02 | ACTION_BASE;
  static final int ACTION_LEFT        = 0x04 | ACTION_BASE;
  static final int ACTION_RIGHT       = 0x08 | ACTION_BASE;
 
  static final int ACTION_BASE2 = 0x10;
  static final int ACTION_WAIT        = 0x00 | ACTION_BASE2;
  static final int ACTION_CLIMB       = 0x01 | ACTION_BASE2;
  static final int ACTION_GRAB        = 0x02 | ACTION_BASE2;
  static final int ACTION_SUCK        = 0x04 | ACTION_BASE2;
  static final int ACTION_TURN_OFF    = 0x08 | ACTION_BASE2;
 
  static final int ACTION_BASE3 = 0x20;
  static final int ACTION_SHOOT = 0x20;
  static final int ACTION_SHOOT_UP    = ACTION_UP    | ACTION_SHOOT;
  static final int ACTION_SHOOT_DOWN  = ACTION_DOWN  | ACTION_SHOOT;
  static final int ACTION_SHOOT_LEFT  = ACTION_LEFT  | ACTION_SHOOT;
  static final int ACTION_SHOOT_RIGHT = ACTION_RIGHT | ACTION_SHOOT;

  static final int ACTION_BASE4 = 0x40;

// --- Percepts - String constansts: --------------------------------------
  static final String PERCEPT_STR_BUMP    = "BUMP";
 
  static final String PERCEPT_STR_GLITTER = "GLITTER";
  static final String PERCEPT_STR_STENCH  = "STENCH";
  static final String PERCEPT_STR_SCREAM  = "SCREAM";
  static final String PERCEPT_STR_BREEZE  = "BREEZE";
 
  static final String PERCEPT_STR_DIRT    = "DIRT";


// Method: HistoryMap Constructor (public)
//
// *Purpose   : intializes a new instance and sets the private variables
//
// *Parameters: none
//
// *Return    : none
//
// *Variables : history_items_ (Vector, class) - stores the HistoryItem's value
//              counter_ (int, class) - current position counter
//              valid_counter_ (int, class) - current validation position counter
//
  public HistoryMap()
  {
    history_items_ = new Vector( 5, 5 );
    setCounter( -1);
    setValidCounter();
  }


// Method: setCounter (public)
//
// *Purpose   : sets the action counter to the given image or the last
//              action
//
// *Parameters: none
//
// *Return    : none
//
// *Variables : history_items_ (Vector, class) - stores the HistoryItem's value
//              counter_ (int, class) - current position counter
//              valid_counter_ (int, class) - current validation position counter
//
  public void setCounter(int counter)
  {
    if ( history_items_ != null )
    {
      if ( counter < history_items_.size() )
        this.counter_ = counter;
      else
        this.counter_ = history_items_.size() - 1;
    } else
      this.counter_ = -1;
  }

  public int getCounter()
  {
    return this.counter_;
  }
// Method: setValidCounter (public)
//
// *Purpose   : set the counter the next action to be validated
//
// *Parameters: none
//
// *Return    : none
//
// *Variables : history_items_ (Vector, class) - stores the HistoryItem's value
//              valid_counter_ (int, class) - current validation position counter
//
  public void setValidCounter()
  {
    int i;
    boolean checked = true;

    valid_counter_ = -1;

    if (history_items_ == null)
      return;

    for ( i = 0; (i < history_items_.size()) && (checked == true) ; i++)
    {
      checked = (( HistoryItem ) (history_items_.elementAt( i ))).getChecked();
      valid_counter_ = i;
    }
  }

// Method: advanceToNextInValid (public)
//
// *Purpose   : set advances the counter to the next action to be validated
//
// *Parameters: none
//
// *Return    : none
//
// *Variables : history_items_ (Vector, class) - stores the HistoryItem's value
//              valid_counter_ (int, class) - current validation position counter
//
  public void advanceToNextInValid()
  {
    boolean checked = true;

    if (history_items_ == null)
    {
      valid_counter_ = -1;
      return;
    }
    if ( valid_counter_ < 0 )
      valid_counter_ = 0;

    while ( ( valid_counter_ < history_items_.size()) && (checked == true) ) 
    {
      checked = (( HistoryItem ) (history_items_.elementAt( valid_counter_++ ))).getChecked();
    }
    valid_counter_--;
  }// End of advanceToNextInValid

// Method: size (public)
//
// *Purpose   : returns the current size of the history-map
//
// *Parameters: none
//
// *Return    : none
//
// *Variables : history_items_ (Vector, class) - stores the HistoryItem's value
//
  public int size()
  {
    if ( history_items_ != null )
    {
      return history_items_.size();
    } else
      return 0;
  }

// Method: elementAt (public)
//
// *Purpose   : returns the history map element at position i
//
// *Parameters: i (int) - the position
//
// *Return    : HistoryItem - the element at position i
//
// *Variables : history_items_ (Vector, class) - stores the HistoryItem's value
//
  public HistoryItem elementAt( int i )
  {
    if ( (history_items_ != null) && 
         (-1 < i) && (i < history_items_.size()) )
      return ( HistoryItem ) history_items_.elementAt( i );
    else
      return null;
  }

// Method: getElement (public)
//
// *Purpose   : returns the history map element the current counter position
//
// *Parameters: none
//
// *Return    : HistoryItem - the element at position i
//
// *Variables : history_items_ (Vector, class) - stores the HistoryItem's value
//              counter_ (int, class) - current position counter
//
  public HistoryItem getElement()
  {
    return elementAt(counter_);
  }

// Method: toValidate (public)
//
// *Purpose   : returns the history map element that is to be validated next
//
// *Parameters: none
//
// *Return    : HistoryItem - the element at position i
//
// *Variables : history_items_ (Vector, class) - stores the HistoryItem's value
//              valid_counter_ (int, class) - current validation position counter
//
  public HistoryItem toValidate()
  {
    advanceToNextInValid(); 
    return elementAt(valid_counter_);
  }


// Method: addElement (public)
//
// *Purpose   : adds a new item to the history-map 
//
// *Parameters: new_element (HistoryItem) - element to add
//
// *Return    : none
//
// *Variables : history_items_ (Vector, class) - stores the HistoryItem's value
//              counter_ (int, class) - current position counter
//
  public void addElement(HistoryItem new_element)
  {
    if ( history_items_ != null )
      history_items_.addElement((Object)new_element);
    else 
    {
      history_items_ = new Vector(5, 5);
      history_items_.addElement((Object)new_element);
    }

    setCounter( counter_ ); // check counters
    setValidCounter();
  }// End of addElement

// Method: removeElement (public)
//
// *Purpose   : removes an item from the history-map
//              * do not use this since history-map is a write-only map
//
// *Parameters: new_element (HistoryItem) - element to delete
//
// *Return    : boolean - wether the removal was successful or not
//
// *Variables : history_items_ (Vector, class) - stores the HistoryItem's value
//              counter_ (int, class) - current position counter
//
  private boolean removeElement(HistoryItem new_element)
  {
    boolean oppvalue = false;

    if ( history_items_ != null )
      oppvalue = history_items_.removeElement((Object)new_element);
    else 
      history_items_ = new Vector(5, 5);

    setCounter( counter_ ); // check counters
    setValidCounter();

    return oppvalue;
  }

// Method: updateHistory ( public )
//
// *Purpose   : stores every action an agent has taken
//
// *Parameters: newaction ( Object ) - the latest action of the agent
//          or: newaction ( String ) - the latest action of the agent
//
// *Return    : none
//
// *Variables : none
//
  public void updateHistory( Action newaction )
  {
    int action_value = String2action( newaction.getName() );
    HistoryItem item = new HistoryItem( action_value );

    this.addElement( item );
  }

  public void updateHistory( String newaction )
  {
    int action_value = String2action( newaction );
    HistoryItem item = new HistoryItem( action_value );

    this.addElement( item );
  }

// Method: validateHistory ( public )
//
// *Purpose   : validates the History map at the current validation pos. 
//              * movement (up, down, left, right) only
//
// *Parameters: percepts ( Vector ) - the percepts that resulted from the
//                                    latest action
//
// *Return    : none
//
// *Variables : none
//
  public void validateHistory( Vector percepts )
  {
    boolean valid= true;
    int i;

    HistoryItem item = toValidate();

    if ( ( item == null ) || ( item.getChecked() ) )
      return; /* nothing to do */

/*
    if (DEBUG)
      System.out.println( "HistoryMap validating >" + item.getName() +
        "< at position " + valid_counter_ );
*/

    i= item.getValue();

    if ( (i > ACTION_BASE) && (i < ACTION_BASE2) )  
    { // only interested in moves
      String tmp_perc;
      if ( percepts != null )
        for ( i = 0 ; i < percepts.size() ; i++ )
        {
          tmp_perc = (( Percept ) percepts.elementAt( i )).getName();
          if ( tmp_perc.equalsIgnoreCase( PERCEPT_STR_BUMP ) )
          {
            valid = false;
            break;
          }
        }
    }

    item.setValid(valid);
  }

// Method: getHistory ( public )
//
// *Purpose   : returns a string with all the agent's actions
//              parted by a newline character
//
// *Parameters: none
//
// *Return    : String - the agent's history
//
// *Variables : history_items_ ( Vector ) - the agent's history Vector
//
  public String getHistory()
  {
    Enumeration actions;
    HistoryItem item;
    StringBuffer giveback = new StringBuffer();

    if (size() > 0)
    {
      actions = history_items_.elements();
      giveback.append( "My actions have been: \n" );
      while( actions.hasMoreElements() == true )
      {
        item = ( HistoryItem )  actions.nextElement();
        giveback.append( action2String( item.getValue() ) );
        if (item.getChecked() == true )
        {
          if ( item.getValid() == true )
            giveback.append( " (+)\n"  );
          else
            giveback.append( " (-)\n"  );
        }
        else
            giveback.append( " (?)\n"  );
      }
      return giveback.toString();
    } else 
      return null;
  }// End of getHistory
 

// Method: lastMove (public)
//
// *Purpose   : returns the last movement action the agent executet
//              (this methode ignores wheter this move was valid or not)
//
// *Parameters: none
//
// *Return    : String - the last move taken
//
// *Variables : history_items_ (Vector, class) - the historymap of the agent
//
  public String lastMove()
  {
    String lastmove = null;
    boolean move = false;
    HistoryItem tmpitem;
    int tmpvalue;
    int i;

    if ( (i = size()) == 0 )
      return null;

    int tmp = -1;
    
    for ( i -= 1 ; ( i >= 0 ) && ( move == false ); i--  )
    {
      tmpitem = elementAt( i );
 
      if (tmpitem == null) 
        break;

      tmpvalue = tmpitem.getValue();

/*
      if ( DEBUG )
        System.out.println( "HistoryMap: lastMove() current move >"+ 
          action2String( tmpvalue ) + "< counter >" + i + "<" );
*/

      if ( (tmpvalue > ACTION_BASE) && (tmpvalue < ACTION_BASE2) ) 
      {
        lastmove = action2String( tmpvalue );
        move = true;
        tmp= i;
      }
    }

    counter_ = tmp;

//    if ( DEBUG )
//      System.out.println(" Last move taken >" +lastmove+ "< ...");

    return lastmove;
  }// End of lastMove


// Method: prevMove (public)
//
// *Purpose   : returns the movement action before the current on (counter_)
//
// *Parameters: none
//
// *Return    : String - the last move taken
//
// *Variables : history_items_ (Vector, class) - the historymap of the agent
//
  public String prevMove()
  {
    String lastmove = null;
    int tmpvalue;
    boolean move = false;
    int i;
    int tmp = -1;

    if ( history_items_ == null )
      return null;

    for ( i = counter_ - 1 ; ( i > -1 ) && ( move == false ); i--  )
    {
      tmpvalue = (( HistoryItem ) (history_items_.elementAt( i ))).getValue();

/*
      if ( DEBUG )
        System.out.println( "HistoryMap: prevMove() current move >" + 
          action2String( tmpvalue ) + "< counter >" + i + "<" );
*/

      if ( (tmpvalue > ACTION_BASE) && (tmpvalue < ACTION_BASE2) ) 
      {
        lastmove = action2String( tmpvalue );
        move = true;
        tmp = i;
      }
    }

    counter_ = tmp;

//    if ( DEBUG )
//      System.out.println( " previous move tried >" +lastmove+ "< ..." );

    return lastmove;
  }// End of prevMove


// Method: lastValidMove (public)
//
// *Purpose   : returns the last valid movement action the agent executed
//
// *Parameters: none
//
// *Return    : String - the last move taken
//
// *Variables : history_items_ (Vector, class) - the historymap of the agent
//
  public String lastValidMove()
  {
    String tmpstr = lastMove();
    HistoryItem item = getElement();
    while ( (item != null) && 
            ( (item.getChecked() == false) || (item.getValid() == false) ) )
    {
      tmpstr = prevMove();
      item = getElement();
    }

//    if ( DEBUG )
//      System.out.println( "HistoryMap: last move executed >" + tmpstr + "<" );

    return tmpstr;
  }// End of lastValidMove


// Method: prevValidMove (public)
//
// *Purpose   : returns the previous valid movement action the agent executed
//
// *Parameters: none
//
// *Return    : String - the last move taken
//
// *Variables : history_items_ (Vector, class) - the historymap of the agent
//
  public String prevValidMove()
  {
    String tmpstr = prevMove();
    HistoryItem item = getElement();

    while ( (item != null) && 
            ( (item.getChecked() == false) || (item.getValid() == false) ) )
    {
      tmpstr = prevMove();
      item = getElement();
    }

    return tmpstr;
  }// End of prevValidMove


// Method: invalidMoves (public)
//
// *Purpose   : returns the all the movement actions that had no effect
//              (Bumps)
//
// *Parameters: none
//
// *Return    : Vector - the last move taken
//
// *Variables : history_items_ (Vector, class) - the historymap of the agent
//
  public Vector invalidMoves()
  {
    Vector tmpvect = new Vector( 1,1 );
    String tmpstr = lastMove();
    HistoryItem item = getElement();

    while ( (item != null) && 
            ( (item.getChecked() == false) || (item.getValid() == false) ) )
    {
      tmpvect.addElement( (Object) tmpstr ); // add the invalid move
      tmpstr = prevMove();
      item = getElement();
    }

    return tmpvect;
  }// End of invalidMoves


// Method: action2String (public, static)
//
// *Purpose   : converts an action (constant value) to a String value
//
// *Parameters: action (int) - the constant value of the action
//   
// *Return    : String - the string representing the action
//
// *Variables : none
//
  public static String action2String(int action)
  {
    switch(action)
    {
      case ACTION_UP   :        return ACTION_STR_UP;
      case ACTION_DOWN :        return ACTION_STR_DOWN;
      case ACTION_LEFT :        return ACTION_STR_LEFT;
      case ACTION_RIGHT:        return ACTION_STR_RIGHT;

      case ACTION_CLIMB:        return ACTION_STR_CLIMB;
      case ACTION_GRAB :        return ACTION_STR_GRAB;
      case ACTION_WAIT :        return ACTION_STR_WAIT;

      case ACTION_SUCK    :     return ACTION_STR_SUCK;
      case ACTION_TURN_OFF:     return ACTION_STR_TURN_OFF;

      case ACTION_SHOOT_UP   :  return ACTION_STR_SHOOT_UP;
      case ACTION_SHOOT_DOWN :  return ACTION_STR_SHOOT_DOWN;
      case ACTION_SHOOT_LEFT :  return ACTION_STR_SHOOT_LEFT;
      case ACTION_SHOOT_RIGHT:  return ACTION_STR_SHOOT_RIGHT;

      default:           return(null);
    }
  }

// Method: String2action (public, static)
//
// *Purpose   : converts an action String to a (constant) value
//
// *Parameters: action (int) - the actions string
//   
// *Return    : int - the action representing the action string
//
// *Variables : none
//
  public static int String2action(String action)
  {
    if (action.equalsIgnoreCase(ACTION_STR_UP))       return ACTION_UP;
    if (action.equalsIgnoreCase(ACTION_STR_DOWN))     return ACTION_DOWN;
    if (action.equalsIgnoreCase(ACTION_STR_LEFT))     return ACTION_LEFT;
    if (action.equalsIgnoreCase(ACTION_STR_RIGHT))    return ACTION_RIGHT;

    if (action.equalsIgnoreCase(ACTION_STR_CLIMB))    return ACTION_CLIMB;
    if (action.equalsIgnoreCase(ACTION_STR_GRAB))     return ACTION_GRAB;
    if (action.equalsIgnoreCase(ACTION_STR_WAIT))     return ACTION_WAIT;

    if (action.equalsIgnoreCase(ACTION_STR_SUCK))     return ACTION_SUCK;
    if (action.equalsIgnoreCase(ACTION_STR_TURN_OFF)) return ACTION_TURN_OFF;

    if (action.equalsIgnoreCase(ACTION_STR_SHOOT_UP))    return ACTION_SHOOT_UP;
    if (action.equalsIgnoreCase(ACTION_STR_SHOOT_DOWN))  return ACTION_SHOOT_DOWN;
    if (action.equalsIgnoreCase(ACTION_STR_SHOOT_LEFT))  return ACTION_SHOOT_LEFT;
    if (action.equalsIgnoreCase(ACTION_STR_SHOOT_RIGHT)) return ACTION_SHOOT_RIGHT;

    return(0);
  }

}// End of HistoryMap
