// file:        ImagePropDecoder.java
// source file: ImagePropDecoder.java
// object file: ImagePropDecoder.class
//
//
// included objects:
//  ImagePropDecoder
//    Desc: implements a decoder for multi-image properties get
//
//
// history:
//   14.04.97 - first version finished by Roman Valehrach
//
//
// critical problems: none
//
// known problems: none
//

import java.awt.Color;
import java.awt.Graphics;
import java.awt.Rectangle;

class ImagePropDecoder extends Object
{
  // private final static boolean DEBUG = false;

  public String  resource;
  public String  base;
  public String  defStr;

  public boolean use_random;
  public int     use_num;
  public int     current_num;
  public boolean use_static;

  protected static DefaultProps props_;


// Constants needed:
  protected static final String DELIMITER   = ".";
  protected static final String TOTAL_ID    = ".TOTAL";
  protected static final String STATIC_ID   = ".STATIC";
  protected static final String RANDOM_ID   = ".RANDOM";

  protected static final String NAME_ID     = ".STR";
  protected static final String IMAGE_ID    = ".IMG";
  protected static final String COLOR_ID    = ".COLOR";

  protected static final String LINK_ID     = "LINK_";
  protected static final int    LINK_ID_LEN = LINK_ID.length();

// Method: DecodeImageProp Constructor (public)
//
// *Purpose   : intializes a new instance that stores the decoded resource
//
// *Parameters: base (String) - the base-name to the image
//              default (String) - if it doesn't work - default to this
//              use_random (boolean) - allways use random !
//              use_num(boolean)     - if possible get the image with this number
//              use_static (boolean) - allways informe of static !
//
// *Return    : none
//
// *Variables : none
//
//
  public ImagePropDecoder(String base, String defStr, 
    boolean use_random, int use_num, boolean use_static)
  {
    this.base        = base;

    if (defStr == null)
      this.defStr = "EMPTY";
    else
      this.defStr = defStr;

    this.use_random  = use_random;
    this.use_num     = use_num;
    this.use_static  = use_static;

    if (this.props_ == null) ;
      this.props_ = new DefaultProps();

    if (this.base != null)
      decodeMultiple();
  }

// Method: decodeMultiple (public)
//
// *Purpose   : method to select the proper image property
//
// *Parameters: none
//
// *Return    : none
//
// *Variables : some counters..
//
  public void decodeMultiple ()
  {
    this.resource = null;
    this.current_num = -1;
    String tmp;

    // System.err.println( "Decoding base.. the base: >" + base + "<");

    int i = this.use_num;
    int max = this.props_.getNumber( base + TOTAL_ID, 1 );

    if ( (max < 1) && (max != this.use_num) ) return;

    if (i > max) i = max; // 'TOTAL'-value defines the !absolute! upper boundrary

    // now search for "STATIC" ...
    if ( (tmp = this.props_.getPropertyNoOutput( base + STATIC_ID )) != null)
    { 
      // override the current seeting
      if ( tmp.equalsIgnoreCase( "true" ) ) this.use_static = true;
      else                                  this.use_static = false;
    }

    // and for "RANDOM"-properties in the DefaultProps
    if ( (tmp = this.props_.getPropertyNoOutput( base + RANDOM_ID)) != null)
    {
      // override the current seeting
      if ( tmp.equalsIgnoreCase( "true" ) ) this.use_random = true;
      else                                  this.use_random = false;
    }
   
    // now select a resource depending on the mode static/random/none
    if ( this.use_static )
    {
      i = this.props_.getStatic( base );

      if (i == 0) {
        i = 1;
        this.props_.newStatic( base, i );
      }
      else {
        // System.err.println("Searched for static: "+ base+ " current value: " + i + ", max:" + max);

        i++;
        if (i > max)
        {
          // this.props_.setStatic( base, 1 ); // return to the first resource
          i = 1;
        } 
          // else {
          // this.props_.increaseStatic( base, 1 ); } // just increase by one to get the next resource

        // this might be a bit faster...
        this.props_.setStatic( base, i ); // just increase by one to get the next resource
      }
    }
    else {
      // random selection
      if ( this.use_random )
        i = (int)(java.lang.Math.abs( (java.lang.Math.random() * max ) + 1 ));
    }


    if ( (i > 1) || (i == this.use_num) )  
    {
      this.resource = base + DELIMITER + i + IMAGE_ID;
      this.current_num = i;
    } else {
      this.resource = base + ".1" + IMAGE_ID; // getting default-name
      this.current_num = 1;
    }
  
    this.resource = this.props_.decodeLink( getName() ); // check the default TOO - when no resource
  }

  public String getName()
  {
    if ( this.resource == null )
      return defStr;
    else
      return this.resource;
  }

  public String decodeColor()
  {
    return this.props_.getLinkableProperty( this.base + DELIMITER + this.current_num + COLOR_ID, "colors.GRAY"  );
  }

  public static String decodeColor( DefaultProps props, String base, int i, String defaultVal )
  {
    return props.getLinkableProperty( base + DELIMITER + i + COLOR_ID, defaultVal  );
  }
}// End of class ImagePropDecoder
