// file:        LinLinList.java
// source file: LinList.java
// object file: LinList.class
//
//
// included objects:
//  LinList
//    Desc: implements a single-node LinList (forward only) with unsorted 
//          (object) elements
//
//
// history:
//   24.04.97 - first version finished by Roman Valehrach
//
//
// critical problems: none
//
// known problems: none
//




class LinList extends Object
{
  protected Object  data;
  protected LinList left;     // ^= forward
  protected LinList right;    // ^= backward

  public LinList() 
  {
    this.data    = null;
    this.right   = null;
  }
      
  public LinList( Object newData )   // initialize a new Node with payload
  {
    this.data    = newData;
    this.right   = null;
  }

  public Object getData()             // get the payload object
  {
    return this.data;
  }

  public Object exchangeData (Object newData) 
  {
    Object temp;
     
    temp = this.data;
    this.data = newData;
      
    return temp;
  }

  public LinList getRight()                // now it's the right
  {
    return this.right;
  }

  public LinList addRightNode( LinList newNode ) 
  {
    if (this.right != null) {
      newNode.addRightNode( this.right ); // to insert nodes that may already have subnodes
      this.right = newNode;
    }
    else {
      this.right = newNode;
    }
   
    return this.right;
  }          

  public LinList addRight( Object toAdd )
  {
    LinList temp = new LinList( toAdd );

    if (this.right != null) {
      temp.addRightNode(this.right);
    }
 
    this.right = temp;

    return this.right;
  }

  public LinList exists( Object extObj )   // again no check for doubled referenzes
  {
    LinList temp;

    if (this.data == extObj) {
      return this;
    }
    else {
      if (this.right != null) {
        return this.right.exists( extObj );
      }  
    }  

    return null;
  }

  public LinList add( Object newData )
  {
    return this.lastElement().addRight( newData );
  }

  public LinList lastElement()  // no recursion to save machine stack
  {
    LinList tmp = this;
    
    while (tmp.right != null)
    {
      tmp= tmp.right;
    }
    return tmp;
  }

  public LinList nextElement() 
  {
    return this.right;
  }

  public void printLinList()  // no recursion to save machine stack
  {
    System.out.println( "Sorted LinList: " );
    LinList tmp = this;

    while ( tmp != null)
    {
      System.out.println( "  " + tmp.getData() );
      tmp = tmp.nextElement();
    }
  }

}

// Function :
// Procedure:
// Method:  (public)
// Method:  (private)
//
// *Purpose   :
//              
//
// *Parameters:
//
// *Return    :
//
// *Variables :
//
