// file:        Pit.java
// source file: Pit.java
// object file: Pit.class
//
//
// included objects:
//  Pit
//    Desc: implements the FieldBase object - Pit
//
//
// history:
//   14.04.97 - first version finished by Roman Valehrach
//
//
// critical problems: none
//
// known problems: none
//

import java.awt.Color;
import java.awt.Graphics;
import java.awt.Rectangle;
import java.awt.image.ImageObserver;


class Pit extends FieldBaseObject implements FieldFillingIfc
{
  // private final static boolean DEBUG = false;

  // ahead defininitions:
  private final static String TYPE_PIT = "PIT";

  private final static String DEFAULT_FOREGROUND_STR = "colors.FOREGROUND";
  private final static String DEFAULT_COLOR_STR      = "colors.fieldobject.PITCOLOR";

// Method: Pit Constructor (public)
//
// *Purpose   : intializes a new instance and sets all (in advance) neccessary objects
//
// *Parameters: name (String, optional) - name of the new instance
//              resource (String) - name of the resource for the FieldObject
//              observer (ImageObserver) - observer for the FieldObject
//
// *Return    : none
//
// *Variables : none
//
//   with set/get-methods:
//              imgref (String) - default reference to an imagemap
//              obs (ImageObserver) - observer for the image
//
  public Pit( String resource, ImageObserver observer )
  {
    super( TYPE_PIT, TYPE_PIT, resource, observer );
  }

  public Pit( String resource )
  {
    super( TYPE_PIT, TYPE_PIT, resource );
  }

  public Pit( ImageObserver observer )
  {
    super( TYPE_PIT, TYPE_PIT, null, observer );
  }

  public Pit()
  {
    super( TYPE_PIT, TYPE_PIT ,null, null );
  }


// Method: defaultColorName      (protected)
//         defaultForegroundName (protected)
//
// *Purpose   : returns a default property identifing string for a color
//              - good for beeing overwritten by subclasses to set an other
//                default property identifing string 
//                (the string to be looked up in the system-property list
//
// *Parameters: none
//
// *Return    : String - the new default color
//
// *Variables : all the nessessary id-strings
//
  public String defaultColorName()      { return DEFAULT_COLOR_STR; }
  public String defaultForegroundName() { return DEFAULT_FOREGROUND_STR; }


// Method: getDisplayName() (public)
//
// *Purpose   : statically returns the PIT-type string
//
// *Parameters: none
//
// *Return    : String - the current type, name or "NONE"
//
// *Variables : none
//
   public String getDisplayName()
   {
     return TYPE_PIT;  // overwrite with static for this fieldobject's purpose
   }


// Method: drawPrimitive (public)
//
// *Purpose   : draws a graphic primitive (a default form of displaying)
//              to the graphics device 
//
// *Parameters: g (Graphics) - the graphics to draw the objects to
//              area (Rectangle) - the position and dimensions of the
//                                 object on screen
//              foreground (Color) - the drawingcolor to use
//
// *Return    : whether the drawing of the Image was successfull or not
//
// *Variables : myimage (ImageReference, class) - pointer to a resource
//
  public boolean drawPrimitive( Graphics g, Rectangle area )
  {
    return ( drawPrimitive(g, area, getColor() ) );
  }

  public boolean drawPrimitive( 
    Graphics g, Rectangle area, Color foreground
  )
  {
//    if ( g == null ) return false;

    if ( foreground != null)
      g.setColor( foreground );

    //Primitive: a simple Box with no distance to the bounds
    try 
    { 
      Rectangle small = new Rectangle ( area.x, 
                                        area.y,
                                        2* (area.width/3), 
                                        2* (area.height/3) );

      small.x += (area.width  - small.width) /2;
      small.y += (area.height - small.height)/2;

      g.fillRect( small.x, small.y, small.width, small.height );
    }
    catch (ArithmeticException e)
    {
      g.fillRect( area.x, area.y, area.width, area.height );
    }
    return true;
  }

// Method: fillsField (public)
//
// *Purpose   : returns whether the current object is as big a the
//              field (i.e. a Wall, Pit etc.) of a Field
//
// *Parameters: none
//
// *Return    : boolean - true when fields base
//
// *Variables : none
//
  public boolean fillsField()
  {
    return true;
  }

/* Examples for additional functionality:

  // validate an agents action in-to, in or out-of a field
  public String vaidateFieldObjectAction ( FieldObject fldobj )
  {
    if there is a step into is ok in action return TRUE through action-string

    if there is no MOVE_.... .. this object can't handle it..
  }

  // may an agent ever leave the field - i.e. a pit may not be left -> no escape
  public boolean leaveAble()
  {
    return false;
  }
*/


}// End of class Pit
