import java.awt.*;
import java.awt.image.ImageObserver;
import java.util.*;

//
// 
// ReflexWumpus
// This class implements an agent for the Wumpus Environment
// that acts like a reflex agent as described in:
// Russel & Norvig: Artificial Intelligence, A Modern Approach
//
//


//
// history:
//   11.03.97 - first version finished by Chris
//
//   11.03.97 - development off. continued by Roman: small changes to
//              displaying
//   12.03.97 - new extensions for displaying images (as per protocol
//              11.03.97) - Roman
//   22.03.97 - revised documentation - Roman
//
//   30.03.97 - added history bookeeping
//
//
// critical problems: none
//
// known problems:
//   * objects have to have a minimal size -> small objects have problems displaying
//     themselfs
//

class ReflexWumpus extends WumpusAgent
{
  private int rand;
  private Action action;  

  private static final String TYPE_AGENT_WUMPUS_REFLEX = "RELEXWUMPUS";


// Method: ReflexWumpus Constructor (public)
//
// *Purpose   : initializes the new instance
//
// *Parameters: name (String) - name of the new instance
//              ammo (int) - ammunition of the agent
//              size (Dimension) - dimension of the agent-environment
//              imgref (String) - default reference to an imagemap
//              obs (ImageObserver) - observer for the image
//
// *Return    : none
//
// *Variables :  only through super
//
  public ReflexWumpus( String name, int ammo, Dimension size )
  {
    super( name, TYPE_AGENT_WUMPUS_REFLEX, ammo, null, null );
  }

  public ReflexWumpus( String name, int ammo, Dimension size, String resource )
  {
    super( name, TYPE_AGENT_WUMPUS_REFLEX, ammo, resource, null );
  }

  public ReflexWumpus( String name, int ammo, Dimension size, 
    ImageObserver observer )
  {
    super( name, TYPE_AGENT_WUMPUS_REFLEX, ammo, null, observer );
  }

  public ReflexWumpus( String name, int ammo, Dimension size, 
    String resource, ImageObserver observer )
  {
    super( name, TYPE_AGENT_WUMPUS_REFLEX, ammo, resource, observer );
  }// End of ReflexWumpus-Constructors


// Method: receivePercepts (public)
//
// *Purpose   : processes the percepts and develops a response via -> reason
//
// *Parameters: percepts (Vector) - percepts to be processed by agentname of the instance
//
// *Return    : none
//
// *Variables : none
//
  public void receivePercepts( Vector percepts )//where the agent receives his percepts
  {
    super.receivePercepts( percepts );
    reason();
  }// End of receivePercepts


// Method: reason (protected)
//
// *Purpose   : generates a random Action and adds it to the agents actions
//
// *Parameters: none
//
// *Return    : none
//
// *Variables : actions (Action, class) - holds the actions of the agent instance
//
  protected void reason()
  {
    actions.removeAllElements();//clear last round action
    //this agent is 'reflexive'; it acts solely on the percepts it has just
    //received
	
    boolean gold = false;
    boolean stench = false;
    boolean breeze = false;
    boolean bump = false;
    boolean scream = false;
    boolean exit = false;
    Object obj;
    String toprocess;

    // extract percepts from the percepts - vector
    Enumeration newpercepts = percepts.elements();
    while( newpercepts.hasMoreElements() == true )
    {
      obj = newpercepts.nextElement();
      if ( obj instanceof Percept)
        toprocess = (( Percept ) obj).getName();
      else
        toprocess = null;

      if (toprocess == null)
      {
      }
      else if( toprocess.equalsIgnoreCase( PERCEPT_STR_GLITTER ) )
        gold = true;
      else if( toprocess.equalsIgnoreCase( PERCEPT_STR_STENCH ) )
        stench = true;
      else if( toprocess.equalsIgnoreCase( PERCEPT_STR_SCREAM ) )
        scream = true;
      else if( toprocess.equalsIgnoreCase( PERCEPT_STR_BREEZE ) )
        breeze = true;
      else if( toprocess.equalsIgnoreCase( PERCEPT_STR_BUMP ) )
        bump = true;
    }

    // react to percepts
    if( gold == true ) // always get the gold
    {
      actions.addElement( ( Object ) ( action = new Action( ACTION_STR_GRAB ) ) );
      updateHistory( ( Object ) action );
      return;
    }

    if( ( stench == true ) && ( scream == false ) ) // always shoot in any direction if arrows are available
    {
      if( this.arrowsLeft() == true )
      {
        rand = ( int )java.lang.Math.abs( ( java.lang.Math.random() * 4 ) );
        switch( rand )
        {
          case 0:
            action = new Action( ACTION_STR_SHOOT_UP );
          break;
 
          case 1:
            action = new Action( ACTION_STR_SHOOT_DOWN );
          break;
 
          case 2:
            action = new Action( ACTION_STR_SHOOT_RIGHT );
          break;

          case 3:
            action = new Action( ACTION_STR_SHOOT_LEFT );
          break;
        }
        actions.addElement( ( Object ) action );
        updateHistory( ( Object ) action );
        return;
      }
    }

    // all other percepts do not help this agent much
    // in this case he just makes a random move
    rand = ( int )java.lang.Math.abs( ( java.lang.Math.random() * 5 ) );
    switch( rand )
    {
      case 0:
        action = new Action( ACTION_STR_UP );
      break;
     
      case 1:
        action = new Action( ACTION_STR_DOWN );
      break;
     
      case 2:
        action = new Action( ACTION_STR_LEFT );
      break;
     
      case 3:
        action = new Action( ACTION_STR_RIGHT );
      break;
     
      case 4:
        action = new Action( ACTION_STR_CLIMB );
      break;
    }

    actions.addElement( ( Object ) action );
    updateHistory( ( Object ) action );
  }
}
