// file:        ReflexWumpus1.java
// source file: ReflexWumpus1.java
// object file: ReflexWumpus1.class
//
//
// included objects:
//  ReflexWumpus1 (is a Backup for ReflexWumpus.java from 7.04.97)
//    Desc: implements a WumpusEnvironment agent based on reflexive
//          movement logic (only acting randomly if there are no reflexis)
//
//
// history:
//   11.03.97 - first version finished by Chris
//
//   
//   11.03.97 - development off. continued by Roman: small changes to 
//              displaying
//   12.03.97 - new extensions for displaying images (as per protocol 
//              11.03.97) - Roman
//   22.03.97 - revised documentation - Roman
//
//   30.03.97 - added history bookeeping
//
//
// critical problems: none
//
// known problems:
//   * objects have to have a minimal size -> small objects have problems displaying
//     themselfs
//

import java.awt.*;
import java.awt.image.ImageObserver;
import java.util.*;


class ReflexWumpus1 extends WumpusAgent
{
  // private static final boolean DEBUG = true;

  private int rand;
  private Action action;  

  private static final String TYPE_AGENT_WUMPUS_REFLEX = "RELEXWUMPUS";


// Method: ReflexWumpus Constructor (public)
//
// *Purpose   : initializes the new instance
//
// *Parameters: name (String) - name of the new instance
//              ammo (int) - ammunition of the agent
//              size (Dimension) - dimension of the agent-environment
//              imgref (String) - default reference to an imagemap
//              obs (ImageObserver) - observer for the image
//
// *Return    : none
//
// *Variables :  only through super
//
  public ReflexWumpus1( String name, int ammo, Dimension size )
  {
    super( name, TYPE_AGENT_WUMPUS_REFLEX, ammo, null, null );
    // ReflexWumpus agent is not interested in its environment
  }

  public ReflexWumpus1( String name, int ammo, Dimension size, 
    String resource )
  {
    super( name, TYPE_AGENT_WUMPUS_REFLEX, ammo, resource, null );
    // ReflexWumpus agent is not interested in its environment

  }// End of ReflexWumpus-Constructors


// Method: receivePercepts (public)
//
// *Purpose   : processes the percepts and develops a response via -> reason
//
// *Parameters: percepts (Vector) - percepts to be processed by agentname of the instance
//
// *Return    : none
//
// *Variables : none
//
  public void receivePercepts( Vector percepts )//where the agent receives his percepts
  {
    super.receivePercepts( percepts );
    reason();
  }// End of receivePercepts


// Method: reason (protected)
//
// *Purpose   : generates a random Action and adds it to the agents actions
//
// *Parameters: none
//
// *Return    : none
//
// *Variables : actions (Action, class) - holds the actions of the agent instance
//
  protected void reason()
  {
    actions.removeAllElements();//clear last round action
    //this agent is 'reflexive'; it acts solely on the percepts it has just
    //received
	
    boolean gold = false;
    boolean stench = false;
    boolean breeze = false;
    boolean bump = false;
    boolean scream = false;
    boolean exit = false;
    Object obj;
    String toprocess;

//    if ( DEBUG )
//      System.out.print("ReflexWumpus2 >"+getName()+"< got the following percepts:... ");

    // extract percepts from the percepts - vector
    Enumeration newpercepts = percepts.elements();
    while( newpercepts.hasMoreElements() == true )
    {
      obj = newpercepts.nextElement();
      if ( obj instanceof Percept)
        toprocess = (( Percept ) obj).getName();
      else
        toprocess = null;

//      if ( DEBUG )
//        System.out.print(">"+toprocess+"<, ");

      if (toprocess == null)
      {
      }
      else if( toprocess.equalsIgnoreCase( PERCEPT_STR_GLITTER ) )
        gold = true;
      else if( toprocess.equalsIgnoreCase( PERCEPT_STR_STENCH ) )
        stench = true;
      else if( toprocess.equalsIgnoreCase( PERCEPT_STR_SCREAM ) )
        scream = true;
      else if( toprocess.equalsIgnoreCase( PERCEPT_STR_BREEZE ) )
        breeze = true;
      else if( toprocess.equalsIgnoreCase( PERCEPT_STR_BUMP ) )
        bump = true;
    }

    // react to percepts
    if( gold == true ) // always get the gold
    {
      actions.addElement( ( Object ) ( action = new Action( ACTION_STR_GRAB ) ) );
      updateHistory( ( Object ) action );
      return;
    }

    if( ( stench == true ) && ( scream == false ) ) // always shoot in any direction if arrows are available
    {
      if( this.arrowsLeft() == true )
      {
        action = new Action( WumpusLib.randomShot() );
        actions.addElement( ( Object ) action );
        updateHistory( ( Object ) action );
        return;
      }
    }

    // all other percepts do not help this agent much
    // in this case he just makes a random move

    action = new Action( WumpusLib.randomMove() );
    actions.addElement( ( Object ) action );
    updateHistory( ( Object ) action );
  }// End of reason


}// End of ReflexWumpus
