// file:        ReflexWumpus2.java
// source file: ReflexWumpus2.java
// object file: ReflexWumpus2.class
//
//
// included objects:
//  ReflexWumpus2
//    Desc: extends the ReflexWumpus with Bump & Breeze percept reaction
//          shooting arrow will now also depend on the last shots taken 
//          on the current field (back to the last move [valid or invalid]) 
//
//          Therefore this agent is an reflexive type with one memory cell
//          for the last move and memory for all the current javelin throws
//
// history:
//   06.04.97 - first version finished by Roman
//              changed reason -> randomMove/Shot
//              added lastMove/Shot, negateMove, nextPossibleMove/Shot, 
//                    excludeAction
//   
// critical problems: none
//
// known problems:
//

import java.awt.*;
import java.awt.image.ImageObserver;
import java.util.*;


class ReflexWumpus2 extends WumpusAgent
{
  // private static final boolean DEBUG = false;

  protected boolean scream = false;
  protected WumpusHistoryMap hist_;

  private static final String TYPE_AGENT_WUMPUS_REFLEX = "RELEX2WUMPUS";


// Method: ReflexWumpus Constructor (public)
//
// *Purpose   : initializes the new instance
//
// *Parameters: name (String) - name of the new instance
//              ammo (int) - ammunition of the agent
//              size (Dimension) - dimension of the agent-environment
//              imgref (String) - default reference to an imagemap
//              obs (ImageObserver) - observer for the image
//
// *Return    : none
//
// *Variables : only through super
//
// *Note      : ReflexWumpus agent is not interested in its environment
//              => Dimension
//
  public ReflexWumpus2( String name, int ammo, Dimension size )
  {
    super( name, TYPE_AGENT_WUMPUS_REFLEX, ammo, null, null );
    hist_ = new WumpusHistoryMap();
  }

  public ReflexWumpus2( String name, int ammo, Dimension size, 
                        String resource )
  {
    super( name, TYPE_AGENT_WUMPUS_REFLEX, ammo, resource, null );
    hist_ = new WumpusHistoryMap();
  }

  public ReflexWumpus2( String name, int ammo, Dimension size, 
                        ImageObserver obs )
  {
    super( name, TYPE_AGENT_WUMPUS_REFLEX, ammo, null, obs );
    hist_ = new WumpusHistoryMap();
  }

  public ReflexWumpus2( String name, int ammo, Dimension size, 
                        String resource, ImageObserver obs )
  {
    super( name, TYPE_AGENT_WUMPUS_REFLEX, ammo, resource, obs );
    hist_ = new WumpusHistoryMap();

  }// End of ReflexWumpus-Constructors



// Method: receivePercepts (public)
//
// *Purpose   : processes the percepts and develops a response via -> reason
//
// *Parameters: percepts (Vector) - percepts to be processed by agentname of the instance
//
// *Return    : none
//
// *Variables : none
//
  public void receivePercepts( Vector percepts )//where the agent receives his percepts
  {
    super.receivePercepts( percepts );
    hist_.validateHistory( percepts );

/* 
    if ( DEBUG ) 
    {
      System.out.println( getName() + ": history... " + hist_.getHistory() );
      String tmpstr = hist_.lastValidMove();
      System.out.println( getName() + ": last valid move .. " + tmpstr);

      Vector tmp= hist_.invalidMoves();
      if (tmp != null) 
      {
        int i;
        System.out.print( getName() + ": Moves unsuccessful:");
        for ( i = 0; i < tmp.size(); i++ )
          System.out.print( " >"+(String)(tmp.elementAt(i)) +"<,");
        System.out.println();
      }
    }
*/

    reason();
  }

// Method: reason (protected)
//
// *Purpose   : generates a random Action and adds it to the agents actions
//
// *Parameters: none
//
// *Return    : none
//
// *Variables : actions (Action, class) - holds the actions of the agent instance
//
  protected void reason()
  {
    Action action;
    String newaction;

    actions.removeAllElements();//clear last round action
    //this agent is 'reflexive'; it acts solely on the percepts it has just
    //received
	
    boolean gold = false;
    boolean stench = false;
    boolean breeze = false;
    boolean bump = false;
    boolean exit = false;
    Object obj;
    String toprocess;

//    if ( DEBUG )
//      System.out.print("ReflexWumpus2 >"+getName()+"< got the following percepts:... ");

    // extract percepts from the percepts - vector
    Enumeration newpercepts = percepts.elements();
    while( newpercepts.hasMoreElements() == true )
    {
      obj = newpercepts.nextElement();
      if ( obj instanceof Percept)
        toprocess = (( Percept ) obj).getName();
      else
        toprocess = null;

//      if ( DEBUG )
//        System.out.print(">"+toprocess+"<, ");

      if (toprocess == null)
      {
      }
      else if( toprocess.equalsIgnoreCase( PERCEPT_STR_GLITTER ) )
      {
        gold = true;   // Gold!... Grab, Grab, ...
      }
      else if( toprocess.equalsIgnoreCase( PERCEPT_STR_STENCH ) )
      {
        stench = true; // 'mumph'... Wumpus dead or alive - he stinks!
      }
      else if( toprocess.equalsIgnoreCase( PERCEPT_STR_SCREAM ) )
      {
        scream = true; // my archfiend is dead! - Hurray!!
      }
      else if( toprocess.equalsIgnoreCase( PERCEPT_STR_BREEZE ) )
      {
        breeze = true; // wait a pit somewhere ahead...
      }
      else if( toprocess.equalsIgnoreCase( PERCEPT_STR_BUMP ) )
      {
        bump = true;   // auch.. my nose.. I bumped my nose... auch
      }
    }

//    if ( DEBUG )
//      System.out.println();

    // react to percepts
    if( gold == true ) // always get the gold
    {
      actions.addElement( ( Object ) ( action = new Action( ACTION_STR_GRAB ) ) );
      updateHistory( action );
      hist_.updateHistory( action );
      return;
    }

    if( stench && !scream ) // I'm still unsave -> noone killed the wumpus...
    {
      if( this.arrowsLeft() == true ) // always shoot in a direction (that 
      {                               // makes sense from my point of knowledge) 
                                      // - if there are still some arrows

        // no need to check BUMP -> agent's rules excude a new move after a 'STENCH'
        // => lastMove (if any) is a VAILD move => exclude the direction
        newaction = WumpusLib.randomShot( hist_.lastShot(), hist_.lastMove() );

        if ( newaction != null ) 
        {
          action = new Action( newaction );
          actions.addElement( ( Object ) action );
          hist_.updateHistory( action );
          updateHistory( action );
          return;
        }
      }

      action = new Action( WumpusLib.negateMove( hist_.lastMove() ) );
      actions.addElement( ( Object ) action );
      hist_.updateHistory( action );
      updateHistory( action );
      return;
    }

    // all other percepts do not help this agent much
    // in this case he just makes a random move

    if ( breeze == true )
    {
//      if ( DEBUG )
//        System.out.println(" Current History: >" + hist_.getHistory());

      newaction = WumpusLib.negateMove( hist_.lastValidMove() ) ;

      if ( newaction == null )        // no movement history => a pit near 
      {                               // the start field => just FLEE
        newaction = WumpusLib.randomMove( ( String ) null );
//        if ( DEBUG )
//          System.out.println( getName() + ": no save place to go I'll try my luck" );
      }

      action = new Action( newaction );
      actions.addElement( ( Object ) action );
      hist_.updateHistory( action );
      updateHistory( action );
      return;
    }

    if ( bump == true ) 
    {
//      if ( DEBUG )
//        System.out.println(" Current History: >" + hist_.getHistory());
      newaction = WumpusLib.randomMove( hist_.lastMove() );
    }
    else // no bump => the last move (if there was one) WAS a valid move
         // => and it's save => do not go back...
      newaction = WumpusLib.randomMove( WumpusLib.negateMove( hist_.lastMove() ) );

    newaction = WumpusLib.checkClimbOut( newaction );

    if ( newaction != null )
    {
      action = new Action( newaction );
      actions.addElement( ( Object ) action );
      hist_.updateHistory( action );
      updateHistory( action );
    }
  }// End of reason

}// End of ReflexWumpus
