// file:        ReflexWumpus3.java
// source file: ReflexWumpus3.java
// object file: ReflexWumpus3.class
//
//
// included objects:
//  ReflexWumpus3
//    Desc: extends the ReflexWumpus2 with a reflexive/Memory-Agent
//          Bump & Breeze & unsuccessful climbs are processed
//
//          through the history_map it knows the and uses:
//            lastValidAction <=> Breeze
//            allShots (all directions a javelin was fired) <=> Stench
//            invalidMoves () <=> Bump
//            
//
//    Therefore this agent is an reflexive type with one memory cell last
//    valid move and all the directions it fired javelins in the current 
//    field;
//
//    The agent is very cautious => if it can kill an Wumpus it steps back
//                                  if it encounters a pit it steps back
//
//    It tries to save javelins/arrows as best as it can
//             to save movement
//             to pick up every gold it can get (the easy way)
//       moves randomly only if it can't help it
//
//    If the agent encounters a pit near the start field it just tries
//    to flee - (should th agent ever forget it's movement history this
//    can lead to a forever loop)
//
// history:
//   06.04.97 - first version finished by Roman
//              changed reason -> randomMove/Shot
//              added lastMove/Shot, negateMove, nextPossibleMove/Shot, 
//                    excludeAction
//   
// critical problems: none
//
// known problems:
//

import java.awt.*;
import java.awt.image.ImageObserver;
import java.util.*;


class ReflexWumpus3 extends WumpusAgent
{
  // private static final boolean DEBUG = false;

  protected boolean          scream      = false;
  protected WumpusHistoryMap hist_;
  protected int              no_climb_[] = {5};

  private static final String TYPE_AGENT_WUMPUS_REFLEX = "RELEX3WUMPUS";

// Method: ReflexWumpus3 Constructor (public)
//
// *Purpose   : initializes the new instance
//
// *Parameters: name (String) - name of the new instance
//              ammo (int) - ammunition of the agent
//              size (Dimension) - dimension of the agent-environment
//              imgref (String) - default reference to an imagemap
//              obs (ImageObserver) - observer for the image
//
// *Return    : none
//
// *Variables : only through super
//
// *Note      : ReflexWumpus agent is not interested in its environment
//              => Dimension
//
  public ReflexWumpus3( String name, int ammo, Dimension size )
  {
    super( name, TYPE_AGENT_WUMPUS_REFLEX, ammo, null, null );
    hist_ = new WumpusHistoryMap();
  }

  public ReflexWumpus3( String name, int ammo, Dimension size, 
                        String resource )
  {
    super( name, TYPE_AGENT_WUMPUS_REFLEX, ammo, resource, null );
    hist_ = new WumpusHistoryMap();
  }

  public ReflexWumpus3( String name, int ammo, Dimension size, 
                        ImageObserver obs )
  {
    super( name, TYPE_AGENT_WUMPUS_REFLEX, ammo, null, obs );
    hist_ = new WumpusHistoryMap();
  }

  public ReflexWumpus3( String name, int ammo, Dimension size, 
                        String resource, ImageObserver obs )
  {
    super( name, TYPE_AGENT_WUMPUS_REFLEX, ammo, resource, obs );
    hist_ = new WumpusHistoryMap();

  }// End of ReflexWumpus-Constructors



// Method: receivePercepts (public)
//
// *Purpose   : processes the percepts and develops a response via -> reason
//
// *Parameters: percepts (Vector) - percepts to be processed by agentname of the instance
//
// *Return    : none
//
// *Variables : none
//
  public void receivePercepts( Vector percepts )//where the agent receives his percepts
  {
    super.receivePercepts( percepts );
    hist_.validateHistory( percepts );

/*
    if ( DEBUG )
    {
      System.out.println( getName() + ": history... " + hist_.getHistory() );
      String tmpstr = hist_.lastValidMove();
      System.out.println( getName() + ": last valid move .. " + tmpstr );

      Vector tmp = hist_.invalidMoves();
      if ( tmp != null ) 
      {
        int i;
        System.out.print( getName() + ": Moves unsuccessful:" );
        for ( i = 0; i < tmp.size(); i++ )
          System.out.print( " >"+(String)(tmp.elementAt(i)) +"<," );
        System.out.println();
      }
    }
*/

    reason();
  }

// Method: reason (protected)
//
// *Purpose   : generates a random Action and adds it to the agents actions
//
// *Parameters: none
//
// *Return    : none
//
// *Variables : actions (Action, class) - holds the actions of the agent instance
//
  protected void reason()
  {
    Action  action;
    Object  obj;
    String  toprocess;
    String  newaction;

    boolean gold   = false;
    boolean stench = false;
    boolean breeze = false;
    boolean bump   = false;
    boolean exit   = false;

    actions.removeAllElements();//clear last round action
    //this agent is 'reflexive'; it acts solely on the percepts it has just
    //received
	
//    if ( DEBUG )
//      System.out.print("ReflexWumpus3 >"+getName()+"< got the following percepts:... ");

    // extract percepts from the percepts - vector
    Enumeration newpercepts = percepts.elements();
    while( newpercepts.hasMoreElements() == true )
    {
      obj = newpercepts.nextElement();
      if ( obj instanceof Percept)
        toprocess = (( Percept ) obj).getName();
      else
        toprocess = null;

//      if ( DEBUG )
//        System.out.print(">"+toprocess+"<, ");

      if (toprocess == null)
      {
      }
      else if( toprocess.equalsIgnoreCase( PERCEPT_STR_GLITTER ) )
      {
        gold = true;   // Gold!... Grab, Grab, ...
      }
      else if( toprocess.equalsIgnoreCase( PERCEPT_STR_STENCH ) )
      {
        stench = true; // 'mumph'... Wumpus dead or alive - he stinks!
      }
      else if( toprocess.equalsIgnoreCase( PERCEPT_STR_SCREAM ) )
      {
        scream = true; // my archfiend is dead! - Hurray!!
      }
      else if( toprocess.equalsIgnoreCase( PERCEPT_STR_BREEZE ) )
      {
        breeze = true; // wait a pit somewhere ahead...
      }
      else if( toprocess.equalsIgnoreCase( PERCEPT_STR_BUMP ) )
      {
        bump = true;   // auch.. my nose.. I bumped my nose... auch
      }
    }
//    if ( DEBUG )
//      System.out.println();

    // react to percepts
    if( gold ) // always get the gold
    {
      actions.addElement( ( Object ) ( action = new Action( ACTION_STR_GRAB ) ) );
      hist_.updateHistory( action );
      updateHistory( action );
      return;
    }

    if( stench && !scream ) // I'm still unsave -> noone killed the wumpus...
    {
      if( this.arrowsLeft() == true ) // always shoot in a direction (that 
      {                               // makes sense from my point of knowledge) 
                                      // - if there are still some arrows

        // lastValidMove() -> agent came from a Field that is SAVE
/*
        if ( DEBUG )
          System.out.println(" A Wumpus is near!!: lastShots: " +
            hist_.lastShots() + " lastValidMove: " + hist_.lastValidMove() );
*/
        newaction = WumpusLib.randomShot( hist_.lastShots(), hist_.lastValidMove() );

        if ( newaction != null ) 
        {
          action = new Action( newaction );
          actions.addElement( ( Object ) action );
          hist_.updateHistory( action );
          updateHistory( action );
          return;
        }
      }

      action = new Action( WumpusLib.negateMove( hist_.lastValidMove() ) );
      actions.addElement( ( Object ) action );
      hist_.updateHistory( action );
      updateHistory( action );
      return;
    }

    // all other percepts do not help this agent much
    // in this case he just makes a random move

    if ( breeze == true )
    {
//      if ( DEBUG )
//        System.out.println(" Current History: >" + hist_.getHistory());

      newaction = WumpusLib.negateMove( hist_.lastValidMove() ) ;

      if ( newaction == null )        // no movement history => a pit near 
      {                               // the start field => just FLEE
        newaction = ACTION_STR_CLIMB;         
//        if ( DEBUG )
//          System.out.println( getName() + ": no save place to go -> fleeing" );
      }
      
      action = new Action( newaction );
      actions.addElement( ( Object ) action );
      hist_.updateHistory( action );
      updateHistory( action );
      return;
    }

    if ( bump == true )  // bumps/climbs are processed in receive-percept
    {                    // => hist_.validateHistory( percepts );
    }

//    if ( DEBUG )
//      System.out.println(" Current History: >" + hist_.getHistory());

    newaction = WumpusLib.randomMove( 
                  hist_.invalidMoves(),
                  no_climb_,
                  WumpusLib.checkClimbOut( // was there a VALID move ?
                    WumpusLib.negateMove( hist_.lastValidMove() ) 
                  ) 
                );

    newaction = WumpusLib.checkClimbOut( newaction );

    if ( newaction != null )
    {
      action = new Action( newaction );
      actions.addElement( ( Object ) action );
      hist_.updateHistory( action );
      updateHistory( action );
    }
  }// End of reason

}// End of ReflexWumpus3
