// file:        SearchableResources.java
// source file: SearchableResources.java
// object file: SearchableResources.class
//
//
// included objects:
//  SearchableResources
//    Desc: implements an searchable reference to mulitple images 
//          references (easily extended structur)
//
//
// history:
//   12.04.97 - creation of class - Roman Valehrach
//
//
// critical problems: none
//
// known problems: none
//

import java.util.Vector;


class SearchableResources extends Resources
{
   private ImageReference def_ref_;
   private boolean        def_img_;

// Method: SearchableResources Constructor (public)
//
// *Purpose   : just calls super
//
// *Parameters: none
//
// *Return    : none
//
// *Variables : images (Vector, class) - stores the references to image 
//                                       resources
//
  SearchableResources() 
  {
    super();
    clearDefaultImageReference();
  }// End of SResources-Constructor


  protected void setImages(Vector images) 
  {
    if (images != null)
    {
      super.setImages( images );
      clearDefaultImageReference();
    }
  }

// Method: searchImageReference (public)
//
// *Purpose   : just calls super
//
// *Parameters: ref (SearchableResources) - the class-instanc to search in
//              name (String) - the name that is beeing searched for
//              ignore_case - whether the search should be case sensitive
//              break_at_first - the search is stopped on the first found
//                               image
//
// *Return    : ImageReference - the image-reference with the searched
//              name;
//
// *Variables : images (Vector, class) - stores the references to image 
//                                       resources
//              def_ref_ (ImageReference, class) - the default reference
//              def_img_ (boolean, class) - the status of the current default
//
  public static ImageReference searchImageReference(
    SearchableResources ref,
    String name,
    boolean ignore_case,
    boolean break_at_first )
  {
    ImageReference img_ref = null;
    ImageReference tmpref = null;
    String tmpstr;
    int i;
    boolean firstfound = false;

    // System.out.println("getActionImage: searching for "+ getActionIcon());

    if ( (ref instanceof Resources) &&
         (ref.images != null) && (name != null) )
    {
      for ( i= 0; i < ref.images.size() && !firstfound; i++ ) 
      {
        tmpref = (ImageReference) (ref.images.elementAt( i ));

        if (tmpref instanceof ImageReference)
        {
          if (ignore_case)
          {
            if ( ( (tmpstr= tmpref.getName()) != null ) &&
                    tmpstr.equalsIgnoreCase( name )     )
            {
              img_ref = tmpref;
              if ( break_at_first )  firstfound = true;
            }
          } else {
            if ( ( (tmpstr= tmpref.getName()) != null ) &&
                    tmpstr.equals( name )               ) 
            {
              img_ref = tmpref;
              if ( break_at_first )  firstfound = true;
            }
          }
        }
      }
    }

    if ( ( img_ref == null ) && ( ref.def_img_ == true ) )
    // nothing found and default set => use it
      return(ref.def_ref_);
    else
      return(img_ref);
  }

  public ImageReference searchImageReference( 
    String name, boolean ignore_case, boolean break_at_first )
  {
    return searchImageReference( this, name, ignore_case, break_at_first );
  }


// Method: searchCheckedImageReference (public)
//
// *Purpose   : searches for an image in the images-Vector
//
// *Parameters: ref (SearchableResources) - the class-instanc to search in
//              name (String) - the name that is beeing searched for
//              ignore_case - whether the search should be case sensitive
//              break_at_first - the search is stopped on the first found
//                               image
//
// *Return    : ImageReference - the image-reference with the searched
//                               name
//
// *Variables : images (Vector, class) - stores the references to image 
//                                       resources
//              def_ref_ (ImageReference, class) - the default reference
//              def_img_ (boolean, class) - the status of the current default
//
  public static ImageReference searchCheckedImageReference(
    SearchableResources ref,
    String name,
    boolean ignore_case,
    boolean break_at_first )
  {
    ImageReference img_ref = null;
    ImageReference tmpref = null;
    String tmpstr;
    int i;
    boolean firstfound = false;

    // System.err.println("searchChecked: searching for: "+ name);

    if ( (ref.images != null) && (name != null) )
    {
      for ( i= 0; i < ref.images.size() && !firstfound; i++ ) 
      {
        tmpref = (ImageReference) (ref.images.elementAt( i ));

        if (tmpref instanceof ImageReference)
        {
        // System.err.println("      current image #"+i+" name:>"+tmpref.getName()+"< Image:>"+tmpref.getImage()+"<");

          if (ignore_case)
          {
            if ( ( (tmpstr= tmpref.getName()) != null ) &&
                    tmpstr.equalsIgnoreCase( name )   &&
                    tmpref.checkImageReference()       )
            {
              img_ref = tmpref;
              if ( break_at_first )  firstfound = true;
              // System.err.println("Found one >"+name+"<-image at position "+i+"!");
            }
          } else {
            if ( ( (tmpstr= tmpref.getName()) != null ) &&
                    tmpstr.equals( name )               &&
                    tmpref.checkImageReference()       )
            {
              // System.err.println("Found one >"+name+"<-image at position "+i+"!");
              img_ref = tmpref;
              if ( break_at_first )  firstfound = true;
            }
          }
        }
      }
    }
    else
    {
      // System.err.println("getActionImage: no image >"+ref.images+"< or action >"+name+"<!");
    }

    if ( ( img_ref == null ) && ( ref.def_img_ == true ) )
    // nothing found and default set => use it
      return(ref.def_ref_);
    else
      return(img_ref);
  }

  public ImageReference searchCheckedImageReference(
    String name, boolean ignore_case, boolean break_at_first )
  {
    return searchCheckedImageReference( this, name, ignore_case, break_at_first );
  }


// Method: searchCheckedImageReferenceNoDefault (public)
//
// *Purpose   : equal to search[Checked]ImageReference 
//              - but it will not  use a default if (no/no valid) image 
//                is found 
//              - this does NOT clear/overwrite the current Default !
//
// *Parameters: ref (SearchableResources) - the class-instanc to search in
//              name (String) - the name that is beeing searched for
//              ignore_case - whether the search should be case sensitive
//              break_at_first - the search is stopped on the first found
//                               image
//
// *Return    : ImageReference - the image-reference with the searched
//                               name
//
// *Variables : images (Vector, class) - stores the references to image 
//                                       resources
//              def_ref_ (ImageReference, class) - the default reference
//              def_img_ (boolean, class) - the status of the current default
//
  public static ImageReference searchImageReferenceNoDefault(
    SearchableResources ref,
    String name,
    boolean ignore_case,
    boolean break_at_first )
  {
    ImageReference last_ref = ref.def_ref_;
    boolean        last_img = ref.def_img_;

    ref.def_ref_ = null;
    ref.def_img_ = false;
    ImageReference tmp = searchImageReferenceNoDefault(
                           ref, 
                           name, 
                           ignore_case, 
                           break_at_first );
    
    ref.def_ref_ = last_ref;
    ref.def_img_ = last_img;
    return tmp;
  }

  public ImageReference searchImageReferenceNoDefault(
    String name, boolean ignore_case,  boolean break_at_first )
  {
    return searchImageReferenceNoDefault( 
      this, name, ignore_case, break_at_first );
  }

  public static ImageReference searchCheckedImageReferenceNoDefault(
    SearchableResources ref,
    String name,
    boolean ignore_case,
    boolean break_at_first )
  {
    ImageReference last_ref = ref.def_ref_;
    boolean        last_img = ref.def_img_;

    ref.def_ref_ = null;
    ref.def_img_ = false;
    ImageReference tmp = searchCheckedImageReferenceNoDefault(
                           ref, 
                           name, 
                           ignore_case, 
                           break_at_first );
    
    ref.def_ref_ = last_ref;
    ref.def_img_ = last_img;
    return tmp;
  }

  public ImageReference searchCheckedImageReferenceNoDefault(
    String name, boolean ignore_case, boolean break_at_first )
  {
    return searchCheckedImageReferenceNoDefault(
             this, name, ignore_case, break_at_first );
  }


// Method: setDefaultImageReference(public)
//
// *Purpose   : searches for the given default image and memorizes it...
//
// *Parameters: def_name (String) - a default if the searched one is not
//                                  found
//              ignore_case - whether the search should be case sensitive
//
// *Return    : boolean - whether setting the default was successfull or not
//
// *Variables : def_ref (ImageReference) - the default reference
//              def_img (boolean) - the status of the current default
//
  public static boolean setDefaultImageReference(
    SearchableResources ref,
    String def_name,
    boolean ignore_case )
  {
    ImageReference tmp_ref = null;

    ref.def_ref_ = null;  // always delete the current default...
    ref.def_img_ = false;

    if ( (def_name != null) && // do not even search on null
         ( (tmp_ref = searchImageReference(ref, def_name, ignore_case, true) ) != null ) )
    {
      ref.def_ref_ = tmp_ref;
      ref.def_img_ = true;
    }

    return ref.def_img_;
  }

  public boolean setDefaultImageReference(
    String def_name, boolean ignore_case )
  {
    return setDefaultImageReference( this, def_name, ignore_case );
  }


  public boolean setDefaultImageReference( String def_name )
  {
    return setDefaultImageReference( this, def_name, true );
  }


// Method: setCheckedDefaultImageReferenceSave(public)
//
// *Purpose   : does not overwrite an already set default image reference
//
// *Parameters: name (String) - the name that is beeing searched for
//              defaultName (String) - a default if the searched one is not
//                                     found
//
// *Return    : boolean - whether setting the default was successfull or not
//
// *Variables : def_ref_ (ImageReference) - the default reference
//              def_img_ (boolean) - the status of the current default
//
  public static boolean setCheckedDefaultImageReference(
    SearchableResources ref, 
    String def_name,
    boolean ignore_case )
  {
    ImageReference tmp_ref = null;

    // System.err.println("setting Default: "+ def_name);
    if ( (def_name != null) && // do not even search on null
         ( (tmp_ref = searchCheckedImageReference(ref, def_name, ignore_case, true) ) != null ) )
    {
      ref.def_ref_ = tmp_ref;
      ref.def_img_ = true;
    }

    return ref.def_img_;
  }

  public boolean setCheckedDefaultImageReference ( 
    String def_name, boolean ignore_case )
  {
    return setCheckedDefaultImageReference ( this, def_name, ignore_case );
  }

  public boolean setCheckedDefaultImageReference ( String def_name )
  {
    return setCheckedDefaultImageReference ( this, def_name, true );
  }


// Method: clearDefaultImageReference(public)
//
// *Purpose   : clears the default ImageReference
//
// *Parameters: ref (SearchableResources) - the ref to the instance
//                                          where the default shall be set
//
// *Return    : none
//
// *Variables : def_ref (ImageReference) - the default reference
//              def_img (boolean) - the status of the current default
//
  public static void clearDefaultImageReference( SearchableResources ref )
  {
    ref.def_ref_ = null;
    ref.def_img_ = false;
  } 
 
  public void clearDefaultImageReference()
  {
    this.def_ref_ = null;
    this.def_img_ = false;
  }  


// Method: usingDefaultImageReference(public)
//
// *Purpose   : returns whether the default ImageReference is set
//
// *Parameters: ref (SearchableResources) - the ref to the instance
//                                          where the default shall be set
//
// *Return    : none
//
// *Variables : def_ref (ImageReference) - the default reference
//              def_img (boolean) - the status of the current default
//
  public static boolean usingDefaultImageReference( 
    SearchableResources ref )
  {
    if (ref instanceof SearchableResources)
      return ref.def_img_;
    else
      return false;
  }  

  public boolean usingDefaultImageReference()
  {
    return this.def_img_;
  }

  public static ImageReference getDefaultImageReference( 
    SearchableResources ref )
  {
    if (ref instanceof SearchableResources)
      return ref.def_ref_;
    else
      return null;
  }  

  public ImageReference getDefaultImageReference()
  {
    return this.def_ref_;
  }


}// End of class SearchableResources
