// file:        StdImageReference.java
// source file: StdImageReference.java
// object file: StdImageReference.class
//
//
// included objects:
//  StdImageReference
//    Desc: implements an drawable reference to a single image (=>standard)
//
//
// history:
//   14.04.97 - first version finished by Roman Valehrach
//
//
// critical problems: none
//
// known problems: none
//

import java.awt.Image;
import java.awt.image.ImageObserver;
import java.awt.Graphics;
import java.awt.Rectangle;
import java.awt.Color;
import java.awt.Point;
import java.awt.Dimension;

import java.awt.Font;
import java.awt.FontMetrics;

class StdImageReference extends ImageReference implements DrawableReferenceIfc
{
  // private final static boolean DEBUG = false;
   

// Method: StdImageReference constructor (public)
//
// *Purpose   : initializes the references to name, image and observer
//              with proper values
//
// *Parameters: picname (String) - new name for the pic
//              pic (Image) - new image
//              obsrvr (ImageObserver) - new observer for the pic
//
// *Return    :
//
// *Variables : string (String, class) - stores the new name
//              image (Image, class) - stores the new image
//              observer (ImageObserver, class) - stores the new observer

  public StdImageReference( String name, Image image, ImageObserver obsrvr ) 
  {
    super( name, image, obsrvr);
  }

  public StdImageReference( String name, Image image ) 
  {
    super( name, image );

  }// End of ImageReference-Constructors


// Method: drawImage (public)
//
// *Purpose   : draws the current image to the graphics device 
//              - depending on the existence of a valid image
//
// *Parameters: g (Graphics) - the graphics to draw the objects to
//              area (Rectangle) - the position and dimensions of the
//                                 object on screen
//              drawColor (Color) - the drawingcolor to use
//
// *Return    : whether the drawing of the Image was successfull or not
//
// *Variables : myimage (ImageReference, class) - pointer to a resource
//
// *add. Note : does not check graphics - must me none-NULL
//
  public boolean drawImage(
    Graphics g, 
    Rectangle area, 
    Color background )
  {
    Image         img = getImage();
    ImageObserver obs = getObserver();

    boolean sized;

//    if ( g != null ) return false;

    if ( image_valid == true || (image_valid = checkImage( img, obs )) )
    {
      if ( (area.width == 0) && (area.height == 0) ) 
        sized = false;
      else 
        sized = true;

      if ( background != null )
      {
        if ( sized )
          g.drawImage( img,
                       area.x,     area.y,
                       area.width, area.height,
                       background,
                       obs );
        else
          g.drawImage( img,
                       area.x,     area.y,
                       background,
                       obs );
      } else {
        if ( sized )
          g.drawImage( img,
                       area.x,     area.y,
                       area.width, area.height,
                       obs );
        else
          g.drawImage( img,
                       area.x,     area.y,
                       obs );
      }
      return true;
    }

    return false;
  }

  public boolean drawImage(
    Graphics g, 
    Rectangle area )
  {
    return drawImage( g, area, null );
  }

// Method: drawName (public)
//
// *Purpose   : draws the current image's name to the graphics device 
//              - uses clipping to fit to drawArea
//
// *Parameters: g (Graphics) - the graphics to draw the objects to
//              area (Rectangle) - the position and dimensions of the
//                                 object on screen
//              drawColor (Color) - the drawingcolor to use
//
// *Return    : whether the drawing of the Image was successfull or not
//
// *Variables : myimage (ImageReference, class) - pointer to a resource
//
// *add. Note : does not check graphics - must me none-NULL
//
  public boolean drawName( Graphics g, Rectangle area,  Color drawColor )
  {
//    if ( g != null ) return false;

    if ( drawColor != null )
      g.setColor( drawColor );

    return drawString(g, getName(), area);
  }

  public boolean drawName( 
    Graphics g, 
    Rectangle area
  )
  {
//    if ( g != null ) return false;

    drawString(g, getName(), area);
    return true;
  }

// Method: draw (public)
//
// *Purpose   : draws the current image, it's name, or nothing on the screen
//              - uses clipping to fit to string to drawArea
//
// *Parameters: g (Graphics) - the graphics to draw the objects to
//              area (Rectangle) - the position and dimensions of the
//                                 object on screen
//              drawColor (Color) - the drawingcolor to use
//              background(Color) - the background-color for the image
//
// *Return    : whether the drawing of the Image was successfull or not
//
// *Variables : none
//
  public boolean draw( Graphics g, Rectangle area )
  {
    if ( g == null ) return false; // the only check

    if ( drawImage(g, area) ) return true;
    //if ( drawName (g, area) ) return true;

    return false;
  }

  public boolean draw( Graphics g, Rectangle area, Color drawColor, 
    Color background )
  {
    if ( g == null ) return false; // the only check

    if ( drawImage(g, area, background) ) return true;
    //if ( drawName (g, area, drawColor ) ) return true;

    return false;
  }


// Method: drawString (public, static)
//
// *Purpose   : draws the current image's name to the graphics device 
//              - uses clipping to fit to drawArea
//
// *Parameters: g (Graphics) - the graphics to draw the objects to
//              area (Rectangle) - the position and dimensions of the
//                                 object on screen
//              drawColor (Color) - the drawingcolor to use
//
// *Return    : whether the drawing of the Image was successfull or not
//
// *Variables : myimage (ImageReference, class) - pointer to a resource
//
// *add. Note : does not check graphics - must me none-NULL
//
  public static boolean drawString( Graphics g, String name, 
    Rectangle area, Color drawColor )
  {
    if (drawColor != null)
      g.setColor( drawColor);

    return drawString (g, name, area);
  }

  public static boolean drawString( 
    Graphics g, 
    String name,
    Rectangle area
  )
  {
//    if ( g != null ) return false;

    Dimension clipdim = new Dimension( area.width, area.height );

    String clipstr= clipString( g, name, clipdim ); 
    Point  offset = new Point( 0, 0 );

    if (clipstr == null) return false;

    try 
    {
      if (area.width != clipdim.width )
        offset.x = ( area.width - clipdim.width )/2;
      else
        offset.x = 0;

      if (area.height != clipdim.height )
        offset.y = area.height - (( area.height - clipdim.height )/2);
      else
        offset.y = area.height; // full down
    }
    catch ( ArithmeticException e )
    {
      System.err.println("drawString() - Error in Calculation: "+ e.getMessage());
    }

    g.drawString( clipstr, area.x + offset.x, area.y + offset.y );
    return true;
  }


// Method: clipString() (public)
//
// *Purpose   : this method is for cutting a string of that is too 
//              long for a given rectangle - with the current font-set
//              - addepts Dimension to clipped String (= the size of the
//                string to draw)
//
// *Parameters: g (Graphics) - the graphics to draw the objects to
//              instr (String) - the string to be clipped
//              area (Rectangle) - the position and dimensions of the
//                                 object on screen
//
// *Return    : String - the String cut to proper size
//              through dim - the size of the draw-string (after clipping)
//
// *Variables : none
//
// *add. Notes: if the dim is to small to draw at the current font a null
//              will be returned and dim will contain all the needed
//              sizes..
//
//              if the font's size is too small the full input-strings 
//                pixel-width with the current font will be returned
//
//              if the font's is too small to draw even a single char
//                the size of the first letter in the input-string is
//                returned (as the minimum-size of the drawrect)
//
  public static String clipString( 
    Graphics g,
    String instr, 
    Dimension dim
  )
  {
    if ( ( g == null ) || ( instr == null ) ) 
    {
      dim.width = 0;
      dim.height= 0;
      return null;
    }


    Font        fnt     = g.getFont();
    FontMetrics met     = g.getFontMetrics();
    Dimension   retdim  = new Dimension( 0, 0);
    String      outstr  = instr;
    int         strlen  = outstr.length()-1;

    retdim.height = fnt.getSize();
    retdim.width  = met.stringWidth(outstr);

    if ( retdim.height > dim.height ) // is the font-height larger than allowed ?
    {
      dim.width = retdim.width;
      dim.height= retdim.height;
      return null;                  // => return - NOT POSSIBLE with this Font
    }

//    if ( DEBUG ) 
//      System.out.println("In-String: >"+instr+"<...");

    while ( (strlen > 0) && (retdim.width > dim.width)  )
    {
      outstr = instr.substring( 0, strlen );
      // retdim.width = met.stringWidth( outstr );
/*
      if ( DEBUG ) 
        System.out.println( "   current Out-String: >"+outstr+
        "<  current char: " + instr.charAt(strlen) +" Len: #"+ strlen );
*/
      retdim.width -= met.charWidth( instr.charAt(strlen) ); // just sub the cut char from the last value..
      strlen--;
    }

    dim.width = retdim.width;       // return the current values for
    dim.height= retdim.height;      // analysis, even if the current

    if ( retdim.width > dim.width ) // width is the still larger than allowed ?
    {
      return null;                  // => return - NOT POSSIBLE with this Font
    } else {
      return outstr;
    }

  }


// Method: drawLine (public, static)
//
// *Purpose   : draws a cross on screen - if the given Area is too small
//              nothing will happen
//
// *Parameters: g (Graphics) - the graphics to draw the objects to
//              x (Point) - defines the start-point
//              y (Point) - defines the end-point
//              linecolor (Color) - the color of the cross, 
//              strength (int) - the line-strength of the cross 
//                               (current min is 1,  no max )
//
// *Return    : none
//
// *Variables : none
//
  public static void drawLine( Graphics g, 
    Point start, Point end, 
    Color linecolor, int strength ) 
  {
    if ( strength < 1 ) return;

    if ( linecolor instanceof Color )
      g.setColor( linecolor );

    g.drawLine( start.x, start.y, end.x, end.y );

    int i=1, y=0;

    while ( (y=(2*i)-1) < strength )
    {
      g.drawLine( start.x+i, start.y, end.x, end.y-i );

      if ( (y+1) < strength )
      {
        g.drawLine( start.x, start.y+i, end.x-i, end.y );
      }
      i++;
    }
  }


// Method: drawCross (public, static)
//
// *Purpose   : draws a cross on screen - if the given Area is too small
//              nothing will happen
//
// *Parameters: g (Graphics) - the graphics to draw the objects to
//              drawArea (Rectangle) - defines the extends of the cross
//              crosscolor (Color) - the color of the cross, 
//              strength (int) - the line-strength of the cross 
//                               (current min is 1,  no max exept the fields size)
//
// *Return    : none
//
// *Variables : none
//
  public static void drawCross( Graphics g, Rectangle drawArea, 
    Color crosscolor, int strength ) 
  {
    if ( crosscolor instanceof Color )
      g.setColor( crosscolor );

    if ( (drawArea.width  < (strength+3)) || 
         (drawArea.height < (strength+3)) ) // this is pointless -> skip draw
      return; 
 
    g.drawLine( drawArea.x, drawArea.y, drawArea.x+drawArea.width-1, drawArea.y+drawArea.height-1 );
    g.drawLine( drawArea.x+drawArea.width-1, drawArea.y, drawArea.x, drawArea.y+drawArea.height-1 );

    int i=1;

    while ( (2*i)-1 < strength )
    {
      // for better visibility 2-pixel lines - shift both lines down by one point
      g.drawLine( drawArea.x, drawArea.y +i, drawArea.x+drawArea.width-1 -i, drawArea.y+drawArea.height-1 );
      g.drawLine( drawArea.x+drawArea.width-1, drawArea.y +i, drawArea.x +i, drawArea.y+drawArea.height-1 );

      if ( (2*i) < strength )
      {
      // for better visibility 3-pixel lines - shift both lines up by one point
      //  this is quite fatal for small image sizes
        g.drawLine( drawArea.x +i, drawArea.y, drawArea.x+drawArea.width-1, drawArea.y+drawArea.height-1 -i );
        g.drawLine( drawArea.x+drawArea.width-1 -i, drawArea.y, drawArea.x, drawArea.y+drawArea.height-1 -i );
      }
      i++;
    }
  }


}// End of class StdImageReference
