// file:        StoreList.java
// source file: StoreList.java
// object file: StoreList.class
//
//
// included objects:
//  StoreList
//    Desc: implements a double-node List (forward/backward) with sorted 
//          insert... and a list (of Strings)
//          payload (sort-base) ... int
//
//
// history:
//   24.04.97 - first version finished by Roman Valehrach
//
//
// critical problems: none
//
// known problems: none
//




class StoreList extends Object
{
  protected int       data;
  protected int       counter;
  protected LinList   list;       // a list of object for the current data
  protected StoreList left;     // ^= forward
  protected StoreList right;    // ^= backward

  public StoreList() 
  {
    this.data    = -1;
    this.counter = 0;
    this.list = null;
    this.right   = null;
    this.left    = null;
  }
      
  public StoreList( int newData, Object storeData )   // initialize a new Node with payload
  {
    this.data    = newData;
    this.counter = 1;
    if (storeData != null) {
      this.list = new LinList( storeData );
    }
    this.right   = null;
    this.left    = null;
  }

  public int getData()             // get the payload object
  {
    return this.data;
  }

  public int getCounter()             // get the payload object
  {
    return this.counter;
  }

  public LinList getList()
  {
    return list;
  }

  public int exchangeData (int newData) 
  {
    int temp;
     
    temp = this.data;
    this.data = newData;
      
    return temp;
  }

  public StoreList getLeft()                 // get the left leafe 
  {
    return this.left;
  }

  public StoreList getRight()                // now it's the right
  {
    return this.right;
  }

  public StoreList addLeftNode(StoreList newNode)
  {
    if (this.left != null) {
      newNode.addLeftNode(this.left);  // to insert nodes that may already have subnodes
      this.left = newNode;
    }
    else {
      this.left = newNode;
    }
   
    this.left.right = this;
    return this.left;
  }          

  public StoreList addRightNode( StoreList newNode ) 
  {
    if (this.right != null) {
      newNode.addRightNode( this.right ); // to insert nodes that may already have subnodes
      this.right = newNode;
    }
    else {
      this.right = newNode;
    }
   
    this.right.left = this;
    return this.right;
  }          

  public StoreList unlinkNode() {
    if (this.left != null)
      this.left.right = this.right;
    if (this.right != null)
      this.right.left = this.left;

    return this.right;
  }

  public StoreList addLeft( int toAdd, Object storeData ) 
  {
    StoreList temp = new StoreList( toAdd, storeData );

    addLeftNode(temp);
    return this.left;
  }

  public StoreList addRight( int toAdd, Object storeData )
  {
    StoreList temp = new StoreList( toAdd, storeData );

    addRightNode(temp);
    return this.right;
  }

  public StoreList exists( int extObj )   // again no check for doubled referenzes
  {
    StoreList temp;

    if (this.data == extObj) {
      return this;
    }
    else if (extObj < this.data) {
      if (this.left != null) { 
        return this.left.exists( extObj );
      }  
    }
    else {
      if (this.right != null) {
        return this.right.exists( extObj );
      }  
    }  

    return null;
  }

/*
  public StoreList checkLeft( int extObj ) 
  {
    if (this.data == extObj) {
      return this;
    } 
    else {
      if (this.left != null) {
        return this.left.checkLeft(extObj);
      }
    }
    return null;
  }

  public StoreList checkRight( int extObj ) 
  {
    if (this.data == extObj) {
      return this;
    } 
    else {
      if (this.right != null) {
        return this.right.checkRight(extObj);
      }
    }
    return null;
  }
*/

  public StoreList insert( int newData )
  {
    return this.insert( newData, null );
  }

  public StoreList insert( int newData, Object storeData )
  {
    StoreList temp;

    if (this.data == newData) {
      this.counter++;
      if (storeData != null) {
        if (list == null)
          this.list = new LinList( storeData );
        else
          this.list.add( storeData );
      }
      return this;
    }
    else if (newData < this.data) { 
      if ( (this.left != null)       &&
           (newData < this.left.data) ) {
        return this.left.insert( newData, storeData );
      }
      else {
        return this.addLeft( newData, storeData );
      }  
    } 
    else {
      if ( (this.right != null)       &&
           (newData > this.right.data) ) {
        return this.right.insert( newData, storeData );
      }
      else {
        return this.addRight( newData, storeData );
      }  
    }  
  }

  public StoreList firstElement() // no recursion to save machine stack
  {
    StoreList tmp = this;
    
    while (tmp.left != null)
    {
      tmp= tmp.left;
    }
    return tmp;
  }

  public StoreList lastElement()  // no recursion to save machine stack
  {
    StoreList tmp = this;
    
    while (tmp.right != null)
    {
      tmp= tmp.right;
    }
    return tmp;
  }

  public StoreList nextElement() // == getRight
  {
    return this.right;
  }

  public void printList()  // no recursion to save machine stack
  {
    System.out.println( "Sorted List: " );
    StoreList tmp = this;
    LinList tmplst;

    while ( tmp != null)
    {
      System.out.println( "  " + tmp.getData() + " (counted: " + getCounter() + ")");
      tmplst = tmp.getList();
      while (tmplst != null) {
        System.out.println( "     " + tmplst.getData() );
        tmplst = tmplst.nextElement();
      }
      tmp = tmp.nextElement();
    }
  }


  public void printFullList()  // no recursion to save machine stack
  {
    System.out.println( "Sorted List: " );
    StoreList tmp = this.firstElement();
    LinList tmplst;

    while ( tmp != null)
    {
      System.out.println( "  " + tmp.getData() + " (counted: " + getCounter() + ")");
      tmplst = tmp.getList();
      while (tmplst != null) {
        System.out.println( "     " + tmplst.getData() );
        tmplst = tmplst.nextElement();
      }
      tmp = tmp.nextElement();
    }
  }

}

// Function :
// Procedure:
// Method:  (public)
// Method:  (private)
//
// *Purpose   :
//              
//
// *Parameters:
//
// *Return    :
//
// *Variables :
//
