// file:        TestAgent.java
// source file: TestAgent.java
// object file: TestAgent.class
//
//
// included objects:
//  TestAgent
//    Desc: implements an with random movement logic
//
//
// history:
//   22.02.97 - first version finished by Chris
//   
//   05.03.97 - development off. continued by Roman:
//              random movement downsized to only 4 way movement
//   07.03.97 - added functions for image drawing - Roman
//   12.03.97 - new extensions for displaying images (as per protocol 11.03.97) - Roman
//              random movement set back to maximum number of movement-ways
//   22.03.97 - added & revised documentation - Roman
//   01.04.97 - added support for Action &  PerceptConstants
//
//
// critical problems: none
//
// known problems:
//   * objects have to have a minimal size -> small objects have problems displaying
//     themselfs
//

import java.awt.*;
import java.awt.image.ImageObserver;
import java.util.*;

class TestAgent extends WumpusAgent
{
  private int rand;
  private Action action;  

  private static final String TYPE_AGENT_WUMPUS_DUMB = "TestAgent";


//Method: TestAgent Constructor (public)
//
// *Purpose   : intializes a new instance and sets a new image (-resource)
//              for the object
//
// *Parameters: name (String) - name of the new instance
//              size (Dimension) - dimension of the agent-environment
//              imgref (String) - default reference to an imagemap
//              obs (ImageObserver) - observer for the image
//              ammo (int) - ammunition of the agent
//
// *Return    : none
//
// *Variables : only through super
//
  public TestAgent( String name, int ammo, Dimension size ) 
  {
    super( name, TYPE_AGENT_WUMPUS_DUMB, ammo );
    // Dumb agent is not interested in its environment
  }

  public TestAgent( String name, int ammo, Dimension size, String imgref )
  {
    super( name, TYPE_AGENT_WUMPUS_DUMB, ammo, imgref );
    // Dumb agent is not interested in its environment
  }

  public TestAgent( String name, int ammo, Dimension size, ImageObserver obs )
  {
    super( name, TYPE_AGENT_WUMPUS_DUMB, ammo, obs );
    // Dumb agent is not interested in its environment
  }

  public TestAgent( String name, int ammo, Dimension size, String imgref, ImageObserver obs )
  {
    super( name, TYPE_AGENT_WUMPUS_DUMB, ammo, imgref, obs );
    // Dumb agent is not interested in its environment
  }// End of TestAgent-Constructors


// Method: receivePercepts (public)
//
// *Purpose   : processes the percepts and develops a response via -> reason
//
// *Parameters: percepts (Vector) - percepts to be processed by agentname of the instance
//
// *Return    : none
//
// *Variables : none
//
  public void receivePercepts( Vector percepts )//where the agent receives his percepts
  {
    super.receivePercepts( percepts );
    reason();
  }// End of receivePercepts


// Method: reason (protected)
//
// *Purpose   : generates a rantom Action and adds it to the agents actions
//
// *Parameters: none
//
// *Return    : none
//
// *Variables : actions (Action) - holds the actions of the agent instance
//
  protected void reason()
  {
    // System.out.println("TestAgent "+getName()+": entering reasoning...");
    // System.out.println("TestAgent "+getName()+": removing prev. elements");

    actions.removeAllElements();//clear last round action
    //this agent is stupid; it forgets about its percepts and just makes
    //one random action
    // System.out.println("TestAgent "+getName()+": generating random movement");
    rand = ( int )java.lang.Math.abs( ( java.lang.Math.random() * 10 ) );
    switch( rand )
    {
      case 0:
        action = new Action( ACTION_STR_UP );
        break;
     
      case 1:
        action = new Action( ACTION_STR_DOWN );
        break;
    
      case 2:
        action = new Action( ACTION_STR_RIGHT );
        break;
     
      case 3:
        action = new Action( ACTION_STR_LEFT );
        break;
     
      case 4:
        action = new Action( ACTION_STR_CLIMB );
        break;
     
      case 5:
        action = new Action( ACTION_STR_GRAB );
        break;

      case 6:
        action = new Action( ACTION_STR_SHOOT_UP );
        break;
     
      case 7:
        action = new Action( ACTION_STR_SHOOT_DOWN );
        break;
     
      case 8:
        action = new Action( ACTION_STR_SHOOT_RIGHT );
        break;

      case 9:
        action = new Action( ACTION_STR_SHOOT_LEFT );
        break;
     
     default:
        action = new Action( ACTION_STR_GRAB );
        break;
    }// end switch

    // System.out.println("TestAgent "+getName()+": adding movement to actions");
    actions.addElement( ( Object ) action );
    updateHistory( ( Object ) action );
    // System.out.println("TestAgent "+getName()+": leaving reasoning!");
  }// End of reason


}// End of class TestAgent
