// file:        VacuumAgent.java
// source file: VacuumAgent.java
// object file: VacuumAgent.class
//
//
// included objects:
//  VacuumAgent
//    Desc: implements a Agent specific to Vacuum-World environments
//
//
// history:
//   22.02.97 - first version finished by Chris
//
//   21.04.97 - added functions from WumpusAgent
//
//
// critical problems: none
//
// known problems: none
//
//
// additional notes:
//   a consuming of the drawn ActionIcon is not trully possible because the 
//   envrionment displays a field multiple times 
//   => only consume after actionOK (Phase 1 =>Phase = 2) and 
//      receivePercepts (Phase 2 =>Phase = 0)
//

import java.awt.*;
import java.awt.image.ImageObserver;
import java.util.*;


abstract class VacuumAgent extends MxNActiveObject implements StatusIfc
{
  // private final static boolean DEBUG = false;

  private final static String TYPE_VACUUM = "VACUUM";

  private final static String DEFAULT_IMAGE = "EMPTY";
  private final static String IMAGE_BASE    = "envs.vacuum.image.agent";

  private final static String DEFAULT_COLOR_STR           = "colors.fieldobject.AGENTCOLOR";
  private final static String DEFAULT_DEATHCOLOR_STR      = "colors.fieldobject.DEATHCOLOR";

  private final static String DEFAULT_CROSSCOLOR_STR      = "colors.fieldobject.CROSSCOLOR";
  private final static String DEFAULT_DEATHCROSSCOLOR_STR = "colors.fieldobject.DEATHCROSSCOLOR";


  private boolean status; //am i stil alive or already a ghost?
  private int score;      //what's the score

  public Vector history; // store all percepts here

  public ActionIcons actionicon;
  public boolean     actionresult;
  protected Color    crosscolor_;


// --- actions - String constansts: ---------------------------------------
  static final String ACTION_STR_UP          = "UP";
  static final String ACTION_STR_DOWN        = "DOWN";
  static final String ACTION_STR_LEFT        = "LEFT";
  static final String ACTION_STR_RIGHT       = "RIGHT";
 
  static final String ACTION_STR_WAIT        = "WAIT";
 
  static final String ACTION_STR_SUCK        = "SUCK";
  static final String ACTION_STR_TURN_OFF    = "TURN_OFF";
 
 
// --- actions - binary coded constansts: ---------------------------------
  static final int ACTION_BASE  = 0x00;
  static final int ACTION_UP          = 0x01 | ACTION_BASE;
  static final int ACTION_DOWN        = 0x02 | ACTION_BASE;
  static final int ACTION_LEFT        = 0x04 | ACTION_BASE;
  static final int ACTION_RIGHT       = 0x08 | ACTION_BASE;
 
  static final int ACTION_BASE2 = 0x10;
  static final int ACTION_WAIT        = 0x00 | ACTION_BASE2;
  static final int ACTION_SUCK        = 0x04 | ACTION_BASE2;
  static final int ACTION_TURN_OFF    = 0x08 | ACTION_BASE2;
 
  static final int ACTION_BASE3 = 0x20;


// --- Percepts - String constansts: --------------------------------------
  static final String PERCEPT_STR_BUMP = "BUMP";
 
  static final String PERCEPT_STR_DUST = "DUST";
  static final String PERCEPT_STR_HOME = "HOME";
 
 
// --- Percepts - binary coded constansts: --------------------------------
  static final int PERCEPT_BASE  = 0x40;
 
  static final int BUMP    = 0x00 | PERCEPT_BASE;

  static final int PERCEPT_BASE2 = 0x80;
  static final int DUST    = 0x00 | PERCEPT_BASE2;
  static final int HOME    = 0x01 | PERCEPT_BASE2;

  static final int PERCEPT_BASE3 = 0x100;
 

// Method: VacuumAgent Constructor (public)
//
// *Purpose   : intializes a new instance and sets a new image (-resource)
//              for the object
//
// *Parameters: name (String) - name of the new instance
//              type (String) - general type of the new instance
//
//    
//              resource (String) - default reference to an imagemap
//
// *Return    : none
// *Variables : as below
//
  public VacuumAgent ( String name, String type, String resource, 
    ImageObserver observer )
  {
    super( name, type, resource, observer );
    status = true;

    if (actionicon == null)
      loadActionResources();

    history = new Vector( 15, 10 );

    crosscolor_ = ColorConstants.getColor( defaultDeathCrossColorName() );
  }// End of main VacuumAgnet-Constructors
 

/* the constructors below just call the above ones, 
   afterwards they change eventually inproperly set variables */

  public VacuumAgent ( String name, String type, String resource )
  {
    this( name, type, resource, null );
  }

  public VacuumAgent ( String name, String type, ImageObserver observer )
  {
    this( name, type, (String)null, observer );
  }

  public VacuumAgent ( String name, String type )
  {
    this( name, type, (String)null, null );

  }// End of VacuumAgnet-Constructors


// Method: baseResName (public)
//
// *Purpose   : returns the basic-Name for the defaultResName- method
//              to search in default-property-list
//              - this is the only thing that needs to be overwritten
//                with defaultResName;
//
// *Parameters: none
//
// *Return    : String - the default string to set
//
// *Variables : ret the temporary return string
//
  public String baseResName() { return IMAGE_BASE; }


// Method: sendAction (public)
//
// *Purpose   : returns the actions of an ActiveObject
//              and sets the action the agent trys to do
//
// *Parameters: none
//
// *Return    : Vector - the current actions of an ActiveObject instance
//
// *Variables : actions (Vector, class) - stores the actions of an ActiveObject
//
  public Vector sendAction() //where the agent returns his actions
  {
    Object tmpobj;
    String action = ACTION_STR_WAIT;

    // before returning set action image to the first action in the vector actions
    if ( ( actions != null ) && 
         ( actions.size() > 0 ) &&
         ( (tmpobj = actions.firstElement()) instanceof Action ) &&
         ( actionicon != null ) )
    {
        action = ((Action) tmpobj).getName();
    }

    if ( actionicon != null )
        actionicon.setAction( action );

    return super.sendAction();
  }// End of sendAction


// Method: receivePercepts (public)
//
// *Purpose   : stores the incoming percepts
//
// *Parameters: perceps (Vector) - the new percept(s) vector
//              and sets the action the agent has done
//
// *Return    : none
//
// *Variables : actions (Vector, class) - stores the new percept
//
  public void receivePercepts( Vector percepts )//where the agent receives his percepts
  {
    super.receivePercepts(percepts);

    if ( actionicon != null ) 
      actionicon.consume();

//    if ( resource instanceof DrawableResources )
//      ((DrawableResources) resource).advanceImagePhase();
  }


// Method: getScore (public)
//
// *Purpose   : returns the current score of an Agent
//
// *Parameters: none
//
// *Return    : int - the current score
//
// *Variables : score (int, class) - the agent's score
//
  public int getScore()
  {
    return score;
  }

// Method: setScore (public)
//
// *Purpose   : sets the agent's new score
//
// *Parameters: points (int) - the points to add to the score
//
// *Return    : none
//
// *Variables : score (int, class) - the agent's score
//
  public void setScore( int points )
  {
    score = points;
  }

// Method: updateScore (public)
//
// *Purpose   : adds the new score to the agents'score
//
// *Parameters: points (int) - the points to add to the score
//
// *Return    : none
//
// *Variables : score (int, class) - the agent's score
//
  public void updateScore( int points )
  {
    score += points;
  }


// Method: getStatus[Str] (public)
//
// *Purpose   : returns whether the agent is still alive or not
//
// *Parameters: none
//
// *Return    : boolean - the agent's current status
//         or : String - the agent's current status
//
// *Variables : status (boolean, class) - the agent's current status
//
  public boolean getStatus()
  {
    return status;
  }

  public String getStatusStr()
  {
 
    if( status == true )
      return "ALIVE";
    else
      return "DEAD";
  }
 
// Method: setStatus (public)
//
// *Purpose   : sets the agent to a new status
//
// *Parameters: newStatus (String) - the new status of the agent
//
//              reason (String) -  the reason for the new status
//                                 can be HOME (in case of 
//                                   the first two set the succes counter 
//                                   'score'  to -1000)
//
// *Return    : boolean - the agent's current status
//
// *Variables : status (boolean, class) - the agent's current status
//
  public void setStatus( String newStatus, String reason ) 
  {
    if( ( newStatus != null ) && 
          newStatus.equalsIgnoreCase( "DEAD" ) ) 
    {
      status = false;
      if( reason != null ) 
      {
        if ( reason.equalsIgnoreCase("EXIT") )
        {
          //actionOK(true);
        } 
        else if ( reason.equalsIgnoreCase("HOME") )
        {
          //actionOK(true);
        } 
        else
        {
          score = -1000;
        }
      }
      else
      {
        score = -1000;
      }
      //crosscolor_ = ColorConstants.getColor( defaultDeathCrossColorName() );
    }
    else
    {
      status = true;
      //crosscolor_ = ColorConstants.getColor( defaultCrossColorName() );
    }

    if ( resource instanceof DrawableResources )
      ((DrawableResources) resource).setImagePhase( status ? 0 : 1 );
  }

 
// Method: updateHistory ( protected )
//
// *Purpose   : stores every action an agent has taken
//
// *Parameters: Action ( Object ) - the latest action of the agent
//
// *Return    : nothing
//
// *Variables :  history ( Vector ) - the agent's history Vector
//
  protected void updateHistory( Object lastaction )
  {
    history.addElement( lastaction );
  }

// Method: getHistory ( public )
//
// *Purpose   : returns a string with all the agent's actions
//              parted by a newline character
//
// *Parameters: none
//
// *Return    : String - the agent's history
//
// *Variables :  history ( Vector ) - the agent's history Vector
//
  public String getHistory()
  {
    Enumeration actions;
    StringBuffer giveback = new StringBuffer();
    actions = history.elements();
    giveback.append( "My actions have been: \n" );
    while( actions.hasMoreElements() == true )
    {
      giveback.append( ( ( Action ) actions.nextElement() ).getName() );
      giveback.append( "\n" );
    }
    return giveback.toString();
  }// End of getHistory
 

// Method: defaultColorName           (public)
//         defaultDeathColorName      (public)
//         defaultCrossColorName      (public)
//         defaultDeathCrossColorName (public)
//
// *Purpose   : returns a default property identifing string for a color
//              - good for beeing overwritten by subclasses to set an other
//                default property identifing string 
//                (the string to be looked up in the system-property list
//
// *Parameters: none
//
// *Return    : String - the new default color
//
// *Variables : all the nessessary id-strings
//
  public String defaultColorName()           { return DEFAULT_COLOR_STR; }
  public String defaultDeathColorName()      { return DEFAULT_DEATHCOLOR_STR; }

  public String defaultCrossColorName()      { return DEFAULT_CROSSCOLOR_STR; }
  public String defaultDeathCrossColorName() { return DEFAULT_DEATHCROSSCOLOR_STR; }

// Method: loadResources (public)
//
// *Purpose   : loads all Resources that can be derived from the 'resname'
//              - resname & ImageObserver must be allready set if differing
//                from default
//              - if getRes() returns null or "EMPTY" the method is not
//                executed
//
// *Parameters: none
//  
// Return     : none
//
// *Variables : resloader (ResourceLoader, class) - the (images-)loader
//              resource (Resources, class ) - the current resources
//
  public void loadResources()
  {
    Image          tmpimg;
    ImageReference tmpref;
    String         fetchres = getRes();

    if ( (fetchres == null) || (fetchres.equals( "EMPTY" )) ) 
      return;
     
    super.loadResources();
    
    tmpimg = resloader.getImage( fetchres, -1, 102 );
    if (tmpimg != null) 
    {
      if (resource == null)
        resource = new DrawableResources();

      tmpref = new StdImageReference( getDisplayName(), tmpimg, getObserver() );
      resource.addImageReference( tmpref );
    }
  }

// Method: loadActionResources (public)
//
// *Purpose   : sets the current action images to the ones provided by
//              the caller
//
// *Parameters: actionimages (Resource) - the new action images
//
// *Return    : none
//
// *Variables : actionimages (ImageReference, class) - reference to a resource
//
  public void loadActionResources()
  {
    if ( this.actionicon == null )
      this.actionicon = new ActionIcons();
    if (resloader == null)
      resloader = new ResourceLoader();

    actionicon.setResources( resloader.getActionResources() );
  }

// Method: setColor (public)
//
// *Purpose   : returns the base-color of the current FieldObject
//              * if null WHITE will be the base-color
//
// *Parameters: color (String) - the new base-color
//
// *Return    : none
//
// *Variables : color_ (ColorConstants, class) - the new base-color
//
  public void setColor( String color )
  {
    super.setColor( color );
  }


// Method: actionOK (public)
//
// *Purpose   : sets the valid-state of the current action-image 
//              and changes to drawphase 2
//
// *Parameters: result (boolean) - the new action images
//
// *Return    : none
//
// *Variables : actionresult (boolean, class) - is the action (icon) indented valid ?
//              drawphase (int, class) - the action icon drawing state
//
  public void actionOK(boolean result)
  {
    actionresult = result;

    if ( actionicon != null ) 
    {
      actionicon.setValid( result );
    }
  }


// Method: draw (public)
//
// *Purpose   : draws the object on screen - depending on the existence
//              (this is done by super)
//              new: draw a cross over image/string if the agent is dead
//
// *Parameters: g (Graphics) - the graphics to draw the objects to
//              drawArea (Rectangle) - the position and dimensions of the
//                                     object on screen
//
// *Return    : none
//
// *Variables : myimage (ImageReference, class) - pointer to a resource
//              status (boolean, class) - status of the agent (true=> alive)
//
  public void draw( Graphics g, Rectangle drawArea )
  {
/*
    if ( DEBUG )
    {
      System.out.println("VacuumAgent "+getName()+": entering Draw function...");
      System.out.println("            params: drawArea.x/y= " +drawArea.x +"/"+ drawArea.y);
      System.out.println("                    drawArea.width/height= " +drawArea.width +"/"+ drawArea.height);
    }
*/
    super.draw( g, drawArea ); 

    if ( getStatusStr().equals("DEAD") ) 
      StdImageReference.drawCross( g, drawArea, crosscolor_, 2 );

    if ( actionicon != null )
    {
      actionicon.draw( g, drawArea ); // do not cross icon out... => for readability
    }

//    if ( DEBUG )
//      System.out.println("VacuumAgent >"+getName()+"<: leaving Draw function.");

  }// End of draw


// Method: reason (protected, abstract)
//
  abstract protected void reason();
 

}//End of class VacuumAgent
