// file:        Wall.java
// source file: Wall.java
// object file: Wall.class
//
//
// included objects:
//  Wall
//    Desc: implements the FieldBase object - Wall, for displaying
//          multiple wall-images
//
//
// history:
//   14.04.97 - first version finished by Roman Valehrach
//
//
// critical problems: none
//
// known problems: none
//

import java.awt.Color;
import java.awt.Graphics;
import java.awt.Rectangle;
import java.awt.image.ImageObserver;


class Wall extends FieldBaseObject implements FieldFillingIfc
{
  // private final static boolean DEBUG = false;

  // ahead defininitions:
  private final static String TYPE_WALL = "WALL";

  private final static String DEFAULT_IMAGE = "EMPTY";
  private final static String IMAGE_BASE    = "images.fields.wall";

  private final static String DEFAULT_COLOR_STR = "colors.fieldobject.WALLCOLOR";

// Method: Wall Constructor (public)
//
// *Purpose   : intializes a new instance and sets all (in advance) neccessary objects
//
// *Parameters: resource (String, optional) - name of the new instance
//              resource (String) - name of the resource for the FieldObject
//              observer (ImageObserver) - observer for the FieldObject
//
// *Return    : none
//
// *Variables : imgref (String) - default reference to an imagemap
//              obs (ImageObserver) - observer for the image
//
  public Wall( String resource, ImageObserver observer )
  {
    super( TYPE_WALL, TYPE_WALL, resource, observer );
  }

  public Wall( String resource )
  {
    super( TYPE_WALL, TYPE_WALL, resource, null );
  }

  public Wall( ImageObserver observer )
  {
    super( TYPE_WALL, TYPE_WALL, null, observer );
  }

  public Wall()
  {
    super( TYPE_WALL, TYPE_WALL, null, null );
  }

// Method: defaultColorName      (protected)
//         defaultForegroundName (protected)
//
// *Purpose   : returns a default property identifing string for a color
//              - good for beeing overwritten by subclasses to set an other
//                default property identifing string 
//                (the string to be looked up in the system-property list
//
// *Parameters: none
//
// *Return    : String - the new default color
//
// *Variables : all the nessessary id-strings
//
  public String defaultColorName()      { return DEFAULT_COLOR_STR; }


// Method: getDisplayName() (public)
//
// *Purpose   : statically returns the NULL-type string (=> no String drawing)
//
// *Parameters: none
//
// *Return    : String - null
//
// *Variables : none
//
   public String getDisplayName()
   {
     return null;  // overwrite with static for this fieldobject's purpose
   }


// Method: defaultResName (public)
//
// *Purpose   : method to select the (property) default resource
//
// *Parameters: none
//
// *Return    : String - the default string to set
//
// *Variables : ret the temporary return string
//
  public String defaultResName()
  {
    ImagePropDecoder decoder= new ImagePropDecoder( IMAGE_BASE, null, true, 1, false);
    String ret = decoder.getName();

    if ( ret == null )
    {
      ret = DEFAULT_IMAGE;
    }
    else
    {
      // System.err.println( getName() + " - colors name is >"+decoder.decodeColor()+"<");
      // setColor( decoder.decodeColor() );
    }

    setColor( ( String ) null );

    // System.err.println( getName() + " - returning resource >"+getColorStr()+"<");
    return ret;
  }


// Method: drawPrimitive (public)
//
// *Purpose   : draws a graphic primitive (a default form of displaying)
//              to the graphics device 
//
// *Parameters: g (Graphics) - the graphics to draw the objects to
//              area (Rectangle) - the position and dimensions of the
//                                 object on screen
//              foreground (Color) - the drawingcolor to use
//
// *Return    : whether the drawing of the Image was successfull or not
//
// *Variables : myimage (ImageReference, class) - pointer to a resource
//
  public boolean drawPrimitive( Graphics g, Rectangle area )
  {
    return ( drawPrimitive(g, area, getColor() ) );
  }

  public boolean drawPrimitive( 
    Graphics g, Rectangle area, Color foreground
  )
  {
//    if ( g == null ) return false;

    if ( foreground != null)
      g.setColor( foreground );

    //Primitive: a simple Box with no distance to the bounds
    g.fillRect( area.x, area.y, area.width, area.height );
    return true;
  }


// Method: fillsField (public)
//
// *Purpose   : returns whether the current object is as big a the
//              field (i.e. a Wall, Pit etc.) of a Field
//
// *Parameters: none
//
// *Return    : boolean - true when fields base
//
// *Variables : none
//
  public boolean fillsField()
  {
    return true;
  }

}// End of class Wall
