//
//			    	   WORLD
//
//  The Applet. Reads HTML Parameters, creates
//  the Environments. Controls the simulation
//  flow. Provides the user interface and event
//  handling.
//
//  Reuse: New Environments need to be 'registered'
//  in the method addEnvironment.
//
//  WARNING: This applet is lacking a lot of error checking.
//  HTML parameters are mostly parsed without error checking. So be
//  sure they are correct (e.g., number of Environments set too high,
//  missing parameters, and so on).
//  Anyone reusing or extending our engine, I just count on your common
//  (programmer) sense.
//
//
//  World was written & documented by
//  		Jrgen Knigsttter, Technical University of Vienna.
//
//  Last changes:  9th May 1997
//


import java.awt.*;
import java.applet.Applet;
import java.lang.*;
import java.util.*;

public class World extends Applet implements Runnable
  {
    int delay;			  // minimum ms per round
    boolean frozen = false;// determines if controlthread has been stopped by
					  // the user

    boolean debug = false; // if set to true, debugging info is displayed


    String worldname;      // the world's name
    Vector env;            // Vector used for storing the Environments
    int numofenv;          // Environments count

    Button [] worldbutton; // buttons Stop All/Continue & Restart
    Checkbox stepmode,stopmode; // choice Stop/Continue or Step Mode
    CheckboxGroup modegroup;    // group for above
    Panel modepanel;            // panel for above
    boolean onestep=false; // Continue/Stop button or Step button
    int stepcount;         // how often the Step button has been pressed

    Scrollbar scrollx=null,
		   scrolly=null; // scrollbars for too big or too many Environments
    boolean scrxused=false,// determines which scrollbars are used
		  scryused=false;

    Thread controlthread;  // Thread in control of the simulation

    Rectangle area;        // display area of world
    Rectangle envarea;	   // display area reserved for environments
    Rectangle envpos[];    // Environment positions on drawing space
    Dimension envsize[];   // Environments' required minimum size
    Dimension appletsize;  // Applet's size

    ResourceLoader resloader; // Ressource loader (needs to be initialized in World)


    //
    // init()
    //
    public void init()
	 {
	   String str;
	   int i;

	   double fps; // maximum frames per second. Usually overridden by
				// Environment's timing

	   showStatus("Initializing...");

	   // get applet size
	   area=new Rectangle();
	   Dimension d = size();
	   appletsize=d;
	   area.x=0;
	   area.y=0;
	   area.width=d.width;
	   area.height=d.height;
	   // set environment display area
	   envarea=new Rectangle(d);
	   envarea.x=20;
	   envarea.y=40;
	   envarea.width=d.width;
	   envarea.height=d.height;
	   envarea.width-=40;
	   envarea.height-=100;


	   // get World name
	   worldname=getParameter("NAME");
	   if(worldname==null)
		worldname="No Worldname";
	   // get maximum frames per second
	   str=getParameter("FPS");

	   if (str != null)
		{
		  try
		    {
			 fps = (Double.valueOf(str)).doubleValue();
		    }
		  catch (NumberFormatException e)
		    {
			 fps=1.0; // Default
		    }
		}
	   else
		fps=1.0;

	   // compute delay
	   delay = (int)((fps > 0) ? (1000 / fps) : 100);



	   // get Environment count
	   str=getParameter("numofenv");
	   if (str != null)
		{
		  try
		    {
			 numofenv = Integer.parseInt(str);
		    }
		  catch (NumberFormatException e)
		    {
			 System.out.println("Incorrect number of environments");
			 numofenv=1; // Default
		    }
		}
	   else
		{
		 System.out.println("No number of environments");
		 numofenv=1;  // Default
		}

	   // create environments
	   env = new Vector();

	   envsize=new Dimension[numofenv];

	   String url=(this.getCodeBase()).toString();
	   url = url.substring(0, url.lastIndexOf('/')+1);
	   resloader = new ResourceLoader( url );

	   showStatus("Initializing...");
	   for(i=0;i<numofenv;i++)
		{
		  addEnvironment(getParameter("envtype"+Integer.toString(i+1,10)),
			getParameter("envname"+Integer.toString(i+1,10)),
			divideParameter(getParameter("envparam"+Integer.toString(i+1,10))),i);
		}

	   // order environments for display
	   orderEnvironments();

	   // prepare Buttons
	   setLayout(null);   // No layout manager. Buttons have fixed sizes
					  // and are always placed at the bottom middle.
	   worldbutton=new Button[2];

	   // Stop All/Continue or Step Button
	   str=getParameter("Step");
	   if(str!=null)
		if(str.toUpperCase().equals("YES"))
		  onestep=true;
	   if(onestep)
		worldbutton[0]=new Button("Step");
	   else
		worldbutton[0]=new Button("Stop All");
	   worldbutton[0].enable();
	   add(worldbutton[0]);
	   worldbutton[0].reshape(area.width/2-160,area.height-45,80,30);

	   // Restart Button
	   worldbutton[1]=new Button(" Restart ");
	   worldbutton[1].enable();
	   add(worldbutton[1]);
	   worldbutton[1].reshape(area.width/2-60,area.height-45,80,30);

	   // Checkboxes
	   modepanel=new Panel();
	   modepanel.setLayout(null);

	   modegroup=new CheckboxGroup();
	   stopmode=new Checkbox("Continue/Stop",modegroup,!onestep);
	   stepmode=new Checkbox("Single Step",modegroup,onestep);
	   stopmode.enable();
	   stepmode.enable();
	   modepanel.add(stopmode);
	   modepanel.add(stepmode);
	   add(modepanel);
	   modepanel.reshape(area.width/2+40,area.height-50,140,40);
	   stopmode.reshape(0,0,140,20);
	   stepmode.reshape(0,20,140,20);

	   // Check fontsize of Checkboxes (Vary form system to system)
	   if(stopmode.getFontMetrics(stopmode.getFont()).getHeight()>18)
		{
		 // Resize checkbox fonts
		 stopmode.setFont(new Font(stopmode.getFont().getName(),Font.PLAIN,18));
		 stepmode.setFont(new Font(stopmode.getFont().getName(),Font.PLAIN,18));
		}

	   validate();
	   stepcount=0; // Start with no steps to do
    }

    //
    // start()
    //
    public void start()
	 {
	   if (frozen && !onestep)
		{
		  // Continue Mode. Pausing by user request. Start has not been
		  // called by user.
		}
	   else
		{
		  // start Sim
		  if (controlthread  == null)
		    {
			 controlthread  = new Thread(this);
		    }
		  controlthread.start();
		  // Recover Button
		  if(onestep)
		    worldbutton[0].setLabel("Step");
		  else
		  worldbutton[0].setLabel("Stop All");
		}
	   showStatus("Simulation running");
	 }

    //
    // stop()
    //
    public void stop()
	 {
	   if(!onestep)
		worldbutton[0].setLabel("Continue");
	   controlthread  = null;
	   showStatus("Simulation stopped");
	 }

    //
    // run()
    //
    // The big control loop.
    //
    public void run()
	 {
	   int i;
	   // Just to be nice, lower this thread's priority
	   // so it can't interfere with other processing going on.
	   Thread.currentThread().setPriority(Thread.MIN_PRIORITY);

	   showStatus("Simulation running");

	   // Remember the starting time.
	   long startTime = System.currentTimeMillis();

	   while (Thread.currentThread() == controlthread )
		{
		  if(onestep)
		    {
			if(stepcount>0)
			  {
			   doOneStep();
			   stepcount--;
			  }
		    }
		  else
		    {
			// Do One Step
			doOneStep();

			// Delay depending on how far we are behind. Many Environments
			// like the WumpusEnvironment have their own delays, thus
			// overriding the set maximum frame rate.
			startTime += delay;
			if(System.currentTimeMillis()<startTime)
			  {
			   try
				{
				 Thread.sleep(Math.max(0,
							   startTime-System.currentTimeMillis()));
				}
			   catch (InterruptedException e)
				{
				 break;
				}
			  }
		    }
		}
	  }

    //
    // doOneStep()
    //
    // Does one Step for each Environment
    //
    private void doOneStep()
	 {
	   Environment en;
	   int i;
	   for(Enumeration e = env.elements() ; e.hasMoreElements() ;)
		{
		 en=(Environment)e.nextElement();
                 // Warning: Next line may need changing if
                 // new environment states are implemented 
		 if(en.status[0]==0)  // Environment is active
		   en.doOneStep();
		}
	 }

    //
    // paint(Graphics g)
    //
    // Handles Applet resizing.
    // Paints the World using paintAll().
    //
    public void paint(Graphics g)
	 {
	  Dimension d;
	  // Check if Applet has been resized
	  d = size();
	  if((d.width!=appletsize.width) || (d.height!=appletsize.height))
	    {
		appletsize=d;
		area.x=0;
		area.y=0;
		area.width=d.width;
		area.height=d.height;
		envarea.x=20;
		envarea.y=40;
		envarea.width=d.width;
		envarea.height=d.height;
		envarea.width-=40;
		envarea.height-=100;

		orderEnvironments();

		g.clearRect(0,0,d.width-1,d.height-1);

		// BUTTONS
		// repaint Buttons: they may have been overpainted
		remove(worldbutton[0]);
		add(worldbutton[0]);
		remove(worldbutton[1]);
		add(worldbutton[1]);
		// Stop All/continue
		worldbutton[0].reshape(area.width/2-160,area.height-45,80,30);
		// Restart
		worldbutton[1].reshape(area.width/2-60,area.height-45,80,30);
		// Mode Choice
		modepanel.reshape(area.width/2+40,area.height-50,140,40);
		// repaint Buttons: they may have been overpainted
		paintComponents(this.getGraphics());
	    }
	  paintAll();
	 }

    //
    // paintAll()
    //
    // Repaints World. Calls paintAll() methods of all its Environments.
    //
    public void paintAll()
	 {
	   int i;
	   Font f,fold;
	   Graphics g=this.getGraphics();

	   // Paint the World's name
	   fold=g.getFont();
	   f=new Font("Courier",1,30);
	   g.setFont(f);
	   g.clearRect(area.x,area.y,area.width-1,area.height-1);
	   g.drawString(worldname, area.width/2-(g.getFontMetrics()).stringWidth(worldname)/2, 30);
	   g.setFont(fold);

	   // Paint a border. Does not show right and lower border on some
	   // Applet viewers.
	   g.drawRect(area.x,area.y,area.width-1,area.height-1);

	   // Call Environments' paintAll()
	   for(Enumeration e = env.elements() ; e.hasMoreElements() ;)
		((MxNEnvironment)e.nextElement()).paintAll();
	 }

    //
    // String[][] divideParameter(String param)
    //
    // Takes the big Environment parameter String (which is taken form the web
    // page by init()) and cuts the Parameters into a handy two-dimensional
    // String array.
    // Example:
    //
    // Before:   ("PAR:VAL, VAL ,VAL ; PAR : VAL;")
    //
    // After:    ("PAR") ("VAL") ("VAL") ("VAL")
    //		  ("PAR") ("VAL")
    //

    String [][] divideParameter(String param)
	 {
	  int i,x,y,startpos,pos[],xcount=0,ycount=0;
	  String finalparam[][];
	  StringBuffer temp=new StringBuffer();
	  pos=new int[400]; // Maximum amount of Values is 400!
		 // This value needs to be raised for very big Environments.


	  // Count parameters
	  for(i=0;i<param.length();i++)
	    {
		if(param.charAt(i)==';')
		  ycount++;
	    }
	  // No ';' at the end of the string
	  if(param.charAt(param.length()-1)!=';')
		  {
		   ycount++;
		   param=new String(param+';');
		  }

	  // Get all parameters
	  finalparam=new String[ycount][];
	  startpos=0;
	  if(debug) System.out.println("Starting. "+ycount+" rows.");
	  for(y=0;y<ycount;y++)
	    {
		// Count values
		xcount=0;
		i=startpos;
		while(param.charAt(i)!=';')
		  {
		   if(param.charAt(i)==',' || param.charAt(i)==':')
			{
			 pos[xcount++]=i;
			 if(debug) System.out.println(xcount+": "+i);
			}
		   i++;
		  }
		pos[xcount]=i;
		if(debug) System.out.println("\nStart cutting. xcount="+xcount);
		// Create Array
		finalparam[y]=new String[xcount+1];
		// Get name
		finalparam[y][0]=param.substring(startpos,pos[0]).trim();
		if(debug) System.out.println("Parametername: "+finalparam[y][0]);
		// Get all values
		for(x=0;x<xcount;x++)
		  {
		   finalparam[y][x+1]=param.substring(pos[x]+1,pos[x+1]).trim();
		   if(debug) System.out.println(finalparam[y][x+1]);
		  }
		// New starting position
		startpos=i+1;
	    }

	 return finalparam; // return the two-dimensional array
    }

    //
    // addEnvironment(String envtype, String envname, String[][] envparam, int i)
    //
    // envtype: 	Which Environment subclass is used.
    // envname: 	Name of the Environment.
    // envparam[][]:	Parameters for the Environment.
    //
    void addEnvironment(String envtype, String envname, String[][] envparam, int i)
	 {
	  Environment e;

	  String url=(this.getCodeBase()).toString();
	  url = url.substring(0, url.lastIndexOf('/')+1);
	  try
	    {
		// ATTENTION REUSERS: This is where new Environments need
		// to be registered. Just insert an
		// 'else if(envtype.equals("MyEnvironment")) {...}
		// before the last else.

		// WumpusEnvironment
		if(envtype.equals("WumpusEnvironment"))
		  e=new WumpusEnvironment(envname,envparam,url);

		// insert else if... here


		// Error: Unknown environment type.
		else
		  {
		   System.out.println("Unknown environment type");
		   System.exit(0);
		   e=new WumpusEnvironment(envname,envparam,url);
		  }
		// Add the new Environment to the Environment Vector.
		env.addElement((Object)e);
	    }
	  // Catch Exceptions during Environment creation.
	  catch (Exception e1)
	    {
		System.out.println("An exception occurred during Environment creation: " + e1.getMessage());
		e1.printStackTrace();
	    }
	 }

    //
    // orderEnvironments()
    //
    // This method gathers the minimum sizes of all Environments, and then
    // searches for an optimal placement upon the display area. It uses the
    // optimumsize(...) method to do so. If the display is larger than the
    // required space, all Environments are resized by the same factor.
    // If the display is too small, the Environments keep their minimum size
    // and scrollbars are used.
    // Aspect ratios are kept in any case, meaning width to height ratios of
    // 5 to 7 will stay 5 to 7, even if there is some width left unused.
    //
    void orderEnvironments()
	 {
	   int i,j;
	   double optquote;	// stores 'optimum quotient' of different placements
	   double newoptquote; // used for comparing of different placements
	   double dx,dy,d;  // used for size multiplicator

	   // initialize rectangles
	   Rectangle envtry[]=new Rectangle[numofenv];  // try placements
	   Rectangle border[]=new Rectangle[numofenv];  // used by optimumsize(.)

	   for(i=0;i<numofenv;i++)
		{
		 envtry[i]=new Rectangle();
		 border[i]=new Rectangle();
		}

	   // get minimum sizes
	   i=0;
	   if(debug) System.out.println("\nStarting Environment Placement");
	   for(Enumeration e = env.elements() ; e.hasMoreElements() ;)
		{
		 envsize[i]=((Environment)e.nextElement()).getMinSize();
		 // put some space between the environments
		 envsize[i].width+=20;
		 envsize[i++].height+=20;
		}

	   // find 'optimal' environment placement
	   // first, get starting values
	   optquote=optimumsize(numofenv,envtry,border,0,0);
	   envpos=envtry;	// starting rectangle formation
	   if(debug) System.out.println("Basic optquote="+optquote);

	   // now, counting backwards to only one environment in the starting
	   // row, placements are compared and the one with the lowest
	   // 'optquotient' (which is >=1) is used
	   for(i=1;i<numofenv;i++)
		{
		 // reset border
		 envtry=new Rectangle[numofenv];
		 border=new Rectangle[numofenv];
		 for(j=0;j<numofenv;j++)
		   {
		    envtry[j]=new Rectangle();
		    border[j]=new Rectangle();
		   }
		 newoptquote=optimumsize(numofenv-i,envtry,border,0,0);
		 if(debug) System.out.println("Optquote inside="+newoptquote);
		 if(optquote>newoptquote)
		   {
		    envpos=envtry;
		    optquote=newoptquote;
		   }
		}


	   // determine and apply size multiplier (if necessary)

	   if(xSize(envpos)<=envarea.width && ySize(envpos)<=envarea.height)
		{
		 // remove old scrollbars (if existant)
		 if(scrollx!=null)
		   remove(scrollx);
		 if(scrolly!=null)
		   remove(scrolly);

		 dx=(double)envarea.width/(double)xSize(envpos);
		 dy=(double)envarea.height/(double)ySize(envpos);
		 if(debug) System.out.println("dx="+dx+", dy="+dy);
		 if(dx<dy)
		   d=dx;
		 else
		   d=dy;

		 if(debug) System.out.println("Multiplier: "+d+"\n");

		 // now resize environment positions
		 for(i=0;i<numofenv;i++)
		   {
		    envpos[i].x*=d;
		    envpos[i].y*=d;
		    envpos[i].width-=20;
		    envpos[i].height-=20;
		    envpos[i].width*=d;
		    envpos[i].height*=d;
		   }
		}
	   // Not enough displaying room. Use Scrollbars
	   else
		{
		 if(debug) System.out.println("No Multiplier: xSize="+xSize(envpos)+", envarea.width="+envarea.width+
						", ySize="+ySize(envpos)+"envarea.height"+envarea.height+"\n");


		 // remove old scrollbars (if existant)
		 if(scrollx!=null)
		   remove(scrollx);
		 if(scrolly!=null)
		   remove(scrolly);

		 // determine which scrollbars are necessary
		 // both:
		 if(((xSize(envpos)>envarea.width) && (ySize(envpos)+20>envarea.height)) ||
		    ((xSize(envpos)+20>envarea.width) && (ySize(envpos)>envarea.height)))
		   {
		    envarea.width-=20;
		    envarea.height-=20;
		    scrxused=scryused=true;
		   }
		 // only x:
		 else if(xSize(envpos)>envarea.width)
		   {
		    envarea.height-=20;
		    scrxused=true;
		    scryused=false;
		   }
		 // only y:
		 else
		   {
		    envarea.width-=20;
		    scrxused=false;
		    scryused=true;
		   }

		 // initialize scrollbars
		 if(scrxused)
		   {
		    i=xSize(envpos)-envarea.width;
		    if(i<0) i=0;
		    scrollx=new Scrollbar(Scrollbar.HORIZONTAL,0,0,0,i);
		    scrollx.enable();
		    scrollx.setLineIncrement((int)(0.5*envarea.width));
		    scrollx.setPageIncrement((int)(0.5*envarea.width));

		    add(scrollx);
		    scrollx.reshape(envarea.x,envarea.y+envarea.height,
						envarea.width,20);
		   }

		 if(scryused)
		   {
		    i=ySize(envpos)-envarea.height;
		    if(i<0) i=0;
		    scrolly=new Scrollbar(Scrollbar.VERTICAL,0,0,0,i);
		    scrolly.enable();
		    scrolly.setLineIncrement((int)(0.5*envarea.height));
		    scrolly.setPageIncrement((int)(0.5*envarea.height));

		    add(scrolly);
		    scrolly.reshape(envarea.x+envarea.width,envarea.y,20,
						envarea.height);
		   }

		 for(i=0;i<numofenv;i++)
		   {
		    envpos[i].width-=20;
		    envpos[i].height-=20;
		   }
		}


	   // optimizing is done, now areas are set
	   i=0;
	   Graphics envgr=(this.getGraphics()).create(envarea.x,
					   envarea.y, envarea.width, envarea.height);

	   for(Enumeration e = env.elements() ; e.hasMoreElements() ;)
		{
		 Environment x;
		 x=((Environment)e.nextElement());
		 x.setArea(envpos[i++], envgr);
		 if(debug) System.out.println("Set Environment to: "+envpos[i-1].toString());
		}

	 }

    //
    // double optimumsize(int num,Rectangle r[],Rectangle border[],
    //		    int bordcount,int pos)
    //
    // num: number of Environment Rectangles to be placed in the first row
    // r[]: already placed Rectangles
    // border[]: contains Rectangles describing all lowest lines (borders)
    //    of already set Rectangles
    // bordcount: how many borders are used
    // pos: which rectangle is to be set
    //
    // This method is used to find optimal placements for Environments.
    // It uses a quite copmlicate algorithm, though the idea is simple. The
    // method is recursive and does the following:
    //
    // First, it fills up the first row with 'num' rectangles.
    // After this till the end: It finds the uppermost place big enough for
    // the next piece (Rectangle with an Environment's minimum size) and
    // place it there. The border pieces are updated and the new position is
    // stored in r[].
    // When all pieces are set, the optimality quotient is returned. The
    // placement with the best optimality quotient is the one which matches
    // the display area's width to height ratio best. All o.q. are 1 or
    // above, with 1 the best o.q..
    //
    // The algorithm is no yet perfect (Well, come up with one! I still would
    // know a few ways to improve it.). However, for a few Environments it is
    // sufficient. If not, rearrange the Environments' order.
    //
    double optimumsize(int num,Rectangle r[],Rectangle border[],
				   int bordcount,int pos)
	 {
	  int i,x;
	  double optx,opty,optenv;	// used for computing optimality quotient

	  // First row. Pieces are simply set side by side.
	  if(pos==0)
	    {
		x=0;
		// set first row pieces
		for(;pos<num;pos++)
		  {
		   if(debug) System.out.println("r["+pos+"].x="+x+";");
		   r[pos].x=x;
		   r[pos].y=0;
		   r[pos].width=envsize[pos].width;
		   r[pos].height=envsize[pos].height;
		   border[pos].x=x;
		   border[pos].y=0;
		   border[pos].width=envsize[pos].width;
		   border[pos].height=envsize[pos].height;
		   x+=envsize[pos].width;


		  }
		bordcount=num;
		// Last piece has been set
		if(num==numofenv)
		  {
		   // Return 'optimality quotient'
		   optx=(double)xSize(r)/ySize(r);
		   opty=(double)envarea.width/envarea.height;
		   if(optx/opty<1)
			return opty/optx;
		   else
			return optx/opty;
		  }
		// Set remaining pieces
		else
		  return optimumsize(num,r,border,bordcount,pos);
	    }
	  // 'Free' piece. Find a place for it.
	  else
	    {
		// Search for uppermost border with enough width including x-space
		// to the left and right of the borderline (putting it as far left
		// as possible)
		if(debug) System.out.println("\nSearching for free space. bordcount="
				+bordcount+", pos="+pos);
		boolean [] bordchecked=new boolean[bordcount]; // Which borders have been checked
		for(i=0;i<bordcount;i++)
		  bordchecked[i]=false;

		int width=envsize[pos].width;
		int height=envsize[pos].height;
		int j=0,bordpos=0;
		int bordx=0,bordx2;
		int x1,x2;
		boolean ok=false;

		// find uppermost border with enough width
		while(!ok)
		  {
		   // count borders left
		   j=0;
		   for(i=0;i<bordcount;i++)
			if(bordchecked[i]==false)
			  {
			   bordpos=i; // to remember the last (& lowest) border
			   j++;
			  }
		   if(debug) System.out.println("Borders left unchecked: "+j);
		   // the last border
		   if(j==1)
			{
			 if(debug) System.out.println("Last Border, "+bordpos);
			 bordx=0;
			 ok=true;
			 j=border[bordpos].height;
			}
		   // not the last border, find uppermost border & check width
		   else
			{
			 // find uppermost border
			 if(debug) System.out.println("Find uppermost B.");
			 j=border[bordpos].height; // set starting value
			 for(i=0;i<bordcount;i++)
			   if((bordchecked[i]==false) && (border[i].height<j))
				{
				 bordpos=i; // to remember the last (& lowest) border
				 j=border[i].height;
				}
			 // check width available at this height
			 if(debug) System.out.println("Uppermost: "+bordpos+", "+border[bordpos].toString());
			 // find left border
			 bordx=0;
			 for(i=0;i<bordcount;i++)
			   {
			    // find borders in the way
			    if(debug) System.out.println("Checking Left Border: "+border[i].toString());
			    if((border[i].height>j)
				  && (border[i].x+border[i].width>bordx)
				  && (border[i].x+border[i].width<=border[bordpos].x))
				 {
				  if(debug) System.out.println("Border is in the way!");
				  bordx=border[i].x+border[i].width;
				 }
			   }
			 // find right border
			 bordx2=35000; // hopefully noone will display anything beyond
						// this width ;)
			 for(i=0;i<bordcount;i++)
			   // find borders in the way
			   if((border[i].height>j)
				 && (border[i].x<bordx2)
				 && (border[i].x>=border[bordpos].x+border[bordpos].width))
				bordx2=border[i].x;
			 // width o.k.? Then position for the rectangle is found
			 if(debug) System.out.println("width required: "+width
					+", actual width: "+bordx2+"-"+bordx+"="+(bordx2-bordx));
			 if(bordx2-bordx>=width)
			   {
			    if(debug) System.out.println("O.K., enough width");
			    ok=true;
			   }
			}

		  }
		// Now, place the new rectangle and reorganize the borders
		// place rectangle
		if(debug) System.out.println("setrect: j="+j+", bordx="+bordx+", height="+height);
		r[pos].x=bordx;
		r[pos].y=j;
		r[pos].width=width;
		r[pos].height=height;

		// reorganize borders

		// as borders are ordered from left to right, find leftmost and
		// rightmost intersecting border, resize them, and insert new
		// border
		// find leftmost & rightmost border
		x1=bordcount-1;
		x2=0;
		for(i=0;i<bordcount;i++)
		  {
		   // check left side
		   if(border[i].inside(r[pos].x,0))
			if(i<x1)
			  x1=i;
		   // check right side
		   if(border[i].inside(r[pos].x+width-1,0))
			if(i>x2)
			  x2=i;
		  }

		// resize next right border
		border[x2]=new Rectangle(bordx+width,0,
							border[x2].x+border[x2].width-bordx-width,
							border[x2].height);
		if(border[x2].width==0)
		  x2++;
		// move borders right of the new border to the left or right
		if(x1==x2) // move borders to the right
		  {
		   for(i=bordcount;i>x2;i--)
			{
			 border[i]=border[i-1];
			}
		   bordcount++;
		  }
		else if(x1+1<x2)
		  {
		   for(i=x2;i<bordcount;i++)
			{
			 border[i+x1-x2+1]=border[i];
			}
		   bordcount+=x1-x2+1;
		  }
		// set new border
		border[x1]=new Rectangle(bordx,0,width,j+height);
	    }

	  // Last piece has been set
	  if(pos==numofenv-1)
	    {
		// Return 'optimality quotient'

		optx=(double)xSize(r)/ySize(r);
		opty=(double)envarea.width/envarea.height;
		if(optx/opty<1)
		  return opty/optx;
		else
		  return optx/opty;
	    }
	  else
	    {
		// Continue with next piece
		pos++;
		return optimumsize(num,r,border,bordcount,pos);
	    }
	 }

    //
    // int xSize(Rectangle r[])
    //
    // Returns rightmost border of the Rectangles r[].
    //
    int xSize(Rectangle r[])
	 {
	  int i,x;
	  x=r[0].x+r[0].width;
	  for(i=1;i<numofenv;i++)
	    if(r[i].x+r[i].width>x)
		 x=r[i].x+r[i].width;
	  return x-20;
	 }

    //
    // int ySize(Rectangle r[])
    //
    // Returns lowest border of the Rectangles r[].
    //
    int ySize(Rectangle r[])
	 {
	  int i,y;
	  y=r[0].y+r[0].height;
	  for(i=1;i<numofenv;i++)
	    if(r[i].y+r[i].height>y)
		 y=r[i].y+r[i].height;
	  return y-20;
	 }

    //
    // delay(int ms)
    //
    // Waits ms milliseconds
    //
    void delay(int ms)
	 {
	  long start=System.currentTimeMillis();
	  while(start+ms>System.currentTimeMillis());
	 }

    //
    // boolean handleEvent(Event e)
    //
    // Event handling. Handles the two Buttons, the two Checkboxesand the
    // Scrollbars.
    //
    public boolean handleEvent(Event e)
	 {
	   Object target = e.target;
	   int i;
	   // Stop All/Continue or Step Button
	   if(target==worldbutton[0] && e.id==Event.ACTION_EVENT)
		{
		  // Step Button
		  if(onestep)
		    {
			stepcount++;
		    }
		  // Stop All/Continue Button
		  else
		    {
			// Continue
			if (frozen)
			  {
			   frozen = false;
			   start();
			  }
			// Stop All
			else
			  {
			   frozen=true;
			   stop();
			  }
		    }
		}
	   // Restart
	   else if(target==worldbutton[1] && e.id==Event.ACTION_EVENT)
		{
		  showStatus("Restarting... Please wait");
                  controlthread.suspend();
		  stop();

		  //wannakillyou=killmenow=false;
		  
		  showStatus("Restarting... Please wait");

		  // garbage collect all old Environments and initialize new ones 
		  env.removeAllElements();
		  env = new Vector();
		  System.gc();
		  
		  this.getGraphics().clearRect(area.x,area.y,area.width-1,area.height-1);

		  DefaultProps.resetStatics(); /* reset all static variables */

		  for(i=0;i<numofenv;i++)
		    {
			 addEnvironment(getParameter("envtype"+Integer.toString(i+1,10)),
				getParameter("envname"+Integer.toString(i+1,10)),
				divideParameter(getParameter("envparam"+Integer.toString(i+1,10))),i);
		    }

		  // set environment display area
		  Dimension d = size();
		  envarea.x=20;
		  envarea.y=40;
		  envarea.width=d.width;
		  envarea.height=d.height;
		  envarea.width-=40;
		  envarea.height-=100;
		  // order environments
		  orderEnvironments();
		  System.gc();
		  paintAll();

		  // Start again
		  frozen = false;
		  if(onestep)
		    stepcount=0;
		  else
		    worldbutton[0].setLabel("Stop All");
		  start();
		  paintAll();
		}
	   // Stop/Continue Mode activated
	   else if(e.target==stopmode && e.id==Event.ACTION_EVENT && onestep)
		{
		 stop();
		 onestep=false;
		 frozen=true;
		 worldbutton[0].setLabel("Continue");
		}
	   // Step Mode activated
	   else if(e.target==stepmode && e.id==Event.ACTION_EVENT && !onestep)
		{
		 onestep=true;
		 frozen=true;
		 stepcount=0;
		 // Check if controlthread was stopped
		 if(controlthread==null)
		   start();
		 worldbutton[0].setLabel("Step");
		}
	   // Scrollbars
	   // Warning: Scrollbars do not work correctly on some viewers. This
	   //          is not because of an error in World.
	   else if((e.id==Event.SCROLL_LINE_UP) ||
			 (e.id==Event.SCROLL_LINE_DOWN) ||
			 (e.id==Event.SCROLL_PAGE_UP) ||
			 (e.id==Event.SCROLL_PAGE_DOWN) ||
			 (e.id==Event.SCROLL_ABSOLUTE))
		{
		 if((e.target==scrollx) || (e.target==scrolly))
		   {
		    i=0;
		    int tx=0,ty=0;

		    // Redistribute drawing space.
		    Graphics envgr=(this.getGraphics()).create(envarea.x,
					   envarea.y, envarea.width, envarea.height);

		    if(scrxused)
			 tx=-scrollx.getValue();
		    if(scryused)
			 ty=-scrolly.getValue();
		    envgr.translate(tx,ty);


		    if(debug) System.out.println("\npx: "+scrollx.getValue()+", py: "+scrolly.getValue()+
								", maxX:"+scrollx.getMaximum()+", LineInc: "+
								scrollx.getLineIncrement()+", PageInc: "+scrollx.getPageIncrement());
		    if(debug) System.out.println("envarea.width: "+envarea.width+", xSize: "+xSize(envpos)+
								", xSize(envpos)-envarea.width: "+(xSize(envpos)-envarea.width));

		    for(Enumeration en = env.elements() ; en.hasMoreElements() ;)
			 {
			  Environment x;
			  x=((Environment)en.nextElement());
			  x.setArea(envpos[i++], envgr);
			  if(debug) System.out.println("Set Environment to: "+envpos[i-1].toString());
			 }
		    // Remove old Graphics objects.
		    System.gc();
		    paintAll(this.getGraphics());
		   }
		}

	   return super.handleEvent(e);
	 }
  }
