// file:        Wumpus.java
// source file: Wumpus.java
// object file: Wumpus.class
//
//
// included objects:
//  Wumpus
//    Desc: implements a Wumpus (monster); inmoveable object - gobbles 
//          agents
//
// history:
//   22.02.97 - first version finished by Chris
//
//   07.03.97 - development off. continued by Roman:
//              added functions for image drawing
//   12.03.97 - new extensions for displaying images (as per protocol 11.03.97) - Roman
//   22.03.97 - added & revised documentation - Roman
//   30.03.97 - changed giveImage- & Draw-function and added giveDeathImage
//   31.03.97 - added new color scheme - Roman
//   06.04.97 - added image/MediaTracker for deathimage, 
//              added getImage, getImageObserver, drawCross
//              unified image drawing style (Draw) - Roman
//   08.04.97 - added crosscolor_ support - Roman
//   11.04.97 - changed Constructors back to old design - property methods
//              set/getRes and set/getImageObserver will handle optional
//              paramethers now - Roman
//   14.04.97 - fitted for major redesign - Roman [pending]
//
// critical problems: none
//
// known problems:
//   * objects have to have a minimal size -> small objects have problems displaying
//     themselfs
//

import java.awt.*;
import java.awt.image.*;
import java.util.*;
import java.lang.*;


class Wumpus extends PassiveObject implements StatusIfc
{
  private boolean status; //am i stil alive or already a ghost?

  // private final static boolean DEBUG = false;


  private final static String TYPE_WUMPUS = "WUMPUS";

  private final static String DEFAULT_COLOR_STR           = "colors.fieldobject.WUMPUSCOLOR";
  private final static String DEFAULT_DEATHCOLOR_STR      = "colors.fieldobject.DEATHCOLOR";

  private final static String DEFAULT_CROSSCOLOR_STR      = "colors.fieldobject.CROSSCOLOR";
  private final static String DEFAULT_DEATHCROSSCOLOR_STR = "colors.fieldobject.DEATHCROSSCOLOR";

  private Color crosscolor_;

// Method: Wumpus Constructor (public)
//
// *Purpose   : intializes a new instance and sets a new image (-resource) 
//              for the object
//
// *Parameters: name (String) - name of the new instance
//              x (int) - 
//              y (int) - 
//
// *Return    : none
//
// *Variables : only through super
//
//   with set/get-methods:
//              imgref (String) - default reference to an imagemap
//              obs (ImageObserver) - observer for the image
//
  public Wumpus( String name, int x, int y )
  {
    super( name, TYPE_WUMPUS );

    // init the new variables:
    status = true;
    crosscolor_ = ColorConstants.getColor( defaultDeathCrossColorName() );
    // for standard crossing
  }

  public Wumpus( String name, int x, int y, String imgref )
  {
    super( name, TYPE_WUMPUS );

    // init the new variables:
    status = true;
    setRes( imgref );
    crosscolor_ = ColorConstants.getColor( defaultDeathCrossColorName() );
    // for standard crossing
  }

// Method: defaultColorName           (protected)
//         defaultDeathColorName      (protected)
//         defaultCrossColorName      (protected)
//         defaultDeathCrossColorName (protected)
//
// *Purpose   : returns a default property identifing string for a color
//              - good for beeing overwritten by subclasses to set an other
//                default property identifing string 
//                (the string to be looked up in the system-property list
//
// *Parameters: none
//
// *Return    : String - the new default color
//
// *Variables : all the nessessary id-strings
//
  public String defaultColorName()           { return DEFAULT_COLOR_STR; }
  public String defaultDeathColorName()      { return DEFAULT_DEATHCOLOR_STR; }

  public String defaultCrossColorName()      { return DEFAULT_CROSSCOLOR_STR; }
  public String defaultDeathCrossColorName() { return DEFAULT_DEATHCROSSCOLOR_STR; }


// Method: getStatus[Str] (public)
//
// *Purpose   : returns whether the agent is still alive or not
//
// *Parameters: none
//
// *Return    : boolean - the agent's current status
//         or : String -  the agent's current status
//
// *Variables : status (boolean, class) - the agent's current status
//
  public boolean getStatus() 
  {
    return status;
  }

  public String getStatusStr() 
  {
    if (status == true) 
      return ( "Alive" );
    else
      return ( "Dead" );
  }

// Method: setStatus (public)
//
// *Purpose   : sets the agent to a new status
//
// *Parameters: newStatus (String) - the new status of the agent
//              reason (String) -  the reason for the new status
//
// *Return    : boolean - the agent's current status
//
// *Variables : status (boolean, class) - the agent's current status
//
  public void setStatus( String newStatus ) 
  {
    if( newStatus.equalsIgnoreCase( "DEAD" ) ) 
    {
      status = false;
    }
    else
    {
      status = true;
    }

    if ( resource instanceof DrawableResources )
      ((DrawableResources) resource).setImagePhase( status ? 0 : 1 );
  }

  public void setStatus( String newStatus, String reason ) 
  {
    this.setStatus( newStatus );
  }


// Method: drawPrimitive (public)
//
// *Purpose   : draws a graphic primitive (a default form of displaying)
//              to the graphics device 
//
// *Parameters: g (Graphics) - the graphics to draw the objects to
//              area (Rectangle) - the position and dimensions of the
//                                 object on screen
//              foreground (Color) - the drawingcolor to use
//
// *Return    : whether the drawing of the Image was successfull or not
//
// *Variables : myimage (ImageReference, class) - pointer to a resource
//
  public boolean drawPrimitive( Graphics g, Rectangle area )
  {
    return ( drawPrimitive(g, area, getColor() ) );
  }

  public boolean drawPrimitive( 
    Graphics g, Rectangle area, Color foreground
  )
  {
//    if ( g == null ) return false;

    if ( foreground != null)
      g.setColor( foreground );

    //Primitive: a simple box-headed Pylon
    if (area.width < 10) return false;
    if (area.height< 10) return false;

    try {
      Rectangle head = new Rectangle( area.x, 
                                      area.y +1,
                                      2*(area.width/3),
                                     2*((area.height-1)/3) );
      head.x += (area.width - head.width)/2;

      Rectangle body = new Rectangle( area.x,
                                      head.y + head.height,
                                      area.width/4,
                                      area.height - head.height);
      body.x += (area.width - body.width)/2;

      g.fillRect( head.x, head.y, head.width, head.height );
      g.fillRect( body.x, body.y, body.width, body.height );

      g.setColor( Color.black );
      g.drawRect( body.x, body.y, body.width, body.height );
      g.drawRect( head.x, head.y, head.width, head.height );

    } 
    catch (ArithmeticException e)
    {
      System.err.println("Error calculating (Gold-)Bar: " + e.getMessage());
      return false;
    }

    return true;
  }


// Method: loadResources (public)
//
// *Purpose   : loads all Resources that can be derived from the 'resname'
//              - resname & ImageObserver must be allready set if differing
//                from default
//              - if getRes() returns null or "EMPTY" the method is not
//                executed
//
// *Parameters: none
//  
// Return     : none
//
// *Variables : resloader (ResourceLoader, class) - the (images-)loader
//              resource (Resources, class ) - the current resources
//
  public void loadResources()
  {
    Image          tmpimg;
    ImageReference tmpref;
    String         fetchres = getRes();

    if ( (fetchres == null) || (fetchres.equals( "EMPTY" )) ) 
      return;
    
    super.loadResources();
    
    tmpimg = resloader.getImage( fetchres, -1, 102 );
    if (tmpimg != null) 
    {
      if (resource == null)
        resource = new DrawableResources();

      tmpref   = new StdImageReference( getDisplayName(), tmpimg, getObserver() );
      resource.addImageReference( tmpref );
    }
  }


// Method: Draw (public)
//
// *Purpose   : draws a the object on screen - depending on the existence
//              of an imagemap with the image or with it's name (no image)
//
// *Parameters: g (Graphics) - the graphics to draw the objects to
//              drawArea (Rectangle) - the position and dimensions of the 
//                                     object on screen
//
// *Return    : none
//
// *Variables : none
//
  public void draw( Graphics g, Rectangle drawArea ) 
  {
    // System.out.println("Wumpus "+getName()+": entering Draw function...");
    // System.out.println("Wumpus: params: drawArea.x/y= " +drawArea.x +"/"+ drawArea.y);
    // System.out.println("Wumpus: drawArea.width/height= " +drawArea.width +"/"+ drawArea.height);

    super.draw(g, drawArea);

    if ( getStatus() == false ) 
      StdImageReference.drawCross( g, drawArea, crosscolor_, 2 );
    
    // System.out.println("Wumpus >"+getName()+"<: leaving Draw function.");
  }// End of Draw


}// End of class Wumpus
