// file:        WumpusAgent.java
// source file: WumpusAgent.java
// object file: WumpusAgent.class
//
//
// included objects:
//  WumpusAgent
//    Desc: implements a Agent specific to Wumpus environments
//
//
// history:
//   22.02.97 - first version finished by Chris
//
//   13.03.97 - development off. continued by Roman:
//              new extensions for displaying images (as per protocol 11.03.97) - Roman
//   23.03.97 - added multiple default image support
//              added & revised documentation - Roman
//   29.03.97 - changed constructors, giveImage, Draw and added 
//              giveDeathImage => to resemble death-image drawing - Roman
//   30.03.97 - added history capabilities to the agent
//              functions getHistory, updateHistory added
//              constructor expanded for member history - Chris
//   01.04.97 - added ActionIcon-Drawing functions (drawAction, 
//              getActionImage, giveActionImageResource)
//              added DefaultColors-support - Roman
//   02.04.97 - added sendAction, receivePercepts
//              sendAction, receivePercepts supports drawaction - Roman
//   04.04.97 - added actionOK (as per meeting 04.04.97)
//              added support for & included set/get/consumeActionIcon
//              added image/MediaTracker for deathimage - Roman 
//   06.04.97 - added getImage, getImageObserver, drawCross , using setRes()
//   08.04.97 - 
//   09.04.97   unified image drawing style (Draw), crosscolor - Roman
//   11.04.97 - changed Constructors back to support the old designs - Roman
//   27.04.97 - added new methods for loading resources - Roman
//   05.05.97 - changed getActionImages to loadActionResources
//              changed documentations - Roman
//
//
// critical problems: none
//
// known problems: none
//
//
// additional notes:
//   a consuming of the drawn ActionIcon is not trully possible because the 
//   envrionment displays a field multiple times 
//   => only consume after actionOK (Phase 1 =>Phase = 2) and 
//      receivePercepts (Phase 2 =>Phase = 0)
//

import java.awt.*;
import java.awt.image.ImageObserver;
import java.util.*;


abstract class WumpusAgent extends MxNActiveObject implements StatusIfc //, FieldFillingIfc
{
  // public boolean fillsField() { return true; }

  // private final static boolean DEBUG = false;

  private final static String TYPE_WUMPUS = "WUMPUS";

  private final static String DEFAULT_IMAGE = "EMPTY";
  private final static String IMAGE_BASE    = "envs.wumpus.image.agent";

  private final static String DEFAULT_COLOR_STR           = "colors.fieldobject.AGENTCOLOR";
  private final static String DEFAULT_DEATHCOLOR_STR      = "colors.fieldobject.DEATHCOLOR";

  private final static String DEFAULT_CROSSCOLOR_STR      = "colors.fieldobject.CROSSCOLOR";
  private final static String DEFAULT_DEATHCROSSCOLOR_STR = "colors.fieldobject.DEATHCROSSCOLOR";


  private boolean status; //am i stil alive or already a ghost?
  private int arrows;     //how many arrows do i possess
  private int score;      //what's the score

  public Vector history; // store all percepts here

  public ActionIcons actionicon;
  public boolean     actionresult;
  protected Color    crosscolor_;


// --- actions - String constansts: ---------------------------------------
  static final String ACTION_STR_UP          = "UP";
  static final String ACTION_STR_DOWN        = "DOWN";
  static final String ACTION_STR_LEFT        = "LEFT";
  static final String ACTION_STR_RIGHT       = "RIGHT";
 
  static final String ACTION_STR_WAIT        = "WAIT";
  static final String ACTION_STR_CLIMB       = "CLIMB";
  static final String ACTION_STR_GRAB        = "GRAB";
 
  static final String ACTION_STR_SHOOT_UP    = "SHOOT_UP";
  static final String ACTION_STR_SHOOT_DOWN  = "SHOOT_DOWN";
  static final String ACTION_STR_SHOOT_LEFT  = "SHOOT_LEFT";
  static final String ACTION_STR_SHOOT_RIGHT = "SHOOT_RIGHT";
 
 
// --- actions - binary coded constansts: ---------------------------------
  static final int ACTION_BASE  = 0x00;
  static final int ACTION_UP          = 0x01 | ACTION_BASE;
  static final int ACTION_DOWN        = 0x02 | ACTION_BASE;
  static final int ACTION_LEFT        = 0x04 | ACTION_BASE;
  static final int ACTION_RIGHT       = 0x08 | ACTION_BASE;
 
  static final int ACTION_BASE2 = 0x10;
  static final int ACTION_WAIT        = 0x00 | ACTION_BASE2;
  static final int ACTION_CLIMB       = 0x01 | ACTION_BASE2;
  static final int ACTION_GRAB        = 0x02 | ACTION_BASE2;
 
  static final int ACTION_SHOOT = 0x20;
  static final int ACTION_SHOOT_UP    = ACTION_UP    | ACTION_SHOOT;
  static final int ACTION_SHOOT_DOWN  = ACTION_DOWN  | ACTION_SHOOT;
  static final int ACTION_SHOOT_LEFT  = ACTION_LEFT  | ACTION_SHOOT;
  static final int ACTION_SHOOT_RIGHT = ACTION_RIGHT | ACTION_SHOOT;


// --- Percepts - String constansts: --------------------------------------
  static final String PERCEPT_STR_BUMP    = "BUMP";
 
  static final String PERCEPT_STR_GLITTER = "GLITTER";
  static final String PERCEPT_STR_STENCH  = "STENCH";
  static final String PERCEPT_STR_SCREAM  = "SCREAM";
  static final String PERCEPT_STR_BREEZE  = "BREEZE";
 
 
// --- Percepts - binary coded constansts: --------------------------------
  static final int PERCEPT_BASE    = 0x40;
 
  static final int PERCEPT_BUMP    = 0x00 | PERCEPT_BASE;
  static final int PERCEPT_GLITTER = 0x01 | PERCEPT_BASE;
  static final int PERCEPT_STENCH  = 0x02 | PERCEPT_BASE;
  static final int PERCEPT_SCREAM  = 0x04 | PERCEPT_BASE;
  static final int PERCEPT_BREEZE  = 0x08 | PERCEPT_BASE;
 
  static final int PERCEPT_BASE2   = 0x80;
 

// Method: WumpusAgent Constructor (public)
//
// *Purpose   : intializes a new instance and sets a new image (-resource)
//              for the object
//
// *Parameters: name (String) - name of the new instance
//              type (String) - general type of the new instance
//              arrowNum (int) - number of arrows the agent has to his disposal
//
//    
//              imgref (String) - default reference to an imagemap
//              obs (ImageObserver) - observer for the image
//
// *Return    : none
// *Variables : only through super
//
  public WumpusAgent ( String name, String type, int arrowNum, 
                       String resource, ImageObserver observer )
  {
    super( name, type, resource, observer );
    status = true;
    if( arrowNum > 0 )
      arrows = arrowNum;
    else
      arrows = 0;

    history = new Vector( 15, 10 );

    if (actionicon == null)
      loadActionResources();

    crosscolor_ = ColorConstants.getColor( defaultDeathCrossColorName() );

  }// End of main WumpusAgnet-Constructors
 

/* the constructors below just call the above ones, 
   afterwards they change eventually inproperly set variables */

  public WumpusAgent( String name, String type )
  {
    this( name, type, 1, (String)null, null );	// one arrow default
  }

  public WumpusAgent( String name, String type, String resource,
    ImageObserver observer )
  {
    this( name, type, 1, resource, observer );	// one arrow default
  }

  public WumpusAgent( String name, String type, ImageObserver observer )
  {
    this( name, type, 1, null, observer );	// one arrow default
  }

  public WumpusAgent( String name, String type, String resource )
  {
    this( name, type, 1, resource, null );	// one arrow default
  }

  public WumpusAgent ( String name, String type, int arrowNum, String resource )
  {
    this( name, type, arrowNum, resource, null );
  }

  public WumpusAgent ( String name, String type, int arrowNum, ImageObserver observer )
  {
    this( name, type, arrowNum, (String)null, observer );
  }

  public WumpusAgent ( String name, String type, int arrowNum )
  {
    this( name, type, arrowNum, (String)null, (ImageObserver)null );

  }// End of WumpusAgnet-Constructors


// Method: baseResName (public)
//
// *Purpose   : returns the basic-Name for the defaultResName- method
//              to search in default-property-list
//              - this is the only thing that needs to be overwritten
//                with defaultResName;
//
// *Parameters: none
//
// *Return    : String - the default string to set
//
// *Variables : ret the temporary return string
//
  public String baseResName() { return IMAGE_BASE; }


// Method: sendAction (public)
//
// *Purpose   : returns the actions of an ActiveObject
//              and sets the action the agent trys to do
//
// *Parameters: none
//
// *Return    : Vector - the current actions of an ActiveObject instance
//
// *Variables : actions (Vector, class) - stores the actions of an ActiveObject
//
  public Vector sendAction() //where the agent returns his actions
  {
    Object tmpobj;
    String action = ACTION_STR_WAIT;

    // before returning set action image to the first action in the vector actions
    if ( ( actions != null ) && 
         ( actions.size() > 0 ) &&
         ( (tmpobj = actions.firstElement()) instanceof Action ) &&
         ( actionicon != null ) )
    {
        action = ((Action) tmpobj).getName();
    }

    if ( actionicon != null )
        actionicon.setAction( action );

    // System.err.println( getName()+" sending action >"+action+"<");

    return super.sendAction();
  }// End of sendAction


// Method: receivePercepts (public)
//
// *Purpose   : stores the incoming percepts
//
// *Parameters: perceps (Vector) - the new percept(s) vector
//              and sets the action the agent has done
//
// *Return    : none
//
// *Variables : actions (Vector, class) - stores the new percept
//
  public void receivePercepts( Vector percepts )//where the agent receives his percepts
  {
    super.receivePercepts(percepts);

    if ( actionicon != null ) 
      actionicon.consume();

//    if ( resource instanceof DrawableResources )
//      ((DrawableResources) resource).advanceImagePhase();
  }


// Method: getScore (public)
//
// *Purpose   : returns the current score of an Agent
//
// *Parameters: none
//
// *Return    : int - the current score
//
// *Variables : score (int, class) - the agent's score
//
  public int getScore()
  {
    return score;
  }

// Method: updateScore (public, final)
//
// *Purpose   : adds the new score to the agents'score
//
// *Parameters: points (int) - the points to add to the score
//
// *Return    : none
//
// *Variables : score (int, class) - the agent's score
//
  public final void updateScore( int points )
  {
    score += points;
  }


// Method: setScore (public, final)
//
// *Purpose   : adds the new score to the agents'score
//
// *Parameters: points (int) - the points to add to the score
//
// *Return    : none
//
// *Variables : score (int, class) - the agent's score
//
  public final void setScore( int points )
  {
    score = points;
  }
 
 
// Method: arrowsLeft (public, final)
//
// *Purpose   : returns true/false if the agent has some
//              errors left to his disposal
//
// *Parameters: none
//
// *Return    : boolean - returns if there are still arrows
//
// *Variables : arrows (int, class) - the agent's number of arrows
//
  public final boolean arrowsLeft()
  {
    if( arrows == 0 )
      return false;
    else
      return true;
  }

// Method: reduceArrows (public, final)
//
// *Purpose   : reduces the number of arrows by one
//
// *Parameters: none
// *Return    : none
//
// *Variables : arrows (int, class) - the agent's number of arrows
//
  public final void reduceArrows()
  {
    if( arrows > 0 )
      arrows --;
  }

// Method: getArrowNum (protected, final)
//
// *Purpose    : allows an agent to know how much arrows are left
//
// *Parameters : none
// *Return     : none
//
// *Variables  : arrows - the agent's number of arrows
  public final int getArrowNum()
  {
    return arrows;
  }


// Method: getStatus[Str] (public)
//
// *Purpose   : returns whether the agent is still alive or not
//
// *Parameters: none
//
// *Return    : boolean - the agent's current status
//         or : String - the agent's current status
//
// *Variables : status (boolean, class) - the agent's current status
//
  public boolean getStatus()
  {
    return status;
  }

  public String getStatusStr()
  {
 
    if( status == true )
      return "ALIVE";
    else
      return "DEAD";
  }
 
// Method: setStatus (public)
//
// *Purpose   : sets the agent to a new status
//
// *Parameters: newStatus (String) - the new status of the agent
//
//              reason (String) -  the reason for the new status
//                                 can be PIT, WUMPUS, EXIT (in case of 
//                                   the first two set the succes counter 
//                                   'score'  to -1000)
//
// *Return    : boolean - the agent's current status
//
// *Variables : status (boolean, class) - the agent's current status
//
  public void setStatus( String newStatus, String reason ) 
  {
    if( ( newStatus != null ) && 
          newStatus.equalsIgnoreCase( "DEAD" ) ) 
    {
      status = false;
      if( reason != null ) 
      {
        if ( reason.equalsIgnoreCase("EXIT") )
        {
          actionOK(true);
        }
        if ( reason.equalsIgnoreCase("WUMPUS") ||
             reason.equalsIgnoreCase("PIT")    )
        {
          score = -10000;
        }
      }
      //crosscolor_ = ColorConstants.getColor( defaultDeathCrossColorName() );
    }
    else
    {
      status = true;
      //crosscolor_ = ColorConstants.getColor( defaultCrossColorName() );
    }

    if ( resource instanceof DrawableResources )
      ((DrawableResources) resource).setImagePhase( status ? 0 : 1 );
  }

  public void setStatus( String newStatus )
  {
    if ( newStatus.equalsIgnoreCase( "DEAD" ) )
      this.setStatus( newStatus, "WUMPUS" );
    else
      this.setStatus( newStatus, "" );
  }

 
// Method: updateHistory ( public )
//
// *Purpose   : stores every action an agent has taken
//
// *Parameters: Action ( Object ) - the latest action of the agent
//
// *Return    : nothing
//
// *Variables :  history ( Vector ) - the agent's history Vector
//
  public void updateHistory( Object lastaction )
  {
    history.addElement( lastaction );
  }

// Method: getHistory ( public )
//
// *Purpose   : returns a string with all the agent's actions
//              parted by a newline character
//
// *Parameters: none
//
// *Return    : String - the agent's history
//
// *Variables :  history ( Vector ) - the agent's history Vector
//
  public String getHistory()
  {
    Enumeration actions;
    StringBuffer giveback = new StringBuffer();
    actions = history.elements();
    giveback.append( "My actions have been: \n" );
    while( actions.hasMoreElements() == true )
    {
      giveback.append( ( ( Action ) actions.nextElement() ).getName() );
      giveback.append( "\n" );
    }
    return giveback.toString();
  }// End of getHistory
 

// Method: defaultColorName           (public)
//         defaultDeathColorName      (public)
//         defaultCrossColorName      (public)
//         defaultDeathCrossColorName (public)
//
// *Purpose   : returns a default property identifing string for a color
//              - good for beeing overwritten by subclasses to set an other
//                default property identifing string 
//                (the string to be looked up in the system-property list
//
// *Parameters: none
//
// *Return    : String - the new default color
//
// *Variables : all the nessessary id-strings
//
  public String defaultColorName()           { return DEFAULT_COLOR_STR; }
  public String defaultDeathColorName()      { return DEFAULT_DEATHCOLOR_STR; }

  public String defaultCrossColorName()      { return DEFAULT_CROSSCOLOR_STR; }
  public String defaultDeathCrossColorName() { return DEFAULT_DEATHCROSSCOLOR_STR; }


// Method: setColor (public)
//
// *Purpose   : returns the base-color of the current FieldObject
//              * if null WHITE will be the base-color
//
// *Parameters: color (String) - the new base-color
//
// *Return    : none
//
// *Variables : color_ (ColorConstants, class) - the new base-color
//
  public void setColor( String color )
  {
    super.setColor( color );
  }


// Method: actionOK (public)
//
// *Purpose   : sets the valid-state of the current action-image 
//              and changes to drawphase 2
//
// *Parameters: result (boolean) - the new action images
//
// *Return    : none
//
// *Variables : actionresult (boolean, class) - is the action (icon) indented valid ?
//              drawphase (int, class) - the action icon drawing state
//
  public void actionOK(boolean result)
  {
    actionresult = result;

    if ( actionicon != null ) 
    {
      actionicon.setValid( result );
    }

    // System.out.println( getName()+" actionOK >"+result+"<");

//    if ( resource instanceof DrawableResources )
//      ((DrawableResources) resource).advanceImagePhase();
  }


// Method: loadResources (public)
//
// *Purpose   : loads all Resources that can be derived from the 'resname'
//              - resname & ImageObserver must be allready set if differing
//                from default
//              - if getRes() returns null or "EMPTY" the method is not
//                executed
//
// *Parameters: none
//  
// Return     : none
//
// *Variables : resloader (ResourceLoader, class) - the (images-)loader
//              resource (Resources, class ) - the current resources
//
  public void loadResources()
  {
    Image          tmpimg;
    ImageReference tmpref;
    String         fetchres = getRes();

    if ( (fetchres == null) || (fetchres.equals( "EMPTY" )) ) 
      return;
    
    super.loadResources();
    
    tmpimg = resloader.getImage( fetchres, -1, 102 );
    if (tmpimg != null) 
    {
      if (resource == null)
        resource = new DrawableResources();

      tmpref   = new StdImageReference( getDisplayName(), tmpimg, getObserver() );
      resource.addImageReference( tmpref );
    }
  }


// Method: loadActionResources (public)
//
// *Purpose   : sets the current action images to the ones provided by
//              the caller
//
// *Parameters: none
//
// *Return    : none
//
// *Variables : actionicons (ActionIcons, class) - reference to a resource
//
  public void loadActionResources()
  {
    // System.out.println("WumpusAgent " +getName()+ ": getting action-images");
    
    if ( this.actionicon == null )
      this.actionicon = new ActionIcons();
    if (resloader == null)
      resloader = new ResourceLoader();

    actionicon.setResources( resloader.getActionResources() );
  }


// Method: draw (public)
//
// *Purpose   : draws the object on screen - depending on the existence
//              (this is done by super)
//              new: draw a cross over image/string if the agent is dead
//
// *Parameters: g (Graphics) - the graphics to draw the objects to
//              drawArea (Rectangle) - the position and dimensions of the
//                                     object on screen
//
// *Return    : none
//
// *Variables : myimage (ImageReference, class) - pointer to a resource
//              status (boolean, class) - status of the agent (true=> alive)
//
  public void draw( Graphics g, Rectangle drawArea )
  {
/*
    if ( DEBUG )
    {
      System.out.println("WumpusAgent "+getName()+": entering Draw function...");
      System.out.println("            params: drawArea.x/y= " +drawArea.x +"/"+ drawArea.y);
      System.out.println("                    drawArea.width/height= " +drawArea.width +"/"+ drawArea.height);
    }
*/
    super.draw( g, drawArea ); 

    if ( getStatusStr().equals("DEAD") ) 
      StdImageReference.drawCross( g, drawArea, crosscolor_, 2 );

    if ( actionicon != null )
    {
      actionicon.draw( g, drawArea ); // do not cross icon out... => for readability
      // for testing the BackGrayFilter-effect on the agent-image's colors
      // actionicon.draw( g, new Rectangle(drawArea.x - drawArea.width/3, drawArea.y + 9*drawArea.height/16, drawArea.width, drawArea.height) ); // do not cross icon out... => for readability
    } 
    else {
//      if ( DEBUG )
//        System.out.println("WumpusAgent >"+getName()+"<: no actionicons loaded");
    }

//    if ( DEBUG )
//      System.out.println("WumpusAgent >"+getName()+"<: leaving Draw function.");

  }// End of draw


// Method: reason (protected, abstract)
//
  abstract protected void reason();
 

}//End of class WumpusAgent
