// file:        WumpusField.java
// source file: WumpusField.java
// object file: WumpusField.class
//
//
// included objects:
//  WumpusField
//    Desc: implements a Wumpus-World specific FieldObject-container 
//          this container can store FieldObjects (strict instance checking)
//          and is extended by new Bases (->FieldBaseObject)
//
//
// history:
//   02.05.97 - first version by Roman Valehrach
//
//   all above changes made by Roman
//
//
// critical problems: (see Field.java for more infos)
//
// known problems: (see Field.java for more infos)
//

import java.util.Vector;
import java.awt.*;
import java.awt.image.*;
import java.net.*;

import java.applet.Applet;


class WumpusField extends Field 
{

// *** Implementation Section: ***

// Method: WumpusField Constructor (public)
//
// *Purpose   : intializes an instance of Field - creates a new Vector,
//              and sets the type, nulls image resources 
//
// *Parameters: fieldtype (String) - defines the type of the Field-Object
//              imgref (String) - default reference to an imagemap
//              obs (ImageObserver) - observer for the image
//
// *Return    : none
//
// *Variables : fieldobjects_ (Vector, class) - a list of FieldObjects in
//                                                the Field-instance
//              type_ (String, class) - the type of the Field-instance
//              resname_(String, class) - default reference to an imagemap
//              myimage_ (Image, class) - pointer to a resource (null)
//
  public WumpusField( String fieldtype, String resource, ImageObserver observer ) 
  {
    super( fieldtype, resource, observer );
  }

  public WumpusField( String fieldtype, String resource ) 
  {
    super( fieldtype, resource );
  }
  public WumpusField( String fieldtype, ImageObserver observer ) 
  {
    super( fieldtype, observer );
  }
  public WumpusField( String fieldtype ) 
  {
    super( fieldtype );
  }


// Method: createFieldBase (public)
//
// *Purpose   : creates the the Fields base depending on the Fields type
//
// *Parameters: none
//
// *Return    : none
//
// *Variables : fieldbases (FieldObject) - the type of the Field-instance
//
  protected void createFieldBase()
  {
    FieldObject fieldbase = null;
    int i = 0;
    String type = getType();

    if (type == null) return;

    if ( type.equalsIgnoreCase("WALL") )
    {
      fieldbase = new Wall(resname_, getObserver());
    }
    else
    if ( type.equalsIgnoreCase("EXIT") )
    {
      fieldbase = new Exit(resname_, getObserver());
    }
    else
    if ( type.equalsIgnoreCase("PIT") )
    {
      fieldbase = new Pit(resname_, getObserver());
    }

    if (fieldbase != null)
    {
//      while ( i < fieldbases.length )
//      {
        this.addObjectNoCheck( fieldbase );
//      }
    }
  }


}// End of class WumpusField
