// file:        WumpusHistoryMap.java
// source file: WumpusHistoryMap.java
// object file: WumpusHistoryMap.class
//
//
// included objects:
//  WumpusHistoryMap
//    Desc: implements an history map for Wumpus-World agents
//
// history:
//   07.04.97 - first version finished by Roman
//   27.04.97 - removed WumpusLib-functions (exporter to own class)
//
//   
// critical problems: none
//
// known problems: none
//

import java.util.Vector;

class WumpusHistoryMap extends HistoryMap
{
  // private static final boolean DEBUG = false;


// Method: HistoryMap Constructor (public)
//
// *Purpose   : just calls the super-class constructor
//
// *Parameters: none
//
// *Return    : none
//
// *Variables : none
//  
   WumpusHistoryMap()
   {
     super();
   }


// Method: validateHistory ( public )
//
// *Purpose   : validates the History map at the current validation pos. 
//              * movement (up, down, left, right, climb) only
//
// *Parameters: percepts ( Vector ) - the percepts that resulted from the
//                                    latest action
//
// *Return    : none
//
// *Variables : none
//
  public void validateHistory( Vector percepts )
  {
    boolean valid= true;
    int i;
    String tmp_perc;

    HistoryItem item = toValidate();

    if ( ( item == null ) || ( item.getChecked() ) )
      return; /* nothing to do */

    i= item.getValue();

    if ( (i > ACTION_BASE) && (i < ACTION_BASE2) )  
    { // only interested in moves
      if ( percepts != null )
        for ( i = 0 ; i < percepts.size() ; i++ )
        {
          tmp_perc = (( Percept ) percepts.elementAt( i )).getName();
          if ( tmp_perc.equalsIgnoreCase( PERCEPT_STR_BUMP ) )
          {
            valid = false;
            break;
          }
        }
    }

    if (i == ACTION_CLIMB)  // we are still in the environment => no success  
      valid = false;

   if ( (i > ACTION_SHOOT) && (i < ACTION_BASE4) ) /* screams validate the shot        */
   {                                               /* though it may not be the object that execuded a shot that then killed the Wumpus - an other agent may have shot at the same time and killed the Wumpus */
      valid = false;                               /* all other percepts invalidate it */
      if ( percepts != null )
        for ( i = 0 ; i < percepts.size() ; i++ )
        {
          tmp_perc = (( Percept ) percepts.elementAt( i )).getName();
          if ( tmp_perc.equalsIgnoreCase( PERCEPT_STR_SCREAM ) )
          {
            valid = true;
            break;
          }
        }
    }

    item.setValid(valid);
  }


// Method: lastMove (public)
//
// *Purpose   : returns the last movement action the agent executet
//              (this methode ignores wheter this move was valid or not)
//
// *Parameters: none
//
// *Return    : String - the last move taken
//
// *Variables : none
//
  public String lastMove()
  {
    String lastmove = null;
    boolean move = false;
    HistoryItem tmpitem;
    int tmpvalue;
    int i;
    int tmp= -1;

    if ( (i = size()) == 0 )
      return null;

    for ( i -= 1 ; ( i >= 0 ) && ( move == false ); i--  )
    {
      tmpitem = elementAt( i );
 
      if (tmpitem == null) 
        break;

      tmpvalue = tmpitem.getValue();

/*
      if ( DEBUG )
        System.out.println( "WumpusHistoryMap: lastMove() current move >"+ 
          action2String( tmpvalue ) + "< counter >" + i + "<" );
*/
      if ( (tmpvalue > ACTION_BASE) && (tmpvalue < ACTION_BASE2) ||
           (tmpvalue == ACTION_CLIMB) ) 
      {
        lastmove = action2String( tmpvalue );
        move = true;
        tmp= i;
      }
    }

    setCounter( tmp );

//    if ( DEBUG )
//      System.out.println(" last move taken >" +lastmove+ "< ...");

    return lastmove;
  }// End of lastMove


// Method: prevMove (public)
//
// *Purpose   : returns the movement action before the current on (counter_)
//
// *Parameters: none
//
// *Return    : String - the previous move taken
//
// *Variables : none
//
  public String prevMove()
  {
    String lastmove = null;
    boolean move = false;
    HistoryItem tmpitem;
    int tmpvalue;
    int i;
    int tmp= -1;

    if ( (i = size()) == 0 )
      return null;

    for ( i = getCounter() - 1 ; ( i >= 0 ) && ( move == false ); i--  )
    {
      tmpitem = elementAt( i );
 
      if (tmpitem == null) 
        break;

      tmpvalue = tmpitem.getValue();

/*
      if ( DEBUG )
        System.out.println( "WumpusHistoryMap: prevMove() current move >"+ 
          action2String( tmpvalue ) + "< counter >" + i + "<" );
*/
      if ( (tmpvalue > ACTION_BASE) && (tmpvalue < ACTION_BASE2) ||
           (tmpvalue == ACTION_CLIMB) ) 
      {
        lastmove = action2String( tmpvalue );
        move = true;
        tmp= i;
      }
    }

    setCounter( tmp );

//    if ( DEBUG )
//      System.out.println(" previous move taken >" +lastmove+ "< ...");

    return lastmove;
  }// End of prevMove


// Method: lastShot (public)
//
// *Purpose   : returns only the last shooting action the agent executed on
//              the current field, (this method stops at the first movement
//              action)
//
// *Parameters: none
//
// *Return    : Vector - all the last shots executed
//
// *Variables : history_items_ (Vector, class) - the historymap of the agent
//
  public String lastShot()
  {
    String lastshot = null;
    int tmpvalue;
    boolean move = false;
    int i;

    if ( super.size() < 1 )
      return null;

    for ( i = super.size() - 1 ; ( i >= 0 ) && ( move == false ); i--  )
    {
      tmpvalue = (( HistoryItem ) (super.elementAt( i ))).getValue();
      if ( (tmpvalue > ACTION_SHOOT) && (tmpvalue < ACTION_BASE4) ) 
      {
        lastshot = action2String( tmpvalue );
        move = true; /* stop here */
      } else
        if (   (tmpvalue > ACTION_BASE) && (tmpvalue < ACTION_BASE2)
           // && (( HistoryItem ) (super.elementAt( i ))).getValid() == true 
           ) 
          move = true;
    }

//    if (DEBUG)
//      System.out.println("Last shot taken: " + lastshot);

    return lastshot;
  }// End of lastShot

// Method: lastShots (public)
//
// *Purpose   : returns the last shooting action the agent executed on
//              the current field, (stops at the first none-shooting 
//              action)
//
// *Parameters: none
//
// *Return    : Vector - all the last shots executed
//
// *Variables : history_items_ (Vector, class) - the historymap of the agent
//
  public Vector lastShots()
  {
    Vector lastshot = new Vector( 1, 1 );
    int tmpvalue;
    boolean noshot = false;
    int i;

    if ( super.size() < 1 )
      return null;

    for ( i = super.size() - 1 ; ( i >= 0 ) && ( noshot == false ); i--  )
    {
      tmpvalue = (( HistoryItem ) (super.elementAt( i ))).getValue();
      if ( (tmpvalue > ACTION_SHOOT) && (tmpvalue < ACTION_BASE4) ) 
        lastshot.addElement( ( Object ) action2String( tmpvalue ) );
      else
        noshot = true;
    }

    return lastshot;
  }// End of lastShots


// Method: allShots (public)
//
// *Purpose   : returns the last shooting action the agent executed on
//              the current field, (this method takes invalid moves in
//              account - all other actions are ignored [have the same
//              effect as infalid moves])
//
// *Parameters: none
//
// *Return    : Vector - all the shots made from the current field
//
// *Variables : history_items_ (Vector, class) - the historymap of the agent
//
  public Vector allShots()
  {
    Vector lastshot = new Vector( 1, 1 );
    int tmpvalue;
    boolean move = false;
    int i;

    if ( super.size() < 1 )
      return null;

    for ( i = super.size() - 1 ; ( i >= 0 ) && ( move == false ); i--  )
    {
      tmpvalue = (( HistoryItem ) (super.elementAt( i ))).getValue();
      if ( (tmpvalue > ACTION_SHOOT) && (tmpvalue < ACTION_BASE4) ) 
        // add shooting action .. and continue
        lastshot.addElement( ( Object ) action2String( tmpvalue ) );
      else
        // is this a VALID move  =>  STOP adding
        if ( (tmpvalue > ACTION_BASE) && (tmpvalue < ACTION_BASE2) &&
            (( HistoryItem ) (super.elementAt( i ))).getValid() == true ) 
          move = true;
      // everything else is ignored... [i.e. "CLIMB" is the last action]
    }

    return lastshot;
  }// End of allShots


}// End of WumpusHistoryMap
