// file:        WumpusLib.java
// source file: WumpusLib.java
// object file: WumpusLib.class
//
//
// included objects:
//  WumpusLib
//    Desc: implements some useful functions for Wumpus-World agents
//
// history:
//   27.04.97 - first version finished by Roman
//
//   
// critical problems: none
//
// known problems: none
//

import java.util.Vector;

class WumpusLib extends Object
{
  // private static final boolean DEBUG = false;


// --- actions - String constansts: ---------------------------------------
  public static final String ACTION_STR_UP          = "UP";
  public static final String ACTION_STR_DOWN        = "DOWN";
  public static final String ACTION_STR_LEFT        = "LEFT";
  public static final String ACTION_STR_RIGHT       = "RIGHT";
 
  public static final String ACTION_STR_WAIT        = "WAIT";
  public static final String ACTION_STR_CLIMB       = "CLIMB";
  public static final String ACTION_STR_GRAB        = "GRAB";
 
  public static final String ACTION_STR_SHOOT_UP    = "SHOOT_UP";
  public static final String ACTION_STR_SHOOT_DOWN  = "SHOOT_DOWN";
  public static final String ACTION_STR_SHOOT_LEFT  = "SHOOT_LEFT";
  public static final String ACTION_STR_SHOOT_RIGHT = "SHOOT_RIGHT";

 
// --- actions - binary coded constansts: ---------------------------------
  public static final int ACTION_BASE  = 0x00;
  public static final int ACTION_UP          = 0x01 | ACTION_BASE;
  public static final int ACTION_DOWN        = 0x02 | ACTION_BASE;
  public static final int ACTION_LEFT        = 0x04 | ACTION_BASE;
  public static final int ACTION_RIGHT       = 0x08 | ACTION_BASE;
 
  public static final int ACTION_BASE2 = 0x10;
  public static final int ACTION_WAIT        = 0x00 | ACTION_BASE2;
  public static final int ACTION_CLIMB       = 0x01 | ACTION_BASE2;
  public static final int ACTION_GRAB        = 0x02 | ACTION_BASE2;
 
  public static final int ACTION_BASE3 = 0x20;
  public static final int ACTION_SHOOT = 0x20;
  public static final int ACTION_SHOOT_UP    = ACTION_UP    | ACTION_SHOOT;
  public static final int ACTION_SHOOT_DOWN  = ACTION_DOWN  | ACTION_SHOOT;
  public static final int ACTION_SHOOT_LEFT  = ACTION_LEFT  | ACTION_SHOOT;
  public static final int ACTION_SHOOT_RIGHT = ACTION_RIGHT | ACTION_SHOOT;

  public static final int ACTION_BASE4 = 0x40;


// ------------------------------------------------------------------------
// MOVE - Support
// ------------------------------------------------------------------------

// Method: randomAction (public, static)
//
// *Purpose   : generates a random action (move, shoot, grab, climb)
//
// *Parameters: none
//   
// *Return    : String - the random action
//
// *Variables : rnd (int) - holds the random number for the move
//
  public static String randomAction()
  {
    int rnd = ( int )java.lang.Math.abs( ( java.lang.Math.random() * 10 ) );

    switch( rnd )
    {
      case 0:  return ACTION_STR_UP;
     
      case 1:  return ACTION_STR_DOWN;
    
      case 2:  return ACTION_STR_RIGHT;
     
      case 3:  return ACTION_STR_LEFT;
     
      case 4:  return ACTION_STR_CLIMB;
     
      case 5:  return ACTION_STR_GRAB;

      case 6:  return ACTION_STR_SHOOT_UP;
     
      case 7:  return ACTION_STR_SHOOT_DOWN;
     
      case 8:  return ACTION_STR_SHOOT_RIGHT;

      case 9:  return ACTION_STR_SHOOT_LEFT;
     
     default:  return ACTION_STR_GRAB;
    }// end switch
  }

  public static Vector addTo ( Vector start, String insert )
  {
    if (insert != null)
      start.addElement( (Object) insert );
    return start;
  }

// Method: randomMove (public, static)
//
// *Purpose   : just calls the randomMove-method with no moves to exclude
//
// *Parameters: none
//   
// *Return    : String - the next move (random)
//
// *Variables : none
//
  public static String randomMove()
  {
    return randomMove( (String) null );
  }

// Method: randomMove (public, static)
//
// *Purpose   : generates a random movement action
//              * if all actions are excluded a null will be returned
//
// *Parameters: exclude (String, optional ) - an action that shall be 
//                                      excluded from the random generation
//          or: exclude (Vector, optional) - all actions to excluded
//         
//              no_climb_num (int, Array, optional) - the number of climbs 
//                                 to skip on all account
//
//              lastMove (String, optional ) - the last action is exclude
//                from the list of possible moves, only if no valid move
//                is possible the last move will be execuded (^= take
//                back the last move, i.e. at dead ends   
//   
// *Return    : String - the next move (random)
//
// *Variables : rnd (int) - holds the random number for the move
//
  public static String randomMove( String exclude )
  {
    int rnd;
    String tmpstr;
    boolean exclusion[] = { false, false, false, false, false };

    if ( exclude != null ) 
    {
      excludeMove ( exclude, exclusion );

/*
      if ( DEBUG )
      {
        System.out.print( "randomMove (String): excluding move >" + exclude + 
          "  result [array]: {" );
        int i;
        for ( i = 0 ; i < exclusion.length ; i++)
        {
          System.out.print( ">" + exclusion[i] + "<, " );
        }
        System.out.println( "}" );
      }
*/
    }

    rnd = ( int )java.lang.Math.abs( ( java.lang.Math.random() * 5 ) );

    return nextPossibleMove( rnd, exclusion, null );
  }// End of randomMove (String)


  public static String randomMove( Vector exclude )
  {
    int rnd;
    int i;
    String tmpstr;
    boolean exclusion[] = { false, false, false, false, false };

    if ( exclude != null ) {
//      if ( DEBUG )
//        System.out.print( "randomMove (Vector): excluding move " );

      for ( i = 0 ; i < exclude.size() ; i++ ) 
      {
        tmpstr = ( String ) ( exclude.elementAt( i ) );
        excludeMove ( tmpstr, exclusion );

//        if ( DEBUG )
//          System.out.print( ">" + tmpstr + "<, " );
      }

/*
      if ( DEBUG )
      {
        System.out.print(" result [array]: {" );
        for ( i = 0 ; i < exclusion.length ; i++)
        {
          System.out.print( ">" + exclusion[i] + "<, " );
        }
        System.out.println( "}" );
      }
*/
    }

    rnd = ( int )java.lang.Math.abs( ( java.lang.Math.random() * 5 ) );

    return nextPossibleMove( rnd, exclusion, null );
  }// End of randomMove (Vector)

  public static String randomMove( Vector exclude, int no_climb_num[] )
  {
    int rnd;
    int i;
    String tmpstr;
    boolean exclusion[] = { false, false, false, false, false };

    if ( exclude != null ) {
//      if ( DEBUG )
//        System.out.print( "randomMove (Vector): excluding move " );

      for ( i = 0 ; i < exclude.size() ; i++ ) 
      {
        tmpstr = ( String ) ( exclude.elementAt( i ) );
        excludeMove ( tmpstr, exclusion );

//        if ( DEBUG )
//          System.out.print( ">" + tmpstr + "<, " );
      }

/*
      if ( DEBUG )
      {
        System.out.print(" result [array]: {" );
        for ( i = 0 ; i < exclusion.length ; i++)
        {
          System.out.print( ">" + exclusion[i] + "<, " );
        }
        System.out.println( "}" );
      }
*/
    }

    rnd = ( int )java.lang.Math.abs( ( java.lang.Math.random() * 5 ) );

    return nextPossibleMove( rnd, exclusion, no_climb_num );
  }// End of randomMove (Vector, noclimb)

  public static String randomMove( Vector exclude, int no_climb_num[],
    String lastMove )
  {
    int rnd;
    int i;
    String tmpstr;
    boolean exclusion[] = { false, false, false, false, false };

//    if ( DEBUG )
//      System.out.print( "randomMove (Vector): excluding move " );

    if ( exclude != null ) {
      for ( i = 0 ; i < exclude.size() ; i++ ) 
      {
        tmpstr = ( String ) ( exclude.elementAt( i ) );
        excludeMove ( tmpstr, exclusion );

//        if ( DEBUG )
//          System.out.print( ">" + tmpstr + "<, " );
      }
    }

    if ( lastMove != null ) 
    {
      excludeMove ( lastMove, exclusion );

//      if ( DEBUG )
//        System.out.print( ">" + lastMove + "<, " );
    }

/*
    if ( DEBUG )
    {
      System.out.print(" result [array]: {" );
      for ( i = 0 ; i < exclusion.length ; i++)
      {
        System.out.print( ">" + exclusion[i] + "<, " );
      }
      System.out.println( "}" );
    }
*/
    rnd = ( int )java.lang.Math.abs( ( java.lang.Math.random() * 5 ) );

    tmpstr = nextPossibleMove( rnd, exclusion, no_climb_num );
    if ( tmpstr == null )
      tmpstr = lastMove;

    return tmpstr;
  }// End of randomMove (Vector, noclimb, lastmove)


// Method: excludeMove (public, static)
//
// *Purpose   : checks wheter the current action is a move and sets the
//              proper slot in the exclusion list to true (^= exclude)
//
// *Parameters: action (String) - the action to be excluded from the list
//              exclude (boolean, Array) - a list with  shots valid/invalid
//
// *Return    : String - the next possible shot (in a sequence)
//
// *Variables : nextshot (String) - holds the current action
//
  public static void excludeMove(String action, boolean exclude[])
  {
    if ( (exclude == null) || (exclude.length < 5) || (action == null) )
      return;

    if (  action.equalsIgnoreCase( ACTION_STR_UP ) )
       exclude[0] = true;
    if (  action.equalsIgnoreCase( ACTION_STR_RIGHT ) )
       exclude[1] = true;
    if (  action.equalsIgnoreCase( ACTION_STR_DOWN ) )
       exclude[2] = true;
    if (  action.equalsIgnoreCase( ACTION_STR_LEFT ) )
       exclude[3] = true;
    if (  action.equalsIgnoreCase( ACTION_STR_CLIMB ) )
       exclude[4] = true;

  }// End of excludeMove


// Method: nextPossibleMove (public, static)
//
// *Purpose   : returns the next possible shooting action in a sequence
//              of five actions 
//              * if all five actions are excluded (null) will be returned
//
// *Parameters: num (int) - the number shot tried
//              exclude (boolean, Array) - a list with move valid/invalid
//              no_clim_numb (int, Array)
//              alternate (String) - an action to execute if no move is 
//                possible valid/invalid
//
// *Return    : String - the next possible shot (in a sequence)
//
// *Variables : nextmove (String) - holds the current action
//
  public static String nextPossibleMove( int num, boolean exclude[], 
                                         int no_climb_num[] )
  {
    String nextmove = null;
    int i;

    if ( exclude.length != 5 ) 
      return null;

    for( i = 0 ; (i < 2) && (nextmove == null) ; i++ )
    {
      if ( num == 0 )
        if ( exclude[0] == false )
          nextmove = ACTION_STR_UP;
        else
          num++;
 
      if ( num == 1 )
        if ( exclude[1] == false )
          nextmove = ACTION_STR_RIGHT;
        else
          num++;

      if (num == 2 )
        if ( exclude[2] == false )
          nextmove = ACTION_STR_DOWN;
        else
          num++;
 
      if ( num == 3 )
        if ( exclude[3] == false )
          nextmove = ACTION_STR_LEFT;
        else
          num++;

      if ( num == 4 )
        if ( exclude[4] == false )
          if ( (no_climb_num != null)    &&
               (no_climb_num.length > 0) && 
               (no_climb_num[0] > 0)     )
          {
            exclude[4] = true;
            no_climb_num[0] -= 1;
            num++;
            if (i > 0) i--; // it might is the second round already...
          }
          else
            nextmove = ACTION_STR_CLIMB;
        else
          num++;

      if ( (num > 4) || (num < 0) ) // try a second time for a possibility
        num = 0;
    }

//    if ( DEBUG )
//      System.out.println(" next possible move >" +nextmove+ "< ...");

    return nextmove; 
  }// End of nextPossibleMove


  public static String nextPossibleMove( 
    int num, boolean exclude[], int no_climb_num[], String alternate)
  // skips climb - when possible
  {
    String nextmove = null;
    int i;
    boolean tryother = false;

    if ( exclude.length != 5 ) 
      return null;

    for( i = 0 ; (i < 3) && (nextmove == null) ; i++ )
    {
      if ( num == 0 )				// =^ Up
        if ( exclude[0] == false )
          nextmove = ACTION_STR_UP;
        else {
          num++;
          tryother = true;
        }
 
      if ( num == 1 )				// =^ Right
        if ( exclude[1] == false )
          nextmove = ACTION_STR_RIGHT;
        else {
          num++;
          tryother = true;
        }

      if (num == 2 )				// =^ Down
        if ( exclude[2] == false )
          nextmove = ACTION_STR_DOWN;
        else {
          num++;
          tryother = true;
        }
 
      if ( num == 3 )				// =^ Left
        if ( exclude[3] == false )
          nextmove = ACTION_STR_LEFT;
        else {
          num++;
          tryother = true;
        }

      if ( tryother ) {  // is this a case of handing over the current number?
        if ( i == 0 ) {  // Yes? - then exclude Climb for the first pass
          num = 0;        
        }
        else {
          if ( alternate != null )
            nextmove = alternate;
          // else try the climb move...
        }
      }

      if ( num == 4 )				// =^ Climb
        if ( exclude[4] == false )
          if ( (no_climb_num != null)    &&
               (no_climb_num.length > 0) && 
               (no_climb_num[0] > 0)     )
          {
            exclude[4] = true; // do not try the climb again in this method
            no_climb_num[0] -= 1;
            num++;
            if (i > 0) i--; // it might is the second round already...
          }
          else
            nextmove = ACTION_STR_CLIMB;
        else
          num++;

      if ( (num > 4) || (num < 0) ) // try a second time for a possibility
        num = 0;
    }

//    if ( DEBUG )
//      System.out.println(" next possible move >" +nextmove+ "< ...");

    return nextmove; 
  }// End of nextPossibleMove


// Method: negateMove (public, static)
//
// *Purpose   : negates the given move
//              * if no movement action is given null will be returned
//
// *Parameters: move (String) - the move to negate
//
// *Return    : String - the negated move taken
//
// *Variables : none
//
  public static String negateMove( String move )
  {
    if ( move != null )
    {
      if ( move.equalsIgnoreCase( ACTION_STR_UP ) )
        return ACTION_STR_DOWN;

      if ( move.equalsIgnoreCase( ACTION_STR_RIGHT ) )
        return ACTION_STR_LEFT;

      if ( move.equalsIgnoreCase( ACTION_STR_DOWN ) )
        return ACTION_STR_UP;

      if ( move.equalsIgnoreCase( ACTION_STR_LEFT ) )
        return ACTION_STR_RIGHT;
    }

    return null;
  }// End of negateMove

// Method: checkClimbOut (public, static)
//
// *Purpose   : checks whether there is an action , usefull to check
//              last actions and execute a climb-out if no possible action
//              would be the result...
//
//              - if there is none (action == null) the Climb-action
//                will be returned
//
// *Parameters: action (String) - the action to execute
//
// *Return    : String - the next action
//
// *Variables : none
//
  public static String checkClimbOut( String action )
  {
    if (action == null)
      return ACTION_STR_CLIMB;
    else
      return action;
  }


// ------------------------------------------------------------------------
// SHOOT - Support
// ------------------------------------------------------------------------

// Method: randomShot (public, static)
//
// *Purpose   : just calls the randomShot-method with no shots to exclude
//
// *Parameters: none
//   
// *Return    : String - the shooting action (random)
//
// *Variables : none
//
  public static String randomShot()
  {
    return randomShot( null );
  }

// Method: randomShot (public, static)
//
// *Purpose   : generates a random shooting action
//
// *Parameters: exclude (Vector) - all shots the agent shall not make
//              lastMove (String, optional) - the last valid movement action
//                exclude the field too from the shoot-direction
//                (It's SAVE - agent came from it & is still alive :-) 
//
// *Return    : String - the next shot (random)
//
// *Variables : rnd (int) - holds the random number for the shot
//
  public static String randomShot( Vector exclude )
  {
    int rnd;
    int i;
    String tmpstr;
    
    boolean exclusion[] = { false, false, false, false };

    if ( exclude != null )
      for ( i = 0 ; i < exclude.size() ; i++ )
        excludeShot ( ( String ) ( exclude.elementAt( i ) ), exclusion );

/*
    if ( DEBUG )
    {
      System.out.print(" exclusion result [array]: {" );
      for ( i = 0 ; i < exclusion.length ; i++)
      {
        System.out.print( ">" + exclusion[i] + "<, " );
      }
      System.out.println( "}" );
    }
*/
    rnd = ( int ) java.lang.Math.abs( ( java.lang.Math.random() * 4 ) );

    return nextPossibleShot( rnd, exclusion );
  }// End of randomShot

  public static String randomShot( Vector exclude, String lastMove )
  {
    int rnd;
    int i;
    String tmpstr;
    
    boolean exclusion[] = { false, false, false, false };

    if ( exclude != null )
      for ( i = 0 ; i < exclude.size() ; i++ )
        excludeShot ( ( String ) ( exclude.elementAt( i ) ), exclusion );

    excludeShot ( noNeedToShoot( lastMove ) , exclusion );

/*
    if ( DEBUG )
    {
      System.out.print(" exclusion result [array]: {" );
      for ( i = 0 ; i < exclusion.length ; i++)
      {
        System.out.print( ">" + exclusion[i] + "<, " );
      }
      System.out.println( "}" );
    }
*/

    rnd = ( int ) java.lang.Math.abs( ( java.lang.Math.random() * 4 ) );

    return nextPossibleShot( rnd, exclusion );
  }// End of randomShot

  public static String randomShot( String exclude, String lastMove )
  {
    int rnd;
    int i;
    String tmpstr;
    
    boolean exclusion[] = { false, false, false, false };

    excludeShot ( exclude, exclusion );

    excludeShot ( noNeedToShoot( lastMove ) , exclusion );

/*
    if ( DEBUG )
    {
      System.out.print(" exclusion result [array]: {" );
      for ( i = 0 ; i < exclusion.length ; i++)
      {
        System.out.print( ">" + exclusion[i] + "<, " );
      }
      System.out.println( "}" );
    }
*/

    rnd = ( int ) java.lang.Math.abs( ( java.lang.Math.random() * 4 ) );

    return nextPossibleShot( rnd, exclusion );
  }// End of randomShot (single-shot)

// Method: excludeShot (public, static)
//
// *Purpose   : checks wheter the current action is a shooting action and 
//              sets the proper slot in the exclusion list 
//              to true (^= exclude)
//
// *Parameters: shot (String) - the shot to be excluded from the list
//              exclude (boolean, Array) - a list with shots valid/invalid
//
// *Return    : none
//
// *Variables : nextshot (String) - holds the current action
//
  public static void excludeShot( String shot, boolean exclude[] )
  {
    if ( (exclude == null ) || ( exclude.length < 4 ) || ( shot == null ) )
      return;

    if ( shot.equalsIgnoreCase( ACTION_STR_SHOOT_UP ) )
    {
      exclude[0] = true;
      return;
    }
    if ( shot.equalsIgnoreCase( ACTION_STR_SHOOT_RIGHT ) )
    {
      exclude[1] = true;
      return;
    }
    if ( shot.equalsIgnoreCase( ACTION_STR_SHOOT_DOWN ) )
    {
      exclude[2] = true;
      return;
    }
    if ( shot.equalsIgnoreCase( ACTION_STR_SHOOT_LEFT ) )
    {
      exclude[3] = true;
      return;
    }
  }// End of excludeShot

// Method: noNeedToShoot (public, static)
//
// *Purpose   : returns the Shooting-direction with is SAVE
//              argumentation line/rules:
//                agent came from a field through the last move-action
//                agent is alive -> it's SAVE
//              i.e.: 
//                Step 1: agent-A in field (1,1): move right
//                Step 2: agent-A in field (2,1): Stench-Percep -> Wumpus
//                        call noNeedToShoot( "RIGHT" ) => shoot_left
//                               no need to shoot left
//                               but up, down, right are unsave...
//
// *Parameters: exclude (Vector) - all shots the agent shall not make
//
// *Return    : String - the next shot (random)
//
// *Variables : rnd (int) - holds the random number for the shot
//
  public static String noNeedToShoot ( String move )
  {
    if ( move == null ) return null;

    if ( move.equalsIgnoreCase( ACTION_STR_UP ) )
      return ACTION_STR_SHOOT_DOWN;

    if ( move.equalsIgnoreCase( ACTION_STR_RIGHT ) )
      return ACTION_STR_SHOOT_LEFT;

    if ( move.equalsIgnoreCase( ACTION_STR_DOWN ) )
      return ACTION_STR_SHOOT_UP;

    if ( move.equalsIgnoreCase( ACTION_STR_LEFT ) )
      return ACTION_STR_SHOOT_RIGHT;

    return null;
  }

// Method: nextPossibleShot (public, static)
//
// *Purpose   : returns the next possible shooting action in a sequence
//              of four 
//              * if all four actions are excluded (null) will be returned
//
// *Parameters: num (int) - the number shot tried
//              exclude (boolean, Array) - a list with  shots valid/invalid
//
// *Return    : String - the next possible shot (in a sequence)
//
// *Variables : nextshot (String) - holds the current action
//
  public static String nextPossibleShot( int num, boolean exclude[] )
  {
    String nextshot = null;
    int i = 0;

    if ( exclude.length != 4 ) 
      return null;

    while( (i++ < 2) && (nextshot == null) )
    {
      if ( num == 0 )
        if ( exclude[0] == false )
          nextshot = ACTION_STR_SHOOT_UP;
        else
          num++;
 
      if ( num == 1 )
        if ( exclude[1] == false )
          nextshot = ACTION_STR_SHOOT_RIGHT;
        else
          num++;

      if (num == 2 )
        if ( exclude[2] == false )
          nextshot = ACTION_STR_SHOOT_DOWN;
        else
          num++;
 

      if ( num == 3 )
        if ( exclude[3] == false )
          nextshot = ACTION_STR_SHOOT_LEFT;
        else
          num++;

      if ( (num > 3) || (num < 0)) // try a second time for a possibility
        num = 0;
    }

//    if ( DEBUG )
//      System.out.println(" next possible shot >" +nextshot+ "< ...");

    return nextshot; 
  }// End of nextPossibleShot


}// End of WumpusLib
